<?

  /**
  * LFXlib development tools
  * Bogdan Stancescu <bogdan@lanifex.com>, December 2002
  * $Id: LFX_development.php,v 1.17 2005/11/10 10:38:13 bogdan Exp $
  * @package LFX
  */
    /**
    * This will dump the parameter you pass and will *return* the dump.
    *
    * Also handles HTML stuff nicely.
    */
  function varDump($variable)
  {
    $result.="<pre>";
    ob_start();
    print_r($variable);
    $result.=htmlentities(ob_get_contents());
    ob_end_clean();
    $result.="</pre>";
    return($result);
  }

    /**
    * This function starts a timer - use {@link getWatch} afterwards to
    * get the time elapsed.
    *
    * Calling this function again with the same parameter will reset the
    * respective watch.
    *
    * @param string $timer the name of the timer to start
    */
  function startWatch($timer)
  {
    global $_LFX;
    list($usec, $sec) = explode(" ",microtime());
    $_LFX["page"]["timers"][$timer]=(float)$usec + (float)$sec;
    return(true);
  }

    /**
    * This function returns the time elapsed in seconds since the named timer was started
    * with {@link startWatch}
    *
    * @param string $timer the timer to read
    * @return float the number of seconds elapsed since the timer was started.
    */
  function getWatch($timer)
  {
    global $_LFX,$_page;
    if (!isset($_LFX["page"]["timers"][$timer])) {
      $_page->debug("TIMER: Warning: timer $timer not initialized. Starting it now and ignoring request. (in getWatch())");
      startWatch($timer);
      return(0);
    }
    list($usec, $sec) = explode(" ",microtime());
    return(((float)$usec + (float)$sec)-$_LFX["page"]["timers"][$timer]);
  }
  
    /**
    * This function writes a new LFXlib system message
    *
    * @param string $message the message to be recorded
    * @param string $severity the recommended set of values is
    *   "debug", "info", "warning", "error" and "critical", but the parameter
    *   itself is a free-form string, and will be recorded as such. The severity
    *   is case-insensitive (auto-convert to lower)
    * @param array $extra an associative array which can contain several extra
    *   information, identified by the array keys. The possible keys can be:
    *   * 'data_dump' - mixed - you can specify whatever value here, and it will
    *     be stored as such
    *   * 'module' - string - if not specified, global
    *     $_LFX['global']['application'] is used
    *     The module name is case-insensitive (auto-convert to upper case).
    *   * 'subsystem' - string - if you don't specify any value here, global
    *     $_LFX['global']['subsystem']; please note that this must be
    *     either manually defined by your script, or defined via
    *     {@link LFX_subsystemStart}()!
    *     The subsystem name is case-insensitive (auto-convert to lower case).
    *   * 'url' - string -  if not specified, the script name and parameters are
    *     determined via $REQUEST_URI and debug_backtrace()
    *   * 'keywords' - string - a comma-separated list of keywords; the recommended
    *     set of keywords is:
    *     * 'subsystem_start' -- typically defined by {@LFX_subsystemStart}()
    *     * 'subsystem_end' -- typically defined by {@LFX_subsystemEnd}()
    *     * 'subsystem_sematch' -- typically defined by the "smart parser"
    *       (not implemented yet)
    * @param integer $backtraceCount this is a private parameter which you
    *   shouldn't need to use; it's used internally in LFXlib to let this
    *   function know how many backtrace entries to skip before reaching
    *   the entry which describes the call from the production code to this
    *   or a related function.
    *
    * @return boolean true on success or false on problems
    */

    function LFX_LogMessage($message,$severity='info',$extra=array(),$backtraceCount=0)
    {
      // Class check
      if (!LFX_include_class('LFX_Message')) {
        global $_page;
        $_page->addError("Failed to include class LFX_Message (in LFX_LogMessage)");
        return(false);
      }

      // Object init
      $msg=new LFX_Message;
      $msg->setAttr('date',time());

      // Message
      $msg->setAttr('message',$message);
      
      // Severity
      $msg->setAttr('severity',strtolower($severity));
      
      // User
      global $_page;
      $msg->setAttr('user',$_page->user->id);

      // Module
      if (!$extra['module']) {
        global $_LFX;
        $extra['module']=$_LFX['global']['application'];
      }
      $msg->setAttr('module',strtoupper($extra['module']));
      
      // Keywords
      if ($extra['keywords']) {
        $msg->setAttr('keywords',$extra['keywords']);
      }
      
      // Subsystem
      if (!$extra['subsystem']) {
        global $_LFX;
        $extra['subsystem']=$_LFX['global']['subsystem'];
      }
      $msg->setAttr('subsystem',strtolower($extra['subsystem']));
      
      // Data dump
      if ($extra['data_dump']) {
        $msg->setAttr('data_dump',serialize($extra['data_dump']));
      }
      
      // URL
      if (!$extra['url']) {
        if (function_exists('debug_backtrace')) {
          $backtrace=debug_backtrace();
          $extra['url']='FILE:'.$backtrace[$backtraceCount+1]['file'].':'.$backtrace[$backtraceCount+1]['line'];
        } else {
          global $_page;
          $_page->addError('Failed to determine the URL via debug_backtrace() (which is not available), and it was also not specified!');
        }
      }
      $msg->setAttr('url',$extra['url']);
      
      // Save object
      return((bool) $msg->save());
    }

      /**
      * This function is used as an output layer for CLI scripts. All parameters
      * are identical to those for {@link LFX_LogMessage}(). However, if global
      * $_LFX['page']['output_type'] is set to 'console', then the parameters
      * are used to generate a line of output in the console (via echo()).
      * If it's set to 'LFX_Messages', the parameters are passed to
      * LFX_LogMessage, which logs them into the messages table. Otherwise,
      * the function outputs both in the console and in the logs a string
      * which warns that an invalid setting has been used. This is to ensure
      * that such coding problems get caught and fixed.
      *
      * One can use this function in conjunction with
      * {@link LFX_parseCLIOutputParams}(), which checks the CLI parameters for
      * a specific set of common parameters, and applies those settings.
      */
    function LFX_CLIOutput($message,$severity='info',$extra=array(),$backtraceCount=0)
    {
      global $_LFX;
      switch($_LFX['page']['output_type']) {
        case 'console':
          if ($severity=='info') {
            echo("$message\n");
          } elseif ($severity=='debug') {
            echo("($message)\n");
          } else {
            echo(strtoupper($severity).": $message\n");
          }
        break;
        case 'LFX_Messages':
          LFX_LogMessage($message,$severity,$extra,$backtraceCount+1);
        break;
        default:
          echo(strtoupper($severity).": $message\n");
          echo("warning: CLI output not defined in this script, ".
            "sending everywhere; use function LFX_parseCLIOutputParams() ".
            "in your code (in LFX_CLIOutput)\n");
          LFX_LogMessage($message,$severity,$extra,$backtraceCount+1);
          LFX_LogMessage("CLI output not defined in this script, ".
            "sending everywhere; use function LFX_parseCLIOutputParams() ".
            "in your code",'warning',NULL,NULL,NULL,NULL,$backtraceCount+1);
        break;
      }
    }
    
      /**
      * A method similar to pecho, which is meant for CLI scripts. The only
      * real difference between the two is the second parameter in this one:
      * if set to a value which evaluates true, and if the system runs
      * with the output set anywhere but the console, then the respective
      * message is skipped altogether. If any of the two conditions fail,
      * the message is sent to {@link LFX_CLIOutput}() with the default
      * parameters.
      *
      * @param string $message the message to output or record
      * @return mixed the output of function LFX_CLIOuput or NULL if the
      *   message is optional and didn't get to executing LFX_CLIOutput
      */
    function cecho($message,$optional=false)
    {
      global $_LFX;
      if (!$optional || ($_LFX['page']['output_type']=='console')) {
        return(LFX_CLIOutput($message,'info',array(),1));
      } else {
        return(NULL);
      }
    }

      /**
      * This function parses the current CLI script's parameters and applies
      * an "LFXlib-default" set of parameters from them.
      *
      * Currently, the following possible parameters are parsed:
      * --output=console -- sets the LFX_CLIOutput output to the console
      * --output=LFX_Messages -- sets the LFX_CLIOutput output to the LFXlib messages log in the database
      * (if none is set, the default is console)
      */
    function LFX_parseCLIOutputParams()
    {
      global $_LFX;
      $_LFX['page']['output_type']='console';
      for($i=0;$i<count($_SERVER['argv']);$i++) {
        switch ($_SERVER['argv'][$i]) {
          case '--output=LFX_Messages':
            $_LFX['page']['output_type']='LFX_Messages';
          break;
          case '--output=console':
            $_LFX['page']['output_type']='console';
          break;
        }
      }
      return(true);
    }
    
      /**
      * A function which records the initialization of a subsystem, and
      * performs a few default initialization steps. A "subsystem" in LFXlib
      * is usually a CLI script. Subsystem examples include the harvester
      * script, the subscription delivery script and the request escalation one.
      *
      * @param string $subsystem uniquely identifies the script within the
      *   module; if the parameter is null, the base name of the script is
      *   used (usually that's enough identification for most subsystems)
      * @param array $ignores an associative array which stores the default
      *   subsystem start operations to ignore. The operation itself is
      *   identified by the key, and the fact that it should be ignored is
      *   indicated by associating boolean TRUE with the key. The possible
      *   keys are:
      *   * 'parseCLIOutputParams' -- skips executing
      *       {@link LFX_parseCLIOutputParams}()
      *   * 'lockCheck' -- skips performing a lock check for the subsystem;
      *       please note that if this is not ignored and a valid lock is
      *       found, then this function itself will warn and exit execution!
      *   * 'lockRegister' -- skips registering a lock for the subsystem
      *   * 'lock' -- skips both the lock check and the lock register
      *   * 'messages' -- skips registering the subsystem in LFX_Messages
      *
      * @return boolean true if all required operations were executed
      *   successfully or false otherwise
      */
    function LFX_subsystemStart($subsystem=NULL,$ignores=array())
    {
      global $_LFX;
      $success=true;

      // Subsystem name
      if ($subsystem===NULL) {
        $backtrace=debug_backtrace();
        $subsystem=basename($backtrace[0]['file'],'.php');
      }

      // CLI Parameters
      if (!$ignores['parseCLIOutputParams']) {
        $success=$success && LFX_parseCLIOutputParams();
      }

      // Checking for other subsystems already registered
      if ($_LFX['global']['subsystem']) {
        LFX_CLIOutput("A subsystem is already registered! ({$_LFX['global']['subsystem']} is registered, and LFX_subsystemStart was called to register $subsystem)","error",array(),1);
        return(false);
      }
      
      // Registering the subsystem in the globals
      $_LFX['global']['subsystem']=$subsystem;
      
      // Lock ignores parsing
      if ($ignores['lock']) {
        $ignores['lockCheck']=true;
        $ignores['lockRegister']=true;
      }
      
      // Lock check
      if (!$ignores['lockCheck']) {
        if (!LFX_include_class('LFX_Lock')) {
          global $_page;
          $_page->addError("Failed to include class LFX_Message (in LFX_subsystemStart)");
          $success=false;
        } else {
          $lock=new LFX_lock($subsystem);
          $lockState=$lock->check();
          if ($lockState=='locked') {
            cecho('Lock in place - exiting\n');
            exit;
          } elseif ($lockState=='dead') {
            LFX_CLIOutput("Dead lock for subsystem $subsystem found on the system! Removed and continuing. (in LFX_subsystemStart)",'warning',array(),1);
            $lock->unlock(1);
          }
        }
      }
      
      if (!$ignore['lockRegister']) {
        $lock=new LFX_lock($subsystem);
        $lock->lock();
      }

      // Registering the subsystem in the messages
      if (!$ignores['messages']) {
        if (!LFX_include_class('LFX_Message')) {
          global $_page;
          $_page->addError("Failed to include class LFX_Message (in LFX_subsystemStart)");
          $success=false;
          $ignores['messages']=true;
        } else {
          $msgSuccess=LFX_LogMessage("Subsystem $subsystem started",
            "info",array('keywords'=>'subsystem_start'),1);
          if (!$msgSuccess) {
            $ignores['messages']=true;
          }
          $success=$success && $msgSuccess;
        }
      }

      $_LFX['global']['subsystem_ignores']=$ignores;
      
      if ($success) {
        cecho("Subsystem $subsystem successfully started.",true);
      } else {
        cecho("Subsystem $subsystem has started with some registration problems.");
      }

      // Done
      return($success);
    }

      /**
      * This function unregisters a subsystem previously registered with
      * {@link LFX_subsystemStart}(). No parameters are needed to identify
      * the subsystem or ignores, the ones for subsystem start are used.
      *
      * @param boolean $die if true (default is false), then this function will
      *   also stop execution, so you don't have to execute an exit() call
      *   in your code.
      * @return boolean true if all required steps for unregistration are
      *   successful and false if any of them fails.
      */
    function LFX_subsystemEnd($die=false)
    {
      global $_LFX;
      $success=true;
      
      if (!$subsystem=$_LFX['global']['subsystem']) {
        LFX_parseCLIOutputParams();
        LFX_CLIOutput("No subsystem is registered! (in LFX_subsystemEnd)","error",array(),1);
        return(false);
      }
      
      $ignore=$_LFX['global']['subsystem_ignores'];
      
      if (!$ignore['lockRegister']) {
        $lock=new LFX_Lock($subsystem);
        $success=$success && $lock->unlock();
      }
      
      if (!$ignore['messages']) {
        $success=$success && LFX_LogMessage("Subsystem $subsystem finished",
            "info",array('keywords'=>'subsystem_end'),1);
      }
      
      if ($success) {
        cecho("Subsystem $subsystem successfully finished.",true);
      } else {
        cecho("Subsystem $subsystem has finished with some registration problems.");
      }

      unset($_LFX['global']['subsystem'],$_LFX['global']['subsystem_ignores']);
      
      if ($die) {
        exit;
      }
      return($success);
    }
    
      /**
      * A simple function to show the current number of queries and timing,
      * along with some labels to identify each one. The id is simply another
      * identificator for each entry, for instance if you run this into a loop.
      *
      * You need to have the debug set to a non-null value to see the output.
      *
      * A future version of this function might use the id to distinguish between
      * entries with the same level, and present consolidated results for that "group"
      * at the end of the page. Not implemented yet because there was no real
      * need for this functionality at the moment.
      *
      * @param string $comment a comment (a label)
      * @param integer $id a sub-identifier, as described above
      * @return void
      */
    function queryCountDebug($comment='(no comment)',$id=0)
    {
      global $_LFX,$_page;
      if ($id) {
        $niceID=" ($id)";
      }
      $last_time=getWatch('LFX_global');
      $last_delta=$last_time-$_LFX['page']['timer_query']['last'];
      $RAM_data='';
      if (function_exists('memory_get_usage')) {
        $memory_delta=memory_get_usage()-$_LFX['page']['memory_usage_cache'];
        $_LFX['page']['memory_usage_cache']=memory_get_usage();
        $RAM_data="; RAM usage: ".niceSize(memory_get_usage())." (delta=".niceSize($memory_delta).")";
      }
      $_LFX['page']['timer_query']['last']=$last_time;
      $_page->debug("<b>QUERIES</b> $comment$niceID: ".
        "{$_LFX['global']['queryCount']} @ sec ".
        number_format($last_time,2).
        " (delta=<b>".number_format($last_delta,3)."</b> sec)".$RAM_data);
    }
    
    function decho($string)
    {
      echo("$string<br />\n");
    }
?>
