<?

  /**
   * Generic LFX functions
   * Bogdan Stancescu <bogdan@lanifex.com>, January 2003
   * $Id: LFX_generic.php,v 1.24 2005/07/14 00:09:57 bogdan Exp $
   * @package LFX
   */

    /**
     * Creates a generic SC descendant.
     * @param string $genericClass the class to extend
     * @param string $classToCreate the class to create now
     * @param string $tableName the table to attach the new class to
     * @param string $idField the name of the "id" field in $tableName
     */
  function LFX_generic($genericClass, $classToCreate, $tableName, $idField, $dbDataSet=NULL)
  {
    if (class_exists($classToCreate)) {
      global $_page;
      $_page->addError("Class $classToCreate already exists! (in LFX_generic())");
      return(false);
    }
    if($dbDataSet != NULL){
      #echo "<pre>This is $dbDataSet";
      $dbDataSet = "\$this->dbData[\"dbDataSet\"]=\"$dbDataSet\";";
    }else{
      $dbDataSet = "";
      #echo "This is default";
    }
    $code="
      class $classToCreate extends $genericClass
      {
        function $classToCreate(\$id=0)
        {
          \$this->$genericClass(\$id);
          unset(\$this->db);
          #unset(\$this->dbData);
          $dbDataSet
          \$this->dbInit(); 
          #var_dump(\$this->db);
          \$this->query('select 1');
          \$fields=mysql_list_fields(\$this->db->Database,
            \"".(addslashes($tableName))."\", \$this->db->Link_ID);
          \$columns = mysql_num_fields(\$fields);
          for (\$i = 0; \$i < \$columns; \$i++) {
            \$tmp[mysql_field_name(\$fields, \$i)]=array();
          }
          \$this->dataStructure=array(
            \"table_name\"=>\"".(addslashes($tableName))."\",
            \"id_field\"=>\"".(addslashes($idField))."\",
            \"fields\"=>\$tmp
          );
          \$this->fillDataStructure();
        }
      }
    ";
    eval($code);
    return(true);
  }

    /**
    * Parses the current $PATH_INFO.
    * Returns an array of path elements with no empty values and no holes
    * @return array indexed array of path elements
    */
  function parsePath()
  {
    global $PATH_INFO;
    $myPath=array_filter((explode('/',$PATH_INFO)),'strlen');
    $result=array();
    while (list(,$dir)=@each($myPath)) {
      $result[]=$dir;
    }
    return($result);
  }
  
    /**
    * Performs an include using LFXlib's Include Manager.
    * Please note this is a function, so if you need to include files
    * which need to work in the global scope you must either make sure
    * they do from within the include, or perform the include in some
    * other way (e.g. manually using $_incReq and $_incMgr from your
    * script's global scope).
    *
    * You can pass any number of string arguments to this function,
    * it will
    */
  function LFX_include()
  {
    global $_incReq,$_incMgr;
    //echo("Before: ".vardump($_incReq));
    // Don't merge the two statements below, seems like some version of PHP was
    // picky about this, we don't need the extra trouble for one local variable.
    $keys=func_get_args();
    $_incReq=array_merge($_incReq,$keys);
    //echo("After: ".vardump($_incReq));
    return(include($_incMgr));
  }
  
    /**
    * Checks whether an include key is valid.
    * @param string $key the key to check
    * @return boolean true if valid, false if invalid
    */
  function LFX_include_probe($key)
  {
    global $_LFX;
    if (!isset($_LFX['global']['master_include_array'][strtolower($key)])) {
      return(false);
    }
    $file=$_LFX['global']['master_include_array'][strtolower($key)];
    if (!is_file($file)) {
      return(false);
    }
    if (!is_readable($file)) {
      return(false);
    }
    return(true);
  }

    /**
    * This function attempts to load a class, if not already loaded,
    * by using its name as an include key.
    *
    * @param string $class the class name
    * @return boolean true if class already exists or if successfully loaded,
    *   or false if unable to load it
    */
  function LFX_include_class($class)
  {
    $classes=func_get_args();
    if (count($classes)>1) {
      return(LFX_include_classes($classes));
    }
    $class=strtolower($class);
    if (class_exists($class)) {
      return(true);
    }
    if (LFX_include_probe($class)) {
      LFX_include($class);
    }
    if (!class_exists($class)) {
      return(false);
    }
    return(true);
  }
  
    /**
    * While this function can be called directly to include multiple classes,
    * it's much more convenient to use {@link LFX_include_class} directly;
    * this actually is a helper function for LFX_include_class.
    *
    * @param array $classes
    * @result boolean true if all classes have been included successfully, or
    *   false otherwise
    */
  function LFX_include_classes($classes)
  {
    $success=true;
    for($i=0;$i<count($classes);$i++) {
      $success&=LFX_include_class($classes[$i]);
    }
    return($success);
  }
  
    /**
    * This function registers foreign dependencies (cross-module dependencies)
    *
    * This function is used when some LFXlib module links to classes in another.
    * In that case you can't just define a dependency in the "other" module,
    * because you have no guarantee that the module which includes the link
    * is actually installed. Therefore, when linking from, say, OPT_Document
    * to LFX_User, you should register the dependency of LFX_user via this
    * function *within* OPT, instead of defining the dependency within LFXlib,
    * since you have no guarantee that OPT will always be installed. That way
    * you can ensure consistency regarding the dependencies from the proper end
    * of the dependency: if the module including the link is installed (in this
    * case, OPT), the dependency is registered. Otherwise, it's neither
    * registered, nor needed (LFXlib objects don't need to define any
    * dependencies on OPT objects as long as OPT is not installed).
    *
    * The recommended method of registering the foreign dependency is to
    * create a separate library in each module, listing the dependencies
    * as calls to this function, and including that library in the module's
    * Basic include key set.
    *
    * The first parameter is the class to which the new dependency is to be
    * attached, and the second is the actual dependency array, structured
    * as usual within any SC descendant's constructor.
    *
    * What this function actually does is defining the relationship between
    * the class name and the dependency structure in a global array. The SC
    * checks for foreign dependencies matching the current class name within
    * SC::fillDataStructure(), and appends the proper dependencies structure
    * to the current object, if any are found.
    *
    * Since this function doesn't do anything which might fail, its result
    * is always true.
    *
    * @param string $class the class to attach the dependency to
    * @param array $dep the dependency descriptor
    * @return boolean true on success
    */
  function LFX_register_dependency($class,$dep)
  {
    global $_LFX;
    $class=strtolower($class);
    if (is_array($_LFX['global']['foreign_deps'][$class])) {
      $_LFX['global']['foreign_deps'][$class]=array_merge(
        $_LFX['global']['foreign_deps'][$class],$dep);
    } else {
      $_LFX['global']['foreign_deps'][$class]=$dep;
    }
    return(true);
  }
  
    /**
    * This function ensures that a directory is useable for PHP,
    * creating any number of parent directories needed, and checking if the
    * final directory is readable and writable. If the second parameter
    * evaluates to true, the "readability" test is skipped.
    *
    * The function returns true only if the directory is available for use,
    * and returns false if it encounters any kind problem (the directory or
    * any of its parents aren't readable or can't be created, etc).
    *
    * @param string $dirName the full path to the directory to be used
    * @param boolean $writeOnly whether the directory will be used for writing
    *   only (default false)
    * @result boolean true on success or false otherwise
    */
  function LFX_use_directory($dirName,$writeOnly=false)
  {
    global $_page;
    if (!$dirName) {
      global $_page;
      $_page->addError("Directory not specified!");
      return(false);
    }
    if ((!is_dir($dirName)) && (!LFX_use_directory(dirname($dirName)))) {
      return(false);
    }
    if (!is_dir($dirName)) {
      if (!@mkdir($dirName,0750)) {
        $_page->addError("Directory $dirName could not be created!");
        return(false);
      }
    }
    if (!$writeOnly && !is_readable($dirName)) {
      $_page->addError("Directory $dirName is not readable!");
      return(false);
    }
    if (!is_writable($dirName)) {
      $_page->addError("Directory $dirName is not writable!");
      return(false);
    }
    return(true);
  }

    /**
    * A simple function to register the current subsystem. Should be used in
    * every major subsystem, because it stores and uses it in LFX_Messages.
    * If you use a unique base script for several other scripts within one
    * subsystem, you can register the subsystem only once, in that common script.
    * Otherwise you'll have to register it in every script within a subsystem.
    *
    * @param string $subsystem the name of the subsystem to register;
    *   please keep it short (5-20 chars) but descriptive and reasonably
    *   unambiguous, as to avoid clashing with other subsystems. Remember that
    *   LFX_Messages also stores the module (LFX, OPT, DMO, etc), so you could
    *   clash with subsystem names from other modules and that wouldn't be a
    *   problem, just don't clash within the module you're working on.
    *
    * @return void
    */
  function LFX_RegisterSubsystem($subsystem)
  {
    global $_LFX;
    $_LFX['global']['subsystem']=$subsystem;
  }
  
    /**
    * This function initializes a variable. It's basically a one-line, easy
    * to read replacement for
    *
    * if (!$var) {
    *   $var=$val;
    * }
    *
    * It also accepts an optional third "compare" parameter, in which case
    * it behaves like
    *
    * if ($var!==$compare) {
    *   $var=$val;
    * }
    *
    * It returns true if it initializes the variable or false if it doesn't
    * (i.e. if the variable already contained something).
    *
    * Please note that due to the way PHP works, this function also works
    * properly on array elements: LFX_initvar($foo['bar'],3) will even work
    * on an unset $foo, changing it to array('bar'=>3).
    *
    * @param mixed $var the variable to initialize
    * @param mixed $val the value to initialize it to
    * @param mixed $compare the pseudo-null value to compare to in order to
    *   decide whether to initialize the variable.
    * @return boolean true if initialized, false otherwise
    */
  function LFX_initvar(&$var, $val, $compare='LFXlib left blank on purpose')
  {
    if ($compare==='LFXlib left blank on purpose') {
      if (!$var) {
        $var=$val;
        return(true);
      } else {
        return(false);
      }
    } else {
      if ($var===$compare) {
        $var=$val;
        return(true);
      } else {
        return(false);
      }
    }
  }

      /**
      * This function merges two permissions, in effect performing a logical
      * OR between the two (e.g. 'A' and '*' will return '*', 'ABW' and 'W'
      * will return 'ABW', '*' and '' will return '*', etc).
      *
      * See {@link LFX_permsFilter} for the opposite function ("AND").
      *
      * @param string $perm1 the first permissions
      * @param string $perm2 the second permissions
      * @return string the resulting permissions
      */
    function LFX_permsMerge($perm1,$perm2)
    {
      if ($perm1==='') {
        return($perm2);
      }
      if ($perm2==='') {
        return($perm1);
      }
      if (strstr($perm1,'*') || strstr($perm2,'*')) {
        return('*');
      }
      $perms=$perm1;
      for($i=0;$i<strlen($perm2);$i++) {
        if (!strstr($perms,$perm2[$i])) {
          $perms.=$perm2[$i];
        }
      }
      return($perms);
    }
    
      /**
      * This function filters two permissions between each other, in effect
      * performing a logical AND between the two (e.g. 'A' and '*' will return
      * 'A', 'AWB' and 'W' will return 'W', '*' and '' will return '', etc).
      *
      * See {@link LFX_permsMerge} for the opposite function ("OR").
      *
      * @param string $perm1 the first permissions
      * @param string $perm2 the second permissions
      * @return string the resulting permissions
      */
    function LFX_permsFilter($perm1,$perm2)
    {
      if (strstr($perm2,'*')) {
        return($perm1);
      }
      if (strstr($perm1,'*')) {
        return($perm2);
      }
      if (($perm2==='') || ($perm1==='')) {
        return('');
      }
      $finalPerm='';
      for($i=0;$i<strlen($perm1);$i++) {
        if(strstr($perm2,$perm1[$i])) {
          $finalPerm.=$perm1[$i];
        }
      }
      return($finalPerm);
    }

    
      /**
      * This function returns the MySQL database structure (a la mysqldump)
      * for a given database key, where database keys are the keys of the
      * associative array $_LFX['global']['dbData'] defined in
      * /etc/LFX_(*)/LFX_dbData.php.
      *
      * @param string $dbKey the database key to dump the structure of
      * @return string the dump
      */
    function LFX_dumpSQLstructure($dbKey)
    {
      global $_LFX_dbData;
      $dbObject=new LFX_Object;
      $db=$dbObject->_doDbInit($_LFX_dbData[$dbKey]);
      if(!$db->query("SHOW TABLES")) {
        return(false);
      }
      $dump='';
      $tables=array();
      while ($db->next_record()) {
        $tables[]=$db->f(0);
      }
      for($i=0;$i<count($tables);$i++) {
        $db->query("SHOW CREATE TABLE {$tables[$i]}");
        $db->next_record();
        $dump.=$db->f(1).";\n\n";
      }
      return($dump);
    }
    
      /**
      * This function returns the IDs of the current PHP sessions. It only works
      * with {@link http://ro.php.net/manual/en/ref.session.php#AEN129461}
      * PHP session.save_handler='files'
      *
      * @return mixed false on error or the indexed array of the session IDs;
      *   please note that the session IDs are 16-byte values represented as
      *   32-character long hexadecimal strings; letters are in lower caps.
      */
    function LFX_getCurrentSessionsIDs()
    {
      if (ini_get('session.save_handler')!='files') {
        // sorry, we only know how to handle files at this time!
        return(false);
      }
      $sessions=array();
      $session_path=session_save_path();
      $d = dir($session_path);
      while (false !== ($entry = $d->read())) {
        if (($entry=='.') || ($entry=='..') || (!is_file("$session_path/$entry"))) {
          continue;
        }
        if (preg_match("/^sess_([0-9a-f]{32})$/",$entry,$matches)) {
          $sessions[]=$matches[1];
        }
      }
      return($sessions);
    }
    
    function LFX_validateHTML($html,$allowMismatches=NULL)
    {
      if ($allowMismatches===NULL) {
        $allowMismatches=array(
          'meta',
          'img',
          'br',
          'hr',
          'input',
          'li',
          'link',
          '?xml',
          '!doctype'
        );
      }
      $tag_stack=array();
      if (!preg_match_all('/<[^>]+>/',$html,$matches)) {
        return(true);
      }
      decho(vardump($matches));
      for($i=0;$i<count($matches[0]);$i++) {
        // Stripping tag of angular brackets and whatver other garbage spaces
        $tag=trim(substr($matches[0][$i],1,-1));
        if (substr($tag,0,3)=='!--') {
          decho("Tag is a comment, skipping");
          continue;
        }
        if (substr($tag,-1)=='/') {
          decho("Tag=$tag; auto-closing tag, continuing");
          continue;
        }
        if (substr($tag,0,1)=='/') {
          decho("Tag=$tag -- it's closing here, let's check for a pair!");
          $repeat=false;
          do {
            if ($repeat) {
              decho("Ignored mismatch on tag $popped");
            }
            $repeat=true;
            $popped=array_pop($tag_stack);
            decho("Popped $popped");
          } while (
            ($popped!=strtolower(trim(substr($tag,1)))) &&
            in_array($popped,$allowMismatches)
          );
          if ($popped==strtolower(trim(substr($tag,1)))) {
            decho("Hey, it's matching, we're going on!");
            continue;
          } else {
            decho("No match, sorry -- I was trying to match tag <b>$popped</b>!");
            decho(vardump($allowMismatches));
            return(false);
          }
        }
        list($tag)=explode(' ',$tag);
        decho("Pushing tag <i>$tag</i> into stack");
        array_push($tag_stack,strtolower(trim($tag)));
      }
      // It we still have stuff in the stack, we're not good
      for($i=0;$i<count($tag_stack);$i++) {
        if (!in_array($tag_stack[$i],$allowMismatches)) {
          decho("Failed match on stack leftover <b>{$tag_stack[$i]}</b>");
          return(false);
        }
      }
      return(true);
    }
    
?>
