<?

  /**
  * LFX Include Library
  * This defines the functions and globals used by the Include Manager.
  * The Include Manager calls itself recursively, that's why these
  * are here - this way they don't get redefined over and over again.
  * Bogdan, November 2002
  * $Id: LFX_include_lib.php,v 1.57 2005/11/28 09:02:21 bogdan Exp $
  * @package LFX
  */
    /**
    * Checks if the includes specified in $incArray are already included
    *
    * This should be used by libraries to check if their dependencies have
    * already been loaded or not. In case they haven't, you should include
    * $_incMgr, providing it with exactly the same array in global variable
    * $_incReq.
    *
    * @param array $incArray an indexed array of strings, each of which
    *   is an include key
    * @return boolean true if all included, false otheriwse
    */
  function _checkIncludes($incArray)
  {
    // Checks if the include keys passed in the indexed array have been included already.
    global $_LFX;
    @reset($incArray);
    while (list(,$inc)=@each($incArray)) {
      $inc=strtolower($inc);
      if (!@in_array($inc, $_LFX['page']['included_includes'])) {
        return(false);
      }
    }
    return(true);
  }

    /**
    * Checks if the file provided is the one which was requested
    * directly via HTTP.
    *
    * Should be called like _checkMeOut(__FILE__) before any output.
    * If the two are the same, it will place a redirection header
    * in the HTTP response. Useful to protect libraries against direct
    * access - discourages a handful of hack attempts.
    */
  function _checkMeOut($file)
  {
    if ($file==$_SERVER['SCRIPT_FILENAME']) {
      echo("This script (<i>$file</i>) should not be accessed directly. Exiting.");
      exit;
    }
  }
  
    /**
    * This function pre-parses the master include array, changing all keys
    * to lowercase, so they can be easily compared afterwards
    */
  function LFX_parse_master_include_array()
  {
    global $_LFX;
    $tmp=$_LFX['global']['master_include_array'];
    $tmp2=array();
    while(list($key,$val)=each($tmp)) {
      $tmp2[strtolower($key)]=$val;
    }
    $_LFX['global']['master_include_array']=$tmp2;
  }

  _checkMeOut(__FILE__);

  if (!is_array($_LFX['global']['master_include_array'])) {
    $_LFX['global']['master_include_array']=array();
  }
  $_LFX['global']['master_include_array']=array_merge(
    $_LFX['global']['master_include_array'],
    array(
      'LFX_lib'          => $_LFX_path.'/LFX_lib.php',
      'LFX_constants'    => $_LFX_inc.'/LFX_constants.php',
      'PHPlib'           => $_LFX_inc.'/phplib/phplib.php',
      'JPGraph'          => $_LFX_inc.'/jpgraph/jpgraph.php',
      'LFX_gui'          => $_LFX_inc.'/LFX_gui.php',
      'LFX_development'  => $_LFX_inc.'/LFX_development.php',
      'LFX_intl'         => $_LFX_inc.'/LFX_intl.php',
      'LFX_generic'      => $_LFX_inc.'/LFX_generic.php',
      'LFX_menu_init'    => $_LFX_inc.'/LFX_menu_init.php',
      'LFX_browser_lang' => $_LFX_inc.'/LFX_browser_lang.php',
      'LFX_query_manager'=> $_LFX_inc.'/LFX_query_manager.php',
      // Class Libraries
      'LFXdiff'          => $_LFX_classes.'/LFXdiff.php',
      'LFX_Object'       => $_LFX_classes.'/LFX_object.php',
      'LFX_Page'         => $_LFX_classes.'/LFX_page.php',
      'LFX_User'         => $_LFX_classes.'/LFX_user.php',
      'LFX_Group'        => $_LFX_classes.'/LFX_group.php',
      'LFX_User_Group'   => $_LFX_classes.'/LFX_user_group.php',
      'LFX_Application'  => $_LFX_classes.'/LFX_application.php',
      'LFX_Applications' => $_LFX_classes.'/LFX_applications.php',
      'LFX_Lock'         => $_LFX_classes.'/LFX_lock.php',
      'LFX_Access'       => $_LFX_classes.'/LFX_access.php',
      'LFX_Subscription' => $_LFX_classes.'/LFX_subscription.php',
      'LFX_Subscriber'   => $_LFX_classes.'/LFX_subscriber.php',
      'LFX_Mail'         => $_LFX_classes.'/LFX_mail.php',
      'LFX_Log'          => $_LFX_classes.'/LFX_log.php',
      'LFX_XML'          => $_LFX_classes.'/LFX_xml.php',
      'LFX_Reference'    => $_LFX_classes.'/LFX_reference.php',
      'LFX_Language'     => $_LFX_classes.'/LFX_language.php',
      'LFX_Global'       => $_LFX_classes.'/LFX_global.php',
      'LFX_Doc_Type'     => $_LFX_classes.'/LFX_doc_type.php',
      'LFX_Calendar'     => $_LFX_classes.'/LFX_calendar.php',
      'LFX_Menu'         => $_LFX_classes.'/LFX_menu.php',
      'LFX_impex'        => $_LFX_classes.'/LFX_impex.php',
      'LFX_message'      => $_LFX_classes.'/LFX_message.php',
      'LFX_Translation'  => $_LFX_classes.'/LFX_translation.php',
      'LFX_IP'           => $_LFX_classes.'/LFX_ip.php',
      'LFX_File'         => $_LFX_classes.'/LFX_file.php',
      'LFX_File_Notif'   => $_LFX_classes.'/LFX_file_notif.php',
      'LFX_List_Arrows'  => $_LFX_classes.'/LFX_list_arrows.php',
      'HTML_Tree_Menu'   => $_LFX_classes.'/HTML_tree_menu.php',
      'LFX_Cache'        => $_LFX_classes.'/LFX_cache.php',
      'LFX_Cache_Session'=> $_LFX_classes.'/LFX_cache_session.php',
      'LFX_Cache_Dates'  => $_LFX_classes.'/LFX_cache_dates.php',
      'LFX_Cache_FS'     => $_LFX_classes.'/LFX_cache_fs.php',
      'LFX_Cache_Rights' => $_LFX_classes.'/LFX_cache_rights.php',
      //'LFX_Full_Debug'   => $_LFX_classes.'/LFX_full_debug.php',
      'LFX_Forum'              => $_LFX_classes.'/LFX_forum.php',
      'LFX_Forum_Message'      => $_LFX_classes.'/LFX_forum_message.php',
      'LFX_Forum_Message_Notif'=> $_LFX_classes.'/LFX_forum_message_notif.php',
      'LFX_Report_Wrapper'     => $_LFX_classes.'/LFX_report_wrapper.php',
      'LFX_Reports_Generic'    => $_LFX_classes.'/LFX_reports_generic.php',
      'LFX_Calendar_Reports'   => $_LFX_classes.'/LFX_calendar_reports.php',
      'LFX_Character_Mapping'  => $_LFX_classes.'/LFX_character_mapping.php',
      'LFX_Personal_Global'    => $_LFX_classes.'/LFX_personal_global.php',
      'LFX_report_parameter'      => $_LFX_classes.'/LFX_report_parameter.php',
      'LFX_report_parameters_link'=> $_LFX_classes.'/LFX_report_parameters_link.php',
      'LFX_report_preset'         => $_LFX_classes.'/LFX_report_preset.php',
      'LFX_report_preset_value'   => $_LFX_classes.'/LFX_report_preset_value.php',
      'LFX_Hierarchical_Graph'    => $_LFX_classes.'/LFX_Hierarchical_Graph.php',
      'LFX_Global_Calendar'    => $_LFX_classes.'/LFX_global_calendar.php',
      'LFX_LDAP_User'    => $_LFX_classes.'/LFX_LDAP_User.php',
      'LFX_Time_Filter'    => $_LFX_classes.'/LFX_Time_Filter.php',
      'LFX_Time_Filter_Atom'    => $_LFX_classes.'/LFX_Time_Filter_Atom.php',

      // Include Sets
      'LFX_Basic' => array(
        'LFX_intl', 'LFX_generic', 'LFX_Page', 'LFX_User', 'LFX_Group',
        'LFX_User_Group', 'LFX_development', 'LFX_Global',
        'LFX_Language', 'LFX_Reference'
      )

    )
  );
  LFX_parse_master_include_array();
?>
