<?

  /**
  * Central EH Include Manager
  * Bogdan, November 2002
  * $Id: LFX_include_manager.php,v 1.18 2005/10/25 06:46:08 bogdan Exp $
  * @package LFX
  */

  // The include returns true on success or false on error

  _checkMeOut(__FILE__);

  global $_LFX, $_incReq;

  if (!isset($_incReq)) {
    return(true);
  } elseif (!is_array($_incReq)) {
    return(false);
  }

  if (!is_array($_LFX['page']['required_includes'])) {
    $_LFX['page']['required_includes']=array();
  }
  if (!is_array($_LFX['page']['includes_in_progress'])) {
    $_LFX['page']['includes_in_progress']=array();
  }
  // Reversing the $_incReq request because we're using a stack-style array
  $_LFX['page']['required_includes']=array_merge($_LFX['page']['required_includes'],array_reverse($_incReq));
  $_incReq=array();
  if (!is_array($_LFX['page']['included_includes'])) {
    $_LFX['page']['included_includes']=array();
  }
  if (!is_array($_LFX['page']['refused_includes'])) {
    $_LFX['page']['refused_includes']=array();
  }
  while ($incKey=strtolower(array_pop($_LFX['page']['required_includes']))) {
    //echo("Popped value: $incKey<br>");
    if (in_array($incKey, $_LFX['page']['included_includes'])) {
      // echo("Key $incKey already included, skipping.<br>");
      continue;
    }
    if (in_array($incKey, $_LFX['page']['includes_in_progress'])) {
      //echo("Key is currently being included, skipping.<br>");
      continue;
    }
    // Debug starting
    /*
    echo("Including $incKey; included stuff:<pre>\n");
    var_dump($_LFX['page']['included_includes']);
    echo("</pre>\n");
    echo("Required includes: <pre>\n");
    var_dump($_LFX['page']['required_includes']);
    echo("</pre>\n");
    */
    // Debug finished
    // echo("Starting include for $incKey<br />");
    if (!isset($_LFX['global']['master_include_array'][$incKey])) {
      // Let's be cool about this and provide a suggestion;
      // we won't use the module name because the dev might have mixed it
      // up, in which case the first three letters will match with any
      // include key from the module he wrongly specified, thus resulting
      // in an erroneous suggestion, most probably. However, since there
      // may be more than one match with identical relevance in various
      // modules (e.g. DMO_Object and OPT_Object), the top matches with
      // identical similarity are all shown.
      $myXKey=explode('_',$incKey);
      $myKey=strtolower(implode('',array_slice($myXKey,1)));
      $keys=&$_LFX['global']['master_include_array'];
      reset($keys);
      $bestKey=$bestSim=0;
      $otherKeys=array();
      while(list($key)=each($keys)) {
        $xKey=explode('_',$key);
        $compKey=strtolower(implode('',array_slice($xKey,1)));
        if (($bs=similar_text($myKey,$compKey))>$bestSim) {
          $bestSim=$bs;
          $bestKey=$key;
          $otherKeys=array();
        } elseif ($bs==$bestSim) {
          $otherKeys[]=$key;
        }
      }
      if ($otherKeys) {
        $bestKey=implode(', ',$otherKeys)." or $bestKey";
      }
      echo("<font face='Verdana, Arial' size='-1'>");
      echo("<p>Include key <b>$incKey</b> is invalid.</p>");
      if ($bestSim>strlen($myKey)/2) {
        echo("<p>You probably meant <b>$bestKey</b>.<br />Or you forgot ".
          "to define the key in the appropriate Include Manager library file ".
          "(<i>\{$myXKey[0] path}</i>/include/$myXKey[0]_include_lib.php).</p>");
      } else {
        echo("<p>You probably forgot to define the key in the appropriate ".
          "Include Manager library file (<i>\{$myXKey[0] path}</i>/include/$myXKey[0]_include_lib.php).<br />".
          "Or you meant <b>$bestKey</b>.</p>\n");
      }
      if (function_exists('debug_backtrace')) {
        echo("<p>Here's a backtrace of what was executed so far:</p>\n");
        echo("<pre>");
        var_dump(debug_backtrace());
        echo("</pre>");
      }
      echo('<p>Exiting include manager.</p><hr />');
      echo("</font>");
      return(false);
    }
    $incFile=$_LFX['global']['master_include_array'][$incKey];
    if (is_array($incFile)) {
      $_LFX['page']['required_includes']=array_merge($_LFX['page']['required_includes'],array_reverse($incFile));
      // We're safe to do this - if there are any problems including any libraries in this
      // set, the errors will be trapped as usual. The advantage is that we now stored this set,
      // so if it's requested again, it will be skipped without checking all elements inside.
      $_LFX['page']['included_includes'][]=$incKey;
      continue;
    }
    if (!is_file($incFile)) {
      // No better way to respond to errors, because we don't know
      // what functions are available.
      echo("<font face='Verdana, Arial' size='-1'>");
      echo("Include not found: include key=<b>$incKey</b>; file=<b>$incFile</b>.<br />\n");
      echo('Exiting include manager.<hr />');
      echo("</font>");
      return(false);
    }
    unset($_noCode);
    $_LFX_include_manager_inc_key=$incKey;
    // We're storing the includes being currently included in this array
    // because certain includes perform new includes of their own. New includes
    // are *appended* (not prepended) to the list of include requests,
    // while the original include is obviously still in pole position, because
    // that's why it got to be included in the first place. This would result
    // in a nice infinite loop. We avoid this by skipping includes in progress
    // from the big while() loop we're in (the second if() in the loop; the
    // first one checks for already included includes.
    $_LFX['page']['includes_in_progress'][]=$incKey;
    $incResult=include($incFile);
    array_pop($_LFX['page']['includes_in_progress']);
    // In case some include messed up $incKey, we have a backup.
    $incKey=$_LFX_include_manager_inc_key;
    // Includes return true on success, false on error or an array
    // of include keys if they require other includes first.
    if (is_array($incResult)) {
      if ((count($_LFX['page']['refused_includes'])) &&
      (in_array($_LFX['page']['refused_includes'][count($_LFX['page']['refused_includes'])-1],$incResult))) {
        echo("<font face='Verdana, Arial' size='-1'>");
        echo("<p>Cyclic include for $incKey ($incFile) in file {$_SERVER['PHP_SELF']} - it most probably uses one array
          to test what's included and passes another for include requests, or two files are requesting each other directly.</p>
          <p>Exiting include manager.</p>");
        echo("</font>");
        return(false);
      }
      if ($_noCode) {
        array_push($_LFX['page']['included_includes'],$incKey);
        // echo("No code include for $incKey<br />\n");
        $_LFX['page']['required_includes']=array_merge($_LFX['page']['required_includes'],
          array_reverse($incResult));
        reset($_LFX['page']['required_includes']);
        unset($_noCode);
        continue;
      }
      array_push($_LFX['page']['refused_includes'],$incKey);
      array_push($_LFX['page']['required_includes'],$incKey);
      $_LFX['page']['required_includes']=array_merge($_LFX['page']['required_includes'],
        array_reverse($incResult));
    } elseif ($incResult===false) {
      // We don't complain here - the include must have already, and whatever
      // included this file also probably will when it receives a false result.
      echo('Exiting include manager.<br />');
      return(false);
    } else {
      // We don't require all includes to return "true" explicitly - anything
      // but arrays and boolean false values is ok (i.e. no news is good news).
      array_push($_LFX['page']['included_includes'],$incKey);
      if ((count($_LFX['page']['refused_includes'])) &&
      ($_LFX['page']['refused_includes'][count($_LFX['page']['refused_includes'])-1]==$incKey)) {
        array_pop($_LFX['page']['refused_includes']);
      }
      // echo("Success including $incKey<br />");
    }
  }
  return(true);
?>
