<?

  /**
  * Internationalization code for LFXlib
  * $Id: LFX_intl.php,v 1.5 2004/12/06 12:06:12 bogdan Exp $
  * Bogdan Stancescu <bogdan@lanifex.com>, January 2003
  * @package LFX
  */

  /*
    Library PRE-dependencies:
    - LFX_lib.php (for $_LFX['global']['paths']['lang'])
    - LFX_Globals.php (for storeReference setting)
    - LFX_include_manager.php (conditionally, based on storeReference)

    Library POST-dependencies:
    - LFX_Reference (conditionally, based on storeReference)
  */
  
  // Just in case we happen not to be included
  // in the global scope (we typically should be)
  global $_LFX, $_LFX_globals;
  
  if (!function_exists('__l')) {
    // LFXlib text domain (namely LFX_Local)
    // Please note the translations have to reside in
    // $_LFX['global']['paths']['lang']/<language code>/LC_MESSAGES/LFX_Local.mo
    // $_LFX['global']['paths']['lang'] is defined in LFX_lib.php
    // Defining everything conditionally instead of checking at every call
    if (!function_exists('dgettext')) {
      $_LFX['global']['nogettext']=true;
      function __L($msg) {
        global $_LFX;
        $translated=LFX_Translate($msg,$_LFX['page']['language'],'LFXlib');
        $result=LFX_translationWrapper($msg,$translated,'LFX');
        return($result);
      }
    } else {
      $_LFX['global']['nogettext']=false;
      if ($_LFX_globals->getSetting('nativeTranslations')) {
        if ($_LFX_globals->getSetting('storeReference')) {
          $_incReq=array('LFX_Reference');
          include($_incMgr);
          function __L($msg) {
            $tmp_ref=new LFX_Reference;
            $tmp_ref->LFX_checkReference($msg,'LFXlib');
            unset($tmp_ref);
            global $_LFX;
            $translated=LFX_Translate($msg,$_LFX['page']['language'],'LFXlib');
            $result=LFX_translationWrapper($msg,$translated,'LFX');
            return($result);
          }
        } else {
          function __L($msg) {
            global $_LFX;
            $translated=LFX_Translate($msg,$_LFX['page']['language'],'LFXlib');
            $result=LFX_translationWrapper($msg,$translated,'LFX');
            return($result);
          }
        }
      } else {
        bindtextdomain('LFX_Local',$_LFX['global']['paths']['lang']);
        if ($_LFX_globals->getSetting('storeReference')) {
          $_incReq=array('LFX_Reference');
          include($_incMgr);
          function __L($msg) {
            $tmp_ref=new LFX_Reference;
            $tmp_ref->LFX_checkReference($msg,'LFXlib');
            unset($tmp_ref);
            $translated=dgettext('LFX_Local',$msg);
            $result=LFX_translationWrapper($msg,$translated,'LFX');
            return($result);
          }
        } else {
          function __L($msg) {
            $translated=dgettext('LFX_Local',$msg);
            $result=LFX_translationWrapper($msg,$translated,'LFX');
            return($result);
          }
        }
      }
    }

      /**
      * The native LFXlib translation engine
      *
      * @param string $msg the message to translate
      * @param string $lang the language to translate to
      * @param string $module the module
      *
      * @return string the (un)translated string
      */
    function LFX_Translate($msg,$lang,$module)
    {
      if (!LFX_include_class('LFX_Translation')) {
        return($msg);
      }
      $t=new LFX_Translation;
      $t=$t->search(
        array('original','module','lang'),
        // not needed
        //array(str_replace("\r\n","\n",$msg),$module,$lang)
        array($msg,$module,$lang)
      );
      if (!$t) {
        //echo("<b>Not found:</b>'$msg' <b>in module</b> '$module' <b>for lang</b> '$lang'<br />\n");
        return($msg);
      }
      // TO DO: Must check for multiple results!
      $result=($t[0]->getAttr('translated'));
      //echo("<b>Found:</b>'$msg' <b>in module</b> '$module' <b>for lang</b> '$lang'<br /> is $result<br />\n");
      return($result);
    }
    
      /**
      * A global wrapper for all translations. This must be called by all
      * translation functions, just before returning the result, and
      * the result of *this* function must be used as the final result.
      *
      * This way we can apply various transformations and manage all
      * translated strings from a central location.
      *
      * @param string $orig the original string for which translation
      *   is requested by the application
      * @param string $translated the translated string, as returned by
      *   whatever translation engine is currently in use
      * @param string $module the current module
      *
      * @return string the string to be used
      */
    function LFX_translationWrapper($orig,$translated,$module)
    {
      if (strpos(' '.$orig,'@')) {
        LFX_LogMessage("E-mail being translated: $orig",'info',array('keywords'=>'debug','backtrace'=>debug_backtrace()));
      }
      return($translated);
    }
  }

?>
