<?
  /**
  * The LFX hierarchical graphing class
  * Bogdan Stancescu <bogdan@moongate.ro>, March 2004
  * $Id: LFX_Hierarchical_Graph.php,v 1.5 2005/04/15 08:27:24 bogdan Exp $
  * @package LFX
  */
  
  class LFX_Hierarchical_Graph extends LFX_Object
  {
    /**
    * This variable is an associative array with the index
    * as the vertice information, and the value as an
    * associative array contatining the x and y coordinates
    * The structure:
    * vertices=array(
    *            <info> => array(
    *                           'x' => <x_value>,
    *                           'y' => <y_value>
    *                              ));
    */
    
    var $vertices=array();
    
    /** This variable is an associative array with the index
    * a vertices information, and the value as an indexed
    * array with its following nodes as values
    */
    
    var $edges=array();
    
    var $verticeClass='';
    
    /** This variable is an associative array that holds
    * the level for each vertice in the graph. 
    */
    
    var $levels;
    
    /**
    * Just another way to hold the levels of vertices.
    * Indexed array with key as level and value as
    * vertices on that level.
    */
    
    var $levelNodes;
    
    /**
    * Minimum distance between any two vertices on the same level
    */
    
    var $xUnit;
    
    /**
    * Distance between levels.
    */
    
    var $yUnit;
    
    /**
    * Holds the inversed edges. An associative array 
    * of indexed arrays.
    */
    
    var $inversedEdges=array();
    
    function LFX_Hierarchical_Graph()
    {
      if (!function_exists('gd_info')) {
        $this->addError('The GD library is not installed.');
        return(false);
      }
    }
    
/*    function getStructure($root,$depend_name,$label_attr)
    {
      $this->vertices=$this->edges=array();
      $this->verticeClass=get_class($root);
      $this->_grabStructure(array($root),$depend_name,$label_attr,array());
    }
    
    function _grabStructure($objects,$depend_name,$label_attr,$exclude)
    {
      for($i=0;$i<count($objects);$i++) {
        $object=&$objects[$i];
        if (in_array($object->id,$exclude)) {
          continue;
        }
        $this->vertices[$object->id]=array(
          'name'=>$object->getAttr($label_attr));
        }
        $children=$object->getObjects($depend_name);
        for($j=0;$j<count($children);$j++) {
          $this->edges[$object->id][]=$children->id;
        }
        $exclude[]=$object->id;
        $exclude=$this->_grabStructure($children,$depend_name,$label_attr,$exclude);
      }
      return($exclude);
    }*/
    
    function getStructure()
    {
      return($this->vertices);
    }
    
    /**
    * This function will create the current structure of
    * the graph based on the $input_edges. It will also set
    * the horizontal distance (xUnit) and distance between
    * levels(yUnit) for the graph.
    */
    
    function createFromEdges($input_edges,$x,$y,$firstNode)
    {
      $this->xUnit=$x;
      $this->yUnit=$y;
      //Solving the eventual loops
      $this->_solveLoops($input_edges,$firstNode);
      // In case the input array is not sorted by keys
      $arKeys=array_keys($input_edges);
      array_multisort($arKeys);
      $newEdges=array();
      foreach($arKeys as $key) {
        $newEdges[$key]=$input_edges[$key];
      }
      
      if (count($newEdges)) {
        $this->edges=$newEdges;
        foreach($newEdges as $startNode => $endNodes) {
          if (!isset($this->vertices[$startNode])) {
            // Initializing each vertice with coordinates 0
            $this->vertices[$startNode]['x']=$this->vertices[$startNode]['y']=0;
          }
          foreach($endNodes as $endNode) {
            if (!isset($this->vertices[$endNode])) {
              $this->vertices[$endNode]['x']=$this->vertices[$endNode]['y']=0;
            }
          }
        }
        foreach ($this->vertices as $node => $coordinates) {
          if (!isset($this->edges[$node])) {
            $this->edges[$node]=array();
            //$this->inversedEdges[$node]=array();
          }
        }
        $this->determineLevels();
      }
    }
    
    function _solveLoops(&$input_edges,$startNode,$history=array())
    {
      $edges=&$input_edges;
      $history[]=$startNode;
      $size=count($edges[$startNode]);
      if ($size==1) {
        $nextNode=$edges[$startNode][0];
        if (in_array($nextNode,$history)) {
          $edges[$nextNode]=array_unique(array_merge($edges[$nextNode],array($startNode)));
          unset($edges[$startNode][0]);
          $this->inversedEdges[$nextNode][]=$startNode;
        } else {
          $this->_solveLoops($input_edges,$nextNode,$history);
        }
      } elseif ($size>1) {
        for ($i=0;$i<$size;$i++) {
          $nextNode=$edges[$startNode][$i];
          if (in_array($nextNode,$history)) {
            $edges[$nextNode]=array_unique(array_merge($edges[$nextNode],array($startNode)));
            unset($edges[$startNode][$i]);
            $this->inversedEdges[$nextNode][]=$startNode;
          } else {
            $this->_solveLoops($input_edges,$nextNode,$history);
          }
        }
      }
    }
    

    
    /**
    * This function will determine each level's node
    * and will set it in the @levels attribute of
    * the object. It will also create the levelNodes
    * attribute of the object.
    */
    
    function determineLevels() 
    {
      $edges=$this->edges;
      $vertices=&$this->vertices;
      $levels=array();
      while ($levels!=$oldLevels || !isset($oldLevels)) {
        $oldLevels=$levels;
        foreach ($edges as $startNode => $endNodes) {
          if (!isset($levels[$startNode])) {
            $levels[$startNode]=0;
          }
          foreach ($endNodes as $endNode) {
            if (!isset($levels[$endNode])) {
              $levels[$endNode]=0;
            }
            $levels[$endNode]=max($levels[$endNode],$levels[$startNode]+1);
            // Inserting a level between us, if I link to you and you link to me, and we're next to each other.
            /*
            // This seems to break process 31, pls investigate!
            if (in_array($startNode,$edges[$endNode]) && (abs($levels[$endNode]-$levels[$startNode])==1)) {
              if ($levels[$endNode]>$levels[$startNode]) {
                $levels[$endNode]++;
              } else {
                $levels[$startNode]++;
              }
            }
            */
          }
        }
      }
      // Building the levels array indexed on levels
      $levelNodes=array();
      foreach ($levels as $node => $lev) {
        if (!isset($levelNodes[$lev])) {
          $levelNodes[$lev]=array();
        }
        $levelNodes[$lev][]=$node;
      }
      $this->levels=$levels;
      $this->levelNodes=$levelNodes;      
      $this->_renderForFirstDisplay();
    }

    /**
    * This function will set the coordinates for each node
    * according to its level in the graph.
    * Private method. 
    */
        
    function _renderForFirstDisplay()
    {
      $levels=$this->levels;
      $vertices=&$this->vertices;
      $levelNodes=array();
      $xUnit=$this->xUnit;
      $yUnit=$this->yUnit;
      foreach ($levels as $node => $lev) {
        if (!isset($levelNodes[$lev])) {
          $levelNodes[$lev]=array();
        }
        $levelNodes[$lev][]=$node;
      }
      $currY=0;
      for($i=0;$i<count($levelNodes);$i++) {
        $currY+=$yUnit;
        $currX=$xUnit*3;
        foreach($levelNodes[$i] as $node) {
          $currX+=$xUnit;
          $vertices[$node]['x']=$currX;
          $vertices[$node]['y']=$currY;
        }
      }
    }
    
    /**
    * This function creates temporary nodes between
    * vertices which are separated by more than one 
    * level
    */
    
    function createMultiLevelLinks()
    {
      $vertices=&$this->vertices;
      $edges=&$this->edges;
      $levels=&$this->levels;
      $newNodes=$newLeves=array();
      foreach ($edges as $startNode => $endNodes) {
        foreach ($endNodes as $endNode) {
          $difference=$levels[$endNode]-$levels[$startNode];
          if ($difference>1) {
            $prevNode=$startNode;
            $index=array_search($endNode,$edges[$startNode]);
            unset($edges[$startNode][$index]);
            for($i=1;$i<$difference;$i++) {
              $nodeLevel=$level[$startNode]+$i;
              $nodeInfo=$startNode.'-'.$endNode.'-'.$nodeLevel;
              $newNodes[$nodeInfo]=array('x'=>0,'y'=>0,'fake' => true);
              $newEdges[$prevNode][]=$nodeInfo;
              if (is_array($this->inversedEdges[$startNode])) {              
                if (in_array($endNode,$this->inversedEdges[$startNode])) {
                  $this->inversedEdges[$prevNode][]=$nodeInfo;
                }
              }
              $prevNode=$nodeInfo;
              $levels[$nodeInfo]=$nodeLevel;
              // If we got to the last temporary node, we will
              // make a link from it to the end node
              if ($i==$difference-1) {
                if (is_array($this->inversedEdges[$startNode])) {
                  if (in_array($endNode,$this->inversedEdges[$startNode])) {
                    $this->inversedEdges[$nodeInfo][]=$endNode;
                  }
                }
                $newEdges[$nodeInfo][]=$endNode;
              }
            }
          }
        }
      }
      $vertices=$vertices+$newNodes;
      // We're going to merge the edges' array now
      if (count($newEdges)) {  
        foreach ($newEdges as $newStartNode => $newEndNodes) {
          if (!isset($edges[$newStartNode])) {
            $edges[$newStartNode]=array();
          }
          foreach ($newEndNodes as $newEndNode) {
            $edges[$newStartNode][]=$newEndNode;
          }
        }
      }
      // Reindexing the edges arrays
      foreach ($edges as $node => $endNodes) {
        $edges[$node]=array_values($edges[$node]);
      }
      
      $this->determineLevels();
    }
    /**
    * This function will identify crossovers between edges
    * starting at current level and ending at next level
    * @return array an indexed array with details on each
    * crossover
    */
    
    function identifyCrossovers($level)
    {
      $vertices=&$this->vertices;
      $edges=&$this->edges;
      $levels=$this->levels;
      $levelNodes=$this->levelNodes;
      $crossovers=array();
      foreach ($levelNodes[$level] as $parent1Index => $parent1) {
        for ($i=$parent1Index+1;$i<count($levelNodes[$level]);$i++) {
          $parent2=$levelNodes[$level][$i];
          if (isset($edges[$parent1]) && isset($edges[$parent2])) {
            foreach ($edges[$parent1] as $child1) {
              foreach ($edges[$parent2] as $child2) {
                $p1_idx=$vertices[$parent1]['x'];
                $p2_idx=$vertices[$parent2]['x'];
                $c1_idx=$vertices[$child1]['x'];
                $c2_idx=$vertices[$child2]['x'];
                $new_p2_idx=$p2_idx-$p1_idx;
                $new_p1_idx=0;
                $new_c2_idx=$c2_idx-$c1_idx;
                $new_c1_idx=0;
                if ($new_p2_idx * $new_c2_idx < 0) {
                  $crossovers[]=array('p1' => $parent1,
                                      'c1' => $child1,
                                      'p2' => $parent2,
                                      'c2' => $child2
                                      );
                }
              }
            }
          }
        }
      }
      return($crossovers);
    }
    /**
    * This function will return all the crossovers in
    * this graph indexed on their starting levels.
    */
                                                              
    function allCrossovers()
    {
      $vertices=&$this->vertices;
      $edges=&$this->edges;
      $levels=$this->levels;
      $levelNodes=$this->levelNodes;
      $crossovers=array();
      foreach($levelNodes as $level => $nodes) {
        $crossovers[$level]=$this->identifyCrossovers($level);
      }
      return($crossovers);
    }
    
    /**
    * This function does its best to minimize the number of
    * crossovers on each level. In most cases, it will
    * eliminate them all.
    */
    
    function minimizeCrossovers()
    {
      $vertices=&$this->vertices;
      $edges=&$this->edges;
      $levelNodes=$this->levelNodes;
      for($j=0;$j<count($levelNodes)-1;$j++) {
        $currLevel=$j;
        $repeat=true;
        $childrenHistory=array();
        $childrenHistory['order']=array();
        while ($repeat) {
          $currCrossovers=$this->identifyCrossovers($currLevel);
          $currChildren=$this->_getChildrenOrderedByX($currLevel); 
          $currVertices=$vertices;
          if (count($currCrossovers) && (!in_array($currChildren,$childrenHistory['order']))) {
            $crossover=$currCrossovers[0];
            // Interchanging children (their coordinates)
            $child1=$crossover['c1'];
            $child2=$crossover['c2'];
            $x=$vertices[$child1]['x'];
            $vertices[$child1]['x']=$vertices[$child2]['x'];
            $vertices[$child2]['x']=$x;
          } else {
            $repeat=false;
          }
          $childrenHistory['order'][]=$currChildren;
          $childrenHistory['noOfCrossovers'][]=count($currCrossovers);
          $childrenHistory['vertices'][]=$currVertices;
          if (!$repeat) {
            $minValue=50;
            for ($i=0;$i<count($childrenHistory['noOfCrossovers']);$i++) {
              if ($minValue > $childrenHistory['noOfCrossovers'][$i]) {
                $minValue=$childrenHistory['noOfCrossovers'][$i];
                $min=$i;
              }
              // Changing the order among the nodes on the current level
              $vertices=$childrenHistory['vertices'][$min];
            }
          }
        }
      }
    }

    /**
    * This method is used by @minimizeCrossovers().
    * Private method. Should never need to use it.
    */
    
    function _getChildrenOrderedByX($level)
    {
      $vertices=$this->vertices;
      $children=$this->levelNodes[$level+1];
      $k=true;
      while ($k) {
        $k=false;
        for ($i=0;$i<count($children)-1;$i++) {
          if ($vertices[$children[$i]]['x'] > $vertices[$children[$i+1]]['x']) {
            $tmp=$children[$i];
            $children[$i]=$children[$i+1];
            $children[$i+1]=$tmp;
            $k=true;
          }
        }
      }
      return($children);
    }
      
    function weighHorizontally()
    {
      $vertices=&$this->vertices;
      $edges=$this->edges;
      $levels=$this->levels;
      $levelNodes=$this->levelNodes;
      $xUnit=$this->xUnit;
      //Removing the links involved in crossovers
      $allCrossovers=$this->allCrossovers();
      foreach ($allCrossovers as $level => $crossovers) {
        foreach ($crossovers as $crossover) {
          $index=array_search($crossover['c1'],$edges[$crossover['p1']]);
          unset($edges[$crossover['p1']][$index]);
          $index=array_search($crossover['c2'],$edges[$crossover['p2']]);
          unset($edges[$crossover['p2']][$index]);
        }
      }
      $repeat=true;
      while ($repeat) {
        $maxWeight=-2000;
        $minWeight=2000;
        foreach ($vertices as $node => $coordinates) {
          if ($maxWeight < $coordinates['x']) {
            $maxWeight=$coordinates['x'];
          }
          if ($minWeight > $coordinates['x']) {
            $minWeight=$coordinates['x'];
          }
        }
        $oldVertices=$vertices;
        for ($j=0;$j<count($levelNodes)-1;$j++) {
          $level=$j;
          $parents=$levelNodes[$level];
          $movedNodes=array();
          foreach ($parents as $parent) {
            if (count($edges[$parent])) {
              $diff=0;
              if (count($edges[$parent])>1) {
                // If a single parent, one or more children
                $noOfChildren=0;
                $totalWeight=0;
                foreach ($edges[$parent] as $child) {
                  $totalWeight+=$vertices[$child]['x'];
                  $noOfChildren++;
                }
                $newParentWeight=$totalWeight/$noOfChildren;
                $movedNodes[$parent]=$newParentWeight;
                $diff=$newParentWeight-$vertices[$parent]['x'];
              } elseif (count($edges[$parent])==1) {
                // If a single child, one or more parents
                $child=$edges[$parent][0];
                $parentsOfSameChild=array();
                foreach ($levelNodes[$level] as $anotherParent) {
                  if (is_array($edges[$anotherParent])) {
                    if (in_array($child,$edges[$anotherParent])) {
                      $parentsOfSameChild[]=$anotherParent;
                    }
                  }
                }
                $totalWeight=0;
                for ($k=0;$k<count($parentsOfSameChild);$k++) {
                  $totalWeight+=$vertices[$parentsOfSameChild[$k]]['x'];
                }
                if ($parentsOfSameChild) {
                  // Avoiding a division by zero
                  $averageWeight=$totalWeight/(count($parentsOfSameChild));
                } else {
                  $averageWeight=0;
                }
                $diff=$vertices[$child]['x']-$averageWeight;
                for ($k=0;$k<count($parentsOfSameChild);$k++) {
                  $movedNodesKeys=array_keys($movedNodes);
                  if (!in_array($parentsOfSameChild[$k],$movedNodesKeys)) {
                    $movedNodes[$parentsOfSameChild[$k]]=$vertices[$parentsOfSameChild[$k]]['x']+$diff;
                  }
                }
              }
            }
          }
          if (count($movedNodes)) {
//             echo("NOduri mutate pe nivelul $level".vardump($movedNodes));
            $parentLevel=$levels[$parent];
            $leftMostNode=$rightMostNode=$parent; 
/*            $movedNodesIndexed=array_keys($movedNodes);
            $leftNodes=array_diff($levelNodes[$level],$movedNodesIndexed);
            foreach ($leftNodes as $leftNode) {
              foreach ($movedNodes as $movedX) {
                if ($vertices[$leftNode]['x']==$movedX) {
              
                }
              }
            }*/
            foreach ($movedNodes as $node => $x) {
              $vertices[$node]['x']=$x;
            }
            $newMaxWeight=0;
            $newMinWeight=1800;
            foreach ($vertices as $node => $coordinates) {
              if ($newMaxWeight < $coordinates['x']) {
                $newMaxWeight=$coordinates['x'];
              }
              if ($newMinWeight > $coordinates['x']) {
                $newMinWeight=$coordinates['x'];
              }
            }
//             decho("La sfarsit min $minWeight si max $maxWeight");     
            // If so...
            if (($newMinWeight-$minWeight) || ($newMaxWeight-$maxWeight)) {
              $diff=($newMaxWeight-$maxWeight+$newMinWeight-$minWeight)/2;
//               decho("S-a schimbat min sau max in: ".$minWeight." ".$maxWeight." Mutam toate nodurile de deasupra nivelului $parentLevel cu $diff");
              for($i=0;$i<$parentLevel;$i++) {
                foreach ($levelNodes[$i] as $node) {
                  $vertices[$node]['x']+=$diff;
                }
              }
              $minWeight=$newMinWeight;
              $maxWeight=$newMaxWeight;
            }
            
          }
        }
        for ($level=0;$level<count($levelNodes);$level++) {
          $nodes=$levelNodes[$level];
          for ($i=0;$i<count($nodes)-1;$i++) {
            $descendants=array();
            for ($k=$i+1;$k<count($nodes);$k++) {
              if (abs($vertices[$nodes[$k]]['x']-$vertices[$nodes[$i]]['x'])<$xUnit) {
                $diff=$xUnit-($vertices[$nodes[$k]]['x']-$vertices[$nodes[$i]]['x']);
//                 decho("AM GASIT NODURI FOARTE APROPIATE. MUT NODURILE DIN DREAPTA: ");
                for ($j=$k;$j<count($nodes);$j++) {
                  $vertices[$nodes[$j]]['x']+=$diff;
//                   decho($nodes[$j].": {$vertices[$nodes[$j]]['x']}");
                  $descendants=array_merge($this->getAllDescendants($nodes[$j]),$descendants);
                }
              }
            }
            $descendants=array_unique(array_diff($descendants,$this->getAllDescendants($nodes[$i])));
            if (count($descendants)) {
//               decho("S-au mutat noduri pe nivelul $level. Asadar se vor muta cu $diff nodurile: ".vardump($descendants));
              foreach ($descendants as $descendant) {
                $vertices[$descendant]['x']+=$diff;
              }
            }
          }
        }
                
        $doRepeat=false;
        foreach ($vertices as $node => $coordinates) {
//             decho($node.": ".($vertices[$node]['x']/*-$oldVertices[$node]['x']*/)." ");
          if (abs($vertices[$node]['x']-$oldVertices[$node]['x']) > ($xUnit/8)) {
            $doRepeat=true;
          }
        }
        $repeat=$doRepeat;
        if (!$repeat) {
          $maxWeight=0;
          $minWeight=$xUnit*3;
          foreach ($vertices as $coordinates) {
            if ($coordinates['x'] < $minWeight) {
              $minWeight=$coordinates['x'];
            }

            if ($coordinates['x'] > $maxWeight) {
              $maxWeight=$coordinates['x'];
            }
          }
        }

      }
      // We will rearrange the top and bottom elements     
      // Not of major importance
      $firstLevelNodes=$levelNodes[0];
      if (count($firstLevelNodes)==1) {
        $firstNode=$firstLevelNodes[0];
        if (count($edges[$firstNode])==1) {
          $vertices[$firstNode]['x']=$vertices[$edges[$firstNode][0]]['x'];
        }
      }
      $lastLevelNodes=$levelNodes[count($levelNodes)-1];
      if (count($lastLevelNodes)==1) {
        $lastNode=$lastLevelNodes[0];
        $noOfParents=0;
        if ($levelNodes[count($levelNodes)-2]) {
          foreach ($levelNodes[count($levelNodes)-2] as $parent) {
            if (is_array($edges[$parent])) {
              if (in_array($lastNode,$edges[$parent])) {
                $noOfParents++;
                $theParent=$parent;
              }
            }
          }
        }
        if ($noOfParents==1) {
          $vertices[$lastNode]['x']=$vertices[$theParent]['x'];
        }
      }

      // Re-determining the min and max, since the result from the function
      // is completely messed up, because of moving stuff around back and
      // forth.
      $maxWeight=-2000;
      $minWeight=2000;
      foreach ($vertices as $node => $coordinates) {
        if ($maxWeight < $coordinates['x']) {
          $maxWeight=$coordinates['x'];
        }
        if ($minWeight > $coordinates['x']) {
          $minWeight=$coordinates['x'];
        }
      }

      return(array('min' => $minWeight,'max' => $maxWeight));
    }
    
    function getAllDescendants($node) 
    {
      $desc=array();
      $edges=$this->edges;
      if (count($edges[$node])) {
        foreach ($edges[$node] as $descendant) {
          $desc[]=$descendant;
          $desc=array_merge($desc,$this->getAllDescendants($descendant));
        }
      }
      return(array_unique($desc));
    }
    
  }
  
?>
