<?
  /**
  * Object to manage LFX authentication over a LDAP Server.
  * Andrei Amaritei <aandrei@moongate.ro>, October 2005
  * $Id: LFX_LDAP_User.php,v 1.2 2005/11/10 10:39:30 bogdan Exp $
  * @package LFX
  */

  class LFX_LDAP_User extends LFX_Object
  {

     var $connectHandle = '';
     var $schemeArr     = array();

     function LFX_LDAP_User()
     {
        $this->ini_ldap_connection();
     }

     # Public Methods ==============
     /**
      * Method to retrive all users from ldap server with predefined attributes (specified in xml structure).
      * @return mixed Array if success on searching, false if errors due search or binding processes.
      */
     function synchronize_from_ldap()
     {
        $ch = $this->connectHandle;
        $schemeArr = $this->schemeArr;

        # check if any connection opened;
        if (!is_resource($ch)) {
         //$this->addError('You must establish a ldap connection  !');
          return false;
        }

        $baseDN = $schemeArr[0]['children'][1]['children'][3]['value'];
        $class = $schemeArr[0]['children'][1]['children'][0]['value'];
        $filter = "(objectClass={$class})";
        $pwd = strtolower($schemeArr[0]['children'][1]['children'][1]['value']);
        $uname = strtolower($schemeArr[0]['children'][1]['children'][2]['value']);
        # to be continued with all needed fields......
        $attributes = array($pwd, 'cn'); # temporary, for testing purposes the field `cn` (the only existing on our schema)
        $searchR = @ldap_search($ch, $baseDN, $filter, $attributes);
        if (!$this->_ldap_err($searchR)) return false;
        $users = @ldap_get_entries($ch, $searchR);

        # retrive unformatted array of users from ldap server
        return $users;
     }

     /**
      * Method to retrieve a specific user.
      * @param array $matches Array of attributes for matching over ldap.
      * @return mixed Returns false if errors due searching process or single array (matching record)
      */
     function match_against_ldap($matches = array())
     {
        $ch = $this->connectHandle;
        $schemeArr = $this->schemeArr;
        if (!is_resource($ch)) {
          //$this->addError('You must establish a ldap connection !');
          return false;
        }

        $baseDN = $schemeArr[0]['children'][1]['children'][3]['value'];
        $class = $schemeArr[0]['children'][1]['children'][0]['value'];
        $pwd = strtolower($schemeArr[0]['children'][1]['children'][1]['value']);
        $uname = strtolower($schemeArr[0]['children'][1]['children'][2]['value']);
        $build_filter = '';
        foreach($matches as $k=>$v) {
          if (trim($k) == '') continue;
          # (!) if (trim($v) == '' ) $v = "*"; # match for all
          if ($fn == 'password') {
             $v = $this->do_hash($v);
          }
          $fn = $this->find_value_by_tag($k);
          if (!$fn) {
            $this->addError("Array Key `{$k}` not a good match for provided schema !");
          }
          if (empty($v)) $v = '*';
          if (!empty($fn))
            $build_filter .= "({$fn}={$v})";
        }
        $attributes = array($pwd, $uname);
        $filter = "(&{$build_filter}(objectClass={$class}))";
        $searchR = @ldap_search($ch, $baseDN, $filter, $attributes);
        if (!$this->_ldap_err($searchR)) return false;
        $record = @ldap_get_entries($ch, $searchR);
        return $record;
     }

     /**
      * Function which builds the settings xml to be stored in globals data table.
      * Defines the Ldap Server connection settings, credidentials, structure schema (for users node only). 
      * @param array $settings Array of settings. 
      * e.g : $settings = array(
      *                         'url'  =>'moongate.ro',                     # or ip
      *                         'port' =>'389',                             # default port no sasl (default port with sasl:636)
      *                         'dn'   =>'cn=Manager, dc=moongate, dc=ro'   # cn=Manager -> connect User for ldap.
      *                   );
      *       $schema   = array(
      *                         'class'           =>'person',                       # objectClass (node) for users
      *                         'password'        =>'pw',                           # name for password field in ldap schema.
      *                         'uname'           =>'un',                           # name for username field in ldap schema.
      *                         'base_dn'         =>'dc=moongate, dc=ro',
      *                         'password_hashing' =>'SHA1'                         # hash type for passwords.
      *
      *                   ); # (!) Only values to be modified according with those from ldap server schema.
      *
      * @param string ldapRootPwd Password for ldap connection.
      * @return boolean True if succeed False if not.
      */
     function ini_ldap_settings($settings, $schema, $ldapRootPwd='')
     {
       global $_LFX_globals;
       if (!count($settings)) return false;
       if (!count($schema)) return false;

       if (trim($ldapRootPwd) != '') {
          LFX_include_class('LFX_Global');
          $pgl = new LFX_Global();
          $trace = $pgl->search('name', 'LDAP_Password'); # check if global already exists
          if (!$trace) # setting password
          {
            $pgl->setAttr('name', 'LDAP_Password');
            $pgl->setAttr('value', $ldapRootPwd);
            $pgl->setAttr('isPassword', 'on');
            if (!$pgl->save()) {
              return false;
            }
          } else { # changing password
            $pgl = new LFX_Global($trace[0]->id);
            $pgl->setAttr('value', $ldapRootPwd);
            if (!$pgl->save()) return false;
          }
       }
       # building xml
       $xml  = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\r\n";//encoding='ISO-8859-1'?".">\n";
       $xml .= "<document type=\"832\">\r\n";
       $xml .= "<server_settings>\r\n"; 
       $xml .= "  <url>{$settings['url']}</url>\r\n";
       $xml .= "  <port>{$settings['port']}</port>\r\n";
       $xml .= "  <dn>{$settings['dn']}</dn>\r\n";
       $xml .= "</server_settings>\r\n";
       $xml .= "<server_schema>\r\n";
       $xml .= "  <class>{$schema['class']}</class>\r\n";
       $xml .= "  <password>{$schema['password']}</password>\r\n";
       $xml .= "  <uname>{$schema['uname']}</uname>\r\n";
       $xml .= "  <base_dn>{$schema['base_dn']}</base_dn>\r\n";

       # E.g: 
       #      LN -> LAN-Manager-HASH
       #      NT -> NT-HASH
       # ....................... to be added

       $xml .= "  <password_hashing>{$schema['password_hashing']}</password_hashing>\r\n";

       $xml .= "</server_schema>\r\n";
       $xml .= "</document>\r\n";
       # done building xml structure

       if (!$_LFX_globals->saveSetting('LDAP_Settings', $xml)) {
        return false;
       }
       return true;
     }

     /**
      * Method for closing ldap connection
      */
     function destroy()
     {
        $ch = $this->connectHandle;
        if (is_resource($ch)) {
          ldap_close($ch);
        }
        $this->_ldap_err($ch);
     }
     # Private Methods ==============

      /** Establishing Connection To Ldap Server
      *  @return bool True on success, false on error.
      */
     function ini_ldap_connection()
     {
        global $_LFX_globals;
        $glLdap = $_LFX_globals->getSetting('LDAP_Settings');
        if (!empty($glLdap)) {
          LFX_include_class('LFX_XML');
          $prs = new LFX_XML();
          $arrAXml = $prs->sParse($glLdap);
          $this->schemeArr = $arrAXml;
          $url  = $arrAXml[0]['children'][0]['children'][0]['value'];
          $port = $arrAXml[0]['children'][0]['children'][1]['value'];
          $dn   = $arrAXml[0]['children'][0]['children'][2]['value'];
          $this->connectHandle = $handle = @ldap_connect("ldap://$url", $port);
          if (!$this->_ldap_err($handle)) return false;
          $connectPwd = $_LFX_globals->getSetting('LDAP_Password');
          $bindH = @ldap_bind($handle, $dn, $connectPwd);
          return true;
         } else return false;
      }

      /** Managing errors from ldap functions */
      function _ldap_err($handle = '')
      {
        if (is_resource($handle)) {
          if ($n = @ldap_errno($handle)) {
            $this->addError(@ldap_err2str($n));
            return false;
          }
        } else { return(false); }
        return true;
        /*
          if ($err = ldap_error($handle) !== 'Success' && is_string($err)) {
            $this->addError($err);
            return false;
          } else
              if (is_numeric($errno = ldap_error($handle))) {
                $this->addError(ldap_err2str($errno));
                return false;
              }
        */
        # return true;
      }

      function find_value_by_tag($pat)
      {
        $pat = strtoupper($pat);
        $schemeArr = $this->schemeArr;
        if (empty($schemeArr)) {
          $this->addError('No scheme array found ! Most probably you haven\'t set settings with (ini_ldap_settings) method !');
        }
        $schF = $schemeArr[0]['children'][1]['children'];
        for($i=0; $i<count($schF); $i++) {
          if ($schF[$i]['tag'] == $pat) {
            return $schF[$i]['value'];
            break;
          }
        }
        return false;
      }

      /**
       * Method to hash values (especially passwords)
       * Known Types : NT, LM, ..... (to be added)
       * @access private
       * @param string $pwd Password to be hashed.
       * @param string $hashType Hash Type to be Applied
       * @return mixed Hash Value Or Plain Text if No Hashing Method Provided
       */
      function do_hash($value) 
      {
          if (trim($value) == '') return false;
          $hashType = $this->find_value_by_tag('password_hashing');
          switch($hashType) 
          {
            case 'NT' : return $this->ntPasswordHash($value); break;
            case 'LM' : return $this->lmPasswordHash($value); break;

            default   : return $value; # plain text password return (cuz no hash known applied)
          }
      }

      ################################################################################
      #                          PASSWORD HASHING METHODS                            #
      ################################################################################

      /** Part of PEAR Framework {@link http://pear.php.net}, Package Crypt_CHAP {@link http://pear.php.net/package/Crypt_CHAP}

      /*
        Copyright (c) 2003, Michael Bretterklieber <michael@bretterklieber.com>
        All rights reserved.

        Redistribution and use in source and binary forms, with or without 
        modification, are permitted provided that the following conditions 
        are met:

        1. Redistributions of source code must retain the above copyright 
          notice, this list of conditions and the following disclaimer.
        2. Redistributions in binary form must reproduce the above copyright 
          notice, this list of conditions and the following disclaimer in the 
          documentation and/or other materials provided with the distribution.
        3. The names of the authors may not be used to endorse or promote products 
          derived from this software without specific prior written permission.

        THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
        ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
        WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
        IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
        INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
        BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
        DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
        OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
        NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
        EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

        This code cannot simply be copied and put under the GNU Public License or 
        any other GPL-like (LGPL, GPL2) License.
      */

      /**
      * Generates a hash from the NT-HASH.
      * @param  string  $nthash The NT-HASH
      * @return string
      */    
      function ntPasswordHashHash($nthash) 
      {
        $h = mhash(MHASH_MD4, $nthash);
        return $h;
      }
      
      function ntPasswordHash($password = null) 
      {
        if (isset($password)) {
            return $this->bin2str(mhash(MHASH_MD4, $this->str2unicode($password))); # custom modified
        } else {
            return $this->bin2str(mhash(MHASH_MD4, $this->str2unicode($this->password))); # custom modified
        }
      }
      
      /**
      * Converts ascii to unicode.
      *
      * @access public
      * @return string
      */
      function str2unicode($str) 
      {
          $uni = '';
          $str = (string) $str;
          for ($i = 0; $i < strlen($str); $i++) {
              $a = ord($str{$i}) << 8;
              $uni .= sprintf("%X", $a);
          }
          return pack('H*', $uni);
      }
      
      /**
     * Generates the LAN-Manager-HASH from the given plaintext password.
     *
     * @access public
     * @return string
     */
    function lmPasswordHash($password = null)
    {
        $plain = isset($password) ? $password : $this->password;

        $plain = substr(strtoupper($plain), 0, 14);
        while (strlen($plain) < 14) {
             $plain .= "\0";
        }
        
        # custom modified
        return $this->bin2str($this->_desHash(substr($plain, 0, 7)) . $this->_desHash(substr($plain, 7, 7)));
    }

    /**
     * Generates an irreversible HASH.
     *
     * @access private
     * @return string
     */
    function _desHash($plain)
    {
        $key = $this->_desAddParity($plain);
        $td = mcrypt_module_open(MCRYPT_DES, '', MCRYPT_MODE_ECB, '');
        $iv = mcrypt_create_iv (mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
        mcrypt_generic_init($td, $key, $iv);
        $hash = mcrypt_generic($td, 'KGS!@#$%');
        mcrypt_generic_deinit($td);
        mcrypt_module_close($td);
        
        return $hash;
    }

    /**
     * Adds the parity bit to the given DES key.
     *
     * @access private
     * @param  string  $key 7-Bytes Key without parity
     * @return string
     */
    function _desAddParity($key)
    {
        static $odd_parity = array(
                1,  1,  2,  2,  4,  4,  7,  7,  8,  8, 11, 11, 13, 13, 14, 14,
                16, 16, 19, 19, 21, 21, 22, 22, 25, 25, 26, 26, 28, 28, 31, 31,
                32, 32, 35, 35, 37, 37, 38, 38, 41, 41, 42, 42, 44, 44, 47, 47,
                49, 49, 50, 50, 52, 52, 55, 55, 56, 56, 59, 59, 61, 61, 62, 62,
                64, 64, 67, 67, 69, 69, 70, 70, 73, 73, 74, 74, 76, 76, 79, 79,
                81, 81, 82, 82, 84, 84, 87, 87, 88, 88, 91, 91, 93, 93, 94, 94,
                97, 97, 98, 98,100,100,103,103,104,104,107,107,109,109,110,110,
                112,112,115,115,117,117,118,118,121,121,122,122,124,124,127,127,
                128,128,131,131,133,133,134,134,137,137,138,138,140,140,143,143,
                145,145,146,146,148,148,151,151,152,152,155,155,157,157,158,158,
                161,161,162,162,164,164,167,167,168,168,171,171,173,173,174,174,
                176,176,179,179,181,181,182,182,185,185,186,186,188,188,191,191,
                193,193,194,194,196,196,199,199,200,200,203,203,205,205,206,206,
                208,208,211,211,213,213,214,214,217,217,218,218,220,220,223,223,
                224,224,227,227,229,229,230,230,233,233,234,234,236,236,239,239,
                241,241,242,242,244,244,247,247,248,248,251,251,253,253,254,254);
        $bin = '';
        for ($i = 0; $i < strlen($key); $i++) {
            $bin .= sprintf('%08s', decbin(ord($key{$i})));
        }
        $str1 = explode('-', substr(chunk_split($bin, 7, '-'), 0, -1));
        $x = '';
        foreach($str1 as $s) {
            $x .= sprintf('%02s', dechex($odd_parity[bindec($s . '0')]));
        }
        return pack('H*', $x);
    }
    
    # custom added
    function bin2str($fh)
    {
      $fh_ascii = '';
      for($i = 0; $i < strlen($fh); $i++) {
       $tmp = strtoupper(dechex(ord($fh[$i])));
       $fh_ascii .= (strlen($tmp)>1) ? $tmp : '0'.$tmp;
      }
      return $fh_ascii;
    }
}
?>