<?
/*
create table `time_filter_atom`
   (
      `atom_id` bigint(20) NOT NULL auto_increment,
      `atom_handle_function` varchar(255) NOT NULL,
      `atom_params` text DEFAULT NULL,
      `atom_parent` bigint(10) DEFAULT '0',
      `time_filter_id` smallint(6) DEFAULT '0',
      PRIMARY KEY (`atom_id`)
   );

   # time_filter_atom holds `filters` (e.g on sunday;)
   # handle_function is the name of function/ class which will handle the specified type of `filter`
   # params the params to be passed to handle function (keapt in a serialized array)
   # atom_parent self relation to time_filter_atom; points to a `chunk` (see below)
   # a record in time_filter_atom will be `chunk` (which defines a bracket) like this: atom_handle_function will hold string `chunk`; atom_params will hold boolean operation to be applied
   # time_filter_id => link to time_filter table.
*/

  _checkMeOut(__FILE__);

class LFX_Time_Filter_Atom extends LFX_Object
{
  var $timePoint = '';
  var $last_time_point = '';
  var $operations;

  function LFX_Time_Filter_Atom($id=0)
  {
    $this->LFX_Object($id);

    $tmp['id']=
    $tmp['handler']=
    $tmp['params']=
    $tmp['parent']=
    $tmp['filter_id']=
    array();

    $tmp['id']['fld_name'] = 'atom_id';
    $tmp['handler']['fld_name'] = 'atom_handle_function';
    $tmp['params']['fld_name'] = 'atom_params';
    $tmp['parent']['fld_name'] = 'atom_parent';
    $tmp['filter_id']['fld_name'] = 'time_filter_id';

    // == DEFINING LINKS ==
    $tmp['parent']['link_class'] = 'LFX_Time_Filter_Atom';
    $tmp['filter_id']['link_class'] = 'LFX_Time_Filter';

    $this->dataStructure=array(
        "table_name" => "time_filter_atom",
          "id_field" => "atom_id",
            "fields" => $tmp
    );
    $this->fillDataStructure();

    $this->operations = array(
     'and'=>', but only',
     'or'=>', and also'
    );
  }

  function setCurrTime($time = '')
  {
    if (!$time) $time = time();
    $this->currTime = $time;
  }

  function addAtoms($arridx, $parent, $atoms)
  {
    foreach($atoms as $k=>$v) {
      if ($v['parent'] == $arridx) {
        $time_atom_obj = new LFX_Time_Filter_Atom();
        if ($v['isChunk'] != 1) {
            $func = $v['type'].'_formatParams';
            //$params = $time_atom_obj->$func(trim($v['html']));
             $params = $time_atom_obj->$func(
                                              array(
                                                    'start1' => $v['start1'],
                                                    'end1'   => $v['end1'],
                                                    'start2' => $v['start2'],
                                                    'end2'   => $v['end2']
                                                     )
                                            );
        } else {
          $params = $v['operation'];
        }
        $time_atom_obj->setAttr('params', $params);

        $time_atom_obj->setAttr('handler', ($v['isChunk'] != 1 ? 'handler_'.$v['type'] : 'chunk'));
        if ($arridx != -1) {
          $time_atom_obj->setAttr('parent', $parent);
        } else {
          $time_atom_obj->setAttr('filter_id', $parent);
        }
        $atom_id = $time_atom_obj->save();
        if (!$atom_id) {
          $time_atom_obj->addError('Errors on saving atom with arr id :'.$k);
          return;
        }
        unset($atoms[$k]);
        $this->addAtoms($k, $atom_id, $atoms);
      }
    }
  }

  function time_formatParams($params)
  {
    $params['start_time'] = $params['start1'];
    $params['end_time']   = $params['end1'];

    return(serialize($params));
  }

  function month_formatParams($params)
  {
    $int = array();
    $int['start_date'] = $params['start1'];
    $int['start_month'] = $params['start2'];

    $int['end_date'] = $params['end1'];
    $int['end_month'] = $params['end2'];

    return(serialize($int));
  }

  function week_formatParams($params)
  { 
    return(serialize(array('week_day'=>$params['start1'])));
  }

// = End formating and adding

// = Start processing


  # ======== TIME =============

  function handler_time($params, $boolOp, $timePoint)
  {
    if (!$this->id) {
      $this->addError('Fatal Error on LFX_Time_Filter_Atom::handler_time(): no `id` passed!');
      return(false);
    }
    $params = unserialize($this->getAttr('params'));
    $params[0] = $params['start_time'];
    $params[1] = $params['end_time'];

    return($this->prepare_time_for_and($params, $timePoint, 1));
  }

  function prepare_time_for_and($params, $timePoint, $w)
  {
    $int=$params;
    if (!$params[0] && $params[1]) {
      $pr = $params[1];
    } elseif($params[0] && !$params[1]) {
      $pr = $params[0];
    }
    $timePointM = date('Y-m-d', $timePoint);

    if ($pr) {
      if (strtotime($timePointM.' '.$pr) >= $timePoint) {
        $int[0] = strtotime($timePointM.' -1 day '.$pr);
        $int[1] = strtotime('+1 day',$int[0]);
      } else {
        $int[0] = strtotime($timePointM.' '.$pr);
        $int[1] = strtotime('+1 day',$int[0]);
      }
    } else {
       if($w == 2) {
          $params[0] = date('H:i', $params[0]);
          $params[1] = date('H:i', $params[1]);
       }
       if (strtotime($timePointM.' '.$params[0]) >= $timePoint) {
          $int[0] = strtotime($timePointM.' -1 day '.$params[0]);
          $int[1] = strtotime($timePointM.' -1 day '.$params[1]);
       } else {
          $int[0] = strtotime($timePointM.' '.$params[0]);
          $int[1] = strtotime($timePointM.' '.$params[1]);
       }
    }

    if ($int[0] > $int[1]) {
      $int[1] = strtotime('+1 day', $int[1]);
    }
    return($int);
  }
  # ======== END TIME =============

  # ======== MONTH ================

  function month_filter_case($params)
  {
     $case = null;
    // case = 0
    if (
          $params['start_date'] && $params['start_month'] && $params['end_date'] && $params['end_month']
    ) {
      $case = 0;
    }

    // case 1
    if (
          (!$params['start_date'] && !$params['start_month'] && !$params['end_date'] && $params['end_month'])
       || (!$params['start_date'] && $params['start_month'] && !$params['end_date'] && !$params['end_month'])
    ) {
      $case = 1;
    }

    // case 2
    if (
          ($params['start_date'] && !$params['start_month'] && !$params['end_date'] && !$params['end_month'])
       || (!$params['start_date'] && !$params['start_month'] && $params['end_date'] && !$params['end_month'])
    ) {
      $case = 2;
    }

    // case 3
    if (
          ($params['start_date'] && $params['start_month'] && !$params['end_date'] && !$params['end_month'])
       || ($params['start_date'] && !$params['start_month'] && !$params['end_date'] && $params['end_month'])
       || (!$params['start_date'] && $params['start_month'] && $params['end_date'] && !$params['end_month'])
       || (!$params['start_date'] && !$params['start_month'] && $params['end_date'] && $params['end_month'])
    ) {
      $case = 3;
    }

    // case 4
    if (
       $params['start_date'] &&
       !$params['start_month'] &&
       $params['end_date'] &&
       !$params['end_month']
    ) {
      $case = 4;
    }

    // case 5
    if (
          ($params['start_date'] && $params['start_month'] && $params['end_date'] && !$params['end_month'])
       || ($params['start_date'] && !$params['start_month'] && $params['end_date'] && $params['end_month'])
    ) {
      $case = 5;
    }

    // case 6
    if (
          (!$params['start_date'] && $params['start_month'] && $params['end_date'] && $params['end_month'])
       || ($params['start_date'] && $params['start_month'] && !$params['end_date'] && $params['end_month'])
    ) {
      $case = 6;
    }
    $timePointM = date('Y-m-d', $timePoint);

    // case 7
    if (
          !$params['start_date'] && $params['start_month'] && !$params['end_date'] && $params['end_month']
    ) {
      $case = 7;
    }
    return($case);
  }
  # ======== END TIME =============

  # ======== MONTH ================

  function handler_month($params, $boolOp, $timePoint)
  {
    if (!$this->id) {
      $this->addError('Fatal Error on LFX_Time_Filter_Atom::handler_month(): no `id` passed!');
      return(false);
    }
    $params = unserialize($this->getAttr('params'));

    $int = array();

    $case = $this->month_filter_case($params);

    // case 0
    if ($case == 0) {
      $int[0] = strtotime($params['start_date'].' '.$params['start_month'], $timePoint);
      //include the last day in interval
      $int[1] = strtotime($params['end_date'].' '.$params['end_month'], $timePoint)+3600*24;
      $int[3] = 'year';
    }

    // case 1
    if ($case == 1) {
      $mymonth = ($params['end_month'] ? $params['end_month'] : $params['start_month']);
      $first_day = strtotime('1 '.$mymonth, $timePoint);
      $month = date('n', strtotime('1 '.$mymonth));//j
      $year = date('Y', $timePoint);
      $last_day = strtotime(date('t', mktime(0,0,0, $month, 1, $year)).' '.$mymonth, $timePoint);
      $int[0] = $first_day;
      $int[1] = $last_day+3600*24;
      $int[3] = 'year'; //recursion by
    }

    // case 2
    if ($case == 2) {
      $mydate = ($params['start_date'] ? $params['start_date'] : $params['end_date']);
      $start_day_time = strtotime($mydate.' '.date('M', $timePoint), $timePoint);
      $end_day_time = $start_day_time+3600*24;

      $int[0] = $start_day_time;
      $int[1] = $end_day_time;
      $int[3] = 'month'; // recursion
    }

    // case 3
    if ($case == 3) {
      $mydate = ($params['start_date'] ? $params['start_date'] : $params['end_date']);
      $mymonth = ($params['start_month'] ? $params['start_month'] : $params['end_month']);
      $start_day_time = strtotime($mydate.' '.$mymonth, $timePoint);
      $end_day_time = $start_day_time+3600*24;
      $int[0] = $start_day_time;
      $int[1] = $end_day_time;
      $int[3] = 'year'; // recursion
    }

    // case 4
    if ($case == 4) {
      if ($params['start_date'] > $params['end_date']) {
        $start_date_time = strtotime($params['start_date'].' '.date('M', $timePoint), $timePoint);
        $end_date_time = strtotime('+1 month '.$params['start_date'].' '.date('M', $timePoint), $timePoint);
      } else {
        $start_date_time = strtotime($params['start_date'].' '.date('M', $timePoint), $timePoint);
        $end_date_time = strtotime($params['end_date'].' '.date('M', $timePoint), $timePoint);
      }
      $int[0] = $start_date_time;
      $int[1] = $end_date_time+3600*24;//@@
      $int[3] = 'month'; // recursion
    }

    // case 5
    if ($case == 5) {
      if (!$params['start_month']) $params['start_month'] = $params['end_month'];
      if (!$params['end_month']) $params['end_month'] = $params['start_month'];

      $int[0] = strtotime($params['start_date'].' '.$params['start_month'], $timePoint);
      $int[1] = strtotime($params['end_date'].' '.$params['end_month'], $timePoint);
      $int[3] = 'year';
    }

    // case 6
    if ($case == 6) {
      if (!$params['start_date']) $params['start_date'] = 1;
      if (!$params['end_date']) $params['end_date'] = 1;

      $int[0] = strtotime($params['start_date'].' '.$params['start_month'], $timePoint);
      $int[1] = strtotime($params['end_date'].' '.$params['end_month'], $timePoint);
      $int[3] = 'year';
    }

    // case 7
    if ($case == 7) {
      $int[0] = strtotime('1 '.$params['start_month'], $timePoint);
      $int[1] = strtotime('1 '.$params['end_month'], $timePoint);
      $int[3] = 'year';
    }
    return($int);
  }

  function prepare_month_for_and($params, $timePoint, $w)
  {
    $int = $params;
    $rec = $params[3];
    while ($int[0] >= $timePoint) {
      if ($int[0] < 0) {
        die('Month start interval < 0! Abort! ');
      }
      $int[0] = strtotime('-1 '.$rec, $int[0]);
      $int[1] = strtotime('-1 '.$rec, $int[1]);
    }
    return($int);
  }
  # ======== END MONTH =============

  # ======== START WEEK =============

  function handler_week($params, $boolOp, $timePoint)
  {
    if (!$this->id) {
      $this->addError('Fatal Error on LFX_Time_Filter_Atom::handler_time(): no `id` passed!');
      return(false);
    }
    $params = unserialize($this->getAttr('params'));
    if (strtolower(date('l', $timePoint)) == $params['week_day']) {
      $params[0] = strtotime('this '.$params['week_day'], $timePoint);
    } else {
      $params[0] = strtotime('last '.$params['week_day'], $timePoint+1);//##
    }
    $params[1] = strtotime('+1 day', $params[0]);

    $int[0] = $params[0];
    $int[1] = $params[1];
    return($int);
  }

  function prepare_week_for_and($params, $timePoint, $w)
  {
    $int = $params;
    $weekDay1 = date('l', $int[0]);
    $weekDay2 = date('l', $int[1]);

    if ($weekDay1 == date('l', $timePoint)) {
      $int[0] = strtotime('this '.$weekDay1, $timePoint);
      $int[1] = $int[0]+3600*24;
    }
    if ($int[0] > $timePoint) {
      $int[0] = strtotime('last '.$weekDay1, $timePoint+1);//$params[0]);
      $int[1] = $int[0]+3600*24;
    }
    $this->debug('<ul><font color=green>'.date('r', $timePoint).'</font></ul>');
    $this->debug('<ul><font color=green>'.date('r', $int[0]).'</font></ul>');
    //decho($this->arrdump($int));
    return($int);
  }
  # ======== END WEEK =============

  function is_curr_time_in_int($int, $timePoint)
  {
    $currTime = $timePoint;
    if ($currTime >= $int[0] && $currTime <= $int[1]) {
       return(true);
    }
    return(false);
  }

  // == Generic Functions

  function set_time_point($timePoint)
  {
    $this->timePoint = $timePoint;
  }

  function get_last_interval($int, $timePoint, $w)
  {
      $this->debug('<font color=pink>'.date('r', $timePoint).'</font>');
      $this->debug('<font color=lightblue>'.$this->arrdump($int).'</font>');
      $sint = $int[0];
      $eint = $int[1];
      $type = ($int[4] ? $int[4] : $int[2]);

      switch ($type)
      {
        case 'time' : $fint = $this->prepare_time_for_and($int, $timePoint, $w); break;
        case 'month': $fint = $this->prepare_month_for_and($int, $timePoint, $w); break;
        case 'week' : $fint = $this->prepare_week_for_and($int, $timePoint, $w); break;

        // is chunk (without type)
        default     : $fint = $this->process_filter($int['parent'], $int['parent_type'], $timePoint);
      }
      $this->debug('<font color=blue>'.$this->arrdump($fint).'</font>');
      return($fint);
  }

  // will be passed only intervals which don't have start time in future.
  function and_int($int1, $int2)
  {
    $this->debug(vardump($int1));
    $this->debug(' @@@ '.$this->arrdump($int1).' @@@');
    $this->debug(' @@@ '.$this->arrdump($int2).' @@@');
    $this->debug(' @@@ '.date('r', $int1[0]).' @@@');
    $this->debug(' @@@ '.date('r', $int2[0]).' @@@');
    if ($this->is_overlapping($int1, $int2)) {
      $and_int = array(max($int1[0], $int2[0]), min($int1[1], $int2[1]));
      $this->debug(' #@@@ '.$this->arrdump($and_int).' @@@');
      if ($and_int[0]==$and_int[1]) {
        return(NULL);
      }
      return($and_int);
    }
    return(null);
  }

  function create_description_time($range)
  {
    $range_desc = '';
    if (!$range['start_time'] && $range['end_time']) {
      $time = $range['end_time'];
    }
    if ($range['start_time'] && !$range['end_time']) {
      $time = $range['start_time'];
    }
    if ($range['start_time'] && $range['end_time']) {
      $range_desc .= 'between '.$range['start_time'].' and'.$range['end_time'].' o`clock';
    } else {
      $range_desc .= 'at '.$time.' o`clock';
    }
    return($range_desc);
  }

  function create_description_month($params)
  {
    $case = $this->month_filter_case($params);
    // case 0
    if ($case == 0) {
      $desc_arr['type'] = 'year';
      $desc_arr['delim'] = 'between';
    }

    // case 1
    if ($case == 1) {
      $desc_arr['type'] = 'year';
      $desc_arr['delim'] = 'on';
    }
    // case 2
    if ($case == 2) {
      $desc_arr['type'] = 'month';
      $desc_arr['delim'] = 'on';
    }

    // case 3
    if ($case == 3) {
      $desc_arr['type'] = 'year';
      $desc_arr['delim'] = 'on';
    }

    // case 4
    if ($case == 4) {
      $desc_arr['type'] = 'month';
      $desc_arr['delim'] = 'between';
    }

    // case 5
    if ($case == 5) {
      $desc_arr['type'] = 'year';
      $desc_arr['delim'] = 'between';
    }

    // case 6
    if ($case == 6) {
      $desc_arr['type'] = 'year';
      $desc_arr['delim'] = 'between';
    }

    // case 7
    if ($case == 7) {
      $desc_arr['type'] = 'year';
      $desc_arr['delim'] = 'between';
    }
    $desc_arr['sd'] = ($params['start_date'] ? $this->format_date($params['start_date']) : '');
    $desc_arr['sm'] = ($params['start_month'] ? ucfirst(substr($params['start_month'], 0, 3)) : '');
    $desc_arr['ed'] = ($params['end_date'] ? $this->format_date($params['end_date']) : '');
    $desc_arr['em'] = ($params['end_month'] ? ucfirst(substr($params['end_month'], 0, 3)) : '');

    //$desc_arr['type'].' '.
    $description = $desc_arr['delim'].' '.$desc_arr['sd'].' '.$desc_arr['sm'].($desc_arr['delim'] == 'between' ? ' and ' : '').$desc_arr['ed'].($desc_arr['em'] ? ' ' : '').$desc_arr['em'];
    return($description);
  }

  function format_date($date)
  {
    $last_chr = $date{strlen($date)-1};
    if (strlen($date) == 2 && $date{0} == '1') {
      $ss = 'th';
    } else {
      switch($last_chr)
      {
        case '1': $ss = 'st'; break;
        case '2': $ss = 'nd'; break;
        case '3': $ss = 'rd'; break;
        default : $ss = 'th'; break;
      }
    }
    return($date.$ss);
  }

  function create_description_week($params)
  {
    return('on '.$params['week_day'].'s');
  }

  function create_description($parent, $type='filter', $desc = '')
  {
    $desc .= '';
    if ($type == 'filter') {
      $atoms = $this->search('filter_id', $parent, 'handler');
    } else {
      $atoms = $this->search('parent', $parent, 'handler');
    }

    for($i=0; $i<count($atoms); $i++) {
      if ($atoms[$i]->getAttr('handler') != 'chunk') {
        if ($type == 'filter') {
           $fobj = new LFX_Time_Filter($parent);
           $boolOp = $fobj->getAttr('operation');
        } else {
           $faobj = new LFX_Time_Filter_Atom($parent);
           $boolOp = $faobj->getAttr('params');
        }
        $ftype = substr($atoms[$i]->getAttr('handler'), 8);
        $range = unserialize($atoms[$i]->getAttr('params'));
        $range_desc = '';
        switch($ftype)
        {
          case 'time'  :
              $range_desc .= $this->create_description_time($range);
            break;
          case 'month' :
              $range_desc .= $this->create_description_month($range);
            break;
          case 'week'  :
              $range_desc .= $this->create_description_week($range);
            break;
        }
        $wdesc[] = $range_desc;
      } else {
        $wdesc[] = $this->create_description($atoms[$i]->id, 'parent', $desc);
      }
    }

    if ($type == 'filter') {
      $oo = new LFX_Time_Filter($parent);
      $op = $oo->getAttr('operation');
    } else {
      $oo = new LFX_Time_Filter_Atom($parent);
      $op = $oo->getAttr('params');
    }
    $glue = $this->operations[$op].' ';
    return('('.join($glue, $wdesc).')');
  }

  function delete_filter($parent)
  {
    $this->delete_filter_atoms($parent);
    $tfobj = new LFX_Time_Filter($parent);
    $tfobj->delete();
  }

  function delete_filter_atoms($parent, $type='filter')
  {
    if ($type == 'filter') {
      $atoms = $this->search('filter_id', $parent, 'handler');
    } else {
      $atoms = $this->search('parent', $parent, 'handler');
    }

    for($i=0; $i<count($atoms); $i++) {
      if ($atoms[$i]->getAttr('handler') != 'chunk') {
        $atoms[$i]->delete();
      } else {
        $this->delete_filter_atoms($atoms[$i]->id, 'parent');
      }
    }
    $this->delete($parent);
  }

  function process_filter($parent, $type='filter', $timePoint)
  {
    $this->debug("<table border=1><tr><td bgcolor='#ff5555'>PF</td><td>");
    global $pfcount;
    $pfcount++;
    if ($pfcount > 50) die('Process Filter :: more than 50 cyclics');

    if ($type == 'filter') {
      $atoms = $this->search('filter_id', $parent, 'handler');
    } else {
      $atoms = $this->search('parent', $parent, 'handler');
    }
    for($i=0; $i<count($atoms); $i++) {
      if ($atoms[$i]->getAttr('handler') != 'chunk') {
         if ($type == 'filter') {
            $fobj = new LFX_Time_Filter($parent);
            $boolOp = $fobj->getAttr('operation');
         } else {
            $faobj = new LFX_Time_Filter_Atom($parent);
            $boolOp = $faobj->getAttr('params');
         }
         $func = $atoms[$i]->getAttr('handler');
         $params = $atoms[$i]->getAttr('params');

         $ftype = substr($func, 8, strlen($func));
         $lidx = count($items);
         $tt = $atoms[$i]->$func($params, $boolOp, $timePoint);
         $tt[] = $ftype;
         $tt['parent'] = $atoms[$i]->id;
         $tt['parent_type'] = $ftype;
         $items[] = $tt;
      } else {
         $tt = $this->process_filter($atoms[$i]->id, 'parent', $timePoint);
         $tt[] = 'chunk';
         $tt['parent'] = $atoms[$i]->id;
         $tt['parent_type'] = $ftype;
         $items[] = $tt;
      }
    }

    if ($type == 'filter') {
      $oo = new LFX_Time_Filter($parent);
      $op = $oo->getAttr('operation');
    } else {
      $oo = new LFX_Time_Filter_Atom($parent);
      $op = $oo->getAttr('params');
    }

    if ($op == 'and') {
      $temp = $this->process_chunk_and($items, $timePoint);
      //decho('----items for and----');
      //decho($this->multidump($items));
      //decho('---and---');
      //decho($this->arrdump($temp));
    } elseif($op == 'or') {
      $temp=$this->process_chunk_or($items, $timePoint);
    }
    $this->debug(" PF RETURN ".$this->arrdump($temp));
    $this->debug("</td></tr></table>");
    return($temp);
  }

  function process_chunk_and($fc, $timePoint, $level=0)
  {
    $this->debug("<table border=1><tr><td bgcolor='#eeee00'>AND</td><td>");
    $this->debug(' -------START AND-----');
    $this->debug("time point -- ".date('r', $timePoint)."<br>");

    $this->debug("AND params:");
    $this->debug($this->multidump($fc));

    if (!$this->same_type_atoms_and($fc)) return(NULL);
    $tr = $tlast = $timePoint;
    $tend = 0;
    $wc = 0;
    if (!@$fc[1]) return($this->get_last_interval($fc[0], $tr, 2));
    do
    {
      $wc++;
      if ($wc>20) die('<center>
                          <b><i>Cyclic recursion! Abort! (Possible cause: Invalid time filter defined!)</i></b>
                       </center>
                      ');
      if ($level == 20) die("<center>
                              <b>Too much recursion! (Possible cause: Invalid time filter defined!)</b>
                             </center>
                            ");

      //find oldest range by start time before previous earliest range
      $this->debug("Starting to find oldest");
      for($i=0; $i<count($fc); $i++) {
        $this->debug("AND for() in do() with i=$i");
        list($ts, $te) = $this->get_last_interval($fc[$i], $tr, 2);
        $this->debug("Last int AND $i: ".$this->arrdump(array($ts,$te)));
        if ($ts < $tlast) {
          $tlast = $ts;
          $tend = $te;
          $cfilter = $i;
        }
      }
      $this->debug("Found oldest: ".$this->arrdump(array($tlast,$tend)));
      // end find oldest
      $fin_int = array($tlast, $tend);//, end($fc[$cfilter]));
      $temp_fc = $fc;
      unset($temp_fc[$cfilter]);
      $temp_fc = array_values($temp_fc); //re-index
      $temp_point = min($tend, $timePoint);
      if (count($temp_fc) >= 2) {
        $this->debug("More than two atoms left, going back to AND at timepoint ".date('r',$temp_point)."!");
        $temp_fin_int = $this->process_chunk_and($temp_fc, $temp_point, $level+1);
      } else {
        $this->debug("Single atom left to merge, performing AND at timepoint ".date('r',$temp_point)."!");
        $this->debug("TEMP FC 0 ".$this->arrdump($temp_fc[0]));
        $temp_fin_int = $this->get_last_interval($temp_fc[0], $temp_point, 2);
      }
      $this->debug("TIME point -- ".date('r', $timePoint)."<br>");
      $this->debug("FIN INT AND -- before AND: ".$this->arrdump($fin_int));
      $this->debug('TEMP fin int: '.$this->arrdump($temp_fin_int));
      $fin_int = $this->and_int($fin_int, $temp_fin_int);
      if ($fin_int != null) {
        $this->debug('AND result: '.$this->arrdump($fin_int));
        $this->debug(' -------END AND-----');
        $this->debug("</td></tr></table>");
        return($fin_int);
      }
      $this->debug('FIN INT AND -- after AND: '.$this->arrdump($fin_int));
      $this->debug("NULL result in AND -- time shift from ".date('r',$tr)." to ".date('r',$tlast));
      $tr = $tlast;
    } while($fin_int == null);
  }

  function get_oldest_int($ints)
  {
    $tlast = $ints[0][0];
    $cfilter = 0;
    for($i=1; $i<count($ints); $i++) {
      list($ts, $te) = $ints[$i];
        if ($ts < $tlast) {
          $tlast = $ts;
          $cfilter = $i;
        }
    }
    return($ints[$cfilter]);
  }

  function remake_items($items)
  {
    $items[0][0] = date('d-m-Y H:i', $items[0][0]);
    $items[0][1] = date('d-m-Y H:i', $items[0][1]);

    $items[1][0] = date('d-m-Y H:i', $items[1][0]);
    $items[1][1] = date('d-m-Y H:i', $items[1][1]);

    return($items);
  }

  function same_type_atoms_and($fc)
  {
    //fc[idx][2] = filter's type
    for($i=0; $i<count($fc); $i++) {
      if ($fc[$i][2] == 'chunk' || $fc[$i][3] == 'chunk') continue;
      for($j=$i+1; $j<count($fc); $j++) {
         if ($fc[$j][2] == 'chunk' || $fc[$j][3] == 'chunk') continue;
         if ($fc[$i][2] == $fc[$j][2] &&
         $this->and_int($fc[$i], $fc[$j]) == null) {
           $this->debug('Always NULL');
           return(false);
         }
      }
    }
    return(true);
  }
  //debug function
  function arrdump($arr)
  {
    if (!is_array($arr)) {
      return('Not an array!');
    }
    $ini_data=date('r',$arr[0]).' -- '.date('r',$arr[1])."; ";
    foreach($arr as $key=>$val) {
      if ($key<2) continue;
      $ini_data.=$val."; ";
    }
    $ini_data=substr($ini_data,0,-2);
    return($ini_data);
  }

  //debug function
  function multidump($arr)
  {
    if (!is_array($arr)) {
      return('Not an array!');
    }
    foreach($arr as $key=>$val) {
      $ini_data.="'$key'=>".$this->arrdump($val)."<br />\n";
    }
    return($ini_data);
  }

  function is_overlapping($int1, $int2)
  {
    if ($int1[1] < $int2[0] || $int2[1] < $int1[0]) {
      return(false);
    }
    return(true);
  }

  function or_int($int1, $int2)
  {
    if ($this->is_overlapping($int1, $int2)) {
      return(array(min($int1[0], $int2[0]), max($int1[1], $int2[1])));
    } else {
      return($this->get_previous_int_for_or($int1, $int2));
    }
  }

  function get_previous_int_for_or($int1, $int2)
  {
     if ($int1[0] < $int2[0]) {
       return($int2);
     }
     $temp_int = array($int1[0],$int1[1]);
     return($temp_int);
  }

  function order($fc)
  {
    $no = count($fc);
    for ($i=0; $i<$no-1; $i++) {
      for ($j=0; $j<$no-1-$i; $j++)
        if ($fc[$j+1] < $fc[$j]) {
          $tmp = $fc[$j];
          $fc[$j] = $fc[$j+1];
          $fc[$j+1] = $tmp;
      }
    }
    return($fc);
  }

  function process_chunk_or($fc, $timePoint)
  {
    $this->debug("<table border=1><tr><td bgcolor='#55ff55'>OR</td><td>");
    $this->debug(' -------START OR-----');
    $this->debug('time point OR :'.date('r', $timePoint)."<br>");

    $this->debug($this->multidump($fc));

    $fc = $this->order($fc);
    $fin_int = $this->get_last_interval($fc[0], $timePoint, 2);

    for($i=1; $i<count($fc); $i++) {
      $fin_int = $this->or_int($fin_int, $fc[$i]);
    }

    for($i=0; $i<count($fc); $i++) {
      $fin_int = $this->or_int($fin_int, $this->get_last_interval($fc[0], $fc[0][0], 2));
    }
    //decho(date('r', $timePoint));
    $this->debug('Fin Int OR : '.$this->arrdump($fin_int));
    $this->debug(' -------END OR-----');
    $this->debug("</td></tr></table>");
    return($fin_int);
  }
}
?>