<?
    /**
    * LFX Cache wrapper
    * Sebastian Pascu <sebi@moongate.ro>, April 2005
    * $Id: LFX_cache.php,v 1.2 2005/09/16 07:40:07 bogdan Exp $
    * @package LFX
    */

  _checkMeOut(__FILE__);

  $result=array('LFX_Object');
  if (!_checkIncludes($result)) {
    return($result);
  }

  LFX_include('LFX_Cache_Session', 'LFX_Cache_FS', 'LFX_cache_dates');
  
  class LFX_Cache extends LFX_Object
  {
    var $cacheObject=NULL;
    var $cacheDates=NULL;
    var $module;
    var $name;
    var $cache_id=0;
    var $cache_subid=0;
    var $parameters=array();

    var $cachetype='session';

      /**
      * Constructor, initializes basic cache parameters.
      * @param $my_module module for which we cache, e.g LFX,HD
      * @param $my_type cache type, e.g. 'user','group', it's meant to provide a method of grouping
      *                 different caches
      * @param $my_name cache name, e.g. 'user_list'
      * @param $my_cache_id used to differentiate caches with same above parameters
      * @param $my_cache_subid used to differentiate caches with same above parameters
      * @return object or NULL on error
      */
    function LFX_Cache($my_module='',$my_type='',$my_name='',$my_cache_id=0,$my_cache_subid=0)
    {
      $this->LFX_Object();
      $this->cacheObject=new LFX_Cache_Session();      
      $this->cacheDates=new LFX_Cache_Dates();

      if(!$this->cacheDates) {
        $this=NULL;
        return;
      }
      $this->parameters=array(
        'module' => $my_module,
        'type' => $my_type,
        'name' => $my_name,
        'cache_id' => $my_cache_id,
        'cache_subid' => $my_cache_subid
      );
    }

      /**
      * This function initializes $this->cacheObject depending on the parameter $cache_type
      * @param $cache_type specifies the cache type
      *       currently supported: session, fs
      * @return boolean true on success, false otherwise
      */
    function setCacheType($cache_type='session')
    {
      if(in_array($cache_type,array('session','fs','db'))) {
        $this->cachetype=$cache_type;
        switch($cache_type) {
          case 'session':
            $this->cacheObject=new LFX_Cache_Session();
            break;
          case 'fs':
            if(!$this->cacheObject=new LFX_Cache_FS()) {
              // we could fall back to session storage...
              $this->setCacheType('session');
            }
            break;
          case 'db':
            return(false);
            break;
        }
        return(true);
      } else {
        return(false);
      }
    }

      /**
      * Simple cache type accessor.
      * @return string cache type that's currently initialized
      */
    function getCacheType()
    {
      return($this->cachetype);
    }

      /**
      * This function re-initializes parameters after the object creation
      * @return true on success, false if no parameters given.
      */
    function setParams($params=array())
    {
      if(!count($params)) {
        return(false);
      }
      foreach($params as $name => $value) {
        $this->parameters[$name]=$value;
      }
      return(true);
    }

      /**
      * This function is a wrapper for the sublayer's readCache()
      * @return boolean, true on success, false otherwise
      */
    function readCache()
    {
     LFX_include('LFX_Global');
     $gl=new LFX_global;
     if($gl->getSetting('disable_cache',NULL,$_SESSION['LFX_project'])) {
       return(false);
     }
     if(!$this->isExpired()) {
        //decho("cache good for ".$this->parameters['module'].".".$this->parameters['type'].".".$this->parameters['name'].".".              $this->parameters['cache_id'].".".$this->parameters['cache_subid']);
        return($this->cacheObject->readCache(
                    $this->parameters['module'],
                    $this->parameters['type'],
                    $this->parameters['name'],
                    $this->parameters['cache_id'],
                    $this->parameters['cache_subid']
                )
        );
      } else {
        //decho("cache not good for ".$this->parameters['module'].".".$this->parameters['type'].".".$this->parameters['name'].".".           $this->parameters['cache_id'].".".$this->parameters['cache_subid']);
        return(false);
      }
    }

      /**
      * This function is a wrapper for the sublayer's saveCache()
      * @param $data data to be cached
      * @return boolean, true on success, false otherwise
      */
    function saveCache($data)
    {
      if($this->cacheObject->saveCache($data,$this->parameters['module'],$this->parameters['type'],
                      $this->parameters['name'],$this->parameters['cache_id'],$this->parameters['cache_subid'])) {
        // update expiration date only if cache sublayer saved successfully the data
        $this->cacheDates->updateExpirationDate($this->parameters['module'],$this->parameters['type'],$this->parameters['name'],
              $this->parameters['cache_id'],$this->parameters['cache_subid'],1);
        return(true);
      } else {
        return(false);
      }
    }

      /**
      * This function return the expiration date of the current cache
      * @return timestamp compatible with php time()
      */
    function getExpirationDate()
    {
      return($this->cacheDates->getExpirationDate($this->parameters['module'],$this->parameters['type'],$this->parameters['name'],$this->parameters['cache_id'],$this->parameters['cache_subid']));
    }

      /**
      * This function return the creation date of the current cache
      * @return timestamp compatible with php time()
      */
    function getCreationDate()
    {
      return($this->cacheObject->getCreationDate($this->parameters['module'],$this->parameters['name'],$this->parameters['cache_id'],$this->parameters['cache_subid']));
    }

      /**
      * This function updates the expiration date on the current cache
      */
    function setExpiration($expiration_date=-1)
    {
      $this->cacheDates->updateExpirationDate($this->parameters['module'],$this->parameters['type'],$this->parameters['name'],$this->parameters['cache_id'],$this->parameters['cache_subid'],$expiration_date);
    }

      /**
      * This function checks if cache has expired.
      * @return boolean, true if expired, false otherwise
      */
    function isExpired()
    {
      $exp=$this->getExpirationDate();
      if(($exp==1) || (time() < $exp)) {
        return(false);
      } else {
        return(true);
      }
    }

      /**
      * This function offers a mechanism of expiring multiple caches
      * If only the module and type are given, all caches determined by those attributes
      * will be marked as expired.
      *
      * @param $module the module for which we expire the caches
      * @param $type the type for which we expire the caches
      * @param $names optional, names of the caches we want expired
      * @param $ids optional, ids of the caches we want expired
      * @param $subids optional, subids of the caches we want expired
      */
    function multiExpire($module,$type,$names=array(),$ids=array(),$subids=array())
    {
      global $_LFX_globals;
      $_LFX_globals->saveSetting('cache_access_valid','invalid',NULL,$_SESSION['LFX_project']);
      return($this->cacheDates->multiExpire($module,$type,$names,$ids,$subids));
    }


    function rebuildCache()
    {
      global $_LFX_globals, $_LFX_path;
      exec('which nice',$nice);
      exec('which php',$php);
      if(count($nice) && count($php)) {
        $command='nice -n 19 php -q '.$_LFX_path.'/new_access/cache_rebuild_cli.php > /dev/null 2>&1 &';
        // now run in background the cache rebuild script.
        exec($command);
        // end cache invalidation
        return(true);
      }
      $this->addError('php or nice were not found in the $PATH. Cache rebuilding failed.');
      return(false);
    }
  }

?>
