<?

  /**
  * The DocType class
  * Bogdan Stancescu <bogdan@moongate.ro>, September 2003
  * $Id: LFX_doc_type.php,v 1.5 2005/07/14 00:09:57 bogdan Exp $
  * @package LFX
  */

  _checkMeOut(__FILE__);

  $result=array('LFX_Object');
  if (!_checkIncludes($result)) {
    return($result);
  }

    /**
    * LFX doctype
    */
  class LFX_DocType extends LFX_Object
  {
      /**
      * Class Constructor
      */
    function LFX_DocType($id=0)
    {
      $this->LFX_object($id);

      // == DEFINING ATTRIBUTES ==
      $tmp['ext']=        // The document extension matching this type
      $tmp['name']=       // The document type, human description
      $tmp['doc_mime']=   // The document MIME type
      $tmp['icon']=       // The actual document icon (16x16 pixels image)
      $tmp['icon_mime']=  // The icon MIME type (may be GIF, JPEG, PNG, etc).
      $tmp['special']=    // If this is non-zero, this is a special icon type
                          // (e.g. directory icon)
      array();

      // == DEFINING FLAGS ==
      $tmp['icon']['flags']['file']=true;
      $tmp['icon']['flags']['large']=true;

      // == DEFINING DATASTRUCTURE ==
      $this->dataStructure=array(
        'table_name'=>'doc_type',
        'id_field'=>'id',
        'fields'=>$tmp
      );
      $this->fillDataStructure();

    }

    function createDefault()
    {
      $dt=new LFX_DocType;
      $dt->setAttrs(array(
        'ext'=>'default',
        'name'=>'Default DocType',
        'doc_mime'=>'application/octet-stream',
        'special'=>1
      ));
      if ($dt->save()) {
        return($dt);
      } else {
        // SC::save() will complain, no need to duplicate that.
        return(false);
      }
    }
    
    function defaultIcon()
    {
      $dt=$this->search(array('ext','special'),array('default',1));
      if (count($dt)) {
        return($dt[0]);
      }
      // Not even the default doctype found. Creating and returning
      return($this->createDefault());
    }
    
    function iconH($id=0)
    {
      global $_LFX_url;
      $dt=&$this->defaultObject($id);
      if ($dt) {
        return("<IMG src='$_LFX_url/icon.php?dtID={$dt->id}' WIDTH='16' ".
          "HEIGHT='16' BORDER='0' ALT='".__L("Icon")."' ".
          "TITLE='".$dt->getAttrH('name')."'>");
      }
      $dt=&$this->defaultIcon();
      if ($dt) {
        return($dt->iconH());
      }
      return("<font color='red'><b>!</b></font>");
    }

    function getDocType($fileName)
    {
      if (preg_match('/\.([a-zA-Z0-9_]+)$/',strtolower($fileName),$ext)) {
        $docTypes=$this->search(array('ext','special'),array(strtolower($ext[1]),0));
        if (count($docTypes)) {
          return($docTypes[0]);
        }
      }
      // Either no extension or doctype not found. Falling back to default.
      return($this->defaultIcon());
    }
    
    function getDocMime($fileName)
    {
      if (preg_match('/\.([a-zA-Z0-9_]+)$/',strtolower($fileName),$ext)) {
        $docTypes=$this->search(array('ext','special'),array(strtolower($ext[1]),0));
        if (count($docTypes)) {
          return($docTypes[0]->getAttr('doc_mime'));
        }
      }
      // Either no extension or doctype not found. Falling back to default.
      $default=$this->defaultIcon();
      return($default->getAttr('doc_mime'));
    }
    
    function getDocIcon($fileName)
    {
      if (preg_match('/\.([a-zA-Z0-9_]+)$/',strtolower($fileName),$ext)) {
        $docTypes=$this->search(array('ext','special'),array(strtolower($ext[1]),0));
        if (count($docTypes)) {
          return($docTypes[0]->getAttr('icon'));
        }
      }
      // Either no extension or doctype not found. Falling back to default.
      $default=$this->defaultIcon();
      return($default->getAttr('icon'));
    }
    
    function getDocIconMime($fileName)
    {
      if (preg_match('/\.([a-zA-Z0-9_]+)$/',strtolower($fileName),$ext)) {
        $docTypes=$this->search(array('ext','special'),array(strtolower($ext[1]),0));
        if (count($docTypes)) {
          return($docTypes[0]->getAttr('icon_mime'));
        }
      }
      // Either no extension or doctype not found. Falling back to default.
      $default=$this->defaultIcon();
      return($default->getAttr('icon_mime'));
    }
    
    function exportToXML($docTypes,$embedded=false)
    {
      if (!$docTypes) {
      return(NULL);
      }
      if (!is_array($docTypes)) {
        return(false);
      }
      $result='';
      for($i=0; $i<count($docTypes); $i++){
        $tmp=new LFX_DocType($docTypes[$i]);
        $result.=$tmp->exportSingleXML();
      }
      $tmp='';
      if (!$embedded) {
        $tmp.="<?xml version='1.0' encoding='ISO-8859-1'?".">\n";
      }
      $tmp.="  <doctype_descriptors>\n";
      $tmp.=     $result;
      $tmp.="  </doctype_descriptors>";
      return($tmp);
    }
  
    function exportSingleXML()
    {
      LFX_include_class('LFX_XML');
      $xml=new LFX_XML;
      $ind='   ';
      $result='';
      $result.=$ind."<doctype_descriptor ";
      $result.="name=\"".$xml->escape($this->getAttr('name'))."\" ";
      $result.="ext=\"".$xml->escape($this->getAttr('ext'))."\" ";
      $result.="doc_mime=\"".$xml->escape($this->getAttr('doc_mime'))."\" ";
      $result.="icon=\"".$xml->escape(rawurlencode($this->getAttr('icon')))."\" ";
      $result.="icon_mime=\"".$xml->escape($this->getAttr('icon_mime'))."\" ";
      $result.="special=\"".$xml->escape($this->getAttr('special'))."\" ";
      $result.=">\n";
      $result.=$ind."</doctype_descriptor>\n";
      return($result);
    }
    
    function importXML($xmlin)
    {
      if (!is_array($xmlin)) {
        LFX_include('LFX_XML');
        $xml=new LFX_XML($xmlin);
        $xml=$xml->xml[0];
      } else {
        $xml=$xmlin;
      }
      if ($xml['tag']!='DOCTYPE_DESCRIPTORS') {
        return(false);
      }
      while (list(,$doctypex)=@each($xml['children'])) {
        list($doctype,$jcreated)=$this->importSingleXML($doctypex);
        if (!$doctype) {
          return(false);
        }
      } 
      return(true);
    }
    
    function importSingleXML($xmlData)
    {
      $doctypes=$this->search('name',$xmlData['attributes']['NAME']);
      if (!count($doctypes)) {
        $doctype=new LFX_DocType;
        $doctype->setAttrs(array(
          'name'=>$xmlData['attributes']['NAME'],
          'ext'=>$xmlData['attributes']['EXT'],
          'doc_mime'=>$xmlData['attributes']['DOC_MIME'],
          'icon'=>rawurldecode($xmlData['attributes']['ICON']),
          'icon_mime'=>$xmlData['attributes']['ICON_MIME'],
          'special'=>$xmlData['attributes']['SPECIAL']
        ));
        if (!$doctype->save()) {
          return(NULL);
        }
        $jcreated=true;
      } else {
        $doctype=$doctypes[0];
        $doctype->load();
        if (!$doctype->save()) {
          return(NULL);
        }
        $jcreated=false;
      }
      return(array($doctype,$jcreated));
    }
    
    function docTypeSynch($xml_string)
    {
      if (!is_array($xml_string)) {
        LFX_include('LFX_XML');
        $xml=new LFX_XML($xml_string);
        $xml=$xml->xml[0]; 
      } else {
        $xml=$xml_string;
      }
      $doctype=new LFX_DocType();  
      $doctypesNames=array();
      while (list(,$doctypex)=@each($xml['children'])) {
        $doctypesNames[]=$doctypex['attributes']['NAME'];
      }
      $query="SELECT id,name FROM doc_type";
      $doctype->query($query);
      $existentDoctypesIds=$existentDoctypesNames=array();
      while ($doctype->db->next_record()) {
        $existentDoctypesIds[]=$doctype->db->f(0);
        $existentDoctypesNames[]=$doctype->db->f(1);
      }
      $doctypesToExport=array_diff($existentDoctypesNames,$doctypesNames);
      $n=count($existentDoctypesIds);
      for($i=0;$i<$n;$i++) {
        if (!$doctypesToExport[$i]) {
          unset($existentDoctypesIds[$i]);
        }
      }
      $doctypesIds=array_values($existentDoctypesIds);
      $returnXmlString=$doctype->exportToXML($doctypesIds,true);
      return($returnXmlString);
    }
 }

?>
