<?


  /**
  * The LFX generic file class
  * Bogdan Stancescu <bogdan@moongate.ro>, January 2004
  * $Id: LFX_file.php,v 1.21 2005/04/15 11:49:35 bogdan Exp $
  * @package LFX
  */

  _checkMeOut(__FILE__);

  $result=array('LFX_Object');
  if (!_checkIncludes($result)) {
    return($result);
  }

    /**
    * The generic LFXlib File class
    *
    * How to install files support for a new module:
    *
    * - Copy the table structure for files from the LFXlib database to your
    *    module's.
    *
    *    Make sure you keep the same name for the table, this is critical.
    *    For MySQL this is simple to do from the command prompt. Suppose your
    *    LFXlib database is lfxlib and you're duplicating the structure for a
    *    database named mymodule, and the mymodule database exists:
    *
    *    mysqldump -d lfxlib module_files | mysql mymodule
    *
    * - Copy one of the existing module's file class (NOT this one) and customize
    *    it for your module.
    *
    *    For instance OPT_File is sure to exist, it was created immediately after
    *    this file in order to test cross-module functionality of this class.
    *    Simply copy it to your module's include/classes and rename everything
    *    starting with "OPT" to your module's name. It is critical that you use
    *    the same name instead of OPT as your module's identification string in
    *    your module's super class $module variable (e.g. AGR for Agora, EH for
    *    Event Horizon, DMO for DMO, etc).
    *
    * - Don't forget to register your new file class in the include manager.
    *
    * - Now you're set as far as the file class is concerned. Make sure
    *    the classes using files know their own includes (see below), and make
    *    sure to define method _filePerm() in your class for customized
    *    permissions to files.
    *
    *    One thing you have to check is that every class which will have
    *    files associated properly defines its include key in the class's
    *    $includeKey variable. This is needed because the file object has to
    *    check for permissions, and for that it needs to include your class,
    *    and for that it obviously needs to know the proper include key.
    *
    *    The second thing is to define the _filePerm() method in your class.
    *    Read more on this in the documentation for SC::filePerm().
    *
    * - Finally, as an optional step you can define method
    *    possibleFileNotifications() in the classes using files.
    *    The value returned by that method will be used to allow users to send
    *    notifications. The value is interpreted as follows:
    *
    *    boolean true:
    *      All users in the system are allowed to be notified. This can be a big
    *      problem if you have a lot of users, since all will be shown in a
    *      multiple select list, so please make sure you use this carefully.
    *
    *    indexed array:
    *      Each element in the array can be one of:
    *
    *        LFX_User object:
    *          That user is used;
    *
    *        numeric value:
    *          The user with that id is used;
    *
    *        string:
    *          The user with that e-mail address is used; if no such user is
    *          found, the e-mail address is simply used as passed;
    *
    *        other data type:
    *          Ignored;
    *
    *    associative array:
    *      The array is converted to an indexed array using array_values() and
    *      used as above;
    *
    *    any other value/data type:
    *      *NO* users are shown; the notification control is not shown at all.
    *      A common mistake is returning one single numeric value, or a LFX_User
    *      object, or a string -- all these fall under this category! You *MUST*
    *      pass those within arrays.
    *
    * - If you choose to implement the notification mechanism, you can also take
    *    advantage of customized notifications by implementing method
    *    fileNotification() in your class. If the result of that method is a
    *    LFX_Mail object, LFX_File will simply fill in the addresses the e-mail
    *    has to be sent to, and sends it. Otherwise, a default message is sent.
    *
    */
  class LFX_File extends LFX_Object
  {
    var $url;
    var $myAssociatedClass;
    var $myAssociatedModule;

      /**
      * Class Constructor.
      *
      * Please take a look at this <b>file's</b> documentation for
      * usage overview.
      */
    function LFX_File($id=0)
    {
      $this->LFX_Object($id);

      // == DEFINING ATTRIBUTES ==
      $tmp['title']=
      $tmp['comment']=
      $tmp['doc']=
      $tmp['file_name']=
      $tmp['src']=
      $tmp['date']=
      $tmp['obsoleted_by']=

      $tmp['objectClass']=
      $tmp['objectId']=
      $tmp['objectInclude']=
      array();

      // == DEFINING FIELDS ==
      $tmp['src']['fld_name']='user_id';

      // == DEFINING LINKS ==
      $tmp['src']['link_class']='LFX_User';
      $tmp['obsoleted_by']['link_class']=get_class($this);

      // == DEFINING FLAGS ==
      $tmp['date']['flags']['sqlDate']=true;
      $tmp['date']['flags']['forceSave']=true;
      $tmp['doc']['flags']['short_in_db']=true;
      $tmp['doc']['flags']['large']=true;
      $tmp['doc']['flags']['huge']=true;
      $tmp['comment']['flags']['file']=true;
      $tmp['comment']['flags']['large']=true;

      // == DEFINING DEPENDENCIES ==
      // Yes, stranger mode is correct, we're managing this chain with
      // beforeDelete() and onDelete() methods, see below.
      $tmp2['last_obsolete']=array(
        'class'=>get_class($this),
        'attr'=>'obsoleted_by',
        'on_mod'=>'STRANGER'
      );

      $tmp2['notif']=array(
        'class'=>'LFX_File_Notif',
        'attr'=>'file_id'
      );

      // == DEFINING DATASTRUCTURE ==
      $this->dataStructure=array(
        'table_name'=>'module_files',
        'id_field'=>'id',
        'tree_link_attr'=>'obsoleted_by',
        'fields'=>$tmp,
        'depend'=>$tmp2
      );
      $this->fillDataStructure();
    }

    function getDocType($id=0)
    {
      LFX_include('LFX_DocType');
      $doc=&$this->defaultObject($id);
      $dt=new LFX_DocType;
      return($dt->getDocType($doc->getAttr('file_name')));
    }

      /**
      * Storing the file which obsoletes this file so we can properly submit
      * it for the object *this* one obsoletes *after* deleting this object
      * in onDelete.
      */
    function beforeDelete($id)
    {
      $this->obsoletedId=$this->getAttr('obsoleted_by');
      return(true);
    }

      /**
      * Using this->obsoletedId to properly pass that reference.
      * See {@link beforeDelete} for more info.
      */
    function onDelete($id)
    {
      $myClass=get_class($this);
      $tmp=new $myClass($id);
      if (!($older=$tmp->getObjects('last_obsolete'))) {
        return(true);
      }
      $old=$older[0];
      $old->quickLoad();
      $old->setAttr('obsoleted_by',$this->obsoletedId);
      $old->save();
      return(true);
    }

      /**
      * This method initializes the rendering output for file lists.
      * It should be called by the SC in SC::renderFilesH().
      * @return string the initial HTML for files.
      */
    function startRenderH($perm, $mode)
    {
      if (!LFX_User::permsFilter($perm,'R')) {
        return('');
      }
      $result='';
      $result.="<A NAME='LFX_file'></A>";
      $result.=LFX_sform($this->_makeURL().'#LFX_file','POST','file_form','',"multipart/form-data");
      $result.=LFX_echo("<INPUT type='hidden' name='LFX_Global_Control' value='LFX_File'>");
      $result.=LFX_startTables();
      $result.=LFX_swatchHeader(array('title'=>__L('Associated Files'),
                                      'files'=>''));
      return($result);
    }
    
      /**
      * This method finalizes the rendering output for file lists.
      * It should be called by the SC in SC::renderFilesH().
      * @return string the final HTML for files.
      */
    function endRenderH($perm,$notifU,$notifG,$xtraHTML='',$mode='RW')
    {
      global $LFile_title;
      $result='';
      
      if (LFX_permsFilter($perm,'W') && (LFX_permsFilter($mode,'W'))) {
        $result.=LFX_str();
        $result.=LFX_echo("<td>");

        $result.=LFX_echo("<table cellpadding='1' cellspacing='0'>");
        $result.=LFX_str();
        $result.=LFX_echo("<td valign='top' width='100%'>");

        $result.=LFX_startTables('100%');
        $result.=LFX_swatchHeader(array('title'=>__L('Submit new file')));
        $result.=LFX_str();
        $result.=LFX_echo("<td>");

        $notificationControls=$this->renderNotificationControlsH($notifU,$notifG,$xtraHTML);
        $result.=LFX_startTables('100%');
        $result.=LFX_str();
        if ($notificationControls) {
          $result.=LFX_td(__O("Please first choose the people who should be ".
            "notified, before picking the file to upload!"),NULL,NULL,2);
          $result.=LFX_estr();
        }
        $result.=LFX_rth(__L('Title'));
        $result.=LFX_td("<input type='text' name='LFile_title' value='".htmlspecialchars(LFX_Object::gpcUnslash($LFile_title))."'>");
        $result.=LFX_estr();
        $result.=LFX_rth(__L('File'));
        LFX_include('LFX_Global');
        $myGlobals=new LFX_Global;
        $myGlobals->global_module=$this->myAssociatedModule;
        if($myGlobals->getSetting($this->myAssociatedClass."_file_notification_warning")) {
          global $_page;
          $_page->renderJSscript("
            function fileNotifCheck() {
              var empty=true;
              var control_exists=false;
              var users, groups;
              if(users=document.getElementById('filenotifU')) {
                control_exists=true;
                if(users.selectedIndex!=-1) {
                  empty=false;
                }
              }
              if(groups=document.getElementById('filenotifG')) {
                control_exists=true;
                if(groups.selectedIndex!=-1) {
                  empty=false;
                }
              }
              if(empty && control_exists) {
                return confirm('".__L("Are you sure you want to upload this file without notifying anyone?")."');
              } else {
                return true;
              }
            }
          ");
          $js_check_code="onclick='return fileNotifCheck()'";
        } else {
          $js_check_code="";
        }
        $result.=LFX_td("<input type='file' name='newLFXFile'>".
          "<input type='submit' value='".__L('Upload')."' $js_check_code>".
          "<input type='hidden' name='LFile_action' value='create'>");
        $result.=LFX_etr();
        $result.=LFX_endTables();

        $result.=LFX_echo("</td>");
        $result.=LFX_etr();
        $result.=LFX_endTables();

        $result.=LFX_echo("</td>");
        $result.=LFX_echo($notificationControls);
        $result.=LFX_etr();
        $result.=LFX_echo("</table>");

        $result.=LFX_echo("</td>");
        $result.=LFX_etr();
      }
      
      $result.=LFX_endTables();

      $result.=LFX_eform();
      return($result);
    }

      /**
      * This method renders one file with proper indentation.
      * You don't need to call this directly, call SC::renderFilesH()
      * instead.
      */
    function renderH($perm,$mode,$indent=0)
    {
      $result='';
      $result.=LFX_str();
      $result.=LFX_echo("<td>");
      
      $result.=LFX_echo("<table border='0' cellpadding='0' cellspacing='0' width='100%'");
      $result.=LFX_str();
      $result.=LFX_echo("<td>".str_repeat('&nbsp',4*$indent)."</td><td width='100%'>");
      
      $result.=LFX_startTables('100%');
      $header=array('title'=>$this->getLinkH());
      if (LFX_permsFilter($mode,'W') && LFX_permsFilter($perm,'D')) {
        $header['drop']=$this->_makeURL(
          array('LFile_action'=>'delete','LFile_id'=>$this->id,'LFX_Global_Control'=>'LFX_File'));
      }
      $result.=LFX_swatchHeader($header);
      $result.=LFX_str();
      $result.=LFX_echo("<td>");
      $result.=LFX_startTables('100%');

      if (!$indent && LFX_permsFilter($perm,'U') && LFX_permsFilter($mode,'W')) {
        $result.=LFX_str();
        $result.=LFX_rth("<label for='obsolete_".$this->id."'>".__L('Obsolete with new file')."</label>");
        $result.=LFX_td("<input type='radio' name='LFile_obsolete' value='".$this->id."' id='obsolete_".$this->id."'>");
      }

      $result.=LFX_estr();
      $result.=LFX_rth(__L('Upload date'),NULL,NULL,NULL,NULL,'50%');
      $result.=LFX_td($this->getAttrH('date'));

      $result.=LFX_estr();
      $result.=LFX_rth(__L('Uploaded by'));
      $src=$this->getObject('src');
      global $_page;
      if(($_page->user->canSeeUser($id)===false) && ($_page->user->getRights("Admins","R")===false)) {
        $result.=LFX_td($src->getNameH());
      } else {
        $result.=LFX_td($src->getLinkH());
      }

      $result.=LFX_estr();
      $result.=LFX_rth(__L('File size'));
      $result.=LFX_td($this->getSizeH('doc'));

      $result.=LFX_estr();
      $result.=LFX_rth(__L('File name'));
      $result.=LFX_td($this->getAttrH('file_name'));

      if ($notif=$this->getLoadedObjects('notif')) {
        $notifText='';
        for($i=0;$i<count($notif);$i++) {
          if ($mail=$notif[$i]->getAttr('notif_email')) {
            $tmp=new LFX_User;
            $tmp=$tmp->search('email',$mail);
            if ($tmp) {
              $notifText.=$tmp[0]->getNameH().', ';
            } else {
              $tmp=new LFX_group;
              $tmp=$tmp->search('email',$mail);
              if ($tmp) {
                $notifText.=$tmp[0]->getNameH().', ';
              } else {
                $notifText.=$mail.', ';
              }
            }
          } elseif ($tmp=$notif[$i]->getObject('notif_person')) {
            $notifText.=$tmp->getNameH().', ';
          } else {
            $notifText.="(".__L('no e-mail')."), ";
          }
        }
        if ($notifText) {
          $notifText=substr($notifText,0,-2);
          $result.=LFX_estr();
          $result.=LFX_rth(__L('Notified'));
          $result.=LFX_td($notifText);
        }
      }

      $result.=LFX_etr();
      $result.=LFX_endTables();
      
      $result.=LFX_echo("</td>");
      $result.=LFX_etr();
      $result.=LFX_endTables();
      
      $result.=LFX_echo("</td>");
      $result.=LFX_etr();
      $result.=LFX_echo("</table>");
      
      $result.=LFX_echo("</td>");
      $result.=LFX_etr();
      $myChildren=$this->search('obsoleted_by',$this->id);
      for ($i=0;$i<count($myChildren);$i++) {
        $result.=$myChildren[$i]->renderH($perm,$mode,$indent+1);
      }
      return($result);
    }
      
      /**
      * This method performs the file-specific actions specified by the user
      * in the interface. It should be called by the SC in SC::renderFilesH().
      *
      * @param string $objectClass the class of the object to which the file is attached
      * @param int $objectId the id of the object to which the file is attached
      * @param string $includeKey the include key for the associated class
      * @param string $perm the permissions on this object, see SC::filePerm()
      * @return true on success, false on failure, NULL if no operation was performed.
      */
    function performActions($objectClass, $objectId, $includeKey, $perm, $mode)
    {
      global $_page, $LFile_action, $LFile_id, $LFile_title, $LFile_comment,
        $LFile_obsolete;
      $thisClass=get_class($this);
      if (($_GET['LFX_Global_Control']!='LFX_File') && ($_POST['LFX_Global_Control']!='LFX_File')) {
        return(NULL);
      }
      if (!LFX_permsFilter($mode,'W')) {
        return(NULL);
      }

      // Running action 'create'
      if ($LFile_action=='create') {
        if (!LFX_permsFilter($perm,'W')) {
          $this->addError("You don't have permissions to create files here!");
          return(false);
        }
        if (!is_uploaded_file($_FILES['newLFXFile']['tmp_name'])) {
          return(false);
        }
        if (!$LFile_title) {
          $LFile_title='['.__L('no name').']';
        }
        $file=new $thisClass;
        $fp=fopen($_FILES['newLFXFile']['tmp_name'],'rb');
        $file->setAttr('doc',fread($fp,filesize($_FILES['newLFXFile']['tmp_name'])));
        fclose($fp);

        $file->setAttrs(array(
          'src'=>$_page->user->id,
          'date'=>time(),
          'objectClass'=>$objectClass,
          'objectId'=>$objectId,
          'objectInclude'=>$includeKey));
        $file->setAttrs(array(
          'file_name'=>$_FILES['newLFXFile']['name'],
          'comment'=>$LFile_comment,
          'title'=>$LFile_title,
          ),'gpc');
        if (!($fsResult=$file->save())) {
          return($fsResult);
        }
        LFX_Include($includeKey);
        $object=new $objectClass($objectId);
        if ($LFile_obsolete) {
          if (!LFX_permsFilter($perm,'U')) {
            $this->addError("You don't have permissions to obsolete files here!");
            return(false);
          }
          $obsoleteFile=new $thisClass($LFile_obsolete);
          if ($obsoleteFile->getAttr('obsoleted_by')) {
            $obsoleteFile->addError("Trying to get obsoleted by file ".
             ($file->id)." but already obsoleted by file ".
             ($obsoleteFile->getAttr('obsoleted_by')).
             " (in LFX_File::performActions())");
            return(false);
          } else {
            $obsoleteFile->setAttr('obsoleted_by',$file->id);
            $obsoleteFile->save();
            $message=$object->fileNotification('Created, old file obsoleted',$file);
            $file->sendNotifications(array_merge($_POST['notifU'],$_POST['notifG']),'Created, old file obsoleted',$message);
            return(true);
          }
        }
        $LFile_title='';
        $message=$object->fileNotification('Created',$file);
        $file->sendNotifications(array_merge($_POST['notifU'],$_POST['notifG']),'Created',$message);
        return(true);
      }

      // Running action 'delete'
      if ($LFile_action=='delete') {
        if (!LFX_permsFilter($perm,'D')) {
          $this->addError("You don't have permissions to delete files here!");
          return(false);
        }
        $file=new $thisClass($LFile_id);
        if (!$file->probe()) {
          return(NULL);
        }
        return($file->delete());
      }
      
      return(NULL);
    }

      /**
      * This method renders the notification controls, if variable $notif
      * matches any users to notify. Please check the global documentation
      * for this class for details on the $notif variable and how to use
      * it in your classes.
      */
    function renderNotificationControlsH($notifU,$notifG,$xtraHTML='')
    {
      if (is_array($notifU)) {
        $notifU=array_values($notifU);
      } elseif ($notifU===true) {
        $tmp=new LFX_User;
        $notifU=$tmp->search(NULL,NULL,'email');
      } else {
        unset($notifU);
      }
      if (is_array($notifG)) {
        $notifG=array_values($notifG);
      } elseif ($notifG===true) {
        $tmp=new LFX_Group;
        $notifG=$tmp->search(NULL,NULL,'name');
      } else {
        unset($notifG);
      }
      if (!$notifU && !$notifG) {
        return('');
      }

      // Users
      $numeric_only=true;
      for($i=0;$i<count($notifU);$i++) {
        if (!is_numeric($notifU[$i])) {
          $numeric_only=false;
          break;
        }
      }
      if ($numeric_only) {
        $tmp=new LFX_User;
        $notifU=$tmp->_doLoad($notifU);
      }
      $users=array();
      for($i=0;$i<count($notifU);$i++) {
        $myUser=$notifU[$i];
        if (is_numeric($myUser)) {
          $myUser=new LFX_User($myUser);
          $myUser=array('display'=>$myUser->getNameH(),'value'=>$myUser->getAttrH('email'));
        } elseif (is_object($myUser) && (get_class($myUser)=='lfx_user')) {
          if (!$myUser->getAttr('email')) {
            continue;
          }
          $myUser=array('display'=>$myUser->getNameH(),'value'=>$myUser->getAttrH('email'));
        } elseif (is_string($myUser)) {
          $tmp=new LFX_User;
          if ($myUserObject=$tmp->search('email',$myUser)) {
            $myUser=array('display'=>$myUserObject[0]->getNameH(),'value'=>$myUserObject[0]->getAttrH('email'));
          } else {
            $myUser=array('display'=>htmlspecialchars($myUser,ENT_QUOTES),'value'=>htmlspecialchars($myUser,ENT_QUOTES));
          }
        } else {
          continue;
        }
        $users[]=$myUser;
      }

      // Groups
      $numeric_only=true;
      for($i=0;$i<count($notifG);$i++) {
        if (!is_numeric($notifG[$i])) {
          $numeric_only=false;
          break;
        }
      }
      if ($numeric_only) {
        $tmp=new LFX_Group;
        $notifG=$tmp->_doLoad($notifG);
      }
      $groups=array();
      for($i=0;$i<count($notifG);$i++) {
        $myGroup=$notifG[$i];
        if (is_numeric($myGroup)) {
          $myGroup=new LFX_Group($myGroup);
          if (!$myGroup->getAttr('email')) {
            continue;
          }
          $myGroup=array('display'=>$myGroup->getNameH(),'value'=>$myGroup->getAttrH('email'));
        } elseif (is_object($myGroup) && (get_class($myGroup)=='lfx_group')) {
          if (!$myGroup->getAttr('email')) {
            //echo("No mail for group ".$myGroup->getLinkH());
            continue;
          }
          $myGroup=array('display'=>$myGroup->getNameH(),'value'=>$myGroup->getAttrH('email'));
        } elseif (is_string($myGroup)) {
          $tmp=new LFX_Group;
          if ($myGroupObject=$tmp->search('email',$myGroup)) {
            $myGroup=array('display'=>$tmp->getNameH(),'value'=>$tmp->getAttrH('email'));
          } else {
            $myGroup=array('display'=>htmlspecialchars($myGroup,ENT_QUOTES),'value'=>htmlspecialchars($myGroup,ENT_QUOTES));
          }
        } else {
          continue;
        }
        $groups[]=$myGroup;
      }
      if (!$users && !$groups) {
        return('');
      }
      $result='';
      $result.=LFX_echo("<td valign='top'>");
      $result.=LFX_startTables('100%');
      $result.=LFX_swatchHeader(array('title'=>__L('Notifications')));
      if ($users) {
        $result.=LFX_str();
        $result.=LFX_echo("<td>");
        $result.=LFX_echo("<select name='notifU[]' id='filenotifU' size='".min(5,count($users))."' multiple>");
        for($i=0;$i<count($users);$i++) {
          $result.=LFX_echo("<option value='{$users[$i]['value']}'>{$users[$i]['display']}</option>");
        }
        $result.=LFX_echo("</select>");
        $result.=LFX_echo("</td>");
        $result.=LFX_etr();
      }
      if ($groups) {
        $result.=LFX_str();
        $result.=LFX_echo("<td>");
        $result.=LFX_echo("<select name='notifG[]' id='filenotifG' size='".min(5,count($groups))."' multiple>");
        for($i=0;$i<count($groups);$i++) {
          $result.=LFX_echo("<option value='{$groups[$i]['value']}'>{$groups[$i]['display']}</option>");
        }
        $result.=LFX_echo("</select>");
        $result.=LFX_echo("</td>");
        $result.=LFX_etr();
      }
      $result.=$xtraHTML;
      $result.=LFX_endTables();
      $result.=LFX_echo("</td>");
      return($result);
    }

      // Variable names start with "LFile" which stands for "LFX_File";
      // The difference between this _makeURL and the one in LFX_forum_message
      // is that in this regexp we include the underscore and "File" -- this
      // way we ensure that even when an object uses both files and forums,
      // they will both work independently.
      /**
      * This function gets the current page URL and adds the variables sent through
      * $vars which must be an array of the form '$var_name'=>'var_value'.
      * @return string the parsed URL
      */
    function _makeURL($vars=NULL)
    {
      global $REQUEST_URI,$QUERY_STRING;

      $this->url=ereg_replace('LFile[a-zA-Z_]+=[^&]*','',$REQUEST_URI);
      while(strpos(' '.$this->url,'&&')) {
        $this->url=str_replace('&&','&',$this->url);
      }
      $this->url=ereg_replace('&$','',$this->url);
      $this->url=ereg_replace('\\?\\?$','?',$this->url);
      if ($vars==NULL || !is_array($vars)) {
        return($this->url);
      }

      if (!$QUERY_STRING && substr($this->url,-1)!='?') {
        $this->url.="?";
      }
      while(list($var,$value)=@each($vars)) {
        if (substr($this->url,-1)!='?') {
          $this->url.="&$var=$value";
        } else {
          $this->url.="$var=$value";
        }
      }
      return($this->url);
    }
    
    function getLinkH()
    {
      global $_LFX_url;
      return("<a href='".htmlspecialchars($this->getUrlH())."'>".
        $this->getAttrH('title')."</a>");
    }
    
    function getUrlH()
    {
      global $_LFX_url;
      return("$_LFX_url/serve_file.php?module={$this->module}&id={$this->id}");
    }
    
    function getFullUrlH()
    {
      global $_LFX_fullUrl;
      return("$_LFX_fullUrl/serve_file.php?module={$this->module}&id={$this->id}");
    }
    
    function sendNotifications($notif,$reason,$mail)
    {
      if (!$notif) {
        return(NULL);
      }
      if (!is_object($mail) || (get_class($mail)!='lfx_mail')) {
        global $_page;
        LFX_include('LFX_Mail');
        $mail=new LFX_Mail;
        $mail->subject=sprintf(__O('File %1$s: %2$s'),$this->getAttr('title'),$reason);
        $mail->text=sprintf(__O("File %1\$s (%2\$s) was altered (%3\$s) ".
          "by %4\$s and he/she wanted to notify you about it. You can access the file at this address:\n\n%5\$s"),
          $this->getAttr('title'),$this->getAttr('file_name'),
          __L($reason),$_page->user->getName(),$this->getFullUrlH());
      }
      $mail->addresses=$notif;
      if (LFX_include_class('LFX_File_Notif')) {
        for($i=0;$i<count($notif);$i++) {
          $tmp=new LFX_File_Notif();
          $tmp->setAttrs(array(
            'file_id'=>$this->id,
            'notif_email'=>$notif[$i]
          ));
          $tmp->save();
        }
      } else {
        $this->addError('Failed to include the LFX_File_Notif class!');
      }
      return($mail->send());
    }
  }
?>
