<?
  /**
  * LFX calendar class
  * Bogdan Stancescu <bogdan@moongate.ro>, April 2004
  * $Id: LFX_global_calendar.php,v 1.1 2005/06/09 08:45:46 bogdan Exp $
  * @package LFX
  */
  
  class LFX_Global_Calendar extends LFX_Object
  {

    /**
    * This is a variable in which we cache all calendar data for the
    * current object.
    * If you want to clear the cache, call LFX_Calendar::{@link clearCache}().
    */
    var $calendar_data;
    
    function LFX_Global_Calendar($id=0)
    {
      $this->LFX_Object($id);
      
      $tmp['description']=
      $tmp['date']=
      $tmp['recuring']=
      array();
      
      $tmp['date']['flags']['sqlDate']=true;
        
      $this->dataStructure=array(
        'table_name' => 'global_calendar',
          'id_field' => 'id',
            'fields' => $tmp
      );
      $this->fillDataStructure();
    }
    
    /*
      Schematic structure of the calendar_data array:
      data => (#)
        entry_id => the entry id in the db
        date => xxxxxxxx
        description => "xxx"
        recurring => x
      weekly_data => (a)
        entry_id => the weekly entry id in the db
        values => (#)
          start_time => xxxx
          end_time => xxxx
          free => x
      recurring => (#)
        date => xxxx
      non_recurring => (#)
        date => xxxxxxxx
    */

    
    
    function getCalendarData()
    {
      global $_LFX;
      if ($this->calendar_data) {
        // If we have a current cache, then the system cache should be set.
        // If it isn't, we must clear our cache as well.
        // A much better solution would be to actually store cache time somewhere
        // and place a cache expiry date at clearCache time, because
        // the way we do things now, several objects reading and writing
        // independently will mess things up. We don't expect this to be
        // a problem though, due to the fact that this is PER-PAGE caching.
        // When we move to per-server caching, we must take this into account.
        if ($_LFX['page']['data_cache']['lfx_calendar_data']) {
          return($this->calendar_data);
        }
      }
      if ($_LFX['page']['data_cache']['lfx_calendar_data']) {
        $this->calendar_data=$_LFX['page']['data_cache']['lfx_calendar_data'];
        return($this->calendar_data);
      }
      $data=$this->search();
      $calendar_data=array();
      for($i=0;$i<count($data);$i++) {
        $myData=&$data[$i];
        if ($myData->getAttr('date')<43200) {
          // Found the entry which deals with the weekly data, let's parse it
          $weekData=explode('|',$myData->getAttr('description'));
          if (count($weekData)!=7) {
            $this->addError("Wrong weekly data format! Internal error!");
          } else {
            $myWeekData=array();
            for($j=0;$j<count($weekData);$j++) {
              $myWeekData[]=array(
                'start_time'=>substr($weekData[$j],0,4),
                'end_time'=>substr($weekData[$j],5,4),
                'free'=>(substr($weekData[$j],9,1)=='x'));
            }
            $calendar_data['weekly_data']=array(
              'entry_id'=>$myData->id,
              'values'=>$myWeekData);
          }
        } elseif($myData->getAttr('recurring')) {
          $calendar_data['recurring'][]=date('dm',$myData->getAttr('date'));
        } else {
          $calendar_data['non_recurring'][]=date('dmY',$myData->getAttr('date'));
        }
        $calendar_data['data'][]=array(
          'entry_id'=>$myData->id,
          'date'=>$myData->getAttr('date'),
          'description'=>$myData->getAttr('description'),
          'recurring'=>$myData->getAttr('recurring'));
      }
      $this->calendar_data=$calendar_data;
      $_LFX['page']['data_cache']['lfx_calendar_data']=$calendar_data;
      return($calendar_data);
    }

    function setCalendarData()
    {
      if (!$this->calendar_data) {
        return(false);
      }
      $tempData='';
      $cd=$this->calendar_data;
      for($j=0;$j<count($cd['weekly_data']['values']);$j++) {
        $tempData.=$cd['weekly_data']['values'][$j]['start_time'].'-'.
          $cd['weekly_data']['values'][$j]['end_time'].
          ($cd['weekly_data']['values'][$j]['free']?'x':' ').'|';
      }
      $curWeeklyData=substr($tempData,0,-1);
      $weeklySaved=false;
      for($i=0;$i<count($cd['data']);$i++) {
        $tmp=new LFX_Global_Calendar($cd['data'][$i]['entry_id']);
        if ($tmp->id) {
          $tmp->load();
        }
        $tmp->setAttrs(array(
          'description'=>$cd['data'][$i]['description'],
          'date'=>$cd['data'][$i]['date'],
          'recurring'=>$cd['data'][$i]['recurring']));
        if (($tmp->id) && ($tmp->id==$cd['weekly_data']['entry_id'])) {
          $tmp->setAttr('description',$curWeeklyData);
          $weeklySaved=true;
        }
        $tmp->save();
      }
      if (!$weeklySaved) {
        $tmp=new LFX_Global_Calendar;
        $tmp->setAttrs(array(
          'description'=>$curWeeklyData,
          'date'=>0,
          'recurring'=>1));
        $tmp->save();
      }
      $this->clearCache();
      $this->getCalendarData();
      return(true);
    }
    
    function clearCache()
    {
      unset($_LFX['page']['data_cache']['lfx_calendar_data']);
      unset($this->calendar_data);
    }
 /*   
    function isFree($date)
    {
      $cd=$this->getCalendarData();
      $dow=date('w',$date);
      if ($cd['weekly_data']['values'][$dow]['free']) {
        return(true);
      }
      if (@in_array(date('dm',$date),$cd['recurring'])) {
        return(true);
      }
      if (@in_array(date('dmY',$date),$cd['non_recurring'])) {
        return(true);
      }
      return(false);
    }
   */
   /*
    function workDateDiff($new,$old)
    {
      if ((!is_numeric($new)) || (!is_numeric($old))) {
        return(false);
      }
      if ($old>$new) {
        $tmp=$old;
        $old=$new;
        $new=$tmp;
        unset($tmp);
      }
      $cd=$this->getCalendarData();
      if (!$cd['weekly_data']) {
        return($new-$old);
      }
      $cdow=$dow=date('w',$old); // (Current) Day of the week
      //pecho("Cdow=$cdow<br>");
      $diff=0;
      while($old!=$new) {
        // First, skip to the next working day
        while($this->isFree($old)) {
          //pecho("Skipping free day ".date('r',$old)."<br>");
          $old=$this->skipToNextDay($old);
          $cdow=($cdow+1) % 7;
        }
        $ctime=date('Hi',$old);
        //pecho("Ctime: ".$ctime."<br>");
        // Then make sure that we didn't accidentally end up after business hours.
        // If we did, we must skip to the beginning of the working hours next day.
        if ($ctime>$cd['weekly_data']['values'][$cdow]['end_time']) {
          //pecho("After hours ".date('r',$old)."<br>");
          $old=$this->skipToNextDay($old);
          $cdow=($cdow+1) % 7;
          while($this->isFree($old)) {
            //pecho("(Skipping free day ".date('r',$old).")<br>");
            $old=$this->skipToPreviousDay($old);
            $cdow=($cdow+1) % 7;
          }
          $nextMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
          $old=mktime(substr($nextMorning,0,2),substr($nextMorning,2),0,date('m',$old),date('d',$old),date('Y',$old));
          //pecho("Fixed after hours, ".date('r',$old)."<br>");
        } elseif ($ctime<$cd['weekly_data']['values'][$cdow]['start_time']) {
          //pecho("Too early, ".date('r',$old)."<br>");
          // This is the "other" case, which is the current one: we ended up
          // at midnight in the previous loop, so we must skip to the beginning
          // of the business hours today
          $thisMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
          $old=mktime(substr($thisMorning,0,2),substr($thisMorning,2),0,date('m',$old),date('d',$old),date('Y',$old));
          //pecho("Fixed too early, ".date('r',$old)."<br>");
        }
        //pecho("Starting calculations, date is ".date('r',$old)."; diff so far ".($diff/3600)."<br>");
        if ($old>=$new) {
          //pecho("What do you know, we already went past the deadline during non-business hours! Let's return what we've got so far.<br>");
          return($diff);
        }
        // Ok, we're now guaranteed to be at the beginning of business hours
        // in a working day. Let's check if the interval ends some time today,
        // and return that date if so.
        $thisEvening=$cd['weekly_data']['values'][$cdow]['end_time'];
        $eveningDate=mktime(substr($thisEvening,0,2),substr($thisEvening,2),0,date('m',$old),date('d',$old),date('Y',$old));
        //pecho("Here's this evening's time: ".date('r',$eveningDate)."<br>");
        if ($eveningDate>=$new) {
          //pecho("That's today, returning diff. (".date('r',$eveningDate)." > ".date('r',$new).")<br>");
          return($new-$old+$diff);
        }
        // Bah, no luck, we need to loop again. Ok, let's diff the diff
        // (or do the dew, whatever), and skip to tomorrow. Let's consider
        // DST switches while we're at it, using the current mechanism:
        // * if the time goes "forward" one hour, then we're in the clear,
        //   the 2nd condition above in the next loop will fix it.
        // * if the time goes "backward" one hour (i.e. the second after
        //   23:59:59 today is 23:00:00 *today*) then we branch into two
        //   possible scenarios:
        //   1. Today's work day ends before 23:00. In that case all is well,
        //      the first condition above will catch the discrepancy and
        //      skip to tomorrow, starting of business hours.
        //   2. Today's work day ends after 23:00. Theoretically that would
        //      also result in proper behaviour because in the next loop we'll
        //      count again the hour between 23:00 today and 0:00 tomorrow,
        //      which is right. The only problem is that the same algorithm is
        //      used again -- $eveningDate is calculated using the exact same
        //      technique, $old is calculated using the exact same technique.
        //      That's why we need "$oldDate" -- if we encounter the same date
        //      twice then we skip an extra hour to escape the infinite loop.
        //      (As a matter of fact it wouldn't actually be infinite, $diff
        //      would eventually end up to 0 at some point. The only potential
        //      problem would be that every time this method would have
        //      to run over a time span encompassing that particular DST
        //      switch in this particular situation, it would return dates
        //      within that fateful hour).
        $diff+=$eveningDate-$old;
        $old=mktime(23,59,59,date('m',$old),date('d',$old),date('Y',$old))+1;
        if ($oldDate==$old) {
          //pecho("DST?<br>");
          $old+=3600;
        }
        $oldDate=$old;
        $cdow=($cdow+1) % 7;
        //pecho("Not today, this is what I got as being tomorrow morning: ".date('r',$old)."; diff so far: ".($diff/3600)."<br>");
      }
      return($diff);
    }
*/
    function skipToNextDay($date)
    {
      global $skipNext;
      $skipNext++;
      $day=date('d',$date);
      $month=date('m',$date);
      $year=date('Y',$date);
      $date=mktime(23,59,59,$month,$day,$year)+1;
      if ($day==date('d',$date)) {
        // DST
        $date+=3600;
      }
      return($date);
    }

    function skipToPreviousDay($date)
    {
      $day=date('d',$date);
      $month=date('m',$date);
      $year=date('Y',$date);
      $date=mktime(0,0,0,$month,$day,$year)-1;
      if ($day==date('d',$date)) {
        // DST
        $date-=3600;
      }
      return($date);
    }

      /**
      * Gets an arbitrary date and either returns it as is if it's during business
      * hours, or skips forward to the first business hour available and returns
      * that date.
      *
      * @param integer $date the input date as UNIX timestamp (PHP datetime)
      * @return integer the first business date encountered, as UNIX timestamp
      */
    function skipToBusinessDate($date)
    {
      $cd=$this->getCalendarData();
      if (!$cd['weekly_data']) {
        return($date);
      }
      $cdow=$dow=date('w',$date); // (Current) Day of the week
      // First, skip to the next working day
      while($this->isFree($date)) {
        //pecho("Skipping free day ".date('r',$date)."<br>");
        $date=$this->skipToNextDay($date);
        $cdow=($cdow+1) % 7;
      }
      $ctime=date('His',$date);
      // Then make sure that we didn't accidentally end up after business hours.
      // If we did, we must skip to the beginning of the working hours next day.
      if ($ctime>$cd['weekly_data']['values'][$cdow]['end_time']*100) {
        //pecho("After hours ".date('r',$date)."<br>");
        $date=$this->skipToNextDay($date);
        $cdow=($cdow+1) % 7;
        while($this->isFree($date)) {
          //pecho("(Skipping free day ".date('r',$date).")<br>");
          $date=$this->skipToNextDay($date);
          $cdow=($cdow+1) % 7;
        }
        $nextMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
        $date=mktime(substr($nextMorning,0,2),substr($nextMorning,2),0,date('m',$date),date('d',$date),date('Y',$date));
        //pecho("Fixed after hours, ".date('r',$date)."<br>");
      } elseif ($ctime<$cd['weekly_data']['values'][$cdow]['start_time']*100) {
        //pecho("Too early, ".date('r',$date)."<br>");
        // This is the "other" case, which is the current one: we ended up
        // at midnight in the previous loop, so we must skip to the beginning
        // of the business hours today
        $thisMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
        $date=mktime(substr($thisMorning,0,2),substr($thisMorning,2),0,date('m',$date),date('d',$date),date('Y',$date));
        //pecho("Fixed too early, ".date('r',$date)."<br>");
      }
      return($date);
    }

    function workDateAdd($date,$seconds)
    {
      $cd=$this->getCalendarData();
      if (!$cd['weekly_data']) {
        return($date+$seconds);
      }
      $cdow=$dow=date('w',$date); // (Current) Day of the week
      //pecho("Cdow=$cdow<br>");
      $diff=$seconds;
      while($diff) {
        // First, skip to the next working day
        while($this->isFree($date)) {
          //pecho("Skipping free day ".date('r',$date)."<br>");
          $date=$this->skipToNextDay($date);
          $cdow=($cdow+1) % 7;
        }
        $ctime=date('His',$date);
        // Then make sure that we didn't accidentally end up after business hours.
        // If we did, we must skip to the beginning of the working hours next day.
        if ($ctime>$cd['weekly_data']['values'][$cdow]['end_time']*100) {
          //pecho("After hours ".date('r',$date)."<br>");
          $date=$this->skipToNextDay($date);
          $cdow=($cdow+1) % 7;
          while($this->isFree($date)) {
            //pecho("(Skipping free day ".date('r',$date).")<br>");
            $date=$this->skipToNextDay($date);
            $cdow=($cdow+1) % 7;
          }
          $nextMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
          $date=mktime(substr($nextMorning,0,2),substr($nextMorning,2),0,date('m',$date),date('d',$date),date('Y',$date));
          //pecho("Fixed after hours, ".date('r',$date)."<br>");
        } elseif ($ctime<$cd['weekly_data']['values'][$cdow]['start_time']*100) {
          //pecho("Too early, ".date('r',$date)."<br>");
          // This is the "other" case, which is the current one: we ended up
          // at midnight in the previous loop, so we must skip to the beginning
          // of the business hours today
          $thisMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
          $date=mktime(substr($thisMorning,0,2),substr($thisMorning,2),0,date('m',$date),date('d',$date),date('Y',$date));
          //pecho("Fixed too early, ".date('r',$date)."<br>");
        }
        //pecho("Starting calculations, date is ".date('r',$date)."; time left ".($diff/3600)."<br>");
        // Ok, we're now guaranteed to be at the beginning of business hours
        // in a working day. Let's check if the interval ends some time today,
        // and return that date if so.
        $thisEvening=$cd['weekly_data']['values'][$cdow]['end_time'];
        $eveningDate=mktime(substr($thisEvening,0,2),substr($thisEvening,2),0,date('m',$date),date('d',$date),date('Y',$date));
        if ($date+$diff<=$eveningDate) {
          //pecho("That's today, returning date.<br>");
          return($date+$diff);
        }
        // Bah, no luck, we need to loop again. Ok, let's diff the diff
        // (or do the dew, whatever), and skip to tomorrow. Let's consider
        // DST switches while we're at it, using the current mechanism:
        // * if the time goes "forward" one hour, then we're in the clear,
        //   the 2nd condition above in the next loop will fix it.
        // * if the time goes "backward" one hour (i.e. the second after
        //   23:59:59 today is 23:00:00 *today*) then we branch into two
        //   possible scenarios:
        //   1. Today's work day ends before 23:00. In that case all is well,
        //      the first condition above will catch the discrepancy and
        //      skip to tomorrow, starting of business hours.
        //   2. Today's work day ends after 23:00. Theoretically that would
        //      also result in proper behaviour because in the next loop we'll
        //      count again the hour between 23:00 today and 0:00 tomorrow,
        //      which is right. The only problem is that the same algorithm is
        //      used again -- $eveningDate is calculated using the exact same
        //      technique, $date is calculated using the exact same technique.
        //      That's why we need "$oldDate" -- if we encounter the same date
        //      twice then we skip an extra hour to escape the infinite loop.
        //      (As a matter of fact it wouldn't actually be infinite, $diff
        //      would eventually end up to 0 at some point. The only potential
        //      problem would be that every time this method would have
        //      to run over a time span encompassing that particular DST
        //      switch in this particular situation, it would return dates
        //      within that fateful hour).
        $diff-=$eveningDate-$date;
        $date=mktime(23,59,59,date('m',$date),date('d',$date),date('Y',$date))+1;
        if ($oldDate==$date) {
          //pecho("DST?<br>");
          $date+=3600;
        }
        $oldDate=$date;
        $cdow=($cdow+1) % 7;
        //pecho("Not today, this is what I got as being tomorrow morning: ".date('r',$date)."; time left: ".($diff/3600)." ($diff)<br>");
      }
      return($date);
    }

    function workDateSub($date,$seconds)
    {
      $cd=$this->getCalendarData();
      if (!$cd['weekly_data']) {
        return($date-$seconds);
      }
      $cdow=$dow=date('w',$date); // (Current) Day of the week
      //pecho("Cdow=$cdow<br>");
      $diff=$seconds;
      while($diff) {
        // First, skip to the previous working day
        while($this->isFree($date)) {
          //pecho("Skipping free day ".date('r',$date)."<br>");
          $date=$this->skipToPreviousDay($date);
          $cdow=($cdow+6) % 7;
        }
        $ctime=date('His',$date);
        // Then make sure that we didn't accidentally end up after business hours.
        // If we did, we must skip to the end of the working hours today.
        if ($ctime>$cd['weekly_data']['values'][$cdow]['end_time']*100) {
          //pecho("After hours ".date('r',$date)."<br>");
          $thisEvening=$cd['weekly_data']['values'][$cdow]['end_time'];
          $date=mktime(substr($thisEvening,0,2),substr($thisEvening,2),0,date('m',$date),date('d',$date),date('Y',$date));
          //pecho("Fixed after hours, ".date('r',$date)."<br>");
        } elseif ($ctime<$cd['weekly_data']['values'][$cdow]['start_time']*100) {
          //pecho("Too early, ".date('r',$date)."<br>");
          // This is the "other" case, which is the current one: we ended up
          // at midnight in the previous loop, so we must skip to the end
          // of the business hours the previous day
          $date=$this->skipToPreviousDay($date);
          $cdow=($cdow+6) % 7;
          while($this->isFree($date)) {
            //pecho("(Skipping free day ".date('r',$date).")<br>");
            $date=$this->skipToPreviousDay($date);
            $cdow=($cdow+6) % 7;
          }
          $previousEvening=$cd['weekly_data']['values'][$cdow]['end_time'];
          $date=mktime(substr($previousEvening,0,2),substr($previousEvening,2),0,date('m',$date),date('d',$date),date('Y',$date));
          //pecho("Fixed too early, ".date('r',$date)."<br>");
        }
        //pecho("Starting calculations, date is ".date('r',$date)."; time left ".($diff/3600)."<br>");
        // Ok, we're now guaranteed to be at the end of business hours
        // in a working day. Let's check if the interval starts some time today,
        // and return that date if so.
        $thisMorning=$cd['weekly_data']['values'][$cdow]['start_time'];
        $morningDate=mktime(substr($thisMorning,0,2),substr($thisMorning,2),0,date('m',$date),date('d',$date),date('Y',$date));
        if ($date-$diff>=$morningDate) {
          //pecho("That's today, returning date.<br>");
          return($date-$diff);
        }
        $diff-=$date-$morningDate;
        $date=mktime(0,0,0,date('m',$date),date('d',$date),date('Y',$date))-1;
        if ($oldDate==$date) {
          //pecho("DST?<br>");
          $date-=3600;
        }
        $oldDate=$date;
        $cdow=($cdow+6) % 7;
        //pecho("Not today, this is what I got as being yesterday evening: ".date('r',$date)."; time left: ".($diff/3600)." ($diff)<br>");
      }
      return($date);
    }
  }
  
?>
