<?

  /**
  * The group class
  * Andrei Tinca <andrei@lanifex.com>, November 2002
  * Lucian Silistru <lucian@moongate.ro>, April 2003
  * $Id: LFX_group.php,v 1.27 2005/10/12 13:51:01 bogdan Exp $
  * @package LFX
  */
    /**
    * This is the group class
    * This class lets you create groups, find out which groups a group is
    * included in, find out a group's users etc.
    */

  _checkMeOut(__FILE__);

  $result=array('LFX_Object', 'LFX_Access');
  if (!_checkIncludes($result)) {
    return($result);
  }

  class LFX_Group extends LFX_Access {

    var $accessList=array();

    function LFX_Group($id=0) {
      $this->LFX_Access($id);

      $tmp['name']=
      $tmp['description']=
      $tmp['type']=
      $tmp['deactivated']=
      $tmp['email']=
      $tmp['date_created']=
      $tmp['module']=
      array();

      // Defining DEPENDENCIES
      $tmp2['members']=array(
        'class'=>'LFX_user_group',
        'attr' =>'parent_group');
      $tmp2['user_group_child']=array(
        'class'=>'LFX_user_group',
        'attr' =>'child_group');

      // Defining FLAGS
      $tmp['date_created']['flags']['sqlDate']=true;
      $tmp['date_created']['flags']['forceSave']=true;
      $tmp['description']['flags']['short_in_db']=true;
      $tmp['description']['flags']['large']=true;

      $this->dataStructure=array(
          'table_name'=>'groups',
          'id_field'=>'id',
          'fields'=>$tmp,
          'depend'=>$tmp2,
          'title_attr'=>'name',
          'object_page'=>'/access_manager/show_group?id=%d'
      );

      $this->fillDataStructure();
    }

      /**
      * Checks whether the data to be saved is valid.
      *
      * Will enforce the name field to be at least 4 chars. long, and
      * the comment field to be at least 10 chars. long.
      */

    function beforeSave()
    {
      if(strlen($this->getAttr('name'))<4) {
        $this->addError('The group name must be at least 4 characters long. The group was not saved.');
        return(false);
      }
      if(strlen($this->getAttr('description'))<10) {
        $this->addError('The description must be at least 10 characters long. The group was not saved.');
        return(false);
      }
      return(true);
    }

    function isActive($id=0)
    {
      $group=&$this->defaultObject($id);
      return(!$group->getAttr('deactivated'));
    }
    
      /**
      * Gets the id's of direct members of this group.
      *
      * Returns an array of the form:
      * $members = array(user  = array(
      *                            array(user_id,user_acc),  // Contains user id's and user accesses to group
      *                          ),
      *                  group = array(
      *                            array(group_id,group_acc), // Contains group id's and group accesses to group
      *                          ),  
      *                  size  = value)    // number of users + number of groups
      *
      * @return an array with the id's of the member groups and users.
      */
    function getGroupMembersID($id=0,$show_only_active=NULL)
    {
      $group=&$this->defaultObject($id);
      $my_groups=$group->getObjects('members');
      // Making sure we return an array even if we find no elements
      $members=array(
        'user'=>array(),
        'group'=>array()
      );
      while(list(,$my_group)=@each($my_groups)) {
        if ($u=$my_group->getAttr('child_user')) {
          $show=true;
          if($show_only_active) {
            $usr=new LFX_User($u);
            if ($usr->getAttr('deactivated')) {
              $show=false;
            }
          }
          if($show) {
            $members['user'][]=array($u,$my_group->getAttr('access'));
          }
        } elseif ($g=$my_group->getAttr('child_group')) {
          $show=true;
          if($show_only_active) {
            $grp=new LFX_Group($g);
            if ($grp->getAttr('deactivated')) {
              $show=false;
            }
          }
          if($show) {
            $members['group'][]=array($g,$my_group->getAttr('access'));
          }
        } else {
          // You should keep track of the 33 below and not use duplicates
          $group->addError("Link record with id {$my_group->id} has no elements! This is a possible integrity problem.",
            EH_DV_ANDREI+33, EH_ET_WARNING+EH_EL_UNEXPECTED);
        }
      }
      $members['size'] = sizeOf($members['user'])+sizeOf($members['group']);
      return($members);
    }
    
    function getMemberUsersID($id=0)
    {
      $group=&$this->defaultObject($id);
      
      $all_members = $group->getGroupMembersID($id);
      $users = array();
      $users = $all_members['user'];
      return($users);
    }
    
    function getMemberGroupsID($id=0)
    {
      $group=&$this->defaultObject($id);

      $all_members = $group->getGroupMembersID($id);
      $users = array();
      $users = $all_members['group'];
      return($users);
    }
    
      /**
      * Gets the id's of all members of this group (by direct link or inheritance), recursively
      *
      * Works like getGroupMembersID, but recurses in the members of the group
      * on which you are calling this function.
      *
      * @return an array with the id's of the member groups and users.
      */    
    function getGroupMembersIDRecursive($id=0,$members=array(user=>array(),group=>array()),$r=0)
    {
      $group=&$this->defaultObject($id);
      $my_groups=$group->getObjects('members');
      while(list(,$my_group)=@each($my_groups)) {
        if ($u=$my_group->getAttr('child_user')) {
          $members['user'][]=array($u,$my_group->getAttr('access'));
        } elseif ($g=$my_group->getAttr('child_group')) {
          $cycle=false;
          for($i=0;$i<count($members['group']);$i++) {
            if ($g==$members['group'][$i]) {
              $cycle=true;
              break;
            }
          }
          if ($cycle) {
            // Problem! We already have this member in the member list -- we'll ignore it here, but this should NEVER, EVER happen!
            $g_g=new LFX_Group($g);
            $this->addError("There is a cyclic membership relation between ".
              "group #$g (".$g_g->getNameH().") ".
              "and group #{$group->id} (".$group->getNameH().")! ".
              "Please decide which one of the two relationships you want ".
              "to keep, and delete the other one using the interface.");
            continue;
          }
          $members['group'][]=array($g,$my_group->getAttr('access'));
          $recursive_group = new LFX_Group($g);
          $members = $recursive_group->getGroupMembersIDRecursive($g,$members,1);
        } else {
          // You should keep track of the 33 below and not use duplicates
          $group->addError("Link record with id {$my_group->id} has no elements!
             This is an integrity problem in table 'user_group'. Please contact your
             developer.", EH_DV_ANDREI+33, EH_ET_WARNING+EH_EL_UNEXPECTED);
        }
      }
      
      $members['size'] = sizeOf($members['user'])+sizeOf($members['group']);
      $members = $this->make_unique($members);
      return($members);
    }
    
      /**
      * Gets the id's of the parents of this group
      *
      * @return an array with the id's of the groups which parent this group and 
      *  this groups access rights to them
      */
    function getGroupParentsID($id=0)
    {
      $group=&$this->defaultObject($id);

      // We return an array, at least. Not doing so breaks some other code
      // Which expects an array from this function
/*      
      $parents = array(
        0=>array(),
        1=>array()
      );
*/      
      $my_groups=$group->getObjects('user_group_child');
      while(list(,$my_group)=@each($my_groups)) {
        if ($g=$my_group->getAttr('parent_group')) {
          $parents[]=array($g,$my_group->getAttr('access'));
        }
      }
      return($parents);
    }

      /**
      * Gets the id's of the parents of this group, recursively
      *
      * In addition to the direct parents of the group, it recureses to find out
      * the parents of this group's parents etc.
      *
      * @return an array with the id's of the groups which parent this group and 
      *  this groups access rights to them
      */    
    function getGroupParentsIDRecursive($id=0,$initial=true)
    {
      static $parents;
      $group=&$this->defaultObject($id);

      if ($initial || !isset($parents)) {
        $parents = array();
      }
      
      $my_groups=$group->getObjects('user_group_child');
      while(list(,$my_group)=@each($my_groups)) {
        if ($g=$my_group->getAttr('parent_group')) {
          $parents[]=array($g,$my_group->getAttr('access'));
          $group->getGroupParentsIDRecursive($g,false);
        }
      }
      return($parents);
    }
    
      /**
      * Gets you the id's of the users or groups which can be made members
      *
      * A possible member user is an user which is not already a member group.
      *
      * A possible member group is a group wihch is not already a member of this
      * group, or which is not a parent of this group (either directly or
      * recursively found. The return is in the same form as @
      * link getGroupMembersID
      * @return an array with possible member users and groups.
      */
    function getPossibleMembersID($id=0)
    {
      $group=&$this->defaultObject($id);

      // Get all existing members
      $members = $group->getGroupMembersIDRecursive();

      // -=-=- Generic code above this line, users below -=-=-
      
      // Get all users
      $user = new LFX_User;
      $all_users = $user->searchID();
      $member_users=array();
      for ($i=0;$i<count($members['user']);$i++) {
        $member_users[$i]=$members['user'][$i][0];
      }
      // Intersect
      $possible_members['user'] = array_diff($all_users,$member_users);

      // -=-=- Users above this line, groups below -=-=-
      
      if ($group->id==1) {
        $possible_members['group']=array();
      } else {
        $all_groups = $this->searchID();
        for ($i=0;$i<count($members['group']);$i++) {
          $member_groups[$i]=$members['group'][$i][0];
        }
        // We don't want to allow the admin group to actually
        // become a member in any other group
        $member_groups[]=1;

        // We also get this group's id, since it's not a valid choice
        $member_groups[]=$group->id;

        // We also need to get the parent groups, because these cannot
        // be made members of this group, as well. TO DO: detect circular
        // relations in groups.
        $parent_groups = $group->getGroupParentsIDRecursive();
        for ($i=0;$i<count($parent_groups);$i++) {
          $parent_groups[$i]=$parent_groups[$i][0];
        }

        // Intersect
        $possible_members['group'] = array_diff($all_groups,$member_groups);
        $possible_members['group'] = array_diff($possible_members['group'],$parent_groups);
      }
      /*
      
      PARENT = Function      => CHILD = NOT Functions
      PARENT = Role          => CHILD = NOT Functions
      PARENT = anything else => CHILD = NOT Functions, NOT Roles
      
      */
      
      $allFunctions=$this->searchID('type','Function');
      $possible_members['group']=array_diff($possible_members['group'],$allFunctions);
      if (!in_array($group->getAttr('type'),array('Function','Role'))) {
        $allRoles=$this->searchID('type','Role');
        $possible_members['group']=array_diff($possible_members['group'],$allRoles);
      }
      
      $possible_members['group']=array_values($possible_members['group']);

      // -=-=- Groups above this line, generic code below -=-=-
      
      //  We do this here to avoid doing it in the page
      $possible_members['size'] = sizeOf($possible_members['user'])+sizeOf($possible_members['group']);
      return($possible_members);
    }
      /**
      * Checks the parent groups of the group, and then recurses into all the other
      * groups which parent the group's parent.
      * Returns an array with group id's and access levels for the corresponding
      * group.
      * @return array of the form $list = array(0=>array(group_id,access_level),
      *                                         .
      *                                         .
      *                                         n=>array(group_id,access_level))
      */
    function accessList($id=0,$lvl=1)
    {
      static $currentAccessList, $groupMapping;
      if ($lvl==1) {
        $group=&$this->defaultObject($id);
        if ($group->accessList) {
          return($group->accessList);
        }
        $this->accessList=array();
        $currentAccessList=array();
        $groupMapping=array();
        $id=$group->id;
      }
      // Getting list of memberships for current id
      $link=new LFX_User_Group;
      $links=$link->search('child_group',$id);

      // For each of the memberships
      while(list($k,$v)=@each($links)) {
        // We get the parent group and access to it
        if (!$group=$v->getObject('parent_group')) {
          continue;
        }
        $tmp_acc=$v->getAttr('access');

        // If the group is active then we calculate rights
        if ($group->isActive()) {
          // If this is the first level ($id belongs to the group (base group) requesting the access list)
          // there is no need to perform the intersections. We simply add the access.
          if ($lvl==1) {
            $acc=$tmp_acc;
          } else {
            // Otherwise, we find the access the base group has on the current group
            // We found it before as it is on a lower level
            $tmp_list=array();
            foreach ($currentAccessList as $key => $value) {
              $tmp_list[$key]=$value[0];
            }
            $access_idx=array_search($id,$tmp_list);
            unset($tmp_list);
            $parentAccess=strtoupper($currentAccessList[$access_idx][1]);
            $childAccess=strtoupper($tmp_acc);
            $acc=LFX_permsFilter($childAccess,$parentAccess);
          }
          if(isset($groupMapping[$group->id])) {
            $currentAccessList[$groupMapping[$group->id]][1]=LFX_permsMerge($acc,$currentAccessList[$groupMapping[$group->id]][1]);
          } else {
            $currentAccessList[] = array($group->id,$acc);
            $groupMapping[$group->id]=count($currentAccessList)-1;
          }
        }
        $currentAccessList=$this->accessList($group->id,$lvl+1);
      }
      $lvl--;
      if (!$lvl) {
        $this->accessList=$currentAccessList;
      }
      return($currentAccessList);
    }

      /**
      * Works with an array of type members=array(user=>array(),group=>array())
      *
      * Both user and group are indexed array containing arrays of id's and access rights
      * @return An array of the same form as the one received, with unique id values
      */    
    function make_unique($members)
    {
      // Check that we receive what we expect
      while (list($k,$v) = each($members)) {
        while (@list($key,$value) = each($v)) {
          if (is_array($value[0])) {
            return(false);
          }
        }
      }
      $tmp=array();
      for ($i=0;$i<count($members['user']);$i++) {
        $tmp[$i]=$members['user'][$i][0];
      }
      $tmp=array_unique($tmp);
      for ($i=0;$i<count($members['user']);$i++) {
        if ($tmp[$i]==NULL) {
          unset($members['user'][$i]);
        }
      }
      unset($tmp);
      $tmp=array();
      $members['user']=array_values($members['user']);
      for ($i=0;$i<count($members['group']);$i++) {
        $tmp[$i]=$members['group'][$i][0];
      }
      $tmp=array_unique($tmp);
      for ($i=0;$i<count($members['group']);$i++) {
        if ($tmp[$i]==NULL) {
          unset($members['group'][$i]);
        }
      }
      $members['group']=array_values($members['group']);
      return($members);
    }

      /**
      * Returns groups which can parent a given user/group.
      *
      * A possible parent is a group which is not already a parent of this group,
      * or groups which are not already members of this group recursively.
      *
      * @return an indexed array with the id's of the possible groups
      */
    function getPossibleParentsID($child_id,$child_type) {

      // Check to see that we receive a valid user or group id,
      // and that its type is also correctly specified
      $parents = $constrain = $forbidden = array();

      switch ($child_type) {
          case 'User':
            $child = new LFX_User;
            $load = $child->load($child_id);
            $forbidden = $child->getUserGroupsID();
            for ($i=0;$i<count($forbidden);$i++) {
              $forbidden[$i]=$forbidden[$i][0];
            }
          break;
          case 'Group':
            if ($child_id==1) {
              return(array());
            }
            $child = new LFX_Group;
            $load = $child->load($child_id);
            if ($load) {
              /*

              CHILD = Function      => PARENT = NONE
              CHILD = Role          => PARENT = ROLE, FUNCTION
              CHILD = anything else => PARENT = anything

              */
              if ($child->getAttr('type')=='Function') {
                return(array());
              }
              $parents = $child->getGroupParentsID();
              for ($i=0;$i<count($parents);$i++) {
                $parents[$i]=$parents[$i][0];
              }
              $parents[] = $child_id; // a group cannot parent itself
              $parents[] = 1; // We don't want to allow making any group a member of the admin group
              $children = $child->getGroupMembersIDRecursive();
              $children = $children['group'];
              for ($i=0;$i<count($children);$i++) {
                $children[$i]=$children[$i][0];
              }
              $forbidden=array_merge($parents,$children);
              if ($child->getAttr('type')=='Role') {
                $allFunctions=$this->searchID('type','Function');
                $allRoles=$this->searchID('type','Role');
                $constrain=array_merge($allFunctions,$allRoles);
              }
            }
          break;
          default:
            echo "Invalid child_type, must be either \"User\" or \"Group\"";
            return(false);
      }

      if (!$load) {
        echo "Could not load $child_type with id $child_id.<br>";
        return(false);
      }
      
      $possible_parents = array();
      $all_groups = $this->searchID();
      // echo"<pre>All groups";print_r($all_groups);echo"</pre>";
      // echo"<pre>Parents";print_r($parents);echo"</pre>";
      $possible_parents = array_diff($all_groups,$forbidden);
      if ($constrain) {
        $possible_parents=array_intersect($possible_parents,$constrain);
      }
      // echo"<pre>Possible Parents";print_r($possible_parents);echo"</pre>";
      return($possible_parents);
    }
    
      /**
      * A method to export this group to XML. The XML is only the "blob" which
      * describes *this* group; if you need a properly formatted XML, use
      * {@link multiExportXML} instead.
      *
      * @return string an XML chunk which describes this group
      */
    function exportXML($indent=0,$includeMemberships=false,$goRecursive=true,$goRecursiveDown=true)
    {
      LFX_include_class('LFX_XML');
      $tx=new LFX_XML;
      $ind=str_repeat('  ',$indent).'     ';
      $result='';
      $result.=$ind."<group_descriptor ";
      $result.="name=\"".$tx->escape($this->getAttr('name'))."\" ";
      $result.="date_created=\"".$tx->escape($this->getAttr('date_created'))."\" ";
      $result.="description=\"".$tx->escape($this->getAttr('description'))."\" ";
      $result.="type=\"".$tx->escape($this->getAttr('type'))."\" ";
      $result.="deactivated=\"".$tx->escape($this->getAttr('deactivated'))."\" ";
      $result.="email=\"".$tx->escape($this->getAttr('email'))."\" ";
      $result.="module=\"".$tx->escape($this->getAttr('module'))."\" ";
      $result.=">\n";
      $memberLinks=$this->getObjects('members');
      $validMembers=false;
      if ($goRecursiveDown) {
        for($i=0;$i<count($memberLinks);$i++) {
        set_time_limit(15);
          if (!$memberLinks[$i]->getAttr('child_group')) {
  	        continue;
        	}
          if (!$validMembers) {
            $result.=$ind."  <members>\n";
            $validMembers=true;
          }
          $result.=$ind."    <member ";
      	  $result.="access=\"".$tx->escape($memberLinks[$i]->getAttr('access'))."\" ";
      	  $result.="comments=\"".$tx->escape($memberLinks[$i]->getAttr('comments'))."\" ";
      	  $result.="date_added=\"".$tx->escape($memberLinks[$i]->getAttr('date_added'))."\">\n";
      	  $tmp=$memberLinks[$i]->getObject('child_group');
	        if ($goRecursive) {
            $result.=$tmp->exportXML($indent+3);
	        }
      	  $result.=$ind."    </member>\n";
        }
      }
      if ($validMembers) {
        $result.=$ind."  </members>\n";
      }
      if ($includeMemberships) {
        $memberships=$this->getObjects('user_group_child');
        $validMembers=false;
        for($i=0;$i<count($memberships);$i++) {
          if (!$memberships[$i]->getAttr('parent_group')) {
            continue;
      	  }
          if (!$validMembers) {
            $result.=$ind."  <memberships>\n";
            $validMembers=true;
          }
          $result.=$ind."    <member_of ";
      	  $result.="access=\"".$tx->escape($memberships[$i]->getAttr('access'))."\" ";
      	  $result.="comments=\"".$tx->escape($memberships[$i]->getAttr('comments'))."\" ";
      	  $result.="date_added=\"".$tx->escape($memberships[$i]->getAttr('date_added'))."\">\n";
      	  $tmp=$memberships[$i]->getObject('parent_group');
      	  $result.=$tmp->exportXML($indent+3,false,false);
      	  $result.=$ind."    </member_of>\n";
        }
        if ($validMembers) {
          $result.=$ind."  </memberships>\n";
        }
      }
      $result.=$ind."</group_descriptor>\n";
      return($result);
    }
    
      /**
      * This method returns a proper XML string which describes a set of groups.
      * The groups are passed as an indexed array of id's.
      *
      * @param array $groups the id's of the groups to be exported
      * @param boolean $includeMemberships true if memberships are to be 
      *   included in the XML file
      * @param boolean $embedded true if the XML file is to be included
      *   in another XML file, thus it escapes the XML header
      * @return mixed the XML if the parameter is a valid array, a NULL entry
      *   if the parameter is empty, or false if the parameter is invalid.
      */
    function multiExportXML($groups,$includeMemberships=false,$embedded=false,$goRecursiveDown=true)
    {
      if (!$groups) {
        return(NULL);
      }
      if (!is_array($groups)) {
        return(false);
      }
      $result='';
      for($i=0; $i<count($groups); $i++){
        $tmp=new LFX_Group($groups[$i]);
        $result.=$tmp->exportXML(0,$includeMemberships,false,$goRecursiveDown);
      }
      // Separated the ">" from the "?" in the code as to avoid fooling some
      // PHP editors from thinking that we're escaping from PHP.
      $tmp='';
      if (!$embedded) {
        $tmp.="<?xml version='1.0' encoding='ISO-8859-1'?".">\n";
      }
      $tmp.="  <group_descriptors>\n";
      $tmp.=     $result;
      $tmp.="  </group_descriptors>";
      return($tmp);
    }
    
      /**
      * This private method imports a single group from an XML array.
      */
    function importSingleXML($xmlData,$forceDescriptions=false)
    {
      $groups=$this->search('name',$xmlData['attributes']['NAME']);
      if (!count($groups)) {
        //pecho(vardump($xmlData));
        //pecho("<hr>");
        // Create new group using the rest of the data from the XML
        $group=new LFX_Group;
        $group->setAttrs(array(
          'name'=>$xmlData['attributes']['NAME'],
          'date_created'=>$xmlData['attributes']['DATE_CREATED'],
          'description'=>$xmlData['attributes']['DESCRIPTION'],
          'type'=>$xmlData['attributes']['TYPE'],
          'deactivated'=>$xmlData['attributes']['DEACTIVATED'],
          'email'=>$xmlData['attributes']['EMAIL'],
          'module'=>$xmlData['attributes']['MODULE']));
        if (!$group->save()) {
          return(NULL);
        }
        $jcreated=true;
      } else {
        $group=$groups[0];
        if ($forceDescriptions) {
          $group->setAttr('description',$xmlData['attributes']['DESCRIPTION']);
          $group->save();
        }
        $jcreated=false;
      }
      return(array($group,$jcreated));
    }
    
      /**
      * This method imports an XML which was previously exported with
      * {@link multiExportXML}().
      *
      * @param string $xmlin the XML to import
      * @return boolean true on success or false on failure
      */
    function multiImportXML($xmlin,$forceDescriptions=false)
    {
      if (!is_array($xmlin)) {
        LFX_include('LFX_XML');
        $xml=new LFX_XML($xmlin);
        $xml=$xml->xml[0];
      } else {
        $xml=$xmlin;
      }
      if ($xml['tag']!='GROUP_DESCRIPTORS') {
        return(false);
      }
      while (list(,$xgroup)=@each($xml['children'])) {
        list($group,$jcreated)=$this->importSingleXML($xgroup,$forceDescriptions);
        if (!$group) {
          return(false);
        }
        if ($xgroup['children']) {
          foreach($xgroup['children'] as $child) {
            if ($child['tag']=='MEMBERS') {
              if (!$group->importRecursiveXML($child,$jcreated)) {
                return(false);
	      }
	    } 
	    if ($child['tag']=='MEMBERSHIPS') {
	      if (!$group->importRecursiveXML($child,$jcreated,true)) {
	        return(false);
              }
	      break;
	    }  
          }
        }
      }
      return(true);
    }
    
      /**
      * This is a private method which does the recursive part of the import
      */
    function importRecursiveXML($members,$jcreated=false,$includeMemberships=false)
    {
      foreach($members['children'] as $member) {
        set_time_limit(15);
        if ($member['children'][0]['tag']!='GROUP_DESCRIPTOR') {
          continue;
        }
        list($group,$jcreated)=$this->importSingleXML($member['children'][0]);
        if (!$group) {
          return(false);
        }
        $user_group=new LFX_User_Group;
        if (!$includeMemberships) {
          // We'll first make a quick assessment and if that proves valid,
          // we go on and test thoroughly, thus excluding from thorough tests
          // the flagrant existing links.
          if (!$user_group->search(array('parent_group','child_group'),array($this->id,$group->id))) {
            $possible_members=$this->getPossibleMembersID();
            if (in_array($group->id,$possible_members['group'])) {
              $tmp=new LFX_User_Group;
              $tmp->setAttrs(array(
                'parent_group'=>$this->id,
                'child_group'=>$group->id,
                'access'=>$member['attributes']['ACCESS'],
                'comments'=>$member['attributes']['COMMENTS'],
                'date_added'=>$member['attributes']['DATE_ADDED']));
              if (!$tmp->save()) {
                return(false);
              }
            }
          }
        } else {
          // We'll first make a quick assessment and if that proves valid,
          // we go on and test thoroughly, thus excluding from thorough tests
          // the flagrant existing links.
          if (!$user_group->search(array('parent_group','child_group'),array($group->id,$this->id))) {
            $possible_members=$group->getPossibleMembersID();
            if (in_array($this->id,$possible_members['group'])) {
              $tmp=new LFX_User_Group;
              $tmp->setAttrs(array(
                'parent_group'=>$group->id,
                'child_group'=>$this->id,
                'access'=>$member['attributes']['ACCESS'],
                'comments'=>$member['attributes']['COMMENTS'],
                'date_added'=>$member['attributes']['DATE_ADDED']));
              if (!$tmp->save()) {
                return(false);
              }
            }
          }
        }
        if ($member['children'][0]['children']) {
          foreach ($member['children'][0]['children'] as $child) {
            if ($child['tag']=='MEMBERS') {
              $group->importRecursiveXML($child,$jcreated);
            }
            if ($child['tag']=='MEMBERSHIPS') {
              $group->importRecursiveXML($child,$jcreated,true);
            }
          }
        }
      }
      return(true);
    }
    
    function groupSynch($xml_string)
    {
      if (!is_array($xml_string)) {
        LFX_include('LFX_XML');
        $xml=new LFX_XML($xml_string);
        $xml=$xml->xml[0];
      } else {
        $xml=$xml_string;
      }
      $group=new LFX_Group;  
      $groupNames=array();
      $curModules=array();
      while (list(,$xgroup)=@each($xml['children'])) {
        $groupNames[]=$xgroup['attributes']['NAME'];
        if (!in_array($xgroup['attributes']['MODULE'],$curModules)) {
          $curModules[]=$xgroup['attributes']['MODULE'];
        }
        if ($xgroup['children']) {
          foreach($xgroup['children'] as $child) {
            if ($child['tag']=='MEMBERS') {
              $group->getRecursiveGroups($child,$groupNames);
            }    
          }
        }    
      }
  
      $query=array();
      $query['select']=array('id','name');
      $query['from']=array('groups');
      $cond=array();
      for ($i=0;$i<count($curModules);$i++) {
        $cond[$i]="module=\"".$curModules[$i]."\"";
      }
      $query['where']=array("type"=>"OR","conditions"=>$cond);
      $sql=LFX_buildSQL($query);
      $group->query($sql);
      $existentGroupsIds=$existentGroupsNames=array();
      while ($group->db->next_record()) {
        $existentGroupsIds[]=$group->db->f(0);
        $existentGroupsNames[]=$group->db->f(1);
      }
      $groupsToExport=array_diff($existentGroupsNames,$groupNames);
      $n=count($existentGroupsIds);
      for($i=0;$i<$n;$i++) {
        if (!$groupsToExport[$i]) {
          unset($existentGroupsIds[$i]);
        }
      }
      $groupsIds=array_values($existentGroupsIds);
      $returnXmlString=$group->multiExportXML($groupsIds,true);
      return($returnXmlString);
    }
    
    /**
    * This is an auxiliary function used by the groupSynch 
    * method. You shouldn't ever need to use this code.
    */
    
    function getRecursiveGroups($members,&$groupNames)
   {
      $group=new LFX_Group();
      foreach($members['children'] as $member) {
       if ($member['children'][0]['tag']!='GROUP_DESCRIPTOR') {
         continue;
       }
       $groupNames[]=$member['children'][0]['attributes']['NAME'];
       if ($member['children'][0]['children']) {
         foreach ($member['children'][0]['children'] as $child) {
           if ($child['tag']=='MEMBERS') {
            $group->getRecursiveGroups($child,$groupNames);
           }
         }
       }
     }
   }

  }
?>
