<?
  /**
  * The LFXlib import/export class.
  * Please note that this class only retrieves an object's data into an
  * array and respectively builds an object based on a similar array;
  * it does not actually manage the resulting/incoming files!
  * Bogdan Stancescu <bogdan@lanifex.com>, July 2004
  * @package LFX
  * $Id: LFX_impex.php,v 1.2 2004/07/21 08:15:30 bogdan Exp $
  */
  
  _checkMeOut(__FILE__);
  
    /**
    * This class should be used via its {@link import} and {@link export}
    * methods.
    */
  class LFX_impex extends LFX_Object
  {
    var $objects=array();
    var $data=array();
    
    function LFX_impex($data=NULL)
    {
      if (is_object($data)) {
        $this->parseObject($data);
      } elseif (is_array($data)) {
        $this->parseData($data);
      } elseif ($data!==NULL) {
        $this->addError("Unknown data type in LFX_impex::LFX_impex!");
      }
    }
    
    function parseObject($object)
    {
      if (!is_object($object)) {
        $this->addError('Parameter passed to LFX_impex::parseObject() is not an object: '.vardump($object));
        return(false);
      }
      if (!is_subclass_of($object,'lfx_object')) {
        $this->addError('Object passed to LFX_impex::parseObject() is not a LFXlib object!');
        return(false);
      }
      if (!$object->id) {
        $this->addError('Object passed to LFX_impex::parseObject() does not have any ID set!');
        return(false);
      }
      if (!$object->load()) {
        $this->addError('Object passed to LFX_impex::parseObject() could not be loaded!');
        return(false);
      }
      $this->objects[]=$object;
      $data=array(
        'class'=>get_class($object),
        'id'=>$object->id,
        'attr'=>$object->attr,
        'links'=>array()
      );
      // Ok, now we have the basic object saved -- let's look for its
      // ephemeric links with other objects. We will NOT take care of
      // real dependencies in this method, for now we only care about
      // one to many and many to many MISTRESS relationships.
      // We don't care about WIFE dependencies because that would involve
      // recursivity and that's not covered in this method, and we don't
      // care about STRANGER dependencies because those links are not affected
      // anyway.
      $deps=$object->dataStructure['depend'];
      @reset($deps);
      while(list($depName,$dep)=@each($deps)) {
        if ($dep['on_mod']=='STRANGER') {
          continue;
        }
        $objects=$object->getObjects($depName);
        while (list(,$linked)=@each($objects)) {
          if ($dep['on_mod']=='MISTRESS') {
            $data['links'][]=array(
              'dep'=>$depName,
              'id'=>$linked->id);
          }
        }
      }
      $this->data[]=$data;
      return(true);
    }
    
    function parseData($data)
    {
      if (!is_array($data)) {
        $this->addError('Parameter passed to LFX_impex::parseData is not an array!');
        return(false);
      }
      if (!$data['class'] || !$data['id'] || !$data['attr']) {
        $this->addError('Badly formatted array passed to LFX_impex::parseData()');
      }
      if (!class_exists($data['class'])) {
        if (!LFX_include_probe($data['class'])) {
          $this->addError('Unable to locate include for class '.$data['class']);
          return(false);
        }
        if (!LFX_include($data['class'])) {
          $this->addError('Failed to include '.$data['class']);
          return(false);
        }
        if (!class_exists($data['class'])) {
          $this->addError("Include {$data['class']} successful but class still doesn't exist!");
          return(false);
        }
      }
      $this->data[]=$data;
      $object=new $data['class'];
      $object->id=$data['id'];
      $object->attr=$data['attr'];
      $this->objects[]=$object;
      return($object);
    }

    /*private*/
    function _doImport($data)
    {
      $indexed=true;
      @reset($data);
      while(list($key)=@each($data)) {
        if (!is_numeric($key)) {
          $indexed=false;
          break;
        }
      }
      if (!$indexed) {
        $data=array($data);
      }
      
      $success=true;
      for($i=0;$i<count($data);$i++) {
        $success=$success && ($current=$this->parseData($data[$i]));
        if ($current) {
          $current->insertWithId();
          for($j=0;$j<count($data[$i]['links']);$j++) {
            $link=$data[$i]['links'][$j];
            $success=$success && $current->createLink($link['dep'],$link['id']);
          }
        }
      }
      return($success);
    }
    
    /* private */
    function _doExport($object,$method='archive')
    {
      if (!is_array($object)) {
        $object=array($object);
      }
      $success=true;
      for($i=0;$i<count($object);$i++) {
        $success=$success && ($current=$this->parseObject($object[$i]));
        $exportParse+=getWatch('export');
        if ($current) {
          if ($method=='archive') {
            $objects=$object[$i]->getAllObjects();
            $exportGet+=getWatch('export');
            if ($objects) {
              for($j=0;$j<count($objects);$j++) {
                if (!in_array(get_class($objects[$j]['object'])."_".$objects[$j]['object']->id,$this->objectList)) {
                  $success=$success && $this->_doExport($objects[$j]['object']);
                  $this->objectList[]=get_class($objects[$j]['object'])."_".$objects[$j]['object']->id;
                }
              }
            } elseif ($objects===false) {
              return(false);
            }
          }
        }
      }
      if ($success) {
        return($this->data);
      } else {
        return(false);
      }
    }
    
    function init()
    {
      unset($this->objects,$this->data);
      $this->objectList=array();
    }
    
    function import($data)
    {
      $this->init();
      return($this->_doImport($data));
    }
    
    function export($object)
    {
      $this->init();
      return($this->_doExport($object));
    }
  }
?>
