<?
  /**
  * The LFXlib IP management class
  * Bogdan Stancescu <bogdan@lanifex.com>, July 2004
  * $Id: LFX_ip.php,v 1.2 2004/07/27 14:20:48 bogdan Exp $
  * @package LFX
  */
  
  
    /**
    * Quick usage examples:
    *
    * Masking an IP
    * -------------
    *
    *   LFX_include_class('LFX_ip');
    *   $lfx_ip=new LFX_ip('192.168.0.1');
    *   echo($lfx_ip->mask(32);
    *
    * Masking an IP
    * -------------
    *
    *   LFX_include_class('LFX_ip');
    *   $lfx_ip=new LFX_ip();
    *   echo($lfx_ip->mask(32,'192.168.0.1');
    *
    * (Notice the parameters in the two examples above)
    *
    * Walking a subnet
    * ----------------
    *
    *   LFX_include_class('LFX_ip');
    *   $lfx_ip=new LFX_ip();
    *   if (!$lfx_ip->initRangeWalk('192.168.0.1/23')) {
    *     echo("Bad subnet format!");
    *   }
    *   while($ip=$lfx_ip->rangeWalk()) {
    *     echo("$ip<br />\n");
    *   }
    */
  class LFX_ip extends LFX_object
  {
    var $ip=NULL;
    var $rangeData=array();
    
    /* constructor */
    function LFX_ip($ip=NULL)
    {
      $this->LFX_Object();
      $this->ip=$ip;
    }
    
    /* public */
    function mask($mask,$ip=NULL)
    {
      if(!is_numeric($mask) || ($mask<0) || ($mask>32)) {
        return(false);
      }
      $myIP=$this->formatIP($ip);
      if (!$myIP) {
        echo("WHAT!");
        return(false);
      }
      $masked=array();
      for ($i=0;$i<4;$i++) {
        if ($mask>=8) {
          $myMask=0xff;
        } else {
          $myMask=(0x100-(1 << (8-$mask))) & 0xff;
        }
        $masked[$i]=$myIP[$i] & $myMask;
        $mask-=8;
      }
      return(implode('.',$masked));

    }
    
    /* private */
    function formatIP($ip=NULL)
    {
      if ($ip===NULL) {
        $ip=$this->ip;
      }
      if ($ip===NULL) {
        return(false);
      }
      if (($myIP=explode('.',$ip)) && (count($myIP)==4)) {
        return($myIP);
      } else {
        return(false);
      }
    }
    
    /* private */
    function clearRangeData()
    {
      $this->rangeData=array();
    }
    
    /* public */
    function initRangeWalk($rangeDef=NULL)
    {
      // Ok, now we have to plug parsers here. Let's do it this way:
      // we call a wrapper method which knows all the parsers available
      // and attempts to apply them to the definition we received.
      // If the wrapper finds any matching parser, it lets us know which
      // one it is. We'll then use that parser, and it's keeping track of what's
      // the last IP it returned and whatever internal data it needs.
      // Pretty reasonable, eh?
      if ($rangeDef===NULL) {
        $rangeDef=$this->ip;
      }
      if ($rangeDef===NULL) {
        return(false);
      }
      if ($parser=$this->detectRangeParser($rangeDef)) {
        $this->clearRangeData();
        $this->rangeData=array(
          'parser'=>$parser,
          'rangeDef'=>$rangeDef);
        return(true);
      } else {
        return(false);
      }
    }

    /* private */
    function detectRangeParser($rangeDef)
    {
      if (preg_match('/^[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}\\/[0-9]{1,2}$/',$rangeDef)) {
        return('subnetWalk');
      }
      return(false);
    }
    
    /* public */
    function rangeWalk()
    {
      if (!$this->rangeData) {
        return(false);
      }
      $parser=$this->rangeData['parser'];
      return($this->$parser());
    }
    
    /* private */
    function subnetWalk()
    {
      if (!$this->rangeData['subnetWalkData']) {
        // Let's pre-digest some data here...
        $myData=array();
        $myRangeDef=explode('/',$this->rangeData['rangeDef']);
        $myData['ip']=$this->mask($myRangeDef[1],$myRangeDef[0]);
        $myData['mask']=$myRangeDef[1];
        $myData['lastIP']=ip2long($myData['ip']);
        $this->rangeData['subnetWalkData']=$myData;
        return($myData['ip']);
      }
      $myData=$this->rangeData['subnetWalkData'];
      $nextIPlong=$myData['lastIP']+1;
      $nextIP=long2ip($nextIPlong);
      if ($this->mask($myData['mask'],$nextIP)!=$myData['ip']) {
        return(false);
      }
      $myData['lastIP']=$nextIPlong;
      $this->rangeData['subnetWalkData']=$myData;
      return($nextIP);
    }
  }
?>
