<?
  /**
  * The LFX_List_Arrows class
  * Bogdan Stancescu <bogdan@lanifex.com>, December 2004
  * $Id: LFX_list_arrows.php,v 1.2 2005/01/12 15:28:55 bogdan Exp $
  * @package LFX
  */
  
    /**
    * This class allows you to build multi-criterial lists. Here's an example:
    *
    * < ?
    *   LFX_include_class('LFX_List_Arrows');
    *
    *   // The URL of the page you're working on, with a question mark at the end
    *   $baseURL="$_LFX_url/foo/bar?";
    *
    *   $list1=new LFX_List($baseURL,'ord1','sort1');
    *   $list2=new LFX_List($baseURL,'ord2','sort2');
    *   $_page->registerGetVar('switch',"swicth=$switch");
    *   pecho($list1->render('id')."<br>");
    *   pecho($list1->render('fname')."<br>");
    *   pecho("<br>");
    *   pecho($list2->render('id')."<br>");
    *   pecho($list2->render('fname')."<br>");
    *   pecho("<br>");
    *   pecho(LFX_echo("Switch value is $switch. <a href='$baseURL".$_page->getGetVars('switch')."switch=".(1-$switch)."'>Click to switch</a>"));
    * ? >
    *
    * Notice how you don't need to register any GET variables or anything --
    * you just initialize the class and let it handle everything else, you just
    * call method render().
    */
  class LFX_List_Arrows
  {
    var $orderVar;
    var $sortVar;
    var $listName;
    var $baseURL;
    var $page;

      /**
      * This is the constructor, which initializes the variables and registers
      * them with LFX_Page::registerGetVar().
      *
      * @param string $baseURL the URL to the current page. It MUST end in
      *   such a way as to allow appending GET values directly. If you have
      *   any GET vars in your base URL (e.g. "http://www.mydomain.com/foo?bar=foobar")
      *   then append an ampersand. If not (e.g. "http://www.mydomain.com/foo")
      *   then you append a question mark.
      * @param string $orderVar the GET variable name to be used for passing
      *   the order type for each column. An array of "up" and "down" values
      *   will be built in this GET variable when the user clicks on various
      *   ordering arrows.
      * @param string $sortVar the GET variable name to be used for passing
      *   the sorting criterias. An array of (some of) the values passed to
      *   this class's {@link render}() method will be built in this GET
      *   variable when the user clicks on various ordering arrows.
      * @return void
      */
    function LFX_List_Arrows($baseURL,$orderVar,$sortVar,$extra=NULL)
    {
      if ($extra && is_array($extra)) {
        if
        (
          $extra['page'] &&
          is_object($extra['page']) &&
          (
            (get_class($extra['page'])=='lfx_page') ||
            is_subclass_of($extra['page'],'lfx_page')
          )
        ) {
          $this->page=&$extra['page'];
        }
      }
      if (!$this->page) {
        global $_page;
        $this->page=&$_page;
      }
      global $_LFX;
      $_LFX['page']['internal_list_counter']++;
      $this->listName='LFXlib_unnamed_list_'.$_LFX['page']['internal_list_counter'];
      $this->orderVar=$orderVar;
      $this->sortVar=$sortVar;
      $this->baseURL=$baseURL;
      $baseURLonList=LFX_orderArrows($baseURL,$orderVar,$sortVar,NULL,true);
      $this->page->registerGetVar($this->listName,$baseURLonList['currentURL']);
    }
    
      /**
      * This method renders a pair of ordering arrows. It returns the actual
      * HTML which describes them, so all you have to do is output it.
      *
      * @param string $criteria the criteria for the current arrows' sorting
      * @return string the actual HTML for the current criteria's ordering arrows
      */
    function render($criteria)
    {
      return(
        LFX_echo(
          LFX_orderArrows(
            $this->baseURL.$this->page->getGetVars($this->listName),
            $this->orderVar,
            $this->sortVar,
            $criteria
          )
        )
      );
    }
  }
  
?>
