<?

  /**
  * The LFX_Mail library
  * Bogdan Stancescu <bogdan@lanifex.com>, April 2003
  * $Id: LFX_mail.php,v 1.27 2005/11/28 09:07:30 bogdan Exp $
  * @package LFX
  */
    /**
    * This class used to send e-mail from LFXlib.
    * Please see the LFXlib feature demonstrations for examples.
    * @package LFX
    */
  _checkMeOut(__FILE__);

  $result=array('LFX_Object');
  if (!_checkIncludes($result)) {
    return($result);
  }
  
  class LFX_Mail extends LFX_Object
  {
      /**
      * The subject of this e-mail.
      * Please make sure you ONLY include 7bit ASCII here, or the
      * results may be impredictable for different clients.
      * For *REALLY* standards-compliant environment
      * (i.e. if you can make sure users will not use Outlook or
      * most of the current webmail clients at the time of this writing)
      * then you could also use quoted-printable encoded Unicode.
      */
    var $subject='';
      /**
      * The plain text part of the message
      */
    var $text='';
    var $textCharset='us-ascii';
      /**
      * The HTML part of the message
      */
    var $html='';
    var $htmlCharset='us-ascii';
      /**
      * The attachments to be sent. This MUST be an indexed array
      * of associative arrays, each with the following structure:
      * array(
      *   'filename'=> <the file name, e.g. 'LFXlib_doc.pdf'>
      *   'mime'    => <the mime type of the file, e.g. 'application/pdf'>
      *   'content' => <the actual file content, NOT encoded in any way>
      *   'cid'     => <a content ID, used for inline images>
      * )
      */
    var $attachments=array();
      /**
      * The e-mail addresses to deliver this mail to.
      * This must be an indexed array of e-mail address atoms. Each
      * e-mail address atom is either a string which will be placed
      * in the "To:" (although see {@link send}) section of the
      * outgoing e-mail, or an associative
      * array of the form:
      * array(
      *   'name'=><string to be used as the recipient's name>
      *   'email'=><the e-mail address>
      * ), or an instantiated LFX_User object.
      * The proper format for e-mail addresses is described in RFC 2822,
      * section 3.4. and it's a pretty hairy thing, so unless you're sure
      * you only have valid e-mail addresses in there, you should use either the
      * the objects or the array format instead of just sending
      * mail to '$fname $lname <$email>'.
      */
    var $addresses=array();
      /**
      * The header separator. Some say it should be \n, some say it should be \r\n.
      * The specification says it should be \r\n. So, here it is, change it if
      * the default doesn't work for you.
      */
    var $headSep="\r\n";
      /**
      * The address mail should be originating from (please note this string
      * precisely will be used in the headers, so mind the RFC in in doubt)
      */
    var $from=NULL;
    

    function LFX_Mail()
    {
      // Really no need to do anything here.
      // The constructor exists only because it should
      // be called by descendants, and that would've issued errors.
      $this->LFX_Object();

      // If LFXlib global 'send_mail_queues' is set to 1, then the mail
      // messages are put in a folder and are set periodically with a cron job.

      // This is done for the cases where services like DNS or MX are not availeble,
      // which degrades the real-time performance of the system a lot. This way,
      // we are just 'queuing' the mail and the user doesn't have to wait for the
      // mail to be sent real-time.
      global $_LFX_globals;
      if ($_LFX_globals->getSetting('send_mail_queued')) {
        if (!$_LFX_globals->getSetting('dataDir')) {
          $this->addError("You really should set up that data directory! You can't send queued mail otherwise.");
          $this->forceDirectMail=true;
        } else {
          $this->queue_folder = $_LFX_globals->getSetting('dataDir').'/system_mail_queue';
          if (!is_Dir($_LFX_globals->getSetting('dataDir')) && !mkdir($_LFX_globals->getSetting('dataDir'))) {
            $this->addError("Failed to create LFXlib system directory at ".$_LFX_globals->getSetting('dataDir')."!");
            $this->forceDirectMail=true;
          }
        }
      }
    }

      /**
      * This method sends the current e-mail.
      * Please note the following:
      * - the following fallback rules are applied when deciding how
      *   to format the e-mail:
      *   + if only $this->text is set, the message is sent as plain
      *     text (text/plain);
      *   + if $this->html is set, the message is sent as multipart/alternative
      *     using $this->text as the plain text alternative regardless of
      *     whether $this->text is set or not;
      *   + if $this->attachments is set, the message is sent as
      *     multipart/mixed, with the message itself sent as the first
      *     attachment, using the conventions above.
      *   Please note there currently is no way to send multipart/related
      *   messages. (multipart/related allows one to send HTML *WITH* images
      *   included in the e-mail as opposed to images linked from a server)
      * @param string method any of the following (case sensitive):
      *   "toAll" -- all recipients are sent the same e-mail; all recipients
      *              are listed in the "To:" field of the e-mail; this method
      *              has the advantage of speed over the default, but has privacy
      *              problems (all recipients see all recipients);
      *   "toEach" -- the default. All recipients are sent a different e-mail,
      *               each recipient is listed in the "To:" field of the message
      *               s/he receives. This is the most privacy-safe yet personal,
      *               but also the slowest for large numbers of recipients;
      *   "bccAll" -- all recipients are sent the same e-mail; all recipients
      *               are specified as blind carbon copy addresses. The advantage
      *               over the default is speed, but it's less personal, it's
      *               not a nice policy and spam filters may stop this kind of mail.
      * @param boolean force_sending if true, then it sends the mail now regardless
      *   of whether the current setting is to queue the mail or not. You shouldn't
      *   need this, it's used by the cron job which sends the e-mails from the
      *   queue.
      * @return boolean true on success or false otherwise
      */
    function send($method='toEach', $force_sending=false)
    {
      global $_LFX_globals;
      // If we are queuing (as a setting) and NOT sending now, queue
      if($_LFX_globals->getSetting('send_mail_queued') && !$force_sending && !$this->forceDirectMail) {
        $this->queueMail();
        return(true);
      }
      // If we aren't queueing or we're running in the cron, then send away
      $email=$this->_parse($this->_prepare());
      // Adding From: field
      if ($this->from===NULL) {
        $tmp=new LFX_global();
        $tmp->global_module=$this->module;
        if ($from=$tmp->getSetting('subscription_email')) {
          $this->from=$from;
        } else {
          $this->from='"'.$this->module.' Subscription" <'.$this->module.'>';
        }
      }
      if ($lang=$_LFX_globals->getSetting("system_language")) {
        LFX_include('LFX_Language');
        $langs=new LFX_Language;
        if ($myLang=$langs->search('code',$lang)) {
          $encoding=$myLang[0]->getAttr('encoding');
          LFX_include('LFX_Character_Mapping');
          $this->subject=LFX_Character_Mapping::stripAccents($this->subject,$encoding);
        }
      }
      $email['headers'].='From: '.$this->from.$this->headSep;
      // Debugging -- dumps the e-mail on screen if debug level is 3 or more
      $this->debug($this->mailDump($email),3);
      $addresses=$this->getAddresses();
      switch($method) {
        case 'toEach':
          $totalCount=count($addresses);
          $failedAddrs=array();
          while(list(,$addr)=each($addresses)) {
            if (!$this->doSendMail($addr,$this->subject,$email['content'],$email['headers'])) {
              $failedAddrs[]=$addr;
            }
          }
          if ($failedAddrs) {
            // UGLY! BAD PRACTICE! Shouldn't just dump e-mail addresses in the
            // page for everyone to see! Must fix by sending an e-mail to some
            // admin or something.
            $this->addError("Failed to send ".count($failedAddrs).
              " of $totalCount messages: ".implode('; ',$failedAddrs));
            return(false);
          }
          break;
        case 'toAll':
          if (!$this->doSendMail(implode(',',$addresses),$this->subject,$email['content'],$email['headers'])) {
            $this->addError('Unable to send mail!');
            return(false);
          }
          break;
        case 'bccAll':
          if (!$this->doSendMail('undisclosed-recipients',$this->subject,$email['content'],
          implode(',',$addresses).$this->headSep.$email['headers'])) {
            $this->addError('Unable to send mail!');
            return(false);
          }
          break;
        default:
          $this->addError('Unknown delivery method &quot;$method&quot; in LFX_Mail::send()');
          return(false);
      }
      return(true);
    }
    
    function queueMail(){
      LFX_include('LFX_Lock');
      $myLock=new LFX_Lock('LFX_queued_mail');
      while($myLock->check()==='locked') {
        usleep(20);
      }
      if ($myLock->check()==='dead') {
        $myLock->unlock(true);
      }
      $myLock->lock();
      $content = serialize($this);
      //print_r($content);
      // Now we need to get a fresh file to put the content into
      $file_name = $this->queueGetFile();
      $fp = fopen($file_name,'w');
      fwrite($fp,$content);
      fclose($fp);
      $myLock->unlock();
      return TRUE;
    }
    
    function generateRandomMD5()
    {
      $rnd = rand(1,10000000);
      list($usec, $sec) = explode(" ",microtime());
      $md5 = md5($rnd.$sec.$usec);
      return($md5);
    }
    
    function queueGetFile()
    {
      if(!is_dir($this->queue_folder)) {
        mkdir($this->queue_folder);
      }
      for(
        $myname='first';
        file_exists($filename="{$this->queue_folder}/{$myname}_mail.txt");
        $myname=$this->generateRandomMD5()
      );
      return($filename);
    }
    
    function sendQueued()
    {
      global $_LFX_globals;
      if (!$_LFX_globals->getSetting('send_mail_queued') || !is_dir($this->queue_folder)) {
        return(NULL);
      }
      LFX_include('LFX_Lock');
      $myLock=new LFX_Lock('LFX_queued_mail');
      // We don't want to wait more than 20 seconds here, the next execution
      // will hopefully take care of whatever mail is there.
      while(($myLock->check()==='locked') && ($i<1000)) {
        usleep(20);
        $i++;
      }
      if ($i==1000) {
        echo("Failed to send queued mail due to lock problem. Exiting.\n");
        exit;
      }
      if ($myLock->check()==='dead') {
        $myLock->unlock(true);
      }
      // Note that we first cache the files into an array so we can unlock
      // as soon as possible -- web clients would wait for us otherwise.
      $myLock->lock();
      $dir = opendir($this->queue_folder);
      $myFiles=array();
      // Should first make sure all the mail can fit into memory...
      // What we're doing now is dangerous, if the last mail doesn't fit in the
      // memory then we crash after deleting all other queued mail but before
      // delivering it! Bad, but hopefully not going to happen because we're
      // only pooling for 5 minutes, how much mail can you send in 5 minutes?
      // I hope we'll never find out. ;-)
      while(($file = readdir($dir)) !== false) {
        if (($file=='.') || ($file=='..') || is_dir($this->queue_folder.$file)) {
          continue;
        }
        $filename = "{$this->queue_folder}/$file";
        $fp = fopen($filename,'r');
        $myFiles[] = fread($fp,filesize($filename));
        fclose($fp);
        unlink($filename);
      }
      $myLock->unlock();
      for($i=0;$i<count($myFiles);$i++) {
        $mymail = unserialize($myFiles[$i]);
        $mymail->send("toEach",1);
      }
      echo("Sent $i emails from queue\n");
      return(TRUE);
    }
    
      /**
      * This method parses $this->addresses into an indexed array
      * of all addresses, which can easily be transformed to the proper mail
      * format with implode(',',$array).
      * @return array
      */
    function getAddresses($addresses=NULL)
    {
      if ($addresses===NULL) {
        $addr_to_parse=&$this->addresses;
      } else {
        $addr_to_parse=$addresses;
      }
      // Local variables:

      // addr - temp variable storing the current mail atom
      $addr=NULL;

      // addra - the result
      $addra=array();

      for ($i=0;$i<count($addr_to_parse);$i++) {
        $addr=&$addr_to_parse[$i];
        if (is_object($addr)) {
          // Is it a group?
          if(get_class($addr)=='lfx_group') {
            if ($addr->getAttr('email')) {
              $addra[]=$addr->getAttr('email');
              continue;
            }
            // No luck, we really need to look into this one...
            $members=$addr->getGroupMembersID(0,true);
            $member_u=array();
            for($j=0;$j<count($members['user']);$j++) {
              $member_u[]=$members['user'][$j][0];
            }
            $tmp=new LFX_User;
            $tmp2=$tmp->_doLoad($member_u);

            $member_g=array();
            for($j=0;$j<count($members['group']);$j++) {
              $member_g[]=$members['group'][$j][0];
            }
            $tmp=new LFX_Group;
            $tmp2=array_merge($tmp2,$tmp->_doLoad($member_g));

            $addra=array_merge($addra,$this->getAddresses($tmp2));
            continue;
          }

          // Is it a user?
          if(get_class($addr)=='lfx_user') {
            if (!$addr->getAttr('email') || $addr->getAttr('deactivated')) {
              openlog('LFX:mail',LOG_ODELAY,LOG_LOCAL1);
              syslog(LOG_INFO,"The system attempted to send mail to a deactivated user, or a user without an e-mail address (user ID {$addr->id}, e-mail: ".$addr->getAttr('email').")");
              closelog();
              continue;
            }
            $addra[]='"'.addslashes($addr->getName()).'" <'.($addr->getAttr('email')).'>';
            continue;
          }

          // It's neither a group nor a user -- we don't know what do do with this...
          $this->addError("Address at index $i is not the correct class (".niceClass(get_class($addr))." instead of LFX_User or LFX_Group). Ignoring.");
          continue;
        } elseif (is_array($addr)) {
          if (!$addr['email']) {
            continue;
          }
          $tmp=new LFX_User;
          if ($tmp=$tmp->search('email',$addr['email'])) {
            if ($tmp[0]->getAttr('deactivated')) {
              openlog('LFX:mail',LOG_ODELAY,LOG_LOCAL1);
              syslog(LOG_INFO,"The system attempted to send mail to a deactivated user (user ID {$tmp[0]->id}, email: ".$addr['email'].")");
              closelog();
              continue;
            }
          }
          $addra[]='"'.addslashes($addr['name']).'" <'.$addr['email'].'>';
        } else {
          if (!$addr) {
            // We just ignore empty addresses silently
            continue;
          }
          if (!$this->checkMail($addr)) {
            $this->addError("Address $addr is invalid. Ignoring.");
          }
          $tmp=new LFX_User;
          if ($tmp=$tmp->search('email',$addr)) {
            if ($tmp[0]->getAttr('deactivated')) {
              openlog('LFX:mail',LOG_ODELAY,LOG_LOCAL1);
              syslog(LOG_INFO,"The system attempted to send mail to a deactivated user (user ID {$tmp[0]->id}, email: ".$addr.")");
              closelog();
              continue;
            }
          }
          $addra[]=$addr;
        }
      }
      $addra=array_values(array_unique($addra));
      return($addra);
    }

      /**
      * A simple method to dump whatever the current email
      * contains. This method is only intended for debugging,
      * so output is directed to the browser in a pretty ugly
      * fashion. You must specify the $email array which is
      * the one prepared by {@link _parse}.
      * @param array $email the email structure to dump
      * @return void
      */
    function mailDump($email)
    {
      $tab='&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
      $result=LFX_startTables();
      $result.=LFX_str();
      $result.=LFX_rth('Address');
      $addrs=$this->getAddresses();
      $result.=LFX_td(htmlspecialchars(implode(', ',$addrs)));
      $result.=LFX_estr();
      $result.=LFX_rth('Subject');
      $result.=LFX_td($this->subject);
      $result.=LFX_estr();
      $result.=LFX_rth('Headers');
      $result.=LFX_td(str_replace("\t",$tab,nl2br(htmlspecialchars($email['headers']))));
      $result.=LFX_estr();
      $result.=LFX_rth('Content');
      $result.=LFX_td(str_replace("\t",$tab,nl2br(htmlspecialchars($email['content']))));
      $result.=LFX_etr();
      $result.=LFX_endTables();
      return($result);
    }

      /**
      * This private method recursively parses the array prepared by
      * {@link _prepare} and returns the 7 bit mail content along with
      * the extra headers needed for this e-mail in an associative array
      * of the form:
      * array(
      *   'headers'=><the extra headers>,
      *   'content'=><the actual mail content>)
      */
    function _parse($message)
    {
      // echo("<hr><hr>Starting _parse with this message:<pre>"); var_dump($message); echo("</pre><hr><hr>"); flush();
      $mime=explode('/',$message['mime']);
      $h='';
      $content=$message['content'];
      if ($mime[0]=='text') {
        // If text, do two things:
        // - flow the text;
        // - check if 7bit safe, and quoted-printable encode if not
        $flowed=false;
        if (($mime[1]=='plain') && (!$message['filename'])) {
          // Flow the text if it's not an attachment
          // Flowed format is described by RFC 2646
          // In a couple of words, flowed text is text which is soft-wrapped,
          // each soft-wrapped line being identified by the fact that it
          // ends with a space.
          $sep=$this->findSep($content);
          $content=str_replace($sep," \n",wordwrap($content,66,$sep));
          $flowed=true;
        }
        $h='MIME-Version: 1.0'.$this->headSep; # Paul 01.Nov.2005
        $h.='Content-Type: '.$message['mime'].';'.$this->headSep;
        if ($flowed) {
          $h.="\tformat=flowed;".$this->headSep;
        }
        $h.="\tcharset=".$message['charset'].$this->headSep;
        if($this->safe7bit($message['content'])) {
          $h.='Content-Transfer-Encoding: 7bit'.$this->headSep;
        } else {
          $h.='Content-Transfer-Encoding: quoted-printable'.$this->headSep;
          // A replacement for imap_8bit
          $content=str_replace ('%','=',rawurlencode($content));
          // The following is needed because MTA's need to wrap the
          // mail body, they don't know where, and they mess it up.
          $content=wordwrap(str_replace ('=20',' ',$content));
        }
      } elseif($mime[0]=='multipart') {
        // Recursively parse content, find separator, glue pieces
        // together with separator and top the cake with the header
        // (in this order -- we need to place the header last because
        // otherwise we don't know the separator)
        for ($i=0;$i<count($message['content']);$i++) {
          $myContent[]=$this->_parse($message['content'][$i]);
        }
        $sep=$this->findSep($myContent);
        $h='MIME-Version: 1.0'.$this->headSep; # Paul 01.Nov.2005
        $h.='Content-Type: '.$message['mime'].';'.$this->headSep;
        $h.="\tboundary=\"$sep\"".$this->headSep;
        $content='';
        for ($i=0;$i<count($myContent);$i++) {
          $content.='--'.$sep.$this->headSep.$myContent[$i]['headers'].$this->headSep.$myContent[$i]['content'].$this->headSep.$this->headSep;
        }
        $content.='--'.$sep.'--'.$this->headSep.$this->headSep;
      } else {
        // base64 encode
        // also make sure to include the filename in the header!
        $h='Content-Type: '.$message['mime'].$this->headSep;
        $h.='Content-Transfer-Encoding: base64'.$this->headSep;
        if ($message['cid']) {
          $h.='Content-ID: <'.$message['cid'].'>'.$this->headSep;
        }
        $content=base64_encode($content);
      }
      if ($message['filename']) {
        $h.='Content-Disposition: attachment;'.$this->headSep;
        $h.="\tfilename=\"{$message['filename']}\"".$this->headSep;
      }
      // echo("<hr>Returning stuff from parse:<pre>"); var_dump($h); var_dump($content); echo("</pre>"); flush();
      return(array('headers'=>$h, 'content'=>$content));
    }
      
      /**
      * A method to check for email address validity.
      * Checks for the following:
      * - full e-mail validity, including "xxx xxx" <xxx@xxx.xxx> format
      *   invalid character checking
      * - valid e-mail address characters and format
      *   (accepts xxx[.xxx]@xxx(.xxx)+ format)
      * - checks whether the host exists and accepts mail if the second
      *   parameter is not specifically set to false, disabling this check
      * @param string email the e-mail address to check
      * @param integer tests how many host tests should be performed:
      *                0: only mail format validity checks, no host tests (default)
      *                1: check if the host has an MX record
      *                2: check via SMTP if the user is valid on that host
      * @param boolean true if valid, false otherwise
      */
    function checkMail($email,$tests=0)
    {
      $email=trim($email);
      if (substr($email,-1)=='>') {
        // Uh-oh, we have to parse the full email format!
        // i.e. the "xxx xxx" <xxx@xxx.xxx> format
        // Let's start from the last "@" symbol and work
        // our way from there
        $at=strrpos($email,'@');
        preg_match("/[a-z0-9_\.\-]+/i",substr($email,$at),$host);
        $host=$host[0];
        preg_match_all("/([a-z0-9_\.\-]+)/i",substr($email,0,$at),$user);
        $user=$user[0][count($user[0])-1];
        // The final "3" below comes from:
        // 1, the "@" character
        // 1, the final ">" character
        // 1, substr's zero-based character index as opposed to strlen's one-based count;
        $emailPos=strlen($email)-strlen($host)-strlen($user)-3;
        if (substr($email,$emailPos,1)!='<') {
          return(false);
        }
        // Ok, so now we've parsed the actual e-mail part of the address
        // (i.e. "xxx xxx" <$user@$host>"). Let's now check the "xxx xxx" part,
        // which we'll call the "human" part of the e-mail, since it contains
        // that person's name.
        $human=trim(substr($email,0,$emailPos));
        if ($human[0]=='"') {
          // Ok, it's between double quotes, let's see
          if (substr($human,-1)!='"') {
            // If it starts with quotes, it has to end with quotes
            return(false);
          }
          if (preg_match("/[^\\\\]\"/",substr($human,1,-1))) {
            // It MUST NOT contain unescaped double quotes within
            return(false);
          }
        } else {
          // Ok, not between quotes, let's check for characters
          // which MUST be quoted
          if ((strpos($human,'.')!==false) || (strpos($human,'"')!==false)) {
            return(false);
          }
        }
        $email="$user@$host";
      }
      if (!ereg("^[_a-zA-Z0-9-]+(\.[_a-zA-Z0-9-]+)*@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*\$", $email)) {
        return(false);
      }
      list($emailuser,$emailhost) = explode ("@", $email);
      if (($tests>=1) &&
      (!getmxrr($emailhost.'.', $mxhostarr)) &&
      (gethostbyname($emailhost.'.')==$emailhost)) {
        return(false);
        /*
          If you're wondering why gethostbyname() is called, here's an excerpt
          from a user contributed note at getmxrr():
        
          Please bear in mind that MX records are not mandatory and that RFC974
          clearly states: If no MX records exist for a given host, MTAs should
          interpret this as if there is one MX RR matching the host itself.
          (i.e. you should call gethostbyname() to see if the host has an A record.)
          
          Another note there explains why the final period in host name checks is called for:
          
          Prevent your dns server from 'creating' a valid host name by appending
          the local domain to incomplete emails. Do this by appending a period
          to the domain to be checked.
        */
      }
      if ($tests>=2) {
        $fp = fsockopen ($emailhost, 25);
        set_socket_blocking ($fp, true);
        fputs ($fp, "Helo Local\n");
        fgets ($fp, 2000);
        fgets ($fp, 2000);
        fputs ($fp, "Mail From:<$email> \n");
        fgets ($fp, 2000);
        fputs ($fp, "RCPT to:Lanifex<$email> \n");
        $result= fgets ($fp, 2000);
        //echo("Result: $result<br>");
        $st= substr($result,0,3);
        if ($st!=250) {
          return(false);
        }
      }
      return(true);
    }

      /**
      * This function will recursively check if it can find $needle
      * in any of the elements of the array $haystack, or in the string $haystack.
      * @param string needle the string to search for
      * @param mixed haystack the array or string to check
      * @return boolean true if found, false if not found
      */
    function recursubstr($needle,$haystack,$thread=0)
    {
      if (is_array($haystack)) {
        reset($haystack);
        while(list(,$hay)=each($haystack)) {
          // echo("<b>Thread $thread:</b> Recursubstr($needle, (see below<pre>"); var_dump($hay); echo("</pre>");
          if($this->recursubstr($needle,$hay,$thread+1)) {
            return(true);
          }
        }
        return(false);
      }
      if(strpos($haystack,$needle)===false) {
        return(false);
      }
      return(true);
    }
    
      /**
      * This method builds a random, unique separator per this message.
      * You may pass either a string, in which case it returns a string
      * which is not a substring of the string you pass, or you could pass
      * an entire message array as formatted by {@link _parse} in which case
      * it returns a string which is not a substring of any of the components
      * of the respective message.
      */
    function findSep($msg)
    {
      $right=false;
      list($usec,$sec)=explode(" ",microtime());
      $sepo=md5($sec.$usec);
      while(!$right) {
        if ($sep) {
          $sep=md5($sep);
        } else {
          $sep=$sepo;
        }
        $sep='LFX_'.$sep;
        if ($this->recursubstr($sep,$msg)) {
          continue;
        }
        $right=true;
      }
      return($sep);
    }
      
      /**
      * This method checks whether the string is 7bit safe.
      * In other words, it checks whether there's any character
      * in the extended ASCII range (above 127) and returns false if
      * it finds any. This is obviously slow for long strings,
      * the only optimization considerents are that it doesn't set any
      * variables and it returns on the first non-compliant character.
      * Apart from that, it's still a for(), therefore a slow thing.
      * @param string content the string to check
      * @return boolean true if compliant, false otherwise
      */
    function safe7bit($content)
    {
      for($i=0;$i<strlen($content);$i++) {
        if (ord($content[$i])>127) {
          return(false);
        }
      }
      return(true);
    }
    
      /**
      * This private method prepares all message data into an array
      * which is then used to create the actual message body.
      * Although you shouldn't need to use this method directly, here's
      * what it returns:
      * MAIL-ATOM=array(
      *   'mime'=><the mime type>,
      *   'content'=><either a string or an indexed array of MAIL-ATOMs)
      */
    function _prepare()
    {
      $message=array();
      if ($this->text) {
        $message=array(
          'content'=>$this->text,
          'mime'=>'text/plain',
          'charset'=>$this->textCharset
        );
      }
      if ($this->html) {
        $message=array(
          'mime'=>'multipart/alternative',
          'content'=>array(
            $message,
            array(
              'mime'=>'text/html',
              'content'=>$this->html,
              'charset'=>$this->htmlCharset
            )
          )
        );
        // Cleaning up $message['content'] if $message was empty above.
        if (!$message['content'][0]) {
          $message['content']=array($message['content'][1]);
        }
      }
      if ($this->attachments) {
        $old_message=$message;
        $message=array('mime'=>'multipart/mixed');
        if ($old_message) {
          $message['content']=array($message);
        }
        for ($i=0;$i<count($this->attachments);$i++) {
          $message['content'][]=array(
            'mime'=>$this->attachments[$i]['mime'],
            'content'=>$this->attachments[$i]['content'],
            'filename'=>$this->attachments[$i]['filename'],
            'charset'=>$this->attachments[$i]['charset']
          );
        }
      }
      return($message);
    }
    
      /**
      * Internal wrapper for function mail() -- all calls to system function mail()
      * should go through this method.
      *
      * @param string all all parameters are identical 1:1 with the ones accepted
      *   by the native PHP's mail() function
      * @return boolean exactly the result of function mail()
      */
    function doSendMail($to,$subject,$message,$additional_headers=NULL,$additional_parameters=NULL)
    {
      global $_LFX_globals;
      if ($newTo=$_LFX_globals->getSetting('override_email')) {
        $subject.=" [LFX-REDIR from $to]";
        //$message="This e-mail has been redirected from: $to\n\n".$message;
        $to=$newTo;
      }
      if ($additional_parameters!==NULL) {
        $result=mail($to,$subject,$message,$additional_headers,$additional_parameters);
      } elseif ($additional_headers!==NULL) {
        $result=mail($to,$subject,$message,$additional_headers);
      } else {
        $result=mail($to,$subject,$message);
      }
      return($result);
    }
  }
?>
