<?

  /**
  * LFX_menu class (Based on OPT_Menu)
  * Bogdan Stancescu <bogdan@lanifex.com>, April 2004
  * $Id: LFX_menu.php,v 1.7 2005/11/10 10:40:58 bogdan Exp $
  * @package LFX
  */
    /**
    * The LFX_Menu class used to manage the main LFXlib menu
    */
  class LFX_Menu
  {
    var $children=NULL;
    var $focused=false;
    var $id=0;
    var $fullId='0';
    var $title='[no title]';
    var $url='#';
    var $tdTag='';
    var $tip='';
    var $translationFunction='__L';
    var $access;
    var $hidden=false;
    var $module='LFX';

      /**
      * The constructor. You can pass a menu description to the constructor
      * in order to build the whole menu structure. You could alternatively
      * use the class methods for the same purpose, but the recommended way
      * is to pass the menu description to the constructor.
      *
      * Each entry in the menu description array must have the following structure:
      * "title" (string) => the title of the menu entry to be displayed
      * "url" (string) => the URL to link the respective entry to
      * "tip" (string) => the "tip" to be displayed when the user hovers over the link
      * "focused" (boolean) => [optional] if true, the respective item will be focused
      * "children" (array) => indexed array of entries like this one, for submenu items.
      *
      * @param array $menuDesc a menu description
      */
    function LFX_Menu($menuDesc=NULL)
    {
      global $_LFX_images,$_page,$_LFX_globals;
//       decho("//".$this->module." ".$_SERVER['PHP_SELF']);
      $this->tdTag="<td style=\"text-align: center; background: url('$_LFX_images/menu_bar.png')\" width='1%'>";
      if (is_array($menuDesc)) {
        $this->title='MenuRoot';
//        echo("alert('checking cache constructor ".$_SERVER['PHP_SELF'].$this->module."');");
//         decho("//Last updated: ".date("H:i:s",$_SESSION['menu'][$this->module]['cache']['last_updated']));
//         decho("//Per user: ".date("H:i:s",$_page->user->getSetting('user_rights_changed')));
//         decho("//Globala LFX: ".date("H:i:s",$_LFX_globals->getSetting('global_rights_changed')));
        $global_str='_'.$this->module.'_globals';
        global $$global_str;
        $useJSMenu=$$global_str->getSetting('useJSMenu');
        if ($useJSMenu && (!isset($_SESSION['menu'][$this->module]['cache']['last_updated']) || 
            ($_SESSION['menu'][$this->module]['cache']['last_updated']<$_page->user->getSetting('user_rights_changed'))
            || $_SESSION['menu'][$this->module]['cache']['last_updated']<$_LFX_globals->getSetting('global_rights_changed'))) {
//          echo("alert('".$_SESSION['menu_cache'][$this->module]."');");
//           echo("alert('updating cache');");
          $this->loadArray($menuDesc);
          $_SESSION['menu'][$this->module]['cache']['last_updated']=time();
          $_SESSION['menu'][$this->module]['cache']['items']=$this->dump2JS($this->children,'menu_items');
        } else {
          $this->loadArray($menuDesc);
        }
      }
    }

      /**
      * The function used to load the menu entries for a menu item from a menu
      * description array. Please check {@link LFX_Menu} for the array structure.
      *
      * @param array $menuDesc a menu description
      * @param boolean $preserve if true, child menus will NOT be deleted prior to loading the description
      */
    function loadArray($menuDesc, $preserve=false)
    {
      if (!$preserve) {
        unset($this->children);
      }
      while(list(,$menuEntry)=each($menuDesc)) {
        $myClass=get_class($this);
        $tmp=new $myClass();
        if ($menuEntry['access']) {
          $tmp->access=$menuEntry['access'];
          if ($tmp->checkAccessProblems()) {
            $tmp->hidden=true;
          }
        }
        if ($menuEntry['title']) {
          $tmp->title=$menuEntry['title'];
        }
        if ($menuEntry['url']) {
          $tmp->url=$menuEntry['url'];
        }
        if ($menuEntry['noTranslation']) {
          $tmp->noTranslation=$menuEntry['noTranslation'];
        }
        $tmp->tip=$menuEntry['tip'];
        if ($menuEntry['focused']) {
          $tmp->focused=true;
        }
        $this->addChild($tmp);
        if (is_array($menuEntry['children'])) {
          $this->children[count($this->children)-1]->loadArray($menuEntry['children']);
        }
        unset($tmp);
      }
    }

      /**
      * Method to add a LFX_Menu object as a child to this object
      *
      * @param object $menu a LFX_Menu object to add
      * @return string the full path to the new child
      */
    function addChild(&$menu)
    {
      $menu->id=count($this->children);
      $menu->fullId=$this->fullId.'|'.$menu->id;
      $this->children[]=$menu;
      return($this->id.'|'.$menu->id);
    }

      /**
      * Sets focus on this menu item
      */
    function setFocus()
    {
      $this->focused=true;
    }

      /**
      * Returns the full path to the focused item BELOW this one.
      * Please make sure you only run this on the root of the menu - checking
      * is only performed among the children of the current entry.
      *
      * @param boolean $hidden if true, will recurse through hidden children as well
      * @return mixed the full path to the focused item or false if none
      */
    function getFocus($hidden=false)
    {
      if ($this->focused) {
        return($this->fullId);
      }
      $children=&$this->getChildren($hidden);
      reset($children);
      while(list($index,$child)=each($children)) {
        if ($tmp=$child->getFocus($hidden)) {
          return($tmp);
        }
      }
      return(false);
    }

      /**
      * Returns references to $this->children objects. If $hidden is true,
      * then a reference to the whole array is returned (i.e. hidden children
      * included). Otherwise, a new array is returned, and this new array
      * contains references to each non-hidden child in $this->children.
      *
      * Please note that if you ask for non-hidden children, the indexes
      * will be preserved. In other words, expect "holes" in the array
      * if you ask for non-hidden children only. This has been done this way
      * in order to be compliant with the many operations which rely on
      * the ID's of the children in the structure.
      *
      * @param boolean $hidden whether to include the hidden children in the
      *   result
      * @return array an indexed array of children
      */
    function getChildren($hidden=true)
    {
      if ($hidden) {
        $result=&$this->children;
      } else {
        $result=array();
        for($i=0;$i<count($this->children);$i++) {
          if (!$this->children[$i]->hidden) {
            $result[$i]=&$this->children[$i];
          }
        }
      }
      return($result);
    }

      /**
      * Returns a reference to the menu item specified by path.
      *
      * NOTE ON REFERENCES
      * Although the value is returned by reference, you still have
      * to call this method like $foo=&$bar->getById($myPath) in order
      * to retrieve a reference to the respective object, as opposed to
      * a copy of it. If it's a copy you want, then simply call the method
      * without the ampersand.
      *
      * The id path is simply the zero-based index of every child in the
      * children array sequence path to the desired element separated by vertical
      * bars. For instance, "0|1|6" would be the seventh child of the second child
      * of the first child of the root.
      *
      * Since the menus will most certainly move around, you should not hardcode
      * these values, but rather use {@link getByTitle} with a single value (not path).
      * If that's not possible (if there are duplicates in the titles) then use
      * getByTitle with a path.
      *
      * Please note that this method completely disregards the $hidden attribute.
      *
      * @param string $id the path to the menu entry to retrieve
      * @return mixed the respective object or false if not found
      */
    function &getById($id)
    {
      if (substr($id,0,strlen($this->fullId))!=$this->fullId) {
        return(false);
      }
      $id=substr($id,strlen($this->fullId)+1);
      $ids=explode('|',$id);
      $tmp=&$this;
      for($i=0;$i<count($ids);$i++) {
        if (!is_object($tmp)) {
          return(false);
        }
        $tmp=&$tmp->children[$ids[$i]];
      }
      return($tmp);
    }

      /**
      * Returns a reference to the menu item specified by name.
      *
      * If there are duplicate menu item titles in the menu structure
      * you can pass a name path to this function. Name paths are the same
      * as id paths (documented at the {@link getById} method), only they
      * contain menu titles separated by vertical bars. Please note only the
      * English text is used for paths, so make sure you don't translate the
      * titles before building the path.
      *
      * Please read the note on references at {@link getById}
      *
      * Please note that this method completely disregards the $hidden attribute.
      *
      * @param string $title either a simple title or the full path to a title
      * @param string $path if you don't call this method on the root element,
      * then pass the full path to the current object here (but exclude the current object)
      * @return mixed the reference to the object or false if not found
      */
    function &getByTitle($title, $path='')
    {
      if ($this->title==$title) {
        return($this);
      }
      $titles=explode('|','MenuRoot|'.$title);
      if ((count($titles)>1) &&
        ($this->title==array_pop($titles)) &&
        ($path==implode('|',$titles))) {
        return($this);
      }
      for($i=0;$i<count($this->children);$i++) {
        $tmp=&$this->children[$i]->getByTitle($title,$path.($path?'|':'').$this->title);
        if ($tmp!==false) {
          return($tmp);
        }
      }
      return(false);
    }

      /**
      * Renders a single menu cell - you shouldn't need to call this manually.
      *
      * @return string the rendered cell (one HTML table cell)
      */
    function renderCell()
    {
      $tf=$this->translationFunction;
      if ($this->noTranslation) {
        $myTitle=$this->title;
      } else {
        $myTitle=$tf($this->title);
      }
      $title=str_replace(' ','&nbsp;',htmlspecialchars($myTitle,ENT_QUOTES));
      $result=$title;
      $tip=htmlspecialchars($this->tip,ENT_QUOTES);
      if ($this->focused) {
        $id='menu_focused';
      } elseif($this->getFocus()) {
        $id='menu_focused_path';
      } else {
        $id='menu';
      }
      $result="&nbsp;<a href='{$this->url}' title='".$tf($tip)."' id='$id'>$result</a>&nbsp;";
      $result="{$this->tdTag}$result</td>";
      return(LFX_echo($result));
    }

      /**
      * Renders the children of the current menu entry - you shouldn't need
      * to call this manually.
      *
      * @return string the rendered children (one HTML table).
      */
    function renderChildren()
    {
      global $_LFX_images;
      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      global $_LFX_images;
      $result.="{$this->tdTag}<img src='$_LFX_images/spacer.gif' alt='' height='23' width='2'></td>";
      $children=$this->getChildren(false);
      reset($children);
      $first=true;
      while(list($index,$child)=each($children)) {
        if ($first) {
          $first=false;
        } else {
          $result.=LFX_echo($this->tdTag."<img src='$_LFX_images/menu_sep.png' alt='|'></td>");
        }
        $result.=$child->renderCell();
      }
      $result.=LFX_etr();
      $result.=LFX_echo('</table>');
      return($result);
    }

      /**
      * This is really private, and you should NEVER call this directly.
      *
      * Due to the particular nature of the OPT menu, the top most menu level
      * is significantly different from the rest in terms of HTML tags, because
      * it's made of three cells by two, as opposed to two by two as the other
      * levels are rendered (an extra cell is needed to render the shadow below
      * the OPT header bar). This method renders the "other" levels (the child
      * menu levels) while {@link render} performs the root level rendering.
      * {@link render} also calls this method, which is why you should never
      * need to call this method directly.
      *
      * @return string the current level and the levels below up to one level below
      * the focused item, if any - otherwise only the current level - rendered in HTML
      */
    function renderChildLevel()
    {
      global $_LFX_images;
      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      $result.=LFX_echo('<td>');
      $result.=$this->renderChildren();
      $result.=LFX_echo('</td>');
      $result.=LFX_td("<img src='$_LFX_images/menu_end.png' alt=''>");
      $result.=LFX_etr();

      $result.=LFX_str();
      if (($focused=$this->getFocus()) && (!$this->focused)) {
        $focused=substr($focused,strlen($this->fullId)+1);
        list($focusChild)=explode('|',$focused);
        $children=$this->getChildren(false);
        if (!count($children[$focusChild]->getChildren(false))) {
          $result.=LFX_echo("<td background='$_LFX_images/menu_empty.png' width='100%'>&nbsp;</td>");
        } else {
          $result.=LFX_echo('<td>');
          $result.=$children[$focusChild]->renderChildLevel();
          $result.=LFX_echo('</td>');
        }
      } else {
        $result.=LFX_echo("<td background='$_LFX_images/menu_empty.png' width='100%'>&nbsp;</td>");
      }
      $result.=LFX_echo("<td valign='top'><img src='$_LFX_images/menu_end_empty.png' alt=''></td>");
      $result.=LFX_etr();

      $result.=LFX_echo('</table>');
      return($result);
    }

    function renderJS()
    {
      global $_LFX_url;
      $result=LFX_echo("<div id='main_menu' style='position:relative;left:0px;top:0px;'>&nbsp;</div><br/><br/>");
//      $result.=LFX_echo("<div id='menu_focus' style='position:relative;'>".$this->focusJS($this->getFocus())."</div><br/><br/>");
      // The menu_items JS array
//      decho("checking cache {$this->module}");
      if (!isset($_SESSION['menu'][$this->module]['cache']['items'])) {
        // Practically never getting here, the JS array should already be
        // in the session since the constructor was called
//         decho("no cache {$this->module}");
        $menuItems=$this->dump2JS($this->children,'menu_items');
        $result.=LFX_echo("<script>".$menuItems."</script>");
        $_SESSION['menu'][$this->module]['cache']['items']=$menuItems;
      } else {
//         decho("have cache {$this->module} ".vardump($_SESSION));
        $result.=LFX_echo("<script>".$_SESSION['menu'][$this->module]['cache']['items']."</script>");
      }
      $result.=LFX_echo("<script src='$_LFX_url/JS_menu.php?module={$this->module}'></script>");
      $result.=LFX_echo("<script>build_menu('main_menu');</script>");
      $result.="<script>
                  // Resizing top level
                  topLevel=document.getElementById('main_menu').firstChild;
                  topLevel.style.background=topLevel.firstChild.style.background;
                  topLevel.style.display='block';
                  topLevel.style.borderBottomColor='#888888';
                  topLevel.style.borderBottomWidth='1px';
                  topLevel.style.borderBottomStyle='solid';
                  topLevel.style.height=menuItemHeight;
                </script>";
      return($result);
    }
    
      /**
      * The method to render the menu item and all its children.
      *
      * If interested in technical details, you can check the note on {@link renderChildLevel}
      *
      * @return string the whole menu rendered in HTML, with respect to the focused item.
      */
      
    function render()
    {
      $global_str='_'.$this->module.'_globals';
      global $$global_str;
      if ($$global_str->getSetting('useJSMenu')) {
        return($this->renderJS());
      } else {
        return($this->render_static());
      }
    }
    
    function render_static()
    {
      global $_LFX_images;
      $result=LFX_echo('<!-- Rendering menu -->');
      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      $result.=LFX_echo('<td>');

      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      $result.=LFX_echo('<td>');
      $result.=$this->renderChildren();
      $result.=LFX_echo('</td>');
      $result.=LFX_etr();
      $result.=LFX_echo('</table>');

      $result.=LFX_echo('</td>');
      $result.=LFX_td("<img src='$_LFX_images/menu_end.png' alt=''>");
      $result.=LFX_echo("<td background='$_LFX_images/menu_empty.png' width='100%'>&nbsp;</td>");
      $result.=LFX_etr();

      $result.=LFX_str();
      if (($focused=$this->getFocus())) {
        $focused=substr($focused,strlen($this->fullId)+1);
        list($focusChild)=explode('|',$focused);
        $children=$this->getChildren(false);
        if (!count($children[$focusChild]->getChildren(false))) {
          $result.=LFX_echo("<td background='$_LFX_images/menu_empty.png'><img src='$_LFX_images/spacer.gif' height='3' alt=''></td>");
        } else {
          $result.=LFX_echo('<td>');
          $result.=$children[$focusChild]->renderChildLevel();
          $result.=LFX_echo('</td>');
        }
      } else {
        $result.=LFX_echo("<td background='$_LFX_images/menu_empty.png'>&nbsp;</td>");
      }
      // Might be rendered in the middle of the cell in older agents (non-CSS) - oh well...
      $result.=LFX_echo("<td style='align:left; vertical-align:top'><img src='$_LFX_images/menu_end_empty.png' alt=''></td>");
      $result.=LFX_echo("<td width='100%'><img src='$_LFX_images/spacer.gif' height='3' alt=''></td>");
      $result.=LFX_etr();
      $result.=LFX_echo('</table>');

      $result.=LFX_echo('<!-- Finished menu -->');

      return($result);
    }

      /**
      * Unfocuses whatever focused item there may be below this menu item.
      *
      * @return mixed path to the previously focused item if there was any, false otherwise
      */
    function unfocus()
    {
      if($focus=$this->getFocus()) {
        $focused=&$this->getById($focus);
        $focused->focused=false;
        return($focus);
      }
      return(false);
    }

      /**
      * Focuses an item by title.
      *
      * See comment on {@link getByName} for more information on how to pass titles.
      *
      * Any previously focused item in the menu is naturally unfocused before setting
      * the focus on the desired item - but bear in mind that you can end up with
      * multiple items focused it you call this method on any other item but the menu root.
      *
      * @return boolean true on success or false on failure
      */
    function focus($title,$ignoreAccess=false,$user=NULL)
    {
      $this->unfocus();
      $tmp=&$this->getByTitle($title);
      if (!$ignoreAccess) {
        if (!$user) {
          global $_page;
          $user=&$_page->user;
        }
        if ($tmp && $tmp->checkAccessProblems($user)) {
          $tf=$this->translationFunction;
          $noAccess="You do not have access to this page (menu).";
          if ($tf) {
            echo($tf($noAccess));
          } else {
            echo($noAccess);
          }
          exit;
        }
      }
      if ($tmp!==false) {
//        $this->focusJS($tmp->fullId);
        $tmp->focused=true;
        return(true);
      }
      return(false);
    }

    // Focusing for the JavaScript menu, no longer used
        
    function focusJS($path)
    {
      $path=explode('|',$path);
//       echo(vardump($this));
      $return='';
      $menuItem=$this;
      for ($i=1;$i<count($path);$i++) {
        if ($i>1) {
          $gt='>';
        } else {
          $gt='';
        }
        $return.=" $gt <a href='".$menuItem->children[$path[$i]]->url."'>".$menuItem->children[$path[$i]]->title."</a>";
        $menuItem=$menuItem->children[$path[$i]];
      }
      return($return);
    }

      /**
      * This method checks whether a user has any access prohibitions from
      * seeing this menu entry. It can determine this by checking this menu
      * entry's access variable. That variable can store a structure of the
      * following form:
      *
      * array(
      *   'group'=>array(group1, group2, ...),
      *   'level'=>array(level1, level2, ...)
      * )
      *
      * Both the 'group' and the 'level' arrays are indexed arrays in the
      * structure above, and they both contain strings. The 'groups' array
      * stores the groups the user needs to have access to (OR), and the
      * 'level' array stores their corresponding levels of access (passed to
      * LFX_User::getRights). The indexes must correspond.
      *
      * Alternately, 'level' can be a string, even if 'group' is an array,
      * in which case the same level of access is assumed for all groups.
      *
      * The 'level' entry can also miss altogether, in which case only
      * simple membership in the groups is checked.
      *
      * If only one element is needed in the 'group' array, then it can be
      * passed as a simple string. Therefore the simplest possible form for
      * the access value is array('group'=>group).
      *
      * If you need simple membership checking for selected elements in the
      * 'group' array, then use the array format for 'level' and assign
      * empty string values to the indexes corresponding to the respective groups.
      *
      * This method is used by {@link render}() to check which menu entries
      * should be shown, and it's also used by {@link focus}() which denies
      * access to the page if permission problems arise.
      *
      * This method of assigning permissions is assumed from the beginning to
      * be a simplistic solution to be used only for the menu entries which
      * support such a straightforward mechanism (membership in at least one of
      * a set of groups give access to a given page). If you need a more complex
      * permissions checking mechanism, then you must define the menu array
      * conditionally.
      *
      * @param object $user the user to check permissions for. If not specified,
      *   _$page->user is used instead
      * @return mixed true if $user doesn't have access to this menu entry,
      *   false if the user does have access or NULL if this menu entry does
      *   not employ any access checking
      */
    function checkAccessProblems($user=NULL)
    {
      if (!$access=$this->access) {
        return(NULL);
      }
      if (!$user) {
        global $_page;
        $user=&$_page->user;
      }
      $allow=false;
      if (is_array($access['group'])) {
        for($i=0;$i<count($access['group']);$i++) {
          if (!$access['level']) {
            $level='';
          } elseif (is_array($access['level'])) {
            $level=$access['level'][$i];
          } else {
            $level=$access['level'];
          }
          if ($user->getRights($access['group'][$i],$level)) {
            $allow=true;
            break;
          }
        }
      } else {
        if ($user->getRights($access['group'],$access['level'])) {
          $allow=true;
        }
      }
      return(!$allow);
    }
    
    function dump2JS($menu,$jsArray)
    {
      global $_LFX_path,$_LFX_url,$_LFX;
      $return="$jsArray=new Array();\n";
      $k=-1;
//      echo('alert("rendering menu");');
      for ($i=0;$i<count($menu);$i++) {
        if (!$menu[$i]->hidden) {
          $k++;
          $return.=$jsArray."[$k]=new Array();\n".
              $jsArray."[$k][0]='".addslashes($menu[$i]->title)."';\n".
              $jsArray."[$k][1]='".addslashes($menu[$i]->url)."';\n";
//          echo("//".$menu[$i]->title);
          if (($this->module!='LFX') && $menu[$i]->title=='LFXlib') {
            $hideLFXlibMenu=true;
            include($_LFX_path.'/include/LFX_menu_init.php');
            $lfxlib_items=$_SESSION['menu']['LFX']['cache']['items']?$_SESSION['menu']['LFX']['cache']['items']:$this->dump2JS($lfxlib_menu->children,'menu_items');
            $LFXlib_entry=str_replace('menu_items',$jsArray."[$k][2]",$lfxlib_items);
            $return.=$LFXlib_entry;
          }
          if (is_array($menu[$i]->children)) {
            $return.=$this->dump2JS($menu[$i]->children,$jsArray."[$k][2]");
          }
        }
      }
      return($return);
    }
    
  }

?>
