<?

  /**
  * The LFX_Object class
  * Bogdan Stancescu <bogdan@lanifex.com>, October 2002
  * THIS LIBRARY ONLY WORKS WITH PHP 4+
  * $Id: LFX_object.php,v 1.185 2005/11/28 08:21:15 bogdan Exp $
  * @package LFX
  */

  _checkMeOut(__FILE__);
  // Defining - and checking for - include dependencies
  $_noCode=true;
  $result=array('PHPlib');
  if ((function_exists('_checkIncludes')) && (!_checkIncludes($result))) {
    return($result);
  }
  return(true);

    // Conventions:
    // - public class names are separated by underscores when made of
    //   more than one word (e.g. 'LFX_Object')
    // - public method and variable names are made of concatenated words
    //   when made of more than one word (e.g. 'getObject()')
    // - private names start with an underscore (e.g. '_status')
    // Please follow these conventions when extending this class
    // because it makes it harder to mistake names.
    // You can obviously use whatever conventions you wish inside
    // your methods - these are guidelines for the public stuff.

    /**
    * The main Lanifex object (aka SC)
    * @package LFX
    */
  class LFX_Object
  {
    // This is the super class (or 'mother of all classes').
    // Please make sure all Lanifex classes extend this class
    // (or any of its descendants, that is)

    // ----------------------------------
    //          CLASS VARIABLES
    // ----------------------------------
      /**
      * The id of this object in the database
      */
    var $id;

      /**
      * The attributes of this object - an associative array
      */
    var $attr;

      /**
      * Attribute flags - you shouldn't typically need to use this
      * directly - it's an array like above containing associative
      * arrays marking status flags for each attribute (such as
      * 'modified')
      */
    var $attr_flags;

      /**
      * The data structure of this object
      * Please see the documentation - won't duplicate the whole
      * explanation here.
      */
    var $dataStruct;

      /**
      * Somewhat similar to $this->dataStruct, this one lists the dependencies.
      * Used when removing (deleting) this object from the database.
      */
    var $dependencies;

      /**
      * Array of associative arrays of messages associated to this object.
      *
      * The following format is used for each element:
      *      'id' => error number (see IDW PHP Coding Conventions in OPT)
      *    'type' => error type (idem)
      * 'message' => human-readable message
      *  'obj_id' => the current object id ($this->id)
      *  'sub_id' => subsystem error # (e.g. MySQL error #)
      * Please remember $messages contains an ARRAY of such arrays!
      */
    var $messages;

      /**
      * The database data for the current object.
      * An associative array containing:
      * 'dbDataSet' => a valid key of the $_LFX['global']['dbData'] defined in LFXlib/include/LFX_dbdata.php (RECOMMENDED)
      * -- OR --
      * 'class' => a predetermined class type to use (NOT RECOMMENDED!)
      * -- OR --
      *  'host' => the host to use;
      * 'uname' => the database username;
      *   'pwd' => the password;
      *    'db' => the database to use.
      * Filled in by the constructor based on external data.
      * The following defaults apply:
      * - if 'dbDataSet' is specified, it will use the corresponding data
      * $_LFX['global']['dbData'] and will use DB_PHPLIB as class
      * (DB_PHPLIB is defined in LFXlib/include/phplib/local.inc.php
      * and has no credentials by default)
      * - if 'class' is set but 'host' is not, it will instantiate the
      * specified class and leave all class parameters to default;
      * - if both 'class' and 'host' are set, it will instantiate the
      * specified class and set its host, uname, pwd and db to the
      * respective values;
      * - if neither 'dbDataSet' not 'class' are not set but 'host' is,
      * it will use DB_PHPLIB and the credentials from $this->dbData.
      * - if none of 'dbDataSet', 'class' and 'host' are set, it will issue an error;
      */
    var $dbData;

      /**
      * A DB_IDW object which contains all the stuff needed for this object.
      *
      * Should be initialized by the constructor based on $this->dbData,
      * typically using {@link dbInit}.
      */
    var $db;

      /**
      * Tree-structured objects root cache.
      *
      * For the so many objects which have a tree structure, this should
      * be set to the root of the tree in which the current object resides,
      * if known. If not set, methods which need it MUST be able to find
      * it out using $this->getRoot() which in turn relies on $this->dataStructure
      * to find out which attribute to use for parenting relationship.
      */
    var $rootId=0;

      /**
      * This structure details the way attributes will be shown in the object list.
      */
    var $showAttr;

      /**
      * If set to true, logging on this type of object will not occur.
      */
    var $noLogging=false;

      /**
      * The module in which this object resides. Typically should only be set
      * in top-level objects (LFX_Object, DMO_Object, OPT_Object, etc).
      */
    var $module='LFX';

      /**
      * This variable controls whether the consistency checker should skip any
      * consistency tests for this class. The possible values are:
      * 'link' => the links in this class are not tested for consistency
      * 'depend' => the dependencies of this class are not tested
      * 'all' => no consistency checking is performed for this class.
      * Please note that even if you set this to 'all', other classes which
      * link to or depend upon this class will still be fully checked for
      * consistency, including tests which include this class, unless explicitly
      * disabled in those classes as well.
      * You should not disable the consistency checking for this class, that
      * should only be done for descendants. The definition of the variable is
      * here only for reference.
      */
    var $disableConsistency='';

      /**
      * An internal variable which controls the upper size limit for
      * short_in_db fields which get stored in the database.
      */
    var $short_in_db_limit=65530;

    // ----------------------------------
    //          PUBLIC METHODS
    // ----------------------------------
      /**
      * The constructor. Please *always* run parent
      * constructors *explicitly* when extending a class
      * (unless that has extremely adverse effects to your code)
      * In other words, it is highly recommended that any
      * class LFX_random extending LFX_object runs $this->LFX_object()
      * in the constructor, any class extending LFX_random runs
      * $this->LFX_random() in the constructor and so on.
      * The request for explicit running the constructor means that
      * you should not use
      * < ?
      *   $parent=get_parent_class($this);
      *   $this->$parent();
      * ? >
      * because that fails when your class is extended and will end
      * up calling itself over and over, looping indefinitely.
      * The LFX_Object constructor only sets $this->id, if $id is passed.
      * @param integer $id [optional] The id of the newly instantiated object
      */
    function LFX_object($id=0)
    {
      if ($id) {
        $this->id=$id;
      }
      global $_LFX;
      $this->dbData=array('dbDataSet'=>$_LFX['global']['applications']['LFX']['app_dbdata']);
    }

      /**
      * Initializes $this->db.
      *
      * If $this->dbData is set, it uses the values there to
      * initialize the database class.
      */
    function dbInit()
    {
      unset($this->db);
      $this->db=$this->_doDbInit($this->dbData);
    }

     /**
     * This method is meant for CLI scripts.
     * csecho() is used for message logging
     * csecho() writes messages in LFX_message_log and sys.log
     * if one of the two types of logging is inhibit from globals
     * it isn't writing in that log
     * @param string $message - the message
     * @return writes the message log
     */
    function csecho($message,$severity='info')
    {
      global $_LFX,$_LFX_globals;
      $return=false;
      if (!$_LFX_globals->getSetting('inhibit_LFX_message_output')) {
        LFX_LogMessage($message,$severity);
        $return=true;
      }
      if (!$_LFX_globals->getSetting('inhibit_sys_log_output')) {
        openlog('LFXlib'.':'.$_LFX['global']['application'].':'.$_LFX['global']['subsystem'],LOG_ODELAY,LOG_LOCAL1);
        syslog(LOG_ERR,$message);
        $return=true;
      }
      return $return;
    } 

      /**
      * Adds an error to this object's messages list and performs error management
      *
      * This is the only method you should ever call if you need to output errors,
      * warnings or informational messages. 'Informational messages' means things like
      * "Parent not specified - defaulting to root", not "Please enter e-mail address:"
      */
    function addError($message, $id=0, $type=0, $sub_id=0)
    {
      if (function_exists('debug_backtrace')) {
        $data_dump=debug_backtrace();
        $this->messages[]=
          array('id'=>$id, 'sub_id'=>$sub_id, 'type'=>$type,
            'message'=>$message, 'obj_id'=>$this->id,
            'debug_backtrace'=> $data_dump);
      } else {
        $this->messages[]=
          array('id'=>$id, 'sub_id'=>$sub_id, 'type'=>$type,
            'message'=>$message, 'obj_id'=>$this->id);
        $data_dump=0;
      }

      // log the error
      $severities=array(
        0 => 'critical',
        1 => 'error',
        2 => 'error',
        3 => 'warning',
        4 => 'info'
      );
      if(!$severity=$severities[$type]) {
        $severity='critical';
      }
      global $_LFX;
      $xtra=array(
        'module' => $_LFX['global']['application'],
        'keywords' => '',
        'subsystem' => $_LFX['global']['subsystem'],
        'data_dump' => $data_dump
      );
      openlog('LFXlib'.':'.$_LFX['global']['application'].':'.$_LFX['global']['subsystem'],LOG_ODELAY,LOG_LOCAL1);
      syslog(LOG_ERR,$message);
      
      // Error management
      global $_LFX;
      if (isset($_LFX['page']['debug_level'])) {
        switch($_LFX['page']['debug_level']) {
          case LFX_DL_PROD:
            $stop=$_LFX['global']['error_mgt']['stop'];
            $mail=$_LFX['global']['error_mgt']['mail'];
            $show=$_LFX['global']['error_mgt']['show'];
            break;
          case LFX_DL_DEBUG:
            $stop=LFX_ET_ERROR;
            $mail=LFX_ET_NONE;
            $show=LFX_ET_WARNING;
            break;
          case LFX_DL_DEVEL:
            $stop=LFX_ET_NONE;
            $mail=LFX_ET_NONE;
            $show=LFX_ET_ALL;
            break;
          case LFX_DL_QUIET:
            $stop=LFX_ET_NONE;
            $mail=LFX_ET_NONE;
            $show=LFX_ET_NONE;
            break;
          default:
            //echo("Warning! Debug level set to unknown value! Will assume development status!<br />");
            $stop=LFX_ET_NONE;
            $mail=LFX_ET_NONE;
            $show=LFX_ET_ALL;
        }
      } else {
        // If not set, we'll assume recommended production status
        $stop=LFX_ET_NONE;
        $mail=LFX_ET_NONE;
        $show=LFX_ET_ALL;
      }
      $et=$type >> 8;
      $el=$type & 255;
      if (($show==LFX_ET_ALL) || ($el <= $show)) {
        if (function_exists('LFX_echo')) {
          global $_LFX;
          $_LFX['page']['error_messages']=array_merge(
            $_LFX['page']['error_messages'],$this->messages
          );
          //echo(LFX_echo($this->getErrorsH()));
        } else {
          // Ouch, this is really bad, LFXlib is not stable at all!
          // Probably a bad install, dumping the messages just like that
          // and exiting, God knows what we could end up doing if we continued
          // execution in this state!
          echo("Critical error: LFX_echo is unavailable to SC::addError. Here's the error:");
          echo("<pre>"); var_dump($this->messages); echo("</pre>");
          // We used to exit here... :-)
          //exit;
        }
      } else {
        pecho('<b>Error '.($id?"#$id":'(unknown ID)')."</b>.<br />\n");
      }
      if (($stop==LFX_ET_ALL) || ($el <= $stop)) {
        pecho("Aborting due to errors.<br />\n");
        pecho("</body></html>\n");
        exit;
      }
      $this->messages=array();
    }

      /**
      * Gets executed just after loading or quickloading.
      * Should be overridden by descendants.
      * @param boolean $full true if it's a full load or false if it's a quickload.
      */
    function onLoad($full)
    {
    }

      /**
      * Loads the object with the specified id from the database.
      * Sets the correct $this->id upon success.
      * @param integer $id the id of the object to load;
      *   if not specified, it will use the current id ($this->id)
      * @param boolean $force if false, it will avoid loading the attributes
      *   which have already been set
      * @return boolean true on success or false on error.
      */
    function load($id=0,$force=true)
    {
      if ($result=$this->_doLoad($id, true, $force)) {
        $this->onLoad(true);
      }
      return($result);
    }

      /**
      * Otherwise identical to {@link load}, this doesn't load fields flagged 'large'
      * @return boolean true on success or false on error
      */
    function quickLoad($id=0,$force=true)
    {
      if ($result=$this->_doLoad($id, false, $force)) {
        $this->onLoad(false);
      }
      return($result);
    }

      /**
      * Gets executed just before {@link save} performs the actual saving.
      *
      * Should be overridden by descendants - the SC incarnation returns
      * true unconditionally.
      * @param boolean $new true if this will be an insert
      * @param integer $id the id of the object which will be saved (might
      *   not be set for inserts - see {@link onSave} if you need it)
      * @param boolean $force same as the one passed to {@link save}
      * @return boolean true to proceed saving or false to abort.
      */
    function beforeSave($new, $id, $force)
    {
      return(true);
    }

      /**
      * Gets executed immediately after a {@link save}
      * Should be overridden by descendants.
      * @param boolean $new set if this was an insert
      */
    function onSave($new)
    {
    }

    // TO DO (code duplication): save(), insert() and insertWithId() share
    // quite a lot of duplicate code, should merge them or build extra
    // functions to take over the common functionality.
    
      /**
      * Saves the current object to the database.
      *
      * If $id is not specified, $this->id is used.
      * If $this->id is not set (or evaluates to false) then it
      * adds a new record with the current object's attributes
      * to the database (actually executes $this->insert()).
      * Sets the correct $this->id upon success.
      * @return mixed id on success, false on error
      *   or NULL if {@link beforeSave} aborts the save
      */
    function save($id=0, $force=0)
    {
      $id=$id?$id:$this->id;

      if (!$id) {
        return($this->insert());
      }
      if ($this->beforeSave(false, $id, $force)!==true) {
        return(NULL);
      }
      // Preparing the file fields first, if any
      $fileData=$fileDataFields=array();
      $fields=$this->dataStructure['fields'];
      $saveFiles=false;
      while (list($attName,$dataEntry)=@each($fields)) {
        if (
          $this->attr_flags[$attName]['modified'] &&
          (
            $dataEntry['flags']['file'] ||
            (
              $dataEntry['flags']['short_in_db'] &&
              strlen($this->attr[$attName])>=$this->short_in_db_limit
            )
          )
        ) {
          $fileData[$attName]=$this->attr[$attName];
          $fileDataFields[$attName]=true;
          //echo("fileData[$attName]={$this->attr[$attName]}<br />\n");
          unset($this->attr[$attName]);
          $saveFiles=true;
        } elseif ($dataEntry['flags']['short_in_db']) {
          // We prepend an underscore to these entries in order to ensure that
          // the resulting values can't be interpreted as integers when
          // they're retrieved from the database, even if the field itself
          // happens to only contain an integer value.
          $this->attr[$attName]='_'.$this->attr[$attName];
          $this->attr_flags[$attName]['prepended']=true;
        }
      }
      $query=$this->_makeSaveQuery($force);
      @reset($fields);
      while (list($attName,$dataEntry)=@each($fields)) {
        // Can't use $fileData reliably on the 2nd condition below because
        // the value can be 0 or NULL or empty string, although the field
        // HAS been modified.
        if (($dataEntry['flags']['file'] || $dataEntry['flags']['short_in_db']) && $fileDataFields[$attName]) {
          $this->attr[$attName]=$fileData[$attName];
        } elseif ($dataEntry['flags']['short_in_db'] && $this->attr_flags[$attName]['modified']) {
          $this->attr[$attName]=substr($this->attr[$attName],1);
          if (@is_file($fileName=$this->_getDataFile($attName))) {
            if (!@unlink($fileName)) {
              $this->addError("Failed to delete file $fileName as required ".
                "by the new data size. This particular save process will go ".
                "on with no problems, but you should fix the permissions in ".
                "the LFXlib data directories for future operations on this file!");
            }
          }
        }
        if ($this->attr_flags[$attName]['prepended']) {
          $this->attr[$attName]=substr($this->attr[$attName],1);
          unset($this->attr_flags[$attName]['prepended']);
        }
        unset($this->attr_flags[$attName]['modified']);
      }
      if ($query) {
        $id=addslashes($id);
        $id_fld=$this->dataStructure['id_field'];
        $table=$this->dataStructure['table_name'];
        if ((!$table) || (!$id_fld) || (!$id)) {
          $this->addError('Malformed object data structure! (in SC::save())',
            LFX_DV_BOGDAN+31, LFX_ET_MAJOR+LFX_EL_SC);
          return(false);
        }
        $sql="UPDATE $table SET $query WHERE $id_fld=\"$id\"";
        //echo("Running SAVE query: <br />$sql<br />\n");
        if ($this->query($sql)) {
          // This bit is duplicated below in the elseif, take note!
          $this->log('update');
          $this->_saveFiles($fileData);
          $this->onSave(false);
          return($this->id);
        } else {
          return(false);
        }
      } elseif ($saveFiles) {
        $this->log('update');
        $this->_saveFiles($fileData);
        $this->onSave(false);
        return($this->id);
      } else {
        return($this->id);
      }
    }

      /**
      * Saves the current object to the database by performing an
      * INSERT while specifying the id. You should only need this
      * method for de-archiving operations, otherwise method SC::{@link save}()
      * is what you typically need in day to day operations.
      *
      * Please note that this method does not call beforeSave or afterSave,
      * since this is supposed to be old, "frozen" data which doesn't need to
      * go through those operations again. Also please note that this
      * method disregards the "modified" flag, so you don't have to execute
      * {@link touchAllAttrs}() prior to calling it.
      *
      * @param integer $id the id to use for insert
      * @param boolean $triggerEvents if true, the {@link beforeSave} and
      *   {@link onSave} events will be triggered and their results will be used
      * @return boolean true on success, false on error or NULL if $triggerEvents
      *   evaluates true and {@link beforeSave} returns false
      */
    function insertWithId($id=0, $triggerEvents=false)
    {
      $id=$id?$id:$this->id;

      if (!$id) {
        $this->addError('No id in SC::insertWithId()!');
        return(false);
      }

      if ($triggerEvents && (!$this->beforeSave(true,$id,false))) {
        return(NULL);
      }
      
      // Preparing the file fields first, if any
      $fileData=$fileDataFields=array();
      $fields=$this->dataStructure['fields'];
      while (list($attName,$dataEntry)=@each($fields)) {
        if (
          $dataEntry['flags']['file'] ||
          (
            $dataEntry['flags']['short_in_db'] &&
            strlen($this->attr[$attName])>=$this->short_in_db_limit
          )
        ) {
          $fileData[$attName]=$this->attr[$attName];
          $fileDataFields[$attName]=true;
          unset($this->attr[$attName]);
          unset($this->attr_flags[$attName]['modified']);
        } elseif ($dataEntry['flags']['short_in_db']) {
          // We prepend a space to these entries in order to ensure that
          // the resulting values can't be interpreted as integers when
          // they're retrieved from the database, even if the field itself
          // happens to only contain an integer value.
          $this->attr[$attName]=' '.$this->attr[$attName];
        }
      }
      $query=$this->_makeSaveQuery(true);
      @reset($fields);
      while (list($attName,$dataEntry)=@each($fields)) {
        // Can't use $fileData reliably on the 2nd condition below because
        // the value can be 0 or NULL or empty string, although the field
        // HAS been modified.
        if (($dataEntry['flags']['file'] || $dataEntry['flags']['short_in_db']) && $fileDataFields[$attName]) {
          $this->attr[$attName]=$fileData[$attName];
        } elseif ($dataEntry['flags']['short_in_db']) {
          $this->attr[$attName]=substr($this->attr[$attName],1);
        }
      }
      $id=addslashes($id);
      $id_fld=$this->dataStructure['id_field'];
      $table=$this->dataStructure['table_name'];
      if ((!$table) || (!$id_fld) || (!$id)) {
        $this->addError('Malformed object data structure! (in SC::save())',
          LFX_DV_BOGDAN+31, LFX_ET_MAJOR+LFX_EL_SC);
        return(false);
      }
      $sql="INSERT INTO $table SET $query, $id_fld=\"$id\"";
      //echo("Running INSERT WITH ID query: <br />$sql<br />\n");
      if ($this->query($sql)) {
        $this->_saveFiles($fileData);
        $this->onSave(true);
        return(true);
      } else {
        return(false);
      }
    }

      /**
      * You shouldn't call this directly
      *
      * This is called by {@link save} if no id is specified
      * AND if $this->id is not set.
      * You REALLY don't need to call this; should you ever
      * need to however, please note it accepts no parameters.
      * @return boolean true on success or false on error
      */
    function insert()
    {
      if ($this->beforeSave(true, $id, $force)!==true) {
        return(NULL);
      }
      $table=$this->dataStructure['table_name'];
      if (!$table) {
        $this->addError('Malformed object data structure - no table name! (in SC::insert())',
          LFX_DV_BOGDAN+67,LFX_ET_MAJOR+LFX_EL_SC);
        return(false);
      }

      // Preparing the file fields first, if any
      $fileData=$fileDataFields=array();
      $fields=$this->dataStructure['fields'];
      while (list($attName,$dataEntry)=@each($fields)) {
        if (
          $dataEntry['flags']['file'] ||
          (
            $dataEntry['flags']['short_in_db'] &&
            strlen($this->attr[$attName])>=$this->short_in_db_limit
          )
        ) {
          $fileData[$attName]=$this->getAttr($attName);
          $fileDataFields[$attName]=true;
          //unset($this->attr[$attName]);
          $this->attr_flags[$attName]['modified']=false;
        } elseif ($dataEntry['flags']['short_in_db']) {
          // We prepend a space to these entries in order to ensure that
          // the resulting values can't be interpreted as integers when
          // they're retrieved from the database, even if the field itself
          // happens to only contain an integer value.
          $this->attr[$attName]='_'.$this->attr[$attName];
        }
      }

      $query=$this->_makeSaveQuery(); // We DON'T want to force specifying all fields on inserts

      @reset($fields);
      while (list($attName,$dataEntry)=@each($fields)) {
        // Can't use $fileData reliably on the 2nd condition below because
        // the value can be 0 or NULL or empty string, although the field
        // HAS been modified.
        if (($dataEntry['flags']['file'] || $dataEntry['flags']['short_in_db']) && $fileDataFields[$attName]) {
          $this->attr[$attName]=$fileData[$attName];
        } elseif ($dataEntry['flags']['short_in_db']) {
          $this->attr[$attName]=substr($this->attr[$attName],1);
        }
      }

      if (!$query) {
        $this->addError("Malformed object data structure - no fields!
          (in SC::insert())",LFX_DV_BOGDAN+68,LFX_ET_MAJOR+LFX_EL_SC);
        return(false);
      }
      // $query="$id_fld=\"$id\", ".$query;
      // This entire complication makes the insert pretty expensive, but it's
      // the price we need to pay in order to ensure consistency.
      if ($this->query("INSERT INTO $table SET $query")) {
        $this->id=mysql_insert_id();
        if (!$this->id || !$this->probe()) {
          $this->query("SELECT LAST_INSERT_ID()");
          if (!$this->db->next_record() || !($this->id=$this->db->f(0)) || !$this->probe()) {
            $this->addError('Unable to get mysql_insert_id() when inserting!',
              LFX_DV_BOGDAN+61, LFX_ET_MAJOR+LFX_EL_SQL);
          }
        }
        // $this->id=$id;

        $this->log('insert');
        $this->_saveFiles($fileData);

        $this->onSave(true);
        return($this->id);
      } else {
        return(false);
      }
    }

      /**
      * This method gets to run BEFORE even trying to delete object with id $id.
      *
      * Please note that's no guarantee that the delete will actually succeed,
      * so make sure you correlate whatever you do here with the onDelete() call
      * which gets executed after a successful removal.
      * Should be overridden by SC descendants.
      * @return boolean true to proceed deleting or false to abort
      */
    function beforeDelete($id)
    {
      return(true);
    }

      /**
      * This method gets executed immediately AFTER successfully deleting
      * object with id $id and all its dependencies.
      * Should be overridden by SC descendants.
      * Since you won't have much info about that object and its dependencies
      * after they've been deleted, you may also want to overwrite
      * beforeDelete().
      */
    function onDelete($id)
    {
    }

      /**
      * Similar to SC::{@link getAllObjects}(), this one returns the same list
      * recursively.
      *
      * Please note that this method properly retrieves all dependencies,
      * including files, forums and subscriptions for WIFE dependencies.
      *
      * @param string $depType must be one of the dependency types (e.g. 'WIFE')
      * @return indexed array of associative array of the following form:
      *   'dep'    => the dependency name
      *   'object' => the instantiated object
      */
    function getExhaustiveObjects($depType='WIFE',$excludeList=array())
    {
      $finalList=array();
      $objectList=$this->getAllObjects($depType);
      for($i=0;$i<count($objectList);$i++) {
        $excludeItem=get_class($objectList[$i]['object']).'_'.$objectList[$i]['object']->id;
        if (!in_array($excludeItem,$excludeList)) {
          $finalList[]=$objectList[$i];
          $excludeList[]=$excludeItem;
        }
      }
      for($j=0;$j<count($finalList);$j++) {
        $newList=$finalList[$j]['object']->getExhaustiveObjects($depType,$excludeList);
        for($i=0;$i<count($newList);$i++) {
          $excludeItem=get_class($newList[$i]['object']).'_'.$newList[$i]['object']->id;
          if (!in_array($excludeItem,$excludeList)) {
            $finalList[]=$newList[$i];
            $excludeList[]=$excludeItem;
          }
        }
      }
      return($finalList);
    }

      /**
      * This method returns all objects from all dependencies associated
      * with this object. This method is NOT recursive, it will only return
      * *this* object's dependencies.
      *
      * Please note that this method properly retrieves all dependencies,
      * including files, forums and subscriptions for WIFE dependencies.
      *
      * @param string $depType must be one of the dependency types (e.g. 'WIFE')
      * @return indexed array of associative array of the following form:
      *   'dep'    => the dependency name
      *   'object' => the instantiated object
      */
    function getAllObjects($depType='WIFE')
    {
      $myClass=get_class($this);
      $objects=array();
      $deps=$this->dataStructure['depend'];
      @reset($deps);
      while (list($depName,$dep)=@each($deps)) {
        if ($dep['on_mod']!=$depType) {
          continue;
        }
        if ($tmp=$this->getObjects($depName)) {
          for($i=0;$i<count($tmp);$i++) {
            $objects[]=array('dep'=>$depName,'object'=>$tmp[$i]);
          }
        } elseif ($tmp===false) {
          return(false);
        }
      }
      if ($depType=='WIFE') {
        // Ok, let's retrieve those extras too:
        // First, the subscriptions:
        if (($myClass!='lfx_subscription') && ($myClass!='lfx_subscriber')) {
          if (!LFX_include_class('LFX_Subscription')) {
            $this->addError('Failed to include class LFX_Subscription!');
            return(false);
          }
          $sub=new LFX_Subscription;
          if ($subs=$sub->search(array('objectClass','objectId'),array($myClass,$this->id))) {
            for($i=0;$i<count($subs);$i++) {
              $objects[]=array('dep'=>'lfx_subscription','object'=>$subs[$i]);
            }
          }
        }
        // The files
        if (($myClass!='lfx_file') && !is_subclass_of($this,'lfx_file')) {
          $files=$this->myFiles();
          for($i=0;$i<count($files);$i++) {
            $objects[]=array('dep'=>'lfx_file','object'=>$files[$i]);
          }
        }
        // The forum
        if (($myClass!='lfx_forum') && ($myClass!='lfx_forum_message')) {
          if (($forum=$this->myForum()) && $forum->id) {
            $objects[]=array('dep'=>'lfx_forum','object'=>$forum);
          }
        }

        // The logs
        if ($myClass!='lfx_log') {
          LFX_include_class('lfx_log');
          $log=new LFX_Log;
          $logs=$log->search(array('class','object_id'),array($myClass,$this->id));
          for($i=0;$i<count($logs);$i++) {
            $objects[]=array('dep'=>'lfx_log','object'=>$logs[$i]);
          }
        }
      }
      return($objects);
    }

      /**
      * This method returns all log objects associated with this object
      * @param integer $id the id of the object to retrieve the logs for
      *   (this object by default)
      * @return mixed indexed array of LFX_Log objects or false on error
      */
    function myLogs($id=0)
    {
      $object=&$this->defaultObject($id);
      if (!LFX_include_class('lfx_log')) {
        $this->addError('Failed to include class LFX_Log!');
        return(false);
      }
      $log=new LFX_Log;
      $logs=$log->search(array('class','object_id'),array(get_class($object),$object->id),'entry_date');
      return($logs);
    }

      /**
      * Delete object with specified id, or current object if no id specified
      *
      * Also deletes object dependencies, as specified by $this->dataStructure
      * @param integer $id if set, the object with the respective id will be deleted
      * @param string $purpose may be one of
      *   'regular' => a regular delete is performed
      *   'archive' => an archiving delete is performed, which means two things:
      *     1. beforeDelete() and afterDelete() are not called
      *     2. the logs are also deleted
      * @param array $stack internal variable to keep track of dependencies and avoid cycles
      * @return boolean true on success, false on error or
      *   NULL if {@link beforeSave} aborts deleting.
      */
    function delete($id=0, $purpose='regular', $stack=array())
    {
      $victim=&$this->defaultObject($id);
      $tmp=get_class($this).'#'.$victim->id;
      $stack[]=$tmp;
      // We're inheriting the noLogging property from the current object to
      // the victim... (see (1) below)
      $victim->noLogging=$this->noLogging;
      $id=$victim->id;
      if (!$id) {
        $victim->addError('No ID specified to delete! (in SC::delete())',
          LFX_DV_BOGDAN+69, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      if (($purpose!='archive') && ($victim->beforeDelete($id)!==true)) {
        return(NULL);
      }

      // Managing dependencies
      $mistresses=$victim->getAllObjects('MISTRESS');
      for($i=0;$i<count($mistresses);$i++) {
        $this->dropLink($mistresses[$i]['dep'],$mistresses[$i]['object']);
      }
      $harem=$victim->getAllObjects();
      for($i=0;$i<count($harem);$i++) {
        $harem[$i]['object']->noLogging=$victim->noLogging;
        $tmp=get_class($harem[$i]['object']).'#'.$harem[$i]['object']->id;
        if (@in_array($tmp, $stack)) {
          $victim->addError('Cyclic dependency! '.
            'Object class: '.niceClass($harem[$i]['object'])."; ".
            'Object id: '.$harem[$i]['object']->id.' (in SC::delete())',
            LFX_DV_BOGDAN+65, LFX_ET_MAJOR+LFX_EL_SC);
          return(false);
        }
        if (($purpose=='archive') || ($harem[$i]['dep']!='lfx_log')) {
          if (!$harem[$i]['object']->delete(0, $purpose, $stack)) {
            return(false);
          } else {
            $stack[]=$tmp;
          }
        }
      }

      /*
      $deps=$victim->dataStructure['depend'];
      @reset($deps);
      while (list($depName,$dep)=@each($deps)) {
        if ($dep['on_mod']=='STRANGER') {
          continue;
        }
        $objects=$victim->getObjects($depName);
        while (list(,$object)=@each($objects)) {
          // (1) ...and from the victim to its dependencies
          $object->noLogging=$victim->noLogging;
          $tmp=array(
            'class'=>get_class($object),
            'id'=>$object->id
          );
          if (@in_array($tmp, $stack)) {
            $victim->addError('Cyclic dependency! '.
              "Last dependency checked: $dep; ".
              'Object class: '.$tmp['class']."; ".
              'Object id: '.$tmp['id'].' (in SC::delete())',
              LFX_DV_BOGDAN+65, LFX_ET_MAJOR+LFX_EL_SC);
            return(false);
          }
          $stack[]=$tmp;
          if ($dep['on_mod']=='MISTRESS') {
            if ($dep['type']=='many') {
              $victim->dropLink($depName, $object->id);
            } else {
              $object->quickLoad();
              $object->setAttr($dep['attr'],0);
              $object->save();
            }
          } elseif ($dep['on_mod']=='WIFE') {
            if (!($objDel=$object->delete($object->id, $stack))) {
              return($objDel);
            }
            if ($dep['type']=='many') {
              $victim->dropLink($depName, $object->id);
            }
          } else {
            $victim->addError("Unknown dependency type [{$dep['type']}] (in SC::delete())",
              LFX_DV_BOGDAN+102, LFX_ET_MAJOR+LFX_EL_SC);
            return(false);
          }
        }
      }

      // Deleting subscriptions, if the victim is not a subscription
      // or subscriber itself :)
      $victimClass=get_class($victim);
      if (($victimClass!='lfx_subscription') && ($victimClass!='lfx_subscriber')) {
        if (!LFX_include('LFX_Subscription')) {
          $this->addError('Unable to delete subscriptions due to include error');
          return(false);
        }
        $sub=new LFX_Subscription;
        if ($subs=$sub->search(array('objectClass','objectId'),array($victimClass,$id))) {
          for($i=0;$i<count($subs);$i++) {
            if (!($objDel=$subs[$i]->delete())) {
              return($objDel);
            }
          }
        }
      }
      */

      // Deleting files, if any
      $fields=$this->dataStructure['fields'];
      while (list($attName,$dataEntry)=@each($fields)) {
        if
        (
          (
            $dataEntry['flags']['file'] ||
            $dataEntry['flags']['short_in_db']
          ) &&
          // Leave this for last, due to PHP's lazy evaluation it's more
          // economical!
          is_file($fileName=$this->_getDataFile($attName))
        ) {
          if (!@unlink($fileName)) {
            $this->addError("Unable to delete file $fileName in order to delete ".
              "this object! LEAVING OBJECT IN DATABASE FOR CONSISTENCY! ".
              "Please note dependencies may already have been deleted at this point!");
            return(false);
          }
        }
      }
      $id_fld=$victim->dataStructure['id_field'];
      $table=$victim->dataStructure['table_name'];
      $victim->query("DELETE FROM $table WHERE ".$victim->dataStructure['id_field']."=\"$id\"");
      if ($purpose!='archive') {
        $victim->log('delete');
        $victim->onDelete($id);
      }
      return(true);
    }

      /**
      * This method stores a copy of this object for later comparison.
      * Use {@link thawStatus} to get the copy at a later time.
      * @param string $key a key with which this copy will be associated to
      * @return void
      */
    function freezeStatus($key)
    {
      // Make sure we make a copy here -- apparently in PHP 5 we'll need to
      // run a specific function to get a copy.
      $this->frozenCopies[$key]=$this;
      return(true);
    }

      /**
      * This method thaws a previously stored copy of this object.
      * Use {@link freezeStatus} to make the copy, and pass the same key
      * later on to thawStatus().
      * @param string $key a key associated with the copy to restore
      * @return object the instantiate object representing the copy, false
      *   on error or NULL if there was no copy associated with this key
      */
    function thawStatus($key)
    {
      return($this->frozenCopies[$key]);
    }

      /**
      * Returns an attribute as object
      *
      * For attributes which are defined as links in $this->dataStructure,
      * this method returns an instantiated object representing the respective
      * attribute.
      * @param string $att_name the attribute associated with the object to return
      * @return object the instantiated object representing the attribute or false on error
      */
    function getObject($attName)
    {
      $obj_type=$this->dataStructure['fields'][$attName]['link_class'];
      if (!$obj_type) {
        if ($flex=$this->getFlexObject($attName)) {
          return($flex);
        }
        $this->addError("Link $attName not available in attribute list! (in SC::getObject())",
          LFX_DV_BOGDAN+70, LFX_ET_ERROR+LFX_EL+SC);
        return(false);
      }
      $obj_id=$this->getAttr($attName);
      if (!$obj_id) {
        return(NULL);
      }
      //echo("Instantiating an $obj_type");
      LFX_include_class($obj_type);
      $tmp=new $obj_type;
      $tmp->id=$obj_id;
      return($tmp);
    }

      /**
      * This method returns an instantiated object based on the value of the specified attribute.
      * The attribute MUST contain an object specifier of the form "<object class>#<object id>"
      * (e.g. "DMO_Instance#311").
      *
      * @param string $attName the name of the attribute to retrieve and use
      * @return mixed the instantiated object on success, NULL if object not available or false on error.
      */
    function getFlexObject($attName)
    {
      $att_val=$this->getAttr($attName);
      if (!$att_val) {
        return(NULL);
      }
      if (!preg_match("/^([a-zA-Z_0-9]+)#([0-9]+)$/",$att_val,$matches)) {
        return(NULL);
      }
      if (!LFX_include_class($matches[1])) {
        return(false);
      }
      return(new $matches[1]($matches[2]));
    }
    
      /**
      * Returns the specified dependency list as object ID's
      *
      * @param string $dep_name the dependency to return the list of
      * @param string $order_att the attribute to sort the list by
      * @param boolean $reverse if true, the list will be reversed
      * @return array array of instantiated objects or false on error
      */
    function getLinks($dep_name, $order_att=NULL, $reverse=false)
    {
      if (!$this->id) {
        $this->addError('Dependency search request on object with ID not set! (in SC::getLinks())',
          LFX_DV_BOGDAN+66, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      $dep=$this->dataStructure['depend'][$dep_name];
      if (!$dep) {
        $this->addError("Dependency search request on object which doesn't have dependency defined! (in SC::getLinks())",
          LFX_DV_BOGDAN+71, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      if (($dep['type']=='one') || (!$dep['type'])) {
        if (!LFX_include_class($dep['class'])) {
          $this->addError('Failed to include class '.$dep['class']);
          return(false);
        }
        $tmp=new $dep['class'];
        $ids=$tmp->searchId($dep['attr'], $this->id, $order_att, $reverse);
        if ($ids===false) {
          $this->debug('SC::getLinks() called SC::search() which failed');
        }
        return($ids);
      } elseif ($dep['type']=='many') {
        // Better error management required below - no checking right now!
        if (!$dep['dbData']) {
          $dep['dbData']=$this->dbData;
        }
        $tmObject=new LFX_Object();
        $tmObject->dbData=$dep['dbData'];
        $myId=addslashes($this->id);
        $link_fld=$dep['link_fld_name'];
        if (!$link_fld) {
          $this->addError("SC::getLinks() called but object doesn't have link_fld_name defined!",
            LFX_DV_BOGDAN+100, LFX_ET_MAJOR+LFX_EL_SC);
          return(false);
        }
        $my_fld=$dep['my_fld_name'];
        if (!$my_fld) {
          $this->addError("SC::getLinks() called but object doesn't have my_fld_name defined!",
            LFX_DV_BOGDAN+101, LFX_ET_MAJOR+LFX_EL_SC);
          return(false);
        }
        // No ordering here because we didn't need it yet! When we will, we'll probably have to
        // perform a LEFT JOIN on the object's table. That's going to be a little
        // work because we'll have to also include the "other" object's file in order
        // to properly determine the proper field names, etc.
        $tmObject->query("SELECT `$link_fld` FROM `{$dep['table_name']}` WHERE `$my_fld`=\"$myId\"");
        $tmDb=&$tmObject->db;

        $tmp_array=array();
        while ($tmDb->next_record()) {
          $tmp_array[]=$tmDb->f(0);
        }
        return($tmp_array);
      } else {
        $this->addError('Unknown dependency type ('.($dep['type']).') defined for dependency '.$dep_name.' (in SC::getLinks())',
          LFX_DV_BOGDAN+72, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
    }

      /**
      * Returns the specified dependency list as objects
      *
      * This method uses SC::getLinks() to retrieve the object ID's and
      * SC::instantiate() to actually instantiate them.
      *
      * @param string $dep_name the dependency to return the list of
      * @param string $order_att the attribute to sort the list by
      * @param boolean $reverse if true, the list will be reversed
      * @return array array of instantiated objects or false on error
      */
    function getObjects($dep_name, $order_att=NULL, $reverse=false)
    {
      $class=$this->dataStructure['depend'][$dep_name]['class'];
      if (!$class) {
        $className=get_class($this);
        if (function_exists('niceclass')) {
          $className=niceClass($className);
        }
        $this->addError("Dependency '$dep_name' search request on object '$className' which doesn't have dependency [class] defined in structure! (in SC::getObjects())");
        return(false);
      }
      if (!LFX_include_class($class)) {
        $this->addError("Failed to include class $class for instantiation (in SC::getObjects())");
        return(false);
      }
      $links=$this->getLinks($dep_name, $order_att, $reverse);
      if ($links===false) {
        $this->addError('SC::getObjects() called SC::getLinks() which failed');
        return($links);
      }
      $instantiated=$this->instantiate($links,$class);
      return($instantiated);
    }

      /**
      * Returns the specified dependency list as objects, and also loads
      * them. This method should be typically used instead of {@link getObjects},
      * unless you really only need the instantiated objects with ID's
      * (e.g. if you only need to walk a tree).
      * This method is considerably faster than doing a getObjects() followed
      * by load() or getAttr() on each object, because this executes a single
      * query, as opposed to getObjects() + load() which executes as many
      * queries as there are objects to load, plus one (for getObjects()).
      *
      * This method uses SC::getLinks() to retrieve the object ID's and
      * SC::instantiate() to actually instantiate them.
      *
      * @param string $dep_name the dependency to return the list of
      * @param string $order_att the attribute to sort the list by
      * @param boolean $reverse if true, the list will be reversed
      * @return array array of instantiated objects or false on error
      */
    function getLoadedObjects($dep_name, $order_att=NULL, $reverse=false)
    {
      $class=$this->dataStructure['depend'][$dep_name]['class'];
      if (!$class) {
        $this->addError("Dependency search request on object which doesn't have dependency [class] defined in structure! (in SC::getObjects())");
        return(false);
      }
      if (!LFX_include_class($class)) {
        $this->addError("Failed to include class $class for instantiation (in SC::getObjects())");
        return(false);
      }
      $links=$this->getLinks($dep_name, $order_att, $reverse);
      if ($links===false) {
        $this->addError('SC::getObjects() called SC::getLinks() which failed');
        return($links);
      }
      if (!$links) {
        return(array());
      }
      $tmp=new $class;
      return($tmp->_doLoad($links));
    }

      /**
      * Returns the object specified by the respective id, or a reference
      * to the current object if $id==0. We're using this convention so
      * often within SC descendats that a method to do it was called for.
      * Make sure you also call this method assigning the value of its
      * result by reference: use "$def=&$this->defaultObject($id)" instead
      * of "$def=$this->defaultObject($id)".
      * @param integer id the id to use
      * @return reference a reference to the object with id $id or to $this if $id==0
      */
    function &defaultObject($id=0)
    {
      if (($id==0) || ($id==$this->id)) {
        $result=&$this;
      } else {
        $myClass=get_class($this);
        $result=new $myClass($id);
      }
      return($result);
    }

      /**
      * Sorts an array of instantiated descendants by an attribute.
      *
      * Given an array of instantiated SC descendants, this method will
      * return an indexed array with the elements ordered by the given attribute.
      * The returned array will NOT care about the original keys - it will
      * return an indexed array with ordered keys, and no holes.
      *
      * @param array $array the instantiated objects array to sort
      * @param array $attName the attribute to sort by
      * @param boolean $reverse if true, the array will be sorted in reverse
      * @return array an indexed array of instantiated objects or false on error
      */
    function arraySort($array, $attName, $reverse=false)
    {
      if (!is_array($array)) {
        $this->debug('SC::arraySort() called with the first argument not an array');
        return(false);
      }
      reset($array);
      $tmpArray=array();
      while (list($key,$obj)=each($array)) {
        $tmpArray[$key]=$obj->getAttr($attName);
      }
      if ($reverse) {
        arsort($tmpArray);
      } else {
        asort($tmpArray);
      }
      $result=array();
      while (list($key,)=each($tmpArray)) {
        $result[]=$array[$key];
      }
      return($result);
    }

      /**
      * Creates a link
      *
      * This creates a new entry in the join table for a many to many
      * dependency. See documentation on how to set up a many to many
      * dependency.
      * Also creates a link in a one to many dependency, if that is required.
      * Note: if link already exists, it will behave as if it just created it.
      * @param string $dep_name the many to many dependency to use
      * @param mixed $link the numeric id or instantiated object to link to
      * @return boolean true on success or false on error
      */
    function createLink($dep_name, $link)
    {
      if (!isset($this->dataStructure['depend'][$dep_name])) {
        $this->addError("Link creation requested for undefined dependency ($dep_name) (in SC::createLink())",
          LFX_DV_BOGDAN+73, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      $dep=$this->dataStructure['depend'][$dep_name];
      if (!$this->id) {
        $this->addError('Link creation requested on object with id not set! (in SC::createLink())',
          LFX_DV_BOGDAN+75, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      if (is_numeric($link)) {
        $link_id=$link;
        $link_class=$dep['class'];
      } else {
        $link_class=get_class($link);
        $link_base=$this->getBaseClass($link);
        if ($link_base!='lfx_object') {
          $this->addError("Link creation requested to a $link_class object - but that's not a descendant of LFX_Object (it's a descendant of $link_base) (in SC::createLink())",
            LFX_DV_BOGDAN+76, LFX_ET_ERROR+LFX_EL_SC);
          return(false);
        }
        if ($link_class!=strtolower($dep['class'])) {
          $this->addError("Link creation requested to a $link_class object - but the dependency $dep_name refers to {$dep['class']} objects! (in SC::createLink())",
            LFX_DV_BOGDAN+77, LFX_ET_ERROR+LFX_EL_SC);
          return(false);
        }
        $link_id=$link->id;
      }
      if (!$link_id) {
        $this->addError("Link creation requested but couldn't determine link id! (in SC::createLink())",
          LFX_DV_BOGDAN+78, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // Finally finished checking for errors!
      if ($dep['type']=='many') {
        // [must also check for database problems since we're using direct database calls!]
        if (!$dep['dbData']) {
          $dep['dbData']=$this->dbData;
        }
        $db=$this->_doDbInit($dep['dbData']);
        $join_table=$dep['table_name'];
        $Smy_id=addslashes($this->id);
        $Slink_id=addslashes($link_id);
        $my_fld=$dep['my_fld_name'];
        $link_fld=$dep['link_fld_name'];
        $db->query("SELECT COUNT(*) FROM `$join_table` WHERE `$my_fld`=\"$Smy_id\" AND `$link_fld`=\"$Slink_id\"");
        $db->next_record();
        if ($db->f(0)) {
          // If link already exists, we will NOT issue any error - just exit as if we created it
          // If you need to know if that link exists, just check for it yourself!
          return(true);
        }
        if ($db->query("INSERT INTO $join_table SET $my_fld=\"$Smy_id\", $link_fld=\"$Slink_id\"")) {
          return(true);
        } else {
          return(false);
        }
      } else {
        if (!LFX_include_class($link_class)) {
          $this->addError('Failed to load required class '.$link_class);
          return(false);
        }
        $depObject=new $link_class($link_id);
        if ($depObject->getAttr($dep['attr'])!=$this->id) {
          $depObject->setAttr($dep['attr'],$this->id);
          return($depObject->save());
        } else {
          return(true);
        }
      }
    }

      /**
      * Drops (deletes) a many to many dependency
      *
      * Note: if link doesn't exist, it will behave as if it just deleted it.
      * See {@link createLink} for how to create one.
      * @param string $dep_name the dependency to use
      * @param mixed $link numerical id or instantiated object to drop the link to
      * @return boolean true on success or false on error
      */
    function dropLink($dep_name, $link)
    {
      if (!isset($this->dataStructure['depend'][$dep_name])) {
        $this->addError("Link drop requested for undefined dependency ($dep_name) (in SC::dropLink())",
          LFX_DV_BOGDAN+79, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      $dep=$this->dataStructure['depend'][$dep_name];
      if (!$this->id) {
        $this->addError('Link drop requested on object with id not set! (in SC::dropLink())',
          LFX_DV_BOGDAN+81, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      if (is_numeric($link)) {
        $link_id=$link;
        $link_class=$dep['class'];
      } else {
        $link_class=get_class($link);
        $link_base=$this->getBaseClass($link);
        if ($link_base!='lfx_object') {
          $this->addError("Link drop requested to a $link_class object - but that's not a descendant of LFX_Object (it's a descendant of $link_base) (in SC::dropLink())",
            LFX_DV_BOGDAN+82, LFX_ET_ERROR+LFX_EL_SC);
          return(false);
        }
        if ($link_class!=strtolower($dep['class'])) {
          $this->addError("Link drop requested to a $link_class object - but the dependency $dep_name refers to {$dep['class']} objects! (in SC::dropLink())",
            LFX_DV_BOGDAN+83, LFX_ET_ERROR+LFX_EL_SC);
          return(false);
        }
        $link_id=$link->id;
      }
      if (!$link_id) {
        $this->addError("Link drop requested but couldn't determine link id! (in SC::dropLink())",
          LFX_DV_BOGDAN+84, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // Finally finished checking for errors!
      if ($dep['type']=='many') {
        if (!$dep['dbData']) {
          $dep['dbData']=$this->dbData;
        }
        $db=$this->_doDbInit($dep['dbData']);
        $join_table=$dep['table_name'];
        $Smy_id=addslashes($this->id);
        $Slink_id=addslashes($link_id);
        $my_fld=$dep['my_fld_name'];
        $link_fld=$dep['link_fld_name'];
        if ($db->query("DELETE FROM $join_table WHERE $my_fld=\"$Smy_id\" AND $link_fld=\"$Slink_id\"")) {
          return(true);
        } else {
          return(false);
        }
      } else {
        if (!LFX_include_class($link_class)) {
          $this->addError('Failed to load required class '.$link_class);
          return(false);
        }
        $depObject=new $link_class($link_id);
        if ($depObject->getAttr($dep['attr'])!=0) {
          $depObject->setAttr($dep['attr'],0);
          // We don't want to log such BS
          $depObject->noLogging=true;
          return($depObject->save());
        } else {
          return(true);
        }
      }
    }

      /**
      * Drops ALL many-to-many links of this object associated with the
      * specified dependency.
      * No information on the number of deleted links is returned;
      * even if no links are deleted, this method will return success
      * if it suceeded in performing the query, doesn't care about the
      * actual deleted links. This is done for performance reasons;
      * should you need that information use SC::getObjects() or
      * SC::getLinks() and SC::dropLink().
      *
      * @param string $dep_name the dependency to delete the links for
      * @return boolean true on success or false on failure
      */
    function dropLinks($dep_name)
    {
      if (!isset($this->dataStructure['depend'][$dep_name])) {
        $this->addError("Link drop requested for undefined dependency ($dep_name) (in SC::dropLinks())",
          LFX_DV_BOGDAN+259, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      $dep=$this->dataStructure['depend'][$dep_name];
      if ($dep['type']!='many') {
        $this->addError("Link drop requested on one to one dependency ($dep_name) - use attributes for that! (in SC::dropLinks())",
          LFX_DV_BOGDAN+260, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      if (!$this->id) {
        $this->addError('Link drop requested on object with id not set! (in SC::dropLinks())',
          LFX_DV_BOGDAN+261, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // Finally finished checking for errors!
      if (!$dep['dbData']) {
        $dep['dbData']=$this->dbData;
      }
      $db=$this->_doDbInit($dep['dbData']);
      $join_table=$dep['table_name'];
      $Smy_id=addslashes($this->id);
      $my_fld=$dep['my_fld_name'];
      if ($db->query("DELETE FROM $join_table WHERE $my_fld=\"$Smy_id\"")) {
        return(true);
      } else {
        return(false);
      }
    }

      /**
      * Returns the base class of an object. That is, the lowest-level class
      * which may have been repeatedly inherited upon until getting to that
      * object's class.
      *
      * @param mixed $object The object if object, or the class name if string
      * @return string the base class
      */
    function getBaseClass($object)
    {
      if (is_object($object)) {
        $baseClass=get_class($object);
      } else {
        $baseClass=$object;
      }
      while ($myClass=get_parent_class($baseClass)) {
        $baseClass=$myClass;
      }
      return($baseClass);
    }

      /**
      * Returns the 'real' value of the requested attribute
      *
      * This will not attempt to do anything on the attribute but load
      * the object if the attribute is not set.
      * Note: See $this->dataStructure flags for how dates and NULL values
      * are managed.
      *
      * @param string $attName the attribute to return
      * @return string the value of the attribute or false on error
      */
    function getAttr($attName)
    {
      if (!$attName) {
        $this->addError("Please specify a valid attribute name! (in SC::getAttr())");
        return(false);
      }
      if ($this->attr_flags[$attName]['loaded'] || $this->attr_flags[$attName]['modified']) {
        //echo("Attribute $attName for object ".get_class($this)." #{$this->id} already loaded.<br />\n");
        return($this->attr[$attName]);
      } elseif (!$this->id) {
        return(NULL);
      } else {
        if ($this->dataStructure['fields'][$attName]['flags']['large']) {
          //echo("Attribute $attName for object ".get_class($this)." #{$this->id} is large, performing full load.<br />\n");
          $result=$this->load(0,false);
        } else {
          //echo("Attribute $attName for object ".get_class($this)." #{$this->id} is not large, performing quick load.<br />\n");
          $result=$this->quickLoad(0,false);
        }
        if ($result) {
          return($this->attr[$attName]);
        } else {
          $this->debug('SC::getAttr() called SC::load() or SC::quickLoad() which failed');
          return(false);
        }
      }
    }

      /**
      * Returns an array of attributes, each of the values being retrieved
      * using {@link getAttr}.
      *
      * The $attr_names parameter MUST be an array, and its format dictates
      * the format of the resulting array:
      * - If $attr_names is an indexed array then the values must contain
      *   the attribute names, and the resulting array will be an indexed
      *   array of the corresponding values, with the keys matching;
      * - If $attr_names is an associative array then the keys must contain
      *   the attribute names, and the resulting array will be the same
      *   associative array, with the values of the attributes filled in as
      *   the values of the array;
      * - If $attr_names is a mixed array, the result will be a mixed array
      *   as well, with corresponding keys as described above.
      *
      * @param array $attr_names the attributes to be returned
      * @return array an array containing the attribute values
      */
    function getAttrs($attr_names)
    {
      $result=array();
      while(list($key,$val)=@each($attr_names)) {
        if (is_numeric($key)) {
          $result[$key]=$this->getAttr($val);
        } else {
          $result[$key]=$this->getAttr[$key];
        }
      }
      return($result);
    }

      /**
      * Method to be overridden by descendants which want to provide customized
      * formats for attributes when shown in browsers.
      * This method is called by SC::getAttrH(), and if the result is different
      * from the actual value of the attribute then the value returned here is
      * returned from SC:getAttrH(). If this method returns the attribute as
      * returned by SC::getAttr() then SC::getAttrH() checks if the attribute is
      * flagged as an SQL date, and if that's the case then it returns the date
      * formatted for display. If all else fails, SC::getAttrH() returns the
      * attribute value unchanged, as returned by SC::getAttr().
      * By default, SC::getFormattedAttr() returns the same as SC::getAttr().
      * @param string att_name the attribute to be formatted
      * @return string the un/formatted attribute
      */
    function getFormattedAttr($attName)
    {
      return($this->getAttr($attName));
    }

      /**
      * Formats the unix timestamp for nice displaying.
      */
    function formatDate($date,$full=false)
    {
      if ($date<43200) {
        // That's 12 hours. The date is returned in UTC AFTER the current
        // machine's time zone is applied. Therefore, for a date of 0 (n/a)
        // $date will be -1*time zone.
        return(__L('n/a'));
      }
      global $_LFX_globals;
      if ($full || (LFX_getDisposition()!='browser') || $_LFX_globals->getSetting('full_dates',NULL,$_SESSION['LFX_project'])) {
        return(date("M j, Y G:i",$date));
      }
      if (date('G',$date)+date('i',$date)==0) {
        $hourFormat='';
      } else {
        $hourFormat='G:i';
      }
      if (date('Y')==date('Y',$date)) {
        if (date('z')==date('z',$date)) {
          if ($hourFormat) {
            return(date($hourFormat,$date));
          } else {
            return(__L('Today'));
          }
        } else {
          return(str_replace(' ','&nbsp;',date("M j $hourFormat",$date)));
        }
      } else {
        return(date('M j, Y',$date));
      }
      return(date('r',$date));
    }

      /**
      * Returns the required attribute more suitable for HTML
      *
      * Generally returns the same as {@link getAttr}, only with HTML entities.
      * For attributes flagged date and/or sqlDate, it will return
      * a properly formatted date.
      * Descendants may also override SC::getFormattedAttr() to implement their
      * own format to be returned by this method.
      * @param string att_name the attribute to format
      * @return string
      */
    function getAttrH($attName,$fullDate=false)
    {
      // Returns the attribute value in HTML format.
      // If it's a date, it formats as human-readable.
      $att=$this->getAttr($attName);
      if (($fAtt=$this->getFormattedAttr($attName))!=$att) {
        return($fAtt);
      }
      if (
        ($this->dataStructure['fields'][$attName]['flags']['sqlDate']) ||
        ($this->dataStructure['fields'][$attName]['flags']['date'])
      ) {
        return($this->formatDate($att,$fullDate));
      } else {
        return(htmlspecialchars($att,ENT_QUOTES));
      }
    }

      /**
      * Sets the value of the specified attribute - note $slashes!
      *
      * @param string $attName the attribute to assign a value to
      * @param mixed $attValue the value to assign it. Valid values
      *   are strings, numeric values and the PHP NULL
      * @param string $slashes 'gpc' for strings which came from a GPC transaction,
      *   'runtime' for strings which came from a runtime transaction and
      *   'force' for known-to-be-slashed strings. If specified, it will
      *   strip slashes before setting the attribute, if appropriate.
      * @return boolean true on success or false on error.
      */
    function setAttr($attName, $attValue, $slashes=0)
    {
      if ($slashes==='gpc') {
        $attValue=$this->gpcUnslash($attValue);
      } elseif ($slashes==='runtime') {
        $attValue=$this->runtimeUnslash($attValue);
      } elseif ($slashes==='force') {
        $attValue=stripslashes($attValue);
      }
      if (!isset($this->attr[$attName]) || ($this->attr[$attName]!==$attValue)) {
        $this->attr[$attName]=$attValue;
        $this->attr_flags[$attName]['modified']=true;
        return(true);
      }
      return(true);
    }
    
      /**
      * This method allows you to find out if an attribute has been modified
      * since the object has been loaded from the database. This only works
      * if you alter the value of attributes via {@link setAttr} and
      * {@link setAttrs}; if you make direct modifications in the data structure
      * then those changes won't be caught by this mechanism.
      *
      * @param string $attName the attribute name to return the modified status for
      * @return boolean true if modified, false or NULL otherwise
      */
    function isModified($attName)
    {
      return($this->attr_flags[$att_name]['modified']);
    }

      /**
      * This method touches one object attribute so it's saved at the next
      * object save. By default, SC only saves the attributes edited with
      * {@link setAttr}; this method fakes a change in the attribute value.
      * @param string $attName the name of the attribute to touch
      * @return boolean true on success.
      */
    function touchAttr($attName)
    {
      $this->attr_flags[$attName]['modified']=true;
      return(true);
    }

      /**
      * This method walks all attributes and touches them all. Please note that
      * you must make sure yourself that all attributes are loaded properly,
      * otherwise good values will be deleted from the database when saving.
      */
    function touchAllAttrs()
    {
      @reset($this->attr);
      while(list($key)=@each($this->attr)) {
        $this->touchAttr($key);
      }
      return(true);
    }


      /**
      * Sets a set of attributes of this object based on an
      * associative array.
      *
      * The functionality is simple: just pass an associative
      * array to this method where the keys represent SC attribute
      * names and those attributes will be set to the respective
      * values in the array.
      *
      * This method does *NOT* also save your object - it just sets all
      * the attributes in the array, just as if you'd set each at a
      * time using {@link setAttr}.
      *
      * @param array $attrs associative array containing the attribute
      *   names and values to be set
      * @param string $slashes the same meaning as for SC::{@link setAttr}()
      *   The setting here is applied for all attributes being set.
      * @return boolean true on success or false on error.
      */
    function setAttrs($attrs,$slashes=0)
    {
      if (!is_array($attrs)) {
        $this->addError('Value passed to SC::setAttrs() is not an array!',
          LFX_DV_BOGDAN+258,LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      while(list($attName,$attVal)=each($attrs)) {
        if (!$this->setAttr($attName,$attVal,$slashes)) {
          $this->debug('SC::setAttrs() called SC::setAttr() which failed');
          return(false);
        }
      }
      return(true);
    }

      /**
      * This method instantiates objects based on the parameters it receives.
      * Here are the possible parameter combinations:
      * $ids (read it as "ID's"):
      *   - integer: will return one object instantiated as defined by $class.
      *     Should you need an array of one object, pass an indexed array of one id.
      *   - array: will return an array of objects instantiated as
      *     defined by $class. Will observe keys.
      *   - anything else: false (error)
      * $class:
      *   - missing: will use the current object's class to instantiate (all) the object(s)
      *   - string: will use the specified class to instantiate (all) the object(s)
      *   - indexed array: will use one array entry for each element in $ids. If $ids is
      *     integer, will only use the first entry in $class. If count($ids) is smaller
      *     than count($class), will only use as many elements for $class as needed.
      *     If count($ids) is larger than count($class), will use $wrap to determine
      *     what to do. Please note this will NOT observe keys - it will expect $class
      *     to be an indexed array with no holes.
      *   - anything else: false (error)
      * $wrap:
      *   - missing or evaluating false: will use the last enty in $class for all
      *     remaining elements in $ids (see above)
      *   - evaluating true: will wrap $class around $ids (you know what I mean)
      * @param mixed $ids the ID('s) to instantiate
      * @param mixed $class the class to instantiate
      * @param boolean true to wrap, false to copy
      * @return array array of instantiated objects or false on error
      */
    function instantiate($ids, $class=NULL, $wrap=false)
    {
      if ($class===NULL) {
        $class=get_class($this);
      }
      if (is_numeric($ids)) {
        if (is_array($class)) {
          $class=$class[0];
        }
        $result=new $class($ids);
        return($result);
      }
      if (!is_array($ids)) {
        // Returning an empty array if this is incorrect
        return(array());
        /*
        $this->addError('First parameter should be an array! (in SC::instantiate())',
          LFX_DV_BOGDAN+63, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
        */
      }
      $idx=0;
      $result=array();
      while (list($key,$id)=each($ids)) {
        if (!is_array($class)) {
          $myClass=$class;
        } else {
          if ($idx==count($class)) {
            if ($wrap) {
              $idx=0;
            } else {
              $idx--;
            }
          }
          $myClass=$class[$idx];
          if (!$myClass) {
            $this->addError('Something wrong with the classes parameter passed to '.
              "SC::instantiate() at index $idx - skipping",
              LFX_DV_BOGDAN+64, LFX_ET_WARNING+LFX_EL_SC);
            continue;
          }
        }
        $result[$key]=new $myClass($id);
        $idx++;
      }
      return($result);
    }

      /**
      * Returns the number of results of a search.
      *
      * This method is in no way related to {@link search} except in parameters
      * identity. Apart from that, the results are not actually retrieved - only
      * an SQL COUNT(*) is issued in order to minimize response time. If you need
      * both the objects count and the objects themselves, it is much more preferable
      * to issue a {@link search} or {@link searchId} and count the resulting array
      * because no form of caching is performed if you first issue a searchCount() and
      * then a search().
      *
      * See the documentation of {@link search} for detailed parameter info
      * @return integer number of records on success or false on error
      */
    function searchCount($attName=NULL, $attValue=NULL, $like=false)
    {
      if ($attName===NULL) {
        $where='';
      } elseif (is_array($attName)) {
        if (!is_array($attValue)) {
          $this->addError("Parameter att_name is an array but att_value isn't in SC::searchId()",
            LFX_DV_BOGDAN+205, LFX_ET_ERROR+LFX_EL+SC);
          return(false);
        }
        if (count($attName)!=count($attValue)) {
          $this->addError('Array att_name has different number of elements from att_value in SC::searchId()',
            LFX_DV_BOGDAN+206, LFX_ET_ERROR+LFX_EL+SC);
          return(false);
        }
        $where='WHERE ';
        for ($i=0;$i<count($attName);$i++) {
          $field=$this->dataStructure['fields'][$attName[$i]]['fld_name'];
          if (!$field) {
            $this->addError("Attribute to search by ($attName) not found! (in SC::searchId())",
              LFX_DV_BOGDAN+207, LFX_ET_ERROR+LFX_EL_SC);
            return(false);
          }
          if ($attValue[$i]===NULL) {
            $where.="`$field` IS NULL AND ";
          } else {
            $Satt_value=addslashes($attValue[$i]);
            if($like==false) {
              $where.="`$field`=\"$Satt_value\" AND ";
            } else {
              $where.="`$field` LIKE \"$Satt_value\" AND ";
            }
          }
        }
        $where=substr($where,0,-5);
      } else {
        $field=$this->dataStructure['fields'][$attName]['fld_name'];
        if (!$field) {
          $this->addError("Attribute to search by ($attName) not found! (in SC::searchId())",
            LFX_DV_BOGDAN+84, LFX_ET_ERROR+LFX_EL_SC);
          return(false);
        }
        if (($attValue[$i]===NULL) && ($this->dataStructure['fields'][$attName[$i]]['flags']['NULL'])) {
          $where="WHERE `$field` IS NULL";
        } else {
          $attValue=addslashes($attValue);
          if($like==false){
            $where="WHERE `$field`=\"$attValue\"";
          } else {
            $where="WHERE `$field` LIKE \"$attValue\"";
          }
        }
      }
      $id_fld=$this->dataStructure['id_field'];
      $table=$this->dataStructure['table_name'];
      // To check for malformed dataStructure
      $sql="SELECT COUNT(*) FROM `$table` $where";
      $this->query($sql);
      $this->db->next_record();
      return($this->db->f(0));
    }

      /**
      * Returns the result of a search as object ID's
      *
      * See the documentation of {@link search} for detailed parameter info
      * @return array array of id's on success or false on error
      */
    function searchId($attName=NULL, $attValue=NULL, $order_att=NULL, $reverse=false, $like=false)
    {
      if ($attName===NULL) {
        $where='';
      } elseif (is_array($attName)) {
        if (!is_array($attValue)) {
          $this->addError("Parameter att_name is an array but att_value isn't in SC::searchId()",
            LFX_DV_BOGDAN+205, LFX_ET_ERROR+LFX_EL+SC);
          return(false);
        }
        if (count($attName)!=count($attValue)) {
          $this->addError('Array att_name has different number of elements from att_value in SC::searchId()',
            LFX_DV_BOGDAN+206, LFX_ET_ERROR+LFX_EL+SC);
          return(false);
        }
        $where='WHERE ';
        for ($i=0;$i<count($attName);$i++) {
          $field=$this->dataStructure['fields'][$attName[$i]]['fld_name'];
          if (!$field) {
            $this->addError("Attribute to search by ($attName[$i]) not found! (in SC::searchId())",
              LFX_DV_BOGDAN+207, LFX_ET_ERROR+LFX_EL_SC);
            return(false);
          }
          if (($attValue[$i]===NULL) && ($this->dataStructure['fields'][$attName[$i]]['flags']['NULL'])) {
            $where.="`$field` IS NULL AND ";
          } else {
            $Satt_value=addslashes($attValue[$i]);
            if($like==false) {
              $where.="`$field`=\"$Satt_value\" AND ";
            } else {
              $where.="`$field` LIKE \"$Satt_value\" AND ";
            }
          }
        }
        $where=substr($where,0,-5);
        //echo"$where check<br>";
      } else {
        $field=$this->dataStructure['fields'][$attName]['fld_name'];
        if (!$field) {
          $this->addError("Attribute to search by ($attName) not found! (in SC::searchId())",
            LFX_DV_BOGDAN+84, LFX_ET_ERROR+LFX_EL_SC);
          return(false);
        }
        if ($attValue===NULL) {
          $where="WHERE `$field` IS NULL";
        } else {
          $attValue=addslashes($attValue);
          if($like==false) {
            $where="WHERE `$field`=\"$attValue\"";
          } else {
            $where="WHERE `$field` LIKE \"$attValue\"";
          }
        }
        //echo"Where: $where<br>";
      }
      if ($order_att!==NULL) {
        if ($order_att!==0) {
          $orderby=$this->dataStructure['fields'][$order_att]['fld_name'];
          if (!$orderby) {
            @reset($this->dataStructure['fields']);
            $attList='';
            while(list($tmpAtt)=each($this->dataStructure['fields'])) {
              $attList.="$tmpAtt, ";
            }
            $attList=substr($attList,0,-2).'.';
            $this->addError("Attribute to order by ($order_att) not found! (in SC::searchId()). The valid attributes of this object are: $attList",
              LFX_DV_BOGDAN+85, LFX_ET_ERROR+LFX_EL_SC);
            return(false);
          }
          $orderby="ORDER BY $orderby";
        } else {
          $orderby='ORDER BY '.$this->dataStructure['id_field'];
        }
      }
      if ($orderby && $reverse) {
        $orderby.=' DESC';
      }
      $id_fld=$this->dataStructure['id_field'];
      $table=$this->dataStructure['table_name'];
      // To check for malformed dataStructure
      $sql="SELECT `$id_fld` FROM `$table` $where $orderby";
      //echo"$sql<br>";
      $this->query($sql);
      $tmp_array=array();
      while ($this->db->next_record()) {
        $tmp_array[]=$this->db->f(0);
      }
      return($tmp_array);
    }

      /**
      * Returns the result of a search as instantiated objects
      *
      * If att_name is a string then att_value must be a string.
      * If att_name is an array then att_value must be an array
      * with the same number of elements. If they are arrays, then
      * an AND search is performed for all the elements in the arrays.
      * For instance, if $attName=array('children', 'grandchildren')
      * and $attValue=array(33, 44) then search() will return all objects
      * with the *attribute* 'children' equal to 33 *AND* attribute
      * 'grandchildren' equal to 44. If both are arrays AND $like is true
      * then all comparisons are made with 'LIKE' instead of "=".
      *
      * Note: if no objects are found, it returns an empty array,
      * so make sure you use identity operators to check for errors.
      * Note: This method uses {@link searchId} to search and
      * {@link instantiate} to instantiate - it's a one-liner. Just
      * so you know.
      *
      * @param mixed $attName the attribute(s) to search for
      * @param mixed $attValue the value(s) to match the attribute to
      * @param string $order_att the attribute to sort the result by;
      *   0 in order to sort by id; NULL or unspecified to return unsorted
      * @param boolean $reverse if true, the result will be ordered in reverse
      * @param boolean $like if true, the attribute will be compared to
      *   $attValue via LIKE
      * @return array array of objects on success or false on error
      */
    function search($attName=NULL, $attValue=NULL, $order_att=NULL, $reverse=false, $like=false)
    {
      $ids=$this->searchId($attName, $attValue, $order_att, $reverse, $like);
      if ($ids===false) {
        $this->debug('SC::search() called SC::searchID() which failed');
        return(false);
      } elseif (!$ids) {
        return(array());
      }
      // Switched from instantiate() to _doLoad(), please revert on problems
      // $ins=$this->_doLoad($ids,false,true);
      // seems to be SLOWER somehow!
      $ins=$this->instantiate($ids);
      // And running onLoad for these
      for($i=0;$i<count($ins);$i++) {
        $ins[$i]->onLoad(false);
      }
      //$ins=$this->instantiate($ids);
      if ($ins===false) {
        $this->debug('SC::search() called SC::_doLoad which failed');
        return(false);
      }
      return($ins);
    }

      /**
      * Same as {@link search}, except it initializes the searchId with the
      * true value for $like
      *
      * Note: this will excecute SQL with the like statement, so it will be
      *       slower than the normal search method.
      *
      * --------
      * WARNING!
      * --------
      * This functionality is now available in {@link search} directly, so
      * this function is deprecated and will be removed in a future version
      * of this library (no later than June 2003)
      */
    function searchStr($attName=NULL, $attValue=NULL, $order_att=NULL, $reverse=false)
    {
      $ids=$this->searchId($attName, $attValue, $order_att, $reverse,true);
      if ($ids===false) {
        $this->debug('SC::search() called SC::searchID() which failed');
        return(false);
      }
      $ins=$this->instantiate($ids);
      if ($ins===false) {
        $this->debug('SC::search() called SC::instantiate which failed');
        return(false);
      }
      return($ins);
    }

    // ----------------------------------
    //        LISTING METHODS
    // ----------------------------------

    function getName()
    {
      if ($this->dataStructure['title_attr']) {
        if ($name=$this->getAttr($this->dataStructure['title_attr'])) {
          return($name);
        } else {
          return(__L('n/a'));
        }
      }
      return(false);
    }

      /**
      * Returns the HTML representation of this object - implemented by descendants
      *
      * PLEASE EITHER override this to return a valid HTML representation
      * OR use the method described in the second paragraph below for all your
      * objects.
      *
      * The HTML representation should be a string which you feel is reasonably
      * representative for this object; for instance it could be
      * $this->getAttrH('fname').' '.$this->getAttrH('lname') for a LFX_User, or
      * '#'.$this->id.' '.$this->getAttrH('title') for an OPT_Request, or simply
      * $this->getAttr('name') for a DMO_Instance.
      *
      * If you use a single attribute as representation of this object,
      * linke in the last example above,
      * then you can specify it as the "title_attr" entry in the data structure
      * array you pass in the class constructor; if you do that, then you can
      * skip overriding this method and it will simply return
      * $this->getAttrH(<the attribute you specified>).
      *
      * @return string HTML representation of this object.
      */
    function getNameH()
    {
      if ($this->dataStructure['title_attr']) {
        if ($name=$this->getAttrH($this->dataStructure['title_attr'])) {
          return($name);
        } else {
          return(__L('n/a'));
        }
      }
      return(false);
    }

      /**
      * Should return the inter-page URL to the current object's details page;
      * implemented by descendants.
      *
      * "Inter-page" URL means that the resulting string must contain
      * an URL stripped of its protocol and document root part, as in
      * "/opt/documents" instead of "https://opt.lanifex.com/opt/documents".
      *
      * The aim of this method is to allow SC::{@link getLinkH}() to build
      * its result using SC::{@link getNameH()} without having to override
      * getLinkH().
      *
      * You can avoid overriding this method and SC::{@link getFullUrlH}()
      * by providing an absolute sprintf-compatible path to the current object's page
      * starting from the document root in this object's "object_page" entry
      * in the dataStructure array
      * (e.g. $this->dataStructure["object_page"]="/projects/project_details.php?id=%i").
      *
      * See also SC::{@link getFullLinkH}()
      *
      * @return string the sprintf-compatible HTML link to the current object
      */
    function getUrlH()
    {
      if (!$this->id) {
        return('');
      }
      if ($local=$this->dataStructure['object_page']) {
        $global="_".$this->module;
        global $$global;
        $global=$$global;
        return($global['global']['paths']['url'].sprintf($local,$this->id));
      }
      return(false);
    }

      /**
      * Identical in every other way with SC::{@link getUrlH}(), this method
      * should return a fully-qualified URI to the current object's details
      * page, including the protocol and the host name.
      *
      * The aim of this method is to be used by SC::{@link getFullLinkH}()
      * in order to provide the proper full-fledged URI for use in e-mail
      * notifications.
      *
      * See SC::{@link getUrlH}() for more documentation
      *
      * @return string URI to the current object
      */
    function getFullUrlH()
    {
      if (!$this->id) {
        return('');
      }
      if ($local=$this->dataStructure['object_page']) {
        $global="_".$this->module;
        global $$global;
        $global=$$global;
        return($global['global']['paths']['fullUrl'].sprintf($local,$this->id));
      }
      return(false);
    }

      /**
      * Returns an HTML link to this object - implemented by descendants
      *
      * PLEASE EITHER override this to return a valid HTML link to the details
      * page for this object OR properly implement SC::{@link getNameH}() AND
      * SC::{getUrlH}() for all your classes!
      * (The second approach is recommended.)
      *
      * @return string HTML link to the details page
      * (e.g. "<a href='/home/httpd/EH-Master/<path>?id=33><Object human representation></a>
      * where <object human representation> would be the name of a sensor, the title of a document etc).
      */
    function getLinkH()
    {
      if (($name=$this->getNameH()) && ($url=$this->getUrlH())) {
        return(sprintf('<A HREF="%1$s">%2$s</A>',$url,$name));
      } elseif ($name) {
        // if I have a name, but I don't have an URL
        return($name);
      }
      $this->addError('Method getLinkH() not defined by LFX_Oject descendant! (in SC::getLinkH())',
       LFX_DV_BOGDAN+56, LFX_ET_INFO+LFX_EL_SC);
      return(false);
    }

      /**
      * Returns the full link to the current object's representation.
      * Please see the documentation for SC::{@link getFullUrlH}() for a
      * wider context and more information on this method.
      *
      * @return string
      */
    function getFullLinkH()
    {
      if (($name=$this->getNameH()) && ($url=$this->getFullUrlH())) {
        return(sprintf('<A HREF="%1$s">%2$s</A>',$url,$name));
      }
      $this->addError('Method getFullLinkH() not defined by LFX_Oject descendant! (in SC::getLinkH())',
       LFX_DV_BOGDAN+56, LFX_ET_INFO+LFX_EL_SC);
      return(false);
    }

      /**
      * Descendants should override this to return an HTML link to the list detailed by the dependency
      *
      * @return string HTML link
      */
    function getListLinkH($dep_name, $order_att=NULL, $reverse=false)
    {
      $this->addError('Method getListLinkH() not defined by LFX_Object descendant! (in SC::getListLinkH())',
      LFX_DV_BOGDAN+57, LFX_ET_INFO+LFX_EL_SC);
      return(false);
    }

      /**
      * Descendats should override this to manage attribute representation in the list
      *
      * @param string $attName the name of the attribute to represent
      * @param string $attVal the value as it would be shown
      * @return string HTML to represent this attribute
      */
    function prepareAttrH($attName, $attVal)
    {
      return($attVal);
    }

      /**
      * Returns an HTML representation of the array of objects passed
      *
      * Uses many of the methods below which must be redefined by descendants
      * for custom colors (not recommended). If you need to perform some magic
      * on the data which is displayed *inside* the cells, redefine
      * {@link prepareAttrH} (highly recommended).
      *
      * This method will set $_LFX['page']['getListH']['num_rows'] to the
      * number of rows it will render - may be useful for {@link prepareAttrH}
      *
      * This method also sets $_LFX['page']['getListH']['current_row'] to
      * the KEY of the entry in $obj_list which gets rendered.
      *
      * @return string
      */
    function getListH($obj_list)
    {
      if (!is_array($obj_list)) {
        $this->addError('Parameter should be an array! (in SC::getListH())',
          LFX_DV_BOGDAN+86, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      global $_LFX;
      if (!($_LFX['page']['getListH']['num_rows']=count($obj_list))) {
        return('');
      }
      $obj=$obj_list[0];
      if (!is_object($obj)) {
        $this->addError('Parameter should be an array of objects! (in SC::getListH())',
          LFX_DV_BOGDAN+60, LFX_ERROR+LFX_EL_SC);
        return(false);
      }
      $obj_class=get_class($obj);
      // Checking for properly defined display config
      if ((!isset($obj->showAttr)) || (!is_array($obj->showAttr)) || (!sizeOf($obj->showAttr))){
        $this->addError("HTML list requested, but display configuration not set or set incorrectly! (in SC::getListH())",
          LFX_DV_BOGDAN+54, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // Table start, EH header and body header
      $result.=$obj->startTablesH();
      $result.=$obj->tableHeaderH();
      reset($obj->showAttr);
      $result.=$obj->startTableHeaderH();
      while (list(,$attr_show_data)=each($obj->showAttr['attributes'])) {
        $tmp_hdr=$attr_show_data['header'];
        $result.=$obj->tableHeaderCellH($tmp_hdr);
      }
      $result.=$obj->endTableHeaderH();

      // Table body
      $result.=$obj->startTableBodyH();
      reset($obj_list);
      while (list($key,$obj)=each($obj_list)) {
        if ((!is_object($obj)) || (get_class($obj)!=$obj_class)) {
          $this->addError('Parameter should be an array of objects OF THE SAME TYPE! (in LFX_Object::getListH())',
            LFX_DV_BOGDAN+59, LFX_ERROR+LFX_EL_SC);
          return(false);
        }
        $result.=$obj->startTableBodyLineH();
        reset($this->showAttr['attributes']);
        while (list($attr,$attr_show_data)=each($this->showAttr['attributes'])) {
          $tmp_attr=$obj->getAttrH($attr);
          $_LFX['page']['getListH']['current_row']=$key;
          $tmp_attr=$obj->prepareAttrH($attr, $tmp_attr);
          $result.=$obj->tableBodyCellH($tmp_attr);
        }
        $result.=$obj->endTableBodyLineH();
      }
      $obj=$obj_list[0];
      $result.=$obj->endTableBodyH();

      // Table EH footer and ending
      $result.=$obj->tableFooterH();
      $result.=$obj->endTablesH();
      return($result);
    }

    // ----------------------------------
    //          TREE METHODS
    // ----------------------------------

      /**
      * Returns the root id of the tree in which this object resides
      *
      * Note: depends on proper definition of $this->dataStructure to determine
      * which attribute defines the tree structure.
      * Note: $this->rootId is used to cache the root ID. If you need re-reading
      * the whole tree, use force! :)
      *
      * @param boolean $force if true, forces reading the tree instead of using the cache
      * @return integer numerical root ID or false on error
      */
    function getRootId($force=false)
    {
      if ((!$force) && ($this->rootId)) {
        return($this->rootId);
      }
      $tmp=$this->getParent();
      if ($tmp===false) {
        $this->debug('SC::getRootId() called SC::getParent() which failed');
        return(false);
      }
      if ($tmp===NULL) {
        $this->rootId=$this->id;
        return($this->id);
      }
      $tmp->quickLoad();
      $this->rootId=$tmp->getRootID();
      return($this->rootId);
    }

      /**
      * Returns the object at the root of the tree this object resides in
      *
      * Note: Uses {@link getRootId} to retrieve the root id and {@link instantiate}
      * to instantiate it.
      * @return object
      */
    function getRoot($force=false)
    {
      return($this->instantiate($this->getRootId($force)));
    }

      /**
      * Returns the parent of this object
      *
      * Note: needs proper definition of $this->dataStructure to determine
      * the tree links.
      * @return object the parent of this object or false on error.
      */
    function getParent()
    {
      $parentAtt=$this->dataStructure['tree_link_attr'];
      if (!$parentAtt) {
        $this->addError('Parent requested for an object with no tree_link_attr defined in dataStructure! (in SC::getParent())',
          LFX_DV_BOGDAN+87, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      return($this->getObject($parentAtt));
    }

      /**
      * Instantiates the list of id's returned by {@link getParentIDs}
      * and returns it.
      */
    function getParents()
    {
      return($this->instantiate($this->getParentIds()));
    }

      /**
      * Returns the array of the id's of all parents of this object, tree-wise.
      *
      * The root of the tree in which this object resides will be the element
      * at index 0 in the array, and the parent of this object will be the
      * last element in the array. Each index therefore represents the distance
      * from the root in depth. You can get the distance of this object by using
      * count() on the result of this method.
      *
      * @return array the array of parents or false on error
      */
    function getParentIDs()
    {
      $ids=array();
      $tmp=$this;
      while ($tmp2=$tmp->getParent()) {
        if ($tmp2===false) {
          $this->addError("Tree structure broken at ".$tmp->id);
          return(array_reverse($ids));
        }
        $tmp=&$tmp2;
        if (in_array($tmp->id,$ids)) {
          $this->addError("Loop in structure at id ".$tmp->id);
          return(array_reverse($ids));
        }
        $ids[]=$tmp->id;
      }
      return(array_reverse($ids));
    }

      /**
      * Returns an array of instantiated objects representing the children of this object
      *
      * Note: needs proper definition of $this->dataStructure to determine
      * the tree links.
      * @param string $order_att the attribute to order the results by
      * @param boolean $reverse if true, it will return the objects in reverse order
      * @return array the children of this object or false on error.
      */
    function getChildren($order_att=NULL, $reverse=false)
    {
      $parentAtt=$this->dataStructure['tree_link_attr'];
      if (!$parentAtt) {
        $this->addError('Children requested for an object with no tree_link_attr defined in dataStructure! (in SC::getChildren())',
          LFX_DV_BOGDAN+88, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // We don't use getObjects() because that works on dependencies
      // and we don't know which dependency to use. Unfortunately.
      // Because we'll have to use a weaker mechanism to make developers
      // define proper tree dependencies until it may be too late :(
      return($this->search($parentAtt, $this->id, $order_att, $reverse));
    }

      /**
      * Returns an array of IDs of the children of this object
      *
      * Note: needs proper definition of $this->dataStructure to determine
      * the tree links.
      * @param string $order_att the attribute to order the results by
      * @param boolean $reverse if true, it will return the objects in reverse order
      * @return array the children IDs of this object or false on error.
      */
    function getChildrenIDs($order_att=NULL, $reverse=false)
    {
      $parentAtt=$this->dataStructure['tree_link_attr'];
      if (!$parentAtt) {
        $this->addError('Children IDs requested for an object with no tree_link_attr defined in dataStructure! (in SC::getChildren())',
          LFX_DV_BOGDAN+88, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // We don't use getObjects() because that works on dependencies
      // and we don't know which dependency to use. Unfortunately.
      // Because we'll have to use a weaker mechanism to make developers
      // define proper tree dependencies until it may be too late :(
      return($this->searchID($parentAtt, $this->id, $order_att, $reverse));
    }

      /**
      * Returns an array of instantiated objects representing the children of
      * this object which match the condition(s) $attName=$attValue.
      * You may specify as many conditions as you wish, just like in
      * {link @search}.
      *
      * Note: needs proper definition of $this->dataStructure to determine
      * the tree links.
      * @param mixed $attName the attribute(s) to match
      * @param mixed $attVal the value(s) to match
      * @param string $order_att the attribute to order the results by
      * @param boolean $reverse if true, it will return the objects in reverse order
      * @return array the children of this object or false on error.
      */
    function searchChildren($attName, $attVal, $order_att=NULL, $reverse=false)
    {
      $parentAtt=$this->dataStructure['tree_link_attr'];
      if (!$parentAtt) {
        $this->addError('Children requested for an object with no tree_link_attr defined in dataStructure! (in SC::getChildren())',
          LFX_DV_BOGDAN+88, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      // We don't use getObjects() because that works on dependencies
      // and we don't know which dependency to use. Unfortunately.
      // Because we'll have to use a weaker mechanism to make developers
      // define proper tree dependencies until it may be too late :(
      if (is_array($attName)) {
        $attName[]=$parentAtt;
        $attVal[]=$this->id;
      } else {
        $attName=array($attName, $parentAtt);
        $attVal=array($attVal, $this->id);
      }
      $children=$this->search($attName, $attVal, $order_att, $reverse);
      return($this->_doLoad($children));
    }

      /**
      * Returns an array containing the tree under this object.
      *
      * Elements of the same level are ordered by the provided parameters,
      * and every element is followed by its children. It's rather hard to
      * explain - but the idea is that it does what you expect it to.
      * The array is indexed (order as explained above), and every element
      * is itself an associative array made of these elements:
      * 'level' => (int) <item level in the tree, where 0 is this object>
      * 'object' => (object) <the actual object, with only the ID set>
      * @param string $order_att the attribute to sort siblings by
      * @param boolean $reverse if true, will reverse the order OF SIBLINGS
      *   (NOT of the whole tree)
      * @param integer $level this object's level (used internally, but you
      *   may also use it to shift levels for some reason)
      * @return array array describing the tree or false on error.
      */
    function getTree($order_att=NULL, $reverse=false, $level=0, $justIDs=false)
    {
      if (!$this->id) {
      $this->addError('Tree structure requested for object with id not set! (in SC::getTree())',
        LFX_DV_BOGDAN+89, LFX_ET_ERROR+LFX_EL_SC);
      return(false);
      }
      if ($justIDs) {
        $result[]=array('level'=>$level, 'object'=>$this->id);
      } else {
        $result[]=array('level'=>$level, 'object'=>$this);
      }
      $children=$this->getChildrenIDs($order_att, $reverse);
      if ($children===false) {
        $this->debug('SC::getTree called SC::getChildren() which failed');
        return(false);
      }
      $thisClass=get_class($this);
      while (list(,$child)=@each($children)) {
        //$child->quickLoad();
        $child=new $thisClass($child);
        if ($justIDs) {
          $tmp=$child->getTreeIDs($order_att, $reverse, $level+1);
        } else {
          $tmp=$child->getTree($order_att, $reverse, $level+1);
        }
        @reset($tmp);
        while (list(,$tmp2)=each($tmp)) {
          if ($justIDs) {
            $result[]=$tmp2;
          } else {
            $result[]=$tmp2;
          }
        }
      }
      return($result);
    }
    
      /**
      * Identical in every way with {getTree}, this method returns the
      * IDs of the objects, instead of the objects themselves.
      */
    function getTreeIDs($order_att=NULL,$reverse=false,$level=0)
    {
      return($this->getTree($order_att,$reverse,$level,true));
    }

      /**
      * Returns an HTML formatted tree
      * Uses {@link getTree} to get the tree and {@link getLinkH} to render
      * each element
      * @return string HTML string
      */
    function getTreeH($bold=NULL, $order_att=NULL, $reverse=false, $indent=3)
    {
      if (!method_exists($this, getLinkH)) {
        $this->addError("HTML tree requested, but class doesn't have getLinkH implemented! (in SC::getTreeH())",
          LFX_DV_BOGDAN+90, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      $tree=$this->getTree($order_att, $reverse);
      if ($tree===false) {
        $this->debug('SC::getTreeH() called SC::getTree which failed');
        return(false);
      }
      for ($i=0; $i<$indent; $i++) {
        $indentH.="&nbsp;";
      }
      while (list(,$treeEntry)=@each($tree)) {
        $object=$treeEntry['object'];
        $object->quickLoad();
        for ($i=0; $i<$treeEntry['level']; $i++) {
          $result.=$indentH;
        }
        if (($bold) && ($object->id==$bold)) {
          $result.='<b>'.($object->getLinkH())."</b><br />\n";
        } else {
          $result.=($object->getLinkH())."<br />\n";
        }
      }
      return($result);
    }

    // ----------------------------------
    //        PURE HTML METHODS
    // ----------------------------------

      /**
      * Identical to EH_startTables
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function startTablesH($width="90%")
    {
      // Actual table start
      global $_page;
      $pf=$_page->pf;
      $result.="<div align='center'>\n";
      $result.="<table width='$width' border='0' cellspacing='0' cellpadding='0'><tr><td>\n";
      $result.="<table width='100%' border='0' cellspacing='0' cellpadding='0'><tr><td class=\"tbg\">\n";
      if (!$pf) {
        $result.="<table width='100%' border='0' cellspacing='1' cellpadding='2'>\n";
      } else {
        // This isn't nice but it seems that IE doesn't render the gaps for print so...
        $result.="<table width='100%' border='1' cellspacing='0' cellpadding='2'>\n";
      }
      return($result);
    }

      /**
      * Identical to EH_endTables()
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function endTablesH()
    {
      // Actual table end
      $result.="</table>\n";
      $result.="</td></tr></table>\n";
      $result.="</td></tr></table>\n";
      $result.="</div>\n";
      return($result);
    }
    
      /**
      * Starts body header (outputs '<thead><tr>')
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function startTableHeaderH()
    {
      return("<thead>\n<tr>\n");
    }

      /**
      * Ends body header (outputs "</tr></thead>")
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function endTableHeaderH()
    {
      return("</tr>\n</thead>\n");
    }

      /**
      * Starts table body (outputs '<tbody>')
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function startTableBodyH()
    {
      return("<tbody>\n");
    }

      /**
      * Ends table body (outputs "</tbody>")
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function endTableBodyH()
    {
      return("</tbody>\n");
    }

      /**
      * Starts a table body line (outputs '<tr>')
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function startTableBodyLineH()
    {
      return("<tr>\n");
    }

      /**
      * Ends table body line (outputs "</tr>")
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function endTableBodyLineH()
    {
      return("</tr>\n");
    }

      /**
      * Returns table header (read details)
      *
      * Also known as the EH header, this one is typically
      * a single-cell header which spans over all the table width.
      * It generally contains generic information about the data in the table.
      * It gets rendered above the body header. LFX_Object returns an empty string.
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function tableHeaderH()
    {
      return('');
    }

      /**
      * Returns table footer (read details)
      *
      * Also known as the EH footer, this one is typically
      * a single-cell header which spans over all the table width.
      * It generally contains generic information about the data in the table.
      * It gets rendered imediately before the table end. LFX_Object returns an empty string.
      * Used by {@link getListH} - override if you need specific functionality
      * @return string
      */
    function tableFooterH()
    {
      return('');
    }

      /**
      * Returns a header cell. LFX_Object returns "<td class='thead'>$text</td>\n"
      * Used by {@link getListH} - override if you need specific functionality
      * @param string $text the text to be rendered in the cell.
      * @return string
      */
    function tableHeaderCellH($text)
    {
      return("<td>$text</td>\n");
    }

      /**
      * Returns a table cell. LFX_Object returns "<td>$text</td>\n"
      * Please note $text has already been passed to {@link prepareAttrH}.
      * Used by {@link getListH} - override if you need specific functionality
      * @param string $text the text to be rendered in the cell.
      * @return string
      */
    function tableBodyCellH($text)
    {
      return("<td>$text</td>\n");
    }

    // ---------------------------------
    //      SUBSCRIPTION METHODS
    // ---------------------------------

      /**
      * Must be overridden by descendants to return the subscription
      * text, LFX_Mail object to deliver or false if nothing should be
      * delivered; the default implementation in LFX_Object returns NULL.
      * @param string subId the sub-id of the subscription to be delivered
      * @return mixed a string to be delivered as HTML, an LFX_Mail object
      *    or NULL to inhibit delivering the subscription
      */
    function getSubscription($subId=NULL)
    {
      return(NULL);
    }

      /**
      * This method checks whether the current object is associated with
      * an event subscription and returns either the subscription if any
      * or NULL if not available.
      * @param integer subId the sub-id of the subscription
      * @return object the subscription or NULL
      */
    function mySubscription($subId=NULL)
    {
      $myId=$this->id;
      if (!$myId) {
        return(NULL);
      }
      $myClass=get_class($this);
      if (!LFX_include_class('LFX_Subscription')) {
        $this->addError('Unable to check for subscriptions due to failed include.');
        return(false);
      }
      $tmp=new LFX_Subscription;
      $tmp=$tmp->search(
        array('objectClass','objectId','dueDate','objectSubId'),
        array($myClass,$myId,'25 * * *',$subId));
      if ($tmp) {
        return($tmp[0]);
      }
      return(NULL);
    }

      /**
      * This method creates a subscription associated with this object.
      * If an event subscription already exists and the second parameter is not
      * provided, the respective subscription is returned UNCHANGED
      * (i.e. it will not be renamed to the name you specify here).
      * If the second parameter is provided then the method checks whether a digest
      * subscription with the same time interval is associated with this object
      * and returns it UNCHANGED. Otherwise it creates a new one and returns it.
      * @param string name the name of the subscription
      * @param string dueDate if provided, it will create a digest subscription
      * @param integer subId the sub-id of the subscription to make
      * @return object the subscription or false on error
      */
    function makeSubscription($name,$dueDate=NULL,$subId=NULL)
    {
      $myId=$this->id;
      if (!$myId) {
        $this->addError('Subscription creation requested for ambiguous object! Define id first.');
        return(false);
      }
      $myClass=get_class($this);
      if (!LFX_include_class('LFX_Subscription')) {
        $this->addError('Unable to create subscription due to failed include.');
        return(false);
      }
      $tmp=new LFX_Subscription;
      if ($dueDate===NULL) {
        $mySub=$this->mySubscription($subId);
        if ($mySub) {
          return($mySub);
        }
        $dueDate='25 * * *';
      } else {
        $tmp=$tmp->search(
          array('objectClass','objectId','dueDate','objectSubId'),
          array($myClass,$myId,$dueDate,$subId));
        if ($tmp) {
          return($tmp[0]);
        }
      }
      $tmp=new LFX_Subscription();
      $tmp->setAttrs(array(
        'objectClass'=>$myClass,
        'objectId'   =>$myId,
        'dueDate'    =>$dueDate,
        'objectSubId'=>$subId,
        'name'       =>$name));
      $tmp->save();
      return($tmp);
    }

    // ---------------------------------
    //         FORUM METHODS
    // ---------------------------------

      /**
      * This method returns an instance of the forum associated to an object
      * If the object does not have a forum associated it returns
      * an instance with only 'objectId' and 'objectClass' set.
      * This instance will be saved if a message is posted.
      *
      * @return object LFX_Forum instance
      */
    function myForum()
    {
      if (!$this->id) {
        return(false);
      }

      if (!LFX_include_class('LFX_Forum')) {
        $this->addError('Unable to use forums due to include problems');
        return(false);
      }

      $forum=new LFX_Forum;
      $result=$forum->search(array('objectClass','objectId'),array(get_class($this),$this->id));
      if (count($result)) {
        return($result[0]);
      } else {
        $forum->setAttrs(array('objectId'=>$this->id,'objectClass'=>get_class($this)));
        return($forum);
      }
    }

    function renderForumH($mode='RW')
    {
      if (!LFX_permsFilter($mode,'R')) {
        return('');
      }
      $tmp=$this->myForum();
      $tmp->myAssociatedClass=get_class($this);
      $tmp->myAssociatedModule=$this->module;
      $message=new LFX_Forum_Message($LFMid);
      $result='';
      $result.="<A NAME='LFX_forum'></A>";
      $result.=LFX_sform($message->url.'#LFX_forum','POST','LFForm');
      // Make sure you call {class}->forumXtraHTML() *before* the methods
      // for the possible notifications, forumXtraHTML() manages those extras
      $xtraHTML=$this->forumXtraHTML();
      $result.=$tmp->renderH($this,$this->possibleForumNotificationsU(),
        $this->possibleForumNotificationsG(),$xtraHTML,$mode);
      $result.=LFX_eform();
      return($result);
    }

      /**
      * A placeholder method which can return the possible persons/e-mail
      * addresses where notifications about forums can be sent. If not overriden
      * by descendants which use files, no notification controls are rendered
      * at all.
      *
      * Please see the documentation for the LFX_File class for more details
      * on this method (this is not a mistake, see the LFX_File class, not the
      * LFX_Forum class for documentation, the result is treated the same way as
      * the one received from SC::possibleFileNotifications()).
      */
    function possibleForumNotificationsU()
    {
      return(NULL);
    }
    
    function possibleForumNotificationsG()
    {
      return(NULL);
    }
    
    function forumXtraHTML()
    {
      return('');
    }

      /**
      * A placeholder method which can return a LFX_Mail object representing
      * the forum notification for a given forum message. If the result
      * is anything but a LFX_Mail instantiated object, a default message
      * is sent.
      *
      * @param LFX_Forum_Message message the newly created forum message object
      */
    function forumNotification($message)
    {
      return(NULL);
    }

    // ---------------------------------
    //        LFX_FILE METHODS
    // ---------------------------------

      /**
      * This method returns the current user's rights on the current object's
      * files. Descendants of the SC should implement their own _filePerm()
      * method returning the same type of string for customized behavior.
      *
      * The value returned by this method must be a string containing one or
      * more of these characters (case sensitive):
      * "R" - read rights (allows seeing the file list and the file content)
      * "W" - write rights (allows creating files)
      * "D" - delete rights (allows deleting files, needs R to make sense)
      * "U" - update rights (allows obsoleting files, needs W to make sense)
      *
      * @return string a string representing the current user's permissions
      */
    function filePerm()
    {
      global $_page;
      if ($_page->user->isSuperuser()) {
        return('RWUD');
      } elseif (method_exists($this,'_filePerm')) {
        return($this->_filePerm());
      } else {
        return(NULL);
      }
    }

      /**
      * A simple method to check if the current user has a specific permission
      * on this class's associated files. See {@link filePerm} for the possible
      * file permissions. The input can even be the full word describing the
      * permission and is case-insensitive, this method will only use the
      * first character in the input. For instance you can go like
      * if ($this->haveFilePerm('delete')). "read", "write" and "update" work
      * as well.
      *
      * @param string $perm the permission to check
      * @return boolean true if the current user has the respective permission,
      *   false otherwise.
      */
    function haveFilePerm($perm)
    {
      if (strpos(' '.$this->filePerm(),strtoupper($perm[0]))) {
        return(true);
      }
      return(false);
    }

      /**
      * This method renders the current object's files controls and returns
      * an HTML string which represents them. Once your file classes and tables
      * are set up for a module, you can simply execute
      * pecho($myObject->renderFilesH()) and all the files controls will show
      * in that page.
      *
      * For more information on how to set up your module's file classes and
      * tables, see the documentation in the LFX_File class.
      *
      * @return string the HTML representation of associated files and the
      *   controls to manage the associated files or boolean false on error.
      */
    function renderFilesH($mode='RW')
    {
      // Let's do a quick check first so we don't bother with useless controls
      // if not needed.
      $myPerm=$this->filePerm();
      if (!$myPerm) {
        return('');
      }

      // Yes, this means that all modules will have to define both the include
      // name and the classes this way for file classes.
      // Well, tough luck, we have to keep things separated in the minimum
      // (i.e. we need separate tables for files in distinct modules in order
      // to allow installing/uninstalling modules with no hassle).
      $className=$this->module.'_File';
      if (!LFX_include_class($className)) {
        $this->addError('Unable to render file due to include problem.');
        return(false);
      }
      $myClass=get_class($this);
      $myId=$this->id;
      $tmp=new $className;
      if (!$this->includeKey) {
        $this->addError("You haven't declared the include key in the variable ".
        "\$includeKey for object type ".niceClass($this)."!");
        return(false);
      }
      $tmp->myAssociatedClass=$myClass;
      $tmp->myAssociatedModule=$this->module;
      $tmp->performActions($myClass,$myId,$this->includeKey,$myPerm,$mode);
      $myTopFiles=$this->myFiles();
      $result='';
      $result.=$tmp->startRenderH($myPerm,$mode);
      if (strpos(' '.$myPerm,'R')) {
        for($i=0;$i<count($myTopFiles);$i++) {
          $result.=$myTopFiles[$i]->renderH($myPerm,$mode);
        }
      }
      // Make sure you call {class}->fileXtraHTML() *before* the methods
      // for the possible notifications, fileXtraHTML() manages those extras
      $xtraHTML=$this->fileXtraHTML();
      $result.=$tmp->endRenderH($myPerm,$this->possibleFileNotificationsU(),
        $this->possibleFileNotificationsG(),$xtraHTML,$mode);
      return($result);
    }

      /**
      * A placeholder method which can return the possible persons/e-mail
      * addresses where notifications about files can be sent. If not overriden
      * by descendants which use files, no notification controls are rendered
      * at all.
      *
      * Please see the documentation for the LFX_File class for more details
      * on this method.
      */
    function possibleFileNotificationsU()
    {
      return(NULL);
    }

    function possibleFileNotificationsG()
    {
      return(NULL);
    }

    function fileXtraHTML()
    {
      return('');
    }

      /**
      * A placeholder method which can return a LFX_Mail object representing
      * the file notification for a given notification reason. If the result
      * is anything but a LFX_Mail instantiated object, a default message
      * is sent.
      *
      * @param string message a string containing the reason for the
      *    notification
      * @param file the LFX_File object, initialized
      */
    function fileNotification($reason,$file)
    {
      return(NULL);
    }

      /**
      * This method returns the LFX_File descendants associated with this
      * object. Only the "top level" files are returned (the ones which are
      * not obsoleted by any other files).
      *
      * @param bool $all specifies whether to include obsolete files
      * @return array of instantiated LFX_File descendants
      */
    function myFiles()
    {
      $className=$this->module.'_File';
      if (!LFX_include_class($className)) {
        // Returning an empty array -- no files are used in this module
        return(array());
      }
        $tmp=new $className;
        $myFiles=$tmp->search(
          array('objectClass','objectId','obsoleted_by'),
          array(get_class($this),$this->id,0),'date');
        return($myFiles);
      }

    // ---------------------------------
    //     MISCELLANEOUS METHODS
    // ---------------------------------
      /**
      * This method logs actions on this object unless $this->noLogging
      * is set to true. Even if $this->noLogging is set to true, you can
      * override that manually by setting the second parameter to true.
      *
      * Please note the following conditions have to be met in order for
      * logging to be properly performed:
      * - $this->module has to be set to the proper module name ("EH",
      *   "OPT", "DMO" or "LFXlib" at the time of this writing);
      * @return true on success, false on failure or NULL if logging is disabled
      */
    function log($eventType, $force=false)
    {
      global $_LFX_globals;
      if (($this->noLogging || $_LFX_globals->getSetting('full_inhibit_logging',NULL,$_SESSION['LFX_project']))&& !$force) {
        return(NULL);
      }

      if (!LFX_include_class('LFX_Log')) {
        $this->addError('Unable to log due to include problems');
        return(false);
      }

      if (!isset($this->module)) {
        debug('Object '.(get_class($this)).' has $module not set!');
        $this->module='NOT SET!';
      }
      if (!in_array($eventType,array('insert', 'delete', 'update'))) {
        $this->addError("Unknown event type: $eventType",
          LFX_DV_BOGDAN+193, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      
      $tmp=new LFX_Log();
      global $_page;
      if (isset($_page) && isset($_page->user) && isset($_page->user->id)) {
        $userId=$_page->user->id;
      } else {
        $userId=0;
      }
      if ($_LFX_globals->getSetting('full_audit_log',NULL,$_SESSION['LFX_project'])) {
        if ($eventType=='delete') {
          $tmp->setAttr('data_dump',serialize(array()));
        } else {
          $myAttrs=$logAttrs=$this->attr;
          $logHuge=$_LFX_globals->getSetting('huge_audit_log',NULL,$_SESSION['LFX_project']);
          $logAttrs=array();
          while(list($attr,$value)=each($myAttrs)) {
            if ($logHuge || (!$this->dataStructure['fields'][$attr]['flags']['huge'])) {
              $logAttrs[$attr]=$value;
            }
          }
          $tmp->setAttr('data_dump',serialize($logAttrs));
        }
      }
      $tmp->setAttrs(array(
        'event'=>$eventType,
        'class'=>get_class($this),
        'object_id'=>$this->id,
        'module'=>$this->module,
        'entry_date'=>time(),
        'user'=>$userId));
      return($tmp->save());
    }

      /**
      * Swaps id's with the object represented by the given id
      *
      * You probably won't ever need to do this, but should you ever,
      * this is the method which swaps two objects. Takes care of dependencies.
      */
    function swap($id)
    {
      // Locking everything we'll need - dependencies included
      // Please note this is essential - you'll end up locking
      // tables forever if you try to perform any action on an
      // unlocked table as long as others are locked!
      // If you do happen to do this, you'll have to restart mysqld.
      $locks[]=$this->dataStructure['table_name'];
      $deps=$this->dataStructure['depend'];
      while (list(,$dep)=each($deps)) {
        unset($tmp2);
        $tmp2=new $dep['class'];
        if (!@in_array($tmp2->dataStructure['table_name'], $locks)) {
          $locks[]=$tmp2->dataStructure['table_name'];
        }
      }
      if (!$this->db) {
        $this->dbInit();
      }
      $this->db->lock($locks);

      $tmp=get_class($this);
      $tmp=new $tmp($id);
      $tmp->load();
      $this->load();

      // Swapping...
      $tmpID=$tmp->id;
      $tmp->id=$this->id;
      $this->id=$tmpID;
      $tmp->save($tmp->id,1);
      $this->save($this->id,1);

      // Ok, done swapping - now let's see to the dependencies
      @reset($deps);
      while (list($depName,$dep)=@each($deps)) {
        if ($dep['on_mod']=='STRANGER') {
          continue;
        }

        $depList1=$this->getObjects($depName);
        $depList2=$tmp->getObjects($depName);

        while (list(,$depObject)=@each($depList1)) {
          if ($dep['type']=='many') {
            $depObject->dropLink($depName, $this->id);
            $depObject->createLink($depName, $tmp->id);
          } else {
            $depObject->quickLoad();
            $depObject->setAttr($dep['attr'],$tmp->id);
            $depObject->save();
          }
        }

        while (list(,$depObject)=@each($depList2)) {
          if ($dep['type']=='many') {
            $depObject->dropLink($depName, $tmp->id);
            $depObject->createLink($depName, $this->id);
          } else {
            $depObject->quickLoad();
            $depObject->setAttr($dep['attr'],$this->id);
            $depObject->save();
          }
        }
      }
      $this->db->unlock();
    }

    // ----------------------------------
    //        AUXILIARY METHODS
    // ----------------------------------
      /**
      * Returns true if this object has any messages or false otherwise
      * @return boolean
      */
    function problems()
    {
      if (count($this->messages)) {
        return(true);
      }
      return(false);
    }

      /**
      * Returns an HTML representation of this object's errors
      * Shouldn't be called directly unless you're debugging
      * @return string HTML string
      */
    function getErrorsH()
    {
      global $commandline, $_LFX_error_output, $_LFX_error_reporting, $_LFX;
      if ($_LFX_error_reporting) {
        if (!$commandline) {
          echo("<p><b>");
        }
        echo("There have been errors encountered, but the system is currently ".
        "too unstable to even /manage/ the existing errors. Here is the list ".
        "of errors for this object:\n");
        if (!$commandline) {
          echo("</p></b><pre>");
        }
        var_dump($_LFX['page']['error_messages']);
        if (function_exists('debug_backtrace')) {
          echo("Backtrace dump:\n");
          var_dump(debug_backtrace());
        }
        if (!$commandline) {
          echo("</pre>");
        }
        // Too unstable to go on, fix yer errors first and then we'll see.
        exit;
      }
      $_LFX_error_reporting=true;
      //@reset($this->messages);
      @reset($_LFX['page']['error_messages']);
      $class=get_class($this);
      //while (list(,$err)=@each($this->messages)) {
      while (list(,$err)=@each($_LFX['page']['error_messages'])) {
        if ($err['id']) {
          $err_id='#'.$err['id'];
        } else {
          $err_id='(error ID not set!)';
        }
        if ($err['sub_id']) {
          $err_id.="/".$err['sub_id'];
        }
        $human_sev=array(
          LFX_ET_CRITICAL => 'Critical',
          LFX_ET_MAJOR    => 'Major',
          LFX_ET_ERROR    => 'Error',
          LFX_ET_WARNING  => 'Warning',
          LFX_ET_INFO     => 'Information'
        );
        $sev=$err['type'] & 255;
        $sev=$human_sev[$sev];
        $human_layer=array(
          LFX_EL_GENERIC    => 'Generic',
          LFX_EL_DB         => 'Database',
          LFX_EL_GUI        => 'User Interface',
          LFX_EL_USER       => 'User Input',
          LFX_EL_UNEXPECTED => 'Unexpected State',
          LFX_EL_PERM       => 'Permissions',
          LFX_EL_SC         => 'Super Class'
        );
        $layer=$err['type'] & (255<<8);
        $layer=$human_layer[$layer];
        if ($commandline) {
          $result.="--------- LFXlib Error Message ---------\n";
        } else {
          if (!$_LFX_error_output) {
            //$result.=LFX_echo("<b>LFXlib error</b>: ");
            $result.="
              <script language='Javascript 1.2' type='text/javascript'>
              function showHideDiv(divId)
              {
              	el = document.getElementById(divId);
              	if (el.style.display == 'none')
              	{
              		el.style.display = '';
              	} else {
              		el.style.display = 'none';
              	}
              }
              
              function showErrorsTab()
              {
                showHideDiv('errorsTab');
                // showHideDiv('errorsExplain');
              }
              </script>\n";
            $result.=LFX_startTables('100%');
            $result.=LFX_sthead();
            $result.=LFX_td("<a href='#' onClick='showErrorsTab(); return false;'>".
              __L("Errors have been encountered")."</a>");
            $result.=LFX_ethead();
            $result.=LFX_str();
            $result.=LFX_echo("<td>");
            $result.=LFX_echo("<div id='errorsExplain' align='center'>".
              sprintf(__L("Please click on the link to show them."),"<i>","</i>")."</div>");
            $result.="<div id='errorsTab' style='display: none'><pre>";
          } else {
            $result.="----------------------------------------\n";
          }
        }
        $result.="   Class: ".niceClass($class)."\n";
        $result.="      ID: ".($err['obj_id']?$err['obj_id']:"n/a")."\n";
        $result.="   Error: ".($err['id']?$err['id']:"n/a")."\n";
        $result.=" Message: ".$err['message']."\n";
        $result.="Severity: $sev\n";
        $result.="   Layer: $layer\n";
        $result.="   SubID: ".($err['sub_id']?$err['sub_id']:"n/a")."\n";
        if (function_exists('debug_backtrace')) {
          $result.="Debug data:\n";
          $debugData=$err['debug_backtrace'];
          $tmp='';
          for($i=0;$i<count($debugData);$i++) {
            $tmp2='';
            for($j=0;$j<count($debugData[$i]['args']);$j++) {
              $targ=$debugData[$i]['args'][$j];
              if ($targ) {
                if (is_numeric($targ)) {
                  $tmp2.=$targ;
                } elseif (is_object($targ)) {
                  $tmp2.="[".niceClass(get_class($targ));
                  if ($this->getBaseClass($targ)=='lfx_object') {
                    $tmp2.="#".abs($targ->id);
                  }
                  $tmp2.="]";
                } elseif (is_string($targ)) {
                  $tmp2.='"'.htmlspecialchars($targ).'"';
                } else {
                  $tmp2.='"'.$targ.'"';
                }
              }
              $tmp2.=', ';
            }
            while(substr($tmp2,-2)==', ') {
              $tmp2=substr($tmp2,0,-2);
            }
            if ($debugData[$i]['class']) {
              $tmp3='method '.niceClass($debugData[$i]['class']).'::';
            } else {
              $tmp3='function ';
            }
            global $_LFX_url, $_page;
            if (
              $_page->user->id &&
              $_page->user->getRights('LFX_ShowSource') &&
              !$commandline
            ) {
              $url=array(
                "<a name='".__L("Show source").
                "' href='$_LFX_url/util/show_source.php?line=".
                $debugData[$i]['line']."&amp;path=".
                rawurlencode($debugData[$i]['file'])."#line' target='_blank'>",
                
                "</a>"
              );
            } else {
              $url=array('','');
            }
            if ($debugData[$i]['file']) {
              $tmp.=(count($debugData)-$i).". File $url[0]{$debugData[$i]['file']}, ".
                "line {$debugData[$i]['line']}$url[1] called ".
                "$tmp3{$debugData[$i]['function']}($tmp2)";
            } else {
              $tmp.=(count($debugData)-$i).". PHP internal call to ".
                "$tmp3{$debugData[$i]['function']}($tmp2)";
            }
            $tmp.="\n";
          }
          $result.=$tmp;
        }
        if ($commandline) {
          $result.="----------------------------------------\n";
        }
        $_LFX_error_output=true;
      }
      $_LFX_error_reporting=false;
      return($result);
    }

      /**
      * Auxiliary method to register field names identical to attributes
      *
      * LFX_Object needs explicit declaration of field names, so it doesn't
      * have to check both field names and attribute names in every method.
      * Call this method to duplicate attribute names in field names for
      * the attributes which use the same field name as their name instead
      * of defining field names manually in the constructor.
      */
    function fillDataStructure()
    {
      $myClass=get_class($this);
      
      global $_LFX;
      if (isset($_LFX['global']['data_structure'][$myClass])) {
        $this->dataStructure=&$_LFX['global']['data_structure'][$myClass];
        return true;
      }

      // We'll start by filling in the default field name for attributes
      // which don't define it explicitly
      $fields=$this->dataStructure['fields'];
      reset($fields);
      while(list($attName,$dataDef)=@each($fields)) {
        if (!$fields[$attName]['fld_name']) {
          $fields[$attName]['fld_name']=$attName;
        }
      }
      $this->dataStructure['fields']=$fields;
      
      // Now we need to work on the dependencies. But first let's read in
      // the foreign dependencies, if any (for more information, see
      // function LFX_register_dependency())
      global $_LFX;
      if (is_array($_LFX['global']['foreign_deps'][$myClass])) {
        $this->dataStructure['depend']=array_merge(
          $_LFX['global']['foreign_deps'][$myClass],
          $this->dataStructure['depend']
        );
      }
      
      // Ok, now we have the final dependency structure. Let's check if the
      // on_mod setting is explicitly defined, and fill in the defaults:
      // * for many to many, the default is MISTRESS (when one end is deleted,
      //   just delete the corresponding entries in the join table)
      // * for one to many, the default is WIFE (when the "one" is
      //   deleted, automatically delete the "many")
      $deps=$this->dataStructure['depend'];
      @reset($deps);
      while (list($depName,$dep)=@each($deps)) {
        if (!$dep['on_mod']) {
          if ($dep['type']=='many') {
            $dep['on_mod']='MISTRESS';
          } else {
            $dep['on_mod']='WIFE';
          }
          $deps[$depName]=$dep;
        }
      }
      $this->dataStructure['depend']=$deps;

      // And finally, let's make sure to register the link between this object
      // and its siblings, in case a tree link attribute is defined.
      if ($this->dataStructure['tree_link_attr']) {
        $this->dataStructure
          ['fields'][$this->dataStructure['tree_link_attr']]['link_class']=
          $myClass;
      }
      
      // Ok, now for a simple cosmetic thingie, let's check if we have a
      // title_attr defined, and if not we'll look for some traditional
      // default, such as 'title' or 'name'. We'll use one of those by default
      // if any present among the existing attributes.
      if (!$this->dataStructure['title_attr']) {
        if (isset($this->dataStructure['fields']['title'])) {
          $this->dataStructure['title_attr']='title';
        } elseif (isset($this->dataStructure['fields']['name'])) {
          $this->dataStructure['title_attr']='name';
        }
      }
      
      $_LFX['global']['data_structure'][$myClass]=$this->dataStructure;
      $this->dataStructure=&$_LFX['global']['data_structure'][$myClass];
    }

      /**
      * Returns the field name associated with the given attribute
      *
      * Note: use this if you ever need to query the database directly,
      * instead of hardcoding field names. Field names should never appear
      * in your code, except for the constructor.
      *
      * @param string $attName the attribute to find the field associated to
      * @param boolean $simple set to true if you do NOT want the table name
      *   (the default is returning 'tablename.fieldname')
      */
    function getFieldName($attName, $simple=false)
    {
      if ($this->dataStructure['fields'][$attName]['fld_name']) {
        $fld=$this->dataStructure['fields'][$attName]['fld_name'];
      } else {
        $fld=$attName;
      }
      if ($simple) {
        return($fld);
      }
      $tbl=$this->dataStructure['table_name'];
      if (!$tbl) {
        $this->addError("Extended field name required, but table name not defined! (in SC::getFieldName())",
          LFX_DV_BOGDAN+91, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      return("$tbl.$fld");
    }

      /**
      * Returns the table name associated with the given object
      * @param boolean $full if true, it includes the database name, resulting
      *   in a table name of the form "database.table" -- by default, only the
      *   table name is returned.
      * @return string the table name
      */
    function getTableName($full=false)
    {
      $tableName=$this->dataStructure['table_name'];
      if (!$full) {
        return($tableName);
      } else {
        $db=$this->_doDbInit($this->dbData);
        return($db->Database.".".$tableName);
      }
    }

      /**
      * Returns a date formatted for SQL based on a PHP date
      *
      * You don't need this unless you're performing manual
      * queries. For regular use, just set the sqlDate flag and
      * use {@link getAttr} and {@link setAttr} which work
      * with PHP dates.
      *
      * @param integer $timestamp if not specified it will use the current system time
      * @return string
      */
    function sqlDate($timestamp=false)
    {
      if ($timestamp===false) {
        return(date("Y-m-d H:i:s"));
      } else {
        return(date("Y-m-d H:i:s",$timestamp));
      }
    }

      /**
      * A wrapper for $this->db->query - do not use $this->db->query directly.
      *
      * This will first set up $this->db if needed, and then perform the query.
      * After performing the query, use $this->db methods to retrieve the result.
      *
      * @param mixed $sql (string) the query in plain text or
      *  (array) the query as a LFX_Query_Manager structure
      * @return integer the database result handle (which incidentally you don't need)
      */
    function query($sql)
    {
      if (!is_object($this->db)) {
        $this->dbInit();
      }
      // Let's be nice and accept a query manager structure as well
      if (is_array($sql)) {
        LFX_include('LFX_Query_Manager');
        $sql=LFX_buildSQL($sql);
      }

      /*
      
      TO DO
      -----
        * use execution history to ignore duplicate calls from the same scripts
          (might not be effective with some calls, for instance the process
          list in HelpDesk, should also allow inhibiting this filter)
      
      */
      
      $localExplainRecord=false;
      if ($_SESSION['LFX_debug']['log_bad_db_indexes']) {
        if (strtoupper(substr($sql,0,7))=='SELECT ') {
          $result=$this->db->query("EXPLAIN $sql");
          if ($result) {
            if ($this->db->next_record()) {
              $localExplainRecord=$this->db->Record;
            } else {
              $this->addError('Failed to get next record when trying to log bad db indexes');
            }
          }
        }
      }
      
      $result=$this->db->query($sql);
      
      if (!$result && !$this->inhibitQueryErrors) {
        $this->addError("Database error: {$this->db->Error};\nQuery: $sql;\n".
          // "dbData: ".(serialize($this->dbData)).
          " (in SC::query())",
          LFX_DV_BOGDAN+58, LFX_ET_ERROR+LFX_EL_DB, $this->db->Errno);
      } else {
        global $_LFX;
        $_LFX['global']['queryCount']++;
        /*
        // Used for debugging infinite loop conditions
        if ($_LFX['global']['queryCount']>240) {
          echo("TOO MANY QUERIES!");
          exit;
        }
        */
        $this->debug("Query: $sql",5);
      }

      if ($localExplainRecord) {
        if ($localExplainRecord['rows']-$this->db->num_rows()>3) {
          LFX_LogMessage(
            'Too many rows checked by MySQL! ('.$this->db->num_rows().' returned out of '.$localExplainRecord['rows'].' checked)',
            'warning',
            array(
              'data_dump'=>array(
                'table'=>$localExplainRecord['table'],
                'type'=>$localExplainRecord['type'],
                'possible_keys'=>$localExplainRecord['possible_keys'],
                'key'=>$localExplainRecord['key'],
                'key_length'=>$localExplainRecord['key_len'],
                'ref'=>$localExplainRecord['ref'],
                'rows'=>$localExplainRecord['rows'],
                'extra'=>$localExplainRecord['Extra'],
                'query'=>$sql,
                'history'=>debug_backtrace()
              )
            )
          );
        }
      }

      return($result);
    }

      /**
      * This method ECHOES the dump returned by {@link _dump}.
      * Extensive info about the current object.
      */
    function dump()
    {
      echo($this->_dump());
    }

      /**
      * Returns an HTML representation of the current object.
      *
      * This will return LOTS of information about the current object, and
      * links will be available to dump as well.
      *
      * @return string HTML string
      */
    function _dump()
    {
      global $_LFX_url;
      $dumper="$_LFX_url/util/dump.php";
      if (!$this->id) {
        $id='??';
      } else {
        $id=$this->id;
      }
      $class=get_class($this);

      // -- GLOBALS --
      $na='[n/a]';
      $result="<h1>$class #$id Dump</h1>\n";
      $myLink=$this->getLinkH();

      if ($myLink) {
        $result.="<div align='center'><b>$myLink</b></div>\n";
      }

      // -- CREATE/DELETE
      $result.="<div align='center'>\n";
      $result.="[<a href='$dumper?class=$class&action=create'>".__L("Create New")."</a>]";
      if ($id!='??') {
        $result.="&nbsp;[<a href='$dumper?class=$class&action=delete'>".__L("Delete")."</a>] (".__L("no confirmation!").")";
      }
      $result.="&nbsp;[<a href='$dumper?class=$class'>List</a>]";
      $result.="</div>\n";

      // -- DETAILS --
      $result.="<h2>Details</h2>\n";
      $result.="<table cellspacing='1'>\n";
      $result.="<tr>\n";
      $result.="<td class='thead'><b>".__L("Database")."</b></td>\n";
      $result.='<td>'.$this->db->Database."</td>\n";
      $result.="</tr>\n";
      $result.="<tr>\n";
      $result.="<td class='thead'><b>".__L("Table")."</b></td>\n";
      $result.='<td>'.$this->dataStructure['table_name']."</td>\n";
      $result.="</tr>\n";
      $result.="<tr>\n";
      $result.="<td class='thead'><b>".__L("ID Field")."</b></td>\n";
      $result.='<td>'.$this->dataStructure['id_field']."</td>\n";
      $result.="</tr>\n";
      $result.="</table>\n";

      // -- ATTRIBUTES --
      $result.="<h2>Attributes</h2>\n";
      $result.="<form action='$dumper' method='POST'>\n";
      $result.="<table cellspacing='1'>\n";
      $attList=$this->dataStructure['fields'];
      while (list($attName, $attData)=@each($attList)) {
        $origAttr[$attName]=$this->attr[$attName];
      }
      @reset($attList);
      if ($id!='??') {
        while (list($attName, $attData)=@each($attList)) {
          $getAttr[$attName]=$this->getAttrH($attName);
          $getAttrOrig[$attName]=$this->getAttrH($attName);
        }
        @reset($attList);
        $tmp=new $class($id);
        while (list($attName, $attData)=@each($attList)) {
          $loadAttr[$attName]=$tmp->getAttrH($attName);
        }
        @reset($attList);
      } else {
        $getAttr=array();
        $loadAttr=array();
      }
      while (list($attName, $attData)=@each($attList)) {
        if ($attData['link_class']) {
          $linkClass=$attData['link_class'];
          if ($linkId=$origAttr[$attName]) {
            $tmp=new $linkClass($linkId);
            if ($linkLink=$tmp->getLinkH()) {
              $origAttr[$attName]="$linkLink <a href='$dumper?class=$linkClass&id=$linkId'>[".__L("dump")."]</a> ($linkId)";
            } else {
              $origAttr[$attName]="$linkId <a href='$dumper?class=$linkClass&id=$linkId'>[".__L("dump")."]</a>";
            }
          } else {
            $origAttr[$attName]=$na;
          }
          if ($linkId=$getAttr[$attName]) {
            $tmp=new $linkClass($linkId);
            if ($linkLink=$tmp->getLinkH()) {
              $getAttr[$attName]="$linkLink <a href='$dumper?class=$linkClass&id=$linkId'>[".__L("dump")."]</a> ($linkId)";
            } else {
              $getAttr[$attName]="$linkId <a href='$dumper?class=$linkClass&id=$linkId'>[".__L("dump")."]</a>";
            }
          } else {
            $getAttr[$attName]=$na;
          }
          if ($linkId=$loadAttr[$attName]) {
            $tmp=new $linkClass($linkId);
            if ($linkLink=$tmp->getLinkH()) {
              $loadAttr[$attName]="$linkLink <a href='$dumper?class=$linkClass&id=$linkId'>[".__L("dump")."]</a> ($linkId)";
            } else {
              $loadAttr[$attName]="$linkId <a href='$dumper?class=$linkClass&id=$linkId'>[".__L("dump")."]</a>";
            }
          } else {
            $loadAttr[$attName]=$na;
          }
        }
      }
      @reset($attList);
      $result.="<tr>\n";
      $result.="<td class='thead'><b>".__L("Attribute")."</b></td>\n";
      $result.="<td class='thead'><b>".__L("Object Value")."</b></td>\n";
      $result.="<td class='thead'><b>".__L("getAttr() Value")."</b></td>\n";
      $result.="<td class='thead'><b>".__L("Db Value")."</b></td>\n";
      $result.="<td class='thead'><b>".__L("Edit Value")."</b></td>\n";
      $result.="<td class='thead'><b>".__L("Flags")."</b></td>\n";
      $result.="<td class='thead'><b>".__L("Db Field Name")."</b></td>\n";
      $result.="</tr>\n";
      while (list($attName, $attData)=@each($attList)) {
        $result.="<tr>\n";
        $result.="<td><b>$attName</b></td>\n";
        $result.='<td>'.$origAttr[$attName]."</td>\n";
        $result.='<td>'.$getAttr[$attName]."</td>\n";
        $result.='<td>'.$loadAttr[$attName]."</td>\n";
        $result.="<td><input type='text' name='_LFX_dump_$attName' value='".$getAttrOrig[$attName]."' /></td>\n";
        $flags='';
        while (list($flagName,$flagStat)=@each($attData['flags'])) {
          if ($flagStat) {
            $flags.=" $flagName";
          }
        }
        if (!$flags) {
          $flags=$na;
        }
        $result.="<td>$flags</td>";
        $result.='<td>'.$attData['fld_name']."</td>";
        $result.="</tr>\n";
      }
      $result.="</table>\n";
      $result.="<input type='hidden' name='class' value='$class'>\n";
      $result.="<input type='hidden' name='id' value='$id'>\n";
      $result.="<input type='hidden' name='action' value='edit'>\n";
      $result.="<input type='submit' value='".__L("Edit Object")."'>\n";
      $result.="</form>\n";

      // -- DEPENDENCIES --
      $depList=$this->dataStructure['depend'];
      if (!$depList) {
        $result.="<h2>No dependencies</h2>\n";
      } else {
        $result.="<h2>Dependencies</h2>\n";
        $result.="<table cellspacing='1'>\n";
        $result.="<tr>\n";
        $result.="<td class='thead'><b>".__L("Dependency")."</b></td>\n";
        $result.="<td class='thead'><b>".__L("Type")."</b></td>\n";
        $result.="<td class='thead'><b>".__L("Class")."</b></td>\n";
        $result.="<td class='thead'><b>".__L("Relationship")."</b></td>\n";
        $result.="<td class='thead'><b>".__L("Details")."</b></td>\n";
        $result.="<td class='thead'><b>".__L("Dependents")."</b></td>\n";
        $result.="</tr>\n";
        while (list($depName, $depData)=@each($depList)) {
          $result.="<tr>\n";
          $result.="<td><b>$depName</b></td>\n";
          $depType=$depData['type'];
          $depType=$depType?$depType:'one';
          $result.="<td>".__L("$depType")."</td>\n";
          $result.='<td>'.$depData['class']."</td>\n";
          $depMod=$depData['on_mod'];
          $result.="<td>".__L("$depMod")."</td>\n";
          if ($depType=='one') {
            $depDetail=__L("Dep's attribute").": ".$depData['attr'];
          } else {
            $depDetail=__L('Join table').': '.$depData['table_name']."<br />\n".
              __L('My field').': '.$depData['my_fld_name']."<br />\n".
              __L("Dep's field").": ".$depData['link_fld_name'];
          }
          $result.="<td>$depDetail</td>\n";
          if ($id!=='??') {
            $tmp=$this->getObjects($depName);
          } else {
            $tmp=0;
          }
          $deps='';
          if (!$tmp) {
            $deps=$na;
          } else {
            for ($i=0;$i<count($tmp);$i++) {
              $dep=&$tmp[$i];
              $depClass=get_class($dep);
              $depId=$dep->id;
              if (!($depLink=$dep->getLinkH())) {
                $deps.=$depId;
              } else {
                $deps.="$depLink ($depId)";
              }
              $deps.=" <a href='$dumper?class=$depClass&id=$depId'>[".__L("dump")."]</a><br />\n";
            }
            $deps=substr($deps,0,-8);
          }
          $result.="<td>$deps</td>\n";
          $result.="</tr>\n";
        }
        $result.="</table>\n";
      }
      return($result);
    }

      /**
      * Returns the specified debug message depending on debug settings.
      *
      * Uses $_LFX_debug to check if this message should be returned or not.
      * If global $_LFX_debug is less than the specified level, this message
      * will not be displayed.
      *
      * @param string $msg the debug message
      * @param integer $level minimum verbosity level for this message to pass
      * @param boolean $inhibit by default, this method prepends 'LFX Debug: '
      *   to the debug messages for identification. If this is true, it will inhibit
      *   this prepend, leaving only the message followed by a "<br />\n"
      * @return string
      */
    function _debug($msg, $level=1, $inhibit=false)
    {
      if ($this->is_debug($level)) {
        if ($inhibit) {
          return("$msg<br />\n");
        } else {
          return("<b>LFX Debug</b>: $msg<br />\n");
        }
      } else {
        return('');
      }
    }
    
    function is_debug($level=1)
    {
      global $_LFX_debug, $_page, $_LFX_force_inhibit_debug;
      if (!isset($_LFX_force_inhibit_debug)) {
        if (!$_page || !$_page->user || !$_page->user->id || !$_page->user->getRights('LFX_DebugLevel')) {
          $_LFX_force_inhibit_debug=false;
          return(false);
        }
        $_LFX_force_inhibit_debug=false;
      }
      if ($_LFX_force_inhibit_debug) {
        return(false);
      }
      if ($_LFX_debug>=$level) {
        return(true);
      } else {
        return(false);
      }
    }

      /**
      * This will ECHO the result of a {@link _debug} with the same parameters
      */
    function debug($msg, $level=1, $inhibit=false)
    {
      echo($this->_debug($msg, $level, $inhibit));
    }

      /**
      * ADDS slashes only if magic gpc slashes ARE NOT on
      * @param string $val the string to process
      * @return string the processed string
      */
    function gpcSlash($val)
    {
      if (get_magic_quotes_gpc()) {
        return($val);
      } else {
        return(addslashes($val));
      }
    }

      /**
      * REMOVES slashes only if magic gpc slashes ARE on
      * @param string $val the string to process
      * @return string the processed string
      */
    function gpcUnslash($val)
    {
      if (get_magic_quotes_gpc()) {
        return(stripslashes($val));
      } else {
        return($val);
      }
    }

      /**
      * ADDS slashes only if magic runtime slashes ARE NOT on
      * @param string $val the string to process
      * @return string the processed string
      */
    function runtimeSlash($val)
    {
      if (get_magic_quotes_runtime()) {
        return($val);
      } else {
        return(addslashes($val));
      }
    }

      /**
      * REMOVES slashes only if magic runtime slashes ARE on
      * @param string $val the string to process
      * @return string the processed string
      */
    function runtimeUnslash($val)
    {
      if (get_magic_quotes_runtime()) {
        return(stripslashes($val));
      } else {
        return($val);
      }
    }
    
    // Really private stuff
      /**
      * You definitely don't need this.
      *
      * Builds the save query for {@link save} and {@link insert}.
      * It will only build the "field=value" part of the
      * query, so save() and insert can prepend their
      * respective statements.
      * The unmodified attributes are not saved by default,
      * so if you need to force including the unmodified
      * attributes you should specify $force.
      *
      * @param boolean $force true to force saving all attributes
      * @return string the assignment part of the query
      */
    function _makeSaveQuery($force=false)
    {
      $fields=$this->dataStructure['fields'];
      reset($fields);
      while (list($attName,$dataEntry)=@each($fields)) {
        if ((!$force) &&
            (
             (!isset($this->attr[$attName])) ||
             (!$this->attr_flags[$attName]['modified'])
            ) &&
            (!$dataEntry['flags']['forceSave'])
        ) {
          continue;
        }
        $fld_name=$dataEntry['fld_name'];
        $fld_data=$this->attr[$attName];
        if ((isset($this->attr[$attName])) && ($fld_data===NULL)) {
          $query.="`$fld_name`=NULL, ";
        } elseif ($dataEntry['flags']['sqlDate']) {
          $fld_data=$this->sqlDate($fld_data);
          $query.="`$fld_name`=\"$fld_data\", ";
        } else {
          $fld_data=addslashes($fld_data);
          $query.="`$fld_name`=\"$fld_data\", ";
        }
      }
      $query=substr($query, 0, -2);
      //echo "query is: ".$query."<br>";
      return($query);
    }

      /**
      * A cute little method which checks if the respective object exists
      * in the database. Should be used as a test before a {@link load} for instance.
      * If you try to load an object which doesn't exist, you'll get an error.
      * You can therefore use probe() to check if the entry actually exists, and
      * only proceed loading it if it does.
      * An error can be issued either on database error, or if the id could
      * not be determined -- for instance if you try to use probe with no params
      * on an object which has been neither loaded nor saved.
      * @param integer id the id of the object to probe; by default this object
      * @return mixed true if exists, NULL if it doesn't and false on error
      */
    function probe($id=0)
    {
      if (!$id) {
        $id=$this->id;
      }
      if (!$id) {
        return(false);
      }
      $id_fld=$this->dataStructure['id_field'];
      $table=$this->dataStructure['table_name'];
      $sql="SELECT COUNT(*) FROM `$table` WHERE `$id_fld`=\"$id\"";
      $this->query($sql); // Will have to manage errors!
      if (!$this->db->nf()) {
        // See here why num_fields() is used instead of num_rows():
        // http://www.sanisoft.com/phplib/manual/DB_SqlMethods.php#id2790395
        return(false);
      }
      $this->db->next_record();
      if ($this->db->f(0)) {
        return(true);
      } else {
        return(NULL);
      }
    }
      /**
      * You definitely don't need this.
      *
      * This is called by both {@link load} and {@link quickLoad}
      * to perform the loading. Actually you could call {@link load}
      * and {@link quickLoad} aliases of this method.
      *
      * @param mixed $id the id of the object(s) to load. $this->id will be used
      *   if not specified. If evaluates to false, the current object is loaded;
      *   if string or numeric, it loads the object with that id; if array,
      *   it loads the objects with the id's in the respective indexed array.
      * @param boolean $large true to load large fields
      * @param boolean $force true to force loading all attributes from the
      *   database even if some have already been set
      * @param mixed $order_att specifies an order for the result set. Can be
      *   a string, in which case that attribute is used, can be an array
      *   in which case it should be an indexed array with attributes, or it
      *   can be false, in which case the input order of the id's is preserved.
      * @param mixed $reverse specifies whether the order should be reversed.
      *   Can be a boolean, in which case all attributes to order by are (not)
      *   reversed, or it can be an associative array of boolean values, in
      *   which case the keys of the array specify the attributes for which
      *   those boolean values are applied
      * @return mixed false on error; true on success for single objects or
      *   the indexed array of instantiated objects if multiple objects are
      *   desired. In that case the current object is not affected.
      */
      // TODO: $id should accept free-form arrays, and retain the keys
    function _doLoad($id=0, $large=false, $force=true, $order_att=NULL, $reverse=NULL)
    {
      //echo("Load ($large) on object ".get_class($this)." #{$this->id}<br />\n");
      #unset($this->attr);
      #unset($this->attr_flags);
      if (is_array($id) && !count($id)) {
        return(array());
      }
      $id=$id?$id:$this->id;
      if (!$id) {
        $this->addError('ID not specified! (in SC::_doLoad())',
          LFX_DV_BOGDAN+30, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }

      $id_fld=$this->dataStructure['id_field'];
      if (is_array($id)) {
        reset($id);
        while (list(,$myID)=each($id)) {
          $myID=addslashes($myID);
          $SQLid.="`$id_fld`=\"$myID\" OR ";
        }
        $SQLid=substr($SQLid,0,-4);
        //echo("Hey, array! $SQLid<br />\n");
      } else {
        $id=addslashes($id);
        $SQLid="`$id_fld`=\"$id\"";
      }
      $fields=$this->dataStructure['fields'];
      // To check if $fields is an array and issue error otherwise
      reset($fields); // Just to make sure
      while (list($attName,$dataEntry)=@each($fields)) {
        if ((!$large) && ($dataEntry['flags']['large'])) {
          // Skipping large fields if needed
          continue;
        }
        if (!is_array($id) && !$force && $this->attr_flags[$attName]['modified']) {
          // Skipping fields which have already been set if loading
          // just this object, if not forced to load them
          //echo("Field $attName is modified, skipping!\n");
          $newResult->attr[$attName]=$this->attr[$attName];
          continue;
        } elseif (!is_array($id) && $force && $this->attr_flags[$attName]['modified']) {
          unset($this->attr_flags[$attName]['modified']);
        }
        if ($dataEntry['flags']['sqlDate']) {
          $query.=", UNIX_TIMESTAMP(`{$dataEntry['fld_name']}`) AS {$dataEntry['fld_name']}";
        } else {
          $query.=", `{$dataEntry['fld_name']}`";
        }
        if ($dataEntry['flags']['NULL']) {
          // If it may be null, then we have to check
          // for that as well!
          $query.=", ISNULL(`{$dataEntry['fld_name']}`) AS {$dataEntry['fld_name']}ISNULL";
        }
      }
      $query=substr($query, 2);
      $table=$this->dataStructure['table_name'];
      if ((!$query) || (!$table) || (!$id_fld) || (!$id)) {
        $this->addError('Malformed object data structure! (in SC::_doLoad())',
          LFX_DV_BOGDAN+92, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      $sql="SELECT `$id_fld`, $query FROM `$table` WHERE $SQLid";
      if ($order_att) {
        if (is_array($order_att)) {
          $sortSQL=' ORDER BY ';
          for($i=0;$i<count($order_att);$i++) {
            $order_fld=$this->dataStructure['fields'][$order_att[$i]]['fld_name'];
            if (!$order_fld) {
              $this->addError("Unknown attribute to order by ($order_att[$i], in SC::_doLoad())");
            } else {
              $sortSQL.="$order_fld";
              if ((is_array($reverse) && ($reverse[$order_att])) || ($reverse===true)) {
                $sortSQL.=" DESC";
              }
              $sortSQL.=", ";
            }
          }
          $sql.=substr($sortSQL,0,-2);
        } else {
          $order_fld=$this->dataStructure['fields'][$order_att]['fld_name'];
          if (!$order_fld) {
            $this->addError("Unknown attribute to order by ($order_att, in SC::_doLoad())");
          } else {
            $sql.=" ORDER BY $order_fld";
            if ($reverse) {
              $sql.=" DESC";
            }
          }
        }
      }
      //decho("Running LOAD query: <br />$sql<br />");
      $this->query($sql); // Will have to manage errors!
      $result=array();
      $myClass=get_class($this);
      $newResult=NULL;
      //$recordNo=0;
      while($this->db->next_record()) {
        //$recordNo++;
        reset($fields);
        $newResult=new $myClass;
        $newResult->id=$this->db->f($id_fld);
        //echo("Got record #$recordNo with id={$newResult->id} on query <pre>$sql</pre><br />\n");
        if (!is_array($id)) {
          $newResult->attr=$this->attr;
        }
        while (list($attName,$dataEntry)=@each($fields)) {
          if ((!$large) && ($dataEntry['flags']['large'])) {
            // Skipping large fields if needed
            continue;
          }
          if (!is_array($id) && !$force && $this->attr_flags[$attName]['modified']) {
            // Skipping fields which have already been set if loading
            // just this object, if not forced to load them
            continue;
          }
          // Must take care of 'large' fields here!
          if ($dataEntry['flags']['NULL'] && $this->db->f($dataEntry['fld_name'].'ISNULL')) {
            $newResult->attr[$attName]=NULL;
          } else {
            $newResult->attr[$attName]=$this->db->f($dataEntry['fld_name']);
          }
          if (is_array($id)) {
            $newResult->attr_flags[$attName]['loaded']=true;
          } else {
            $this->attr_flags[$attName]['loaded']=true;
          }
          // Taking care of files
          if ($dataEntry['flags']['file'] || $dataEntry['flags']['short_in_db']) {
            if (!$newResult->attr[$attName]) {
              $newResult->attr[$attName]='';
              continue;
            } elseif (!is_numeric(substr($newResult->attr[$attName],0,1))) {
              // Automagically retrieving data which was previously stored in
              // the database; this way you can change only the class definition
              // to use files and leave the database as is; then simply going
              // through all objects, just loading and saving them will convert
              // the required attributes to file storage. After that, all you
              // need to do is alter the database structure to acknowledge the
              // new format (i.e. change the corresponding field types from
              // mediumtext or blobs or whatever to mediumint).
              $newResult->attr[$attName]=$this->parseDbAttr($attName,$newResult->attr[$attName]);
              continue;
            }
            $myFile=$newResult->_getDataFile($attName);
            global $_LFX_globals;
            $fp=@fopen($myFile,'rb');
            if (!$fp) {
              if (!$_LFX_globals->getSetting('inhibit_file_field_errors',NULL,$_SESSION['LFX_project'])) {
                if (!is_file($myFile)) {
                  $newResult->addError("File $myFile for index '".$newResult->attr[$attName].
                    "' corresponding to attribute $attName does not exist! ".
                    "(in SC::_doLoad())");
                } elseif (!is_readable($myFile)) {
                  $newResult->addError("No permissions to read file $myFile ".
                    "for index ".$this->attr[$attName]." corresponding to ".
                    "attribute $attName (in SC::_doLoad())");
                } else {
                  $newResult->addError("Unknown error opening file $myFile for index ".
                    $this->attr[$attName]." corresponding to attribute ".
                    "$attName (in SC::_doLoad())");
                }
              }
              continue;
            }
            clearstatcache();
            if ($fs=filesize($myFile)) {
              $newResult->attr[$attName]=fread($fp,$fs);
            } else {
              $newResult->attr[$attName]='';
            }
            fclose($fp);
            //echo("Loaded file for attribute $attName associated with ".niceClass(get_class($this))." #{$newResult->id} with value ".$newResult->attr[$attName]."<br />\n");
          }
        }
        $result[$newResult->id]=$newResult;
        //echo("Finished block for record #$recordNo<br />\n");
      }
      //echo("Done all $recordNo records.<br />\n");
      if (!$newResult) {
        $this->addError("Object ".get_class($this)."#".$this->id." not found in database! (SC::_doLoad())",
          LFX_DV_BOGDAN+192, LFX_ET_ERROR+LFX_EL_SC);
        return(false);
      }
      //echo("Entering return block<br />\n");
      if (!is_array($id)) {
        $this->id=$id;
        $this->attr=$newResult->attr;
      } elseif ($order_att) {
        return(array_values($result));
      } else {
        // Got to make sure we return the results in the same order as the
        // ID's
        $finalResult=array();
        for($i=0;$i<count($id);$i++) {
          $finalResult[]=$result[$id[$i]];
        }
        return($finalResult);
      }
      // Debugging after loading stuff
      /*
      echo("<hr /><b>Attributes dump after loading:</b>");
      echo("<pre>\n");
      print_r($this->attr);
      echo("</pre><hr />\n");
      */
      return(true);
    }

      /**
      * This method parses an attribute's raw value, removing the trailing
      * character for short_in_db attributes which need to have it removed.
      * The method conditionally returns one of the following:
      * - the value as it was passed for regular attributes
      * - the value without the first character for short_in_db entries which
      *   are stored in the database
      * - false (PHP boolean) when the attribute is stored in the filesystem
      *   (i.e. "short_in_db" attributes stored in database, or "file" attributes).
      *
      * @param string $attr the attribute name
      * @param string $value the attribute value
      * @return mixed the result, as explained above
      */
    function parseDbAttr($attr,$value)
    {
      if ($this->dataStructure['fields'][$attr]['flags']['short_in_db']) {
        // Stripping the first underscore
        // We used to strip the first character unconditionally until
        // version 1.165. We changed that behaviour to only strip it
        // if it's an underscore (or space, as per the old convention),
        // as to accomodate transparent data type changes (e.g. from
        // plain varchar to short_in_db)
        $firstChar=substr($value,0,1);
        if (($firstChar=='_') || ($firstChar==' ')) {
          $value=substr($value,1);
        } else {
          $value=false;
        }
      } elseif ($this->dataStructure['fields'][$attr]['flags']['file']) {
        $value=false;
      }
      return($value);
    }

      /**
      * This returns the data dir for this type of object.
      *
      * @return mixed (string) the path to the data dir or (bool) false on error
      */
    function _getDataDir()
    {
      if ($this->id==0) {
        $this->addError("SC::_getDataFile() called on object with id not set!");
        return(false);
      }
      if (!$this->dataDir) {
        $className=get_class($this);
        if (!LFX_include_class('LFX_Global')) {
          $this->addError('Unable to determine data dir due to include problems');
          return(false);
        }
        $tmp=new LFX_Global();
        $tmp->global_module=$this->module;
        $this->dataDir=$tmp->getSetting('dataDir',NULL,$_SESSION['LFX_project']);
        if (!$this->dataDir) {
          $this->addError("You must define global variable dataDir in ".
            "module {$this->module} before using this data type.");
          return(false);
        }
        $this->dataDir=$this->dataDir.'/'.$className;
      }
      return($this->dataDir);
    }

      /**
      * This method parses a file index into a properly formatted
      * file specification, including the absolute path.
      *
      * @param integer $attr the SC attribute to "fileize"
      * @return mixed (string) the path and filename or (bool) false on error
      */
    function _getDataFile($attr)
    {
      if(!$this->_getDataDir()) {
        return(false);
      }
      $dataDir=$this->dataDir.'_'.$attr;
      $tmpArray=array();
      for($i=0;$i<4;$i++) $tmpArray[]=0;
      $tmpIndex=$this->id;
      $tmpFName='';
      $i=3;
      while(strlen($tmpIndex)>3) {
        $tmpArray[$i]=substr($tmpIndex,-3);
        $tmpIndex=substr($tmpIndex,0,-3);
        $i--;
      }
      $tmpArray[$i]=$tmpIndex;
      $tmpArray[count($tmpArray)-1]='data_'.$this->id;
      return($dataDir.'/'.implode('/',$tmpArray));
    }

      /**
      * Saves all files associated with this object which are present in
      * $fileData. $fileData should be an associative array with keys
      * containing attribute names and values containing the attribute
      * values ONLY for attributes flagged "file".
      * This method must only be called on objects with the id properly set.
      * @param array $fileData the array of attributes to write
      * @return boolean true on success or false on failure
      */
    function _saveFiles($fileData)
    {
      while(list($attName,$attVal)=each($fileData)) {
        $fileName=$this->_getDataFile($attName);
        if (!$fileName) {
          if (get_class($this)!='lfx_message') {
            $this->addError("SC::_saveFiles() called SC::_getDataFile() which failed");
            return(false);
          } else {
            echo("SC::_saveFiles() called SC::_getDataFile() for a log file, and the call failed. Exiting in order to avoid an infinite loop.");
            exit;
          }
        }
        if (@is_file($fileName) && (!@unlink($fileName))) {
          // Gwah! Should save the data in a temp location;
          // will just throw an error instead, this shouldn't happen;
          // the data is "fresh" anyway, so no real data loss should occur.
          // Will implement better management if needed.
          $this->addError("File $fileName already exists; failed deleting it! DATA LOST!");
          return(false);
        }
        if (!strlen($attVal)) {
          continue;
        }
        $tmp=dirname($fileName);
        $tmpCount=0;
        while(!@is_dir($tmp)) {
          $tmpCount++;
          $tmp=dirname($tmp);
        }
        $success=true;
        for($i=$tmpCount;$success && $i;$i--) {
          $tmp=$fileName;
          for($j=0;$j<$i;$j++) {
            $tmp=dirname($tmp);
          }
          $success=$success && @mkdir($tmp,0770);
          if (!$success) {
            if (get_class($this)=='lfx_message') {
              echo("SC::_saveFiles() tried to create directory $tmp and  failed. Exiting in order to avoid an infinite loop.");
              exit;
            }
            $this->addError("Failed to create directory $tmp needed to store field!");
            return(false);
          }
        }
        $fp=@fopen($fileName,'wb');
        if (!$fp) {
          if (get_class($this)=='lfx_message') {
            echo("SC::_saveFiles() tried to open the file $fileName for writing and failed. Exiting in order to avoid an infinite loop.");
            exit;
          }
          $this->addError("Failed to open file $fileName for writing");
          return(false);
        }
        fwrite($fp,$attVal,strlen($attVal));
        fclose($fp);
        $this->setAttr($attName,$this->id);
      }
      reset($fileData);
      $query='';
      while(list($attName,$attVal)=each($fileData)) {
        $query.=$this->getFieldName($attName,true)."=";
        if (strlen($attVal)) {
          $query.=$this->id;
        } else {
          $query.='0';
        }
        $query.=', ';
      }
      $table=$this->dataStructure['table_name'];
      $id_field=$this->dataStructure['id_field'];
      if ($query) {
        // Ok, pretty ugly, but we need to force saving the fields which we
        // need to force saving, unfortunately.
        $fields=$this->dataStructure['fields'];
        reset($fields);
        while (list($attName,$dataEntry)=@each($fields)) {
          if (!$dataEntry['flags']['forceSave'] || isset($fileData[$attName])) {
            continue;
          }
          $fld_name=$dataEntry['fld_name'];
          $fld_data=$this->attr[$attName];
          if ((isset($this->attr_flags[$attName]['loaded'])) && ($fld_data===NULL)) {
            $query.="$fld_name=NULL, ";
          } elseif ($dataEntry['flags']['sqlDate']) {
            $fld_data=$this->sqlDate($fld_data);
            $query.="$fld_name=\"$fld_data\", ";
          } else {
            $fld_data=addslashes($fld_data);
            $query.="$fld_name=\"$fld_data\", ";
          }
        }
        $query=substr($query, 0, -2);
        //echo("Query is $query<br />\n");
        $query="UPDATE $table SET $query WHERE $id_field={$this->id}";
        $this->query($query);
      }
      return(true);
    }
  
    /**
    * Returns {@link getSize} human-friendly formatted.
    */
    function getSizeH($attr) {
      return(niceSize($this->getSize($attr)));
    }
    
      /**
      * Returns the size of the data in the specified attribute.
      * If that attribute is flagged "file" then it doesn't load its data;
      * instead it uses a filesystem filesize() call to retrieve the size.
      * Otherwise the data is loaded from the database and strlen() is
      * returned. So basically this is the most efficient way of finding out
      * the data size of an attribute.
      * Typically used in conjunction with LFXlib's niceSize() for display.
      * @param string $attr the attribute name
      * @return integer the data size, in bytes
      */
    function getSize($attr)
    {
      if
      (
        $this->dataStructure['fields'][$attr]['flags']['file'] ||
        (
          $this->dataStructure['fields'][$attr]['flags']['short_in_db'] &&
          is_numeric($this->getAttr($attr))
        )
      ) {
        $myFile=$this->_getDataFile($attr);
        if (!is_file($myFile)) {
          return(0);
        }
        clearstatcache();
        return(filesize($myFile));
      } else {
        return(strlen($this->getAttr($attr)));
      }
    }
    
      /**
      * This will use a generic database descriptor like $this->dbData
      * to instantiate a PHPlib database.
      *
      * You shouldn't need to call this directly - it's typically called by
      * {@link dbInit} to initialize $this->db.
      *
      * @param array $dbData the database descriptor
      * @return object PHPlib database instance or false on error
      */
    function _doDbInit($dbData)
    {
      if ($dbData['dbDataSet']) {
        global $_LFX;
        $class='DB_PHPLIB';
        if (!isset($_LFX['global']['dbData'][$dbData['dbDataSet']])) {
          $this->addError("Descendant is trying to use dataset {$dbData['dbDataSet']} but that's not defined in LFX_dbData.php!");
        }
        $dbData=$_LFX['global']['dbData'][$dbData['dbDataSet']];
      } elseif ($dbData['class']) {
        $class=$dbData['class'];
      } else {
        $class='DB_PHPLIB';
      }
      $db=new $class();
      if (isset($dbData['host'])) {
        $db->Host=$dbData['host'];
        $db->Database=$dbData['db'];
        $db->User=$dbData['uname'];
        $db->Password=$dbData['pwd'];
      } elseif ((!$dbData['class']) && (!$dbData['host']) && (!$dbData['dbDataSet'])) {
        $this->addError('Database class not defined by LFX_Object descendant! (in SC::_doDbInit()). '.
            'dbData='.(serialize($dbData)),
          LFX_DV_BOGDAN+62, LFX_EL_MAJOR+LFX_EL_SC);
        return(false);
      }
      $db->Halt_On_Error='no';
      return($db);
    }
    
      /**
      * Clones all attributes of this object in a new object with a new class.
      *
      * This can be useful for conditional polymorphism. For instance,
      * class Shape instantiates area() which returns 0,
      * class Triangle extends Shape (and overrides area()),
      * class Square extends Shape (and overrides area()),
      * and then you can use $shape=new Shape; $shape->setAttr(...whatever...),
      * followed by conditionally instantiating descendants with
      * $shape->cloneInClass('Triangle') or $shape->cloneInClass('Square') --
      * the cool thing is that the new object retains all attributes set with
      * setAttr() on $shape -- afterwards, you can call method area() on any
      * of the resulting objects, and get valid results.
      *
      * If no class name is specified, the parent class of this one is used.
      *
      * @param string $className the new class to instantiate
      * @return mixed the instantiated object on success, or false on failure
      */
    function cloneInClass($className)
    {
      if ($className===NULL) {
       $className=get_parent_class(get_class($this));
      }
      if (!LFX_include_class($className)) {
       return(false);
      }
      $clone=new $className;
      $vars=get_object_vars($this);
      while(list($var,$val)=each($vars)) {
       $clone->$var=$val;
      }
      return($clone);
    }
    
  }
?>
