<?

 /**
 * This is the 'Master' class, which will load your report class
 *
 * This class works with data from the 'reports' table.
 *
 * Essentially, it will load your report and will use your report's methods to:
 *
 * 1) Update for every period (look at the array periods from the constructor)
 * 2) 'Render' it, adding a standard header (Report created on ... for period...)
 *
 */


class LFX_report_wrapper extends LFX_Object
{
  var $periods;
  var $instantiated=false;
  var $db_id;

  /**
  * Constructor, additional to all LFX classes, it defines array periods
  */

  function LFX_report_wrapper($id=NULL,$db_id=NULL,$timestamp=NULL){

    $this->LFX_Object($id);
    $this->db_id=$db_id;
    $this->timestamp=($timestamp==NULL)?time():$timestamp;

    // When a report update runs but finds no data, this is its ouptut
    $this->no_data_msg = 'The report ran but there was no data to build the report on.';

    // Define & load datastructure
    $tmp['name']=
    $tmp['description']=
    $tmp['class_name']=
    $tmp['report_start']=
    $tmp['active']=
    $tmp['visible']=
    $tmp['subs_description']=
    $tmp['subs_per_event']=
    $tmp['no_duration_types']=
    $tmp['module']=
    $tmp['spot_report']=
      array();

    $tmp['report_start']['flags']['forceSave'] = true;
    $tmp['report_start']['flags']['sqlDate'] = true;
    $tmp['report_start']['flags']['NULL'] = true;

    $tmp2['generic_reports']=array(
      'class' => 'LFX_Reports_Generic',
       'attr' => 'reports_id'
      );
    $tmp2['param_link']=array(
      'class' => 'LFX_report_parameters_link',
       'attr' => 'report_id'
      );
    $tmp2['report_preset']=array(
      'class' => 'LFX_report_preset',
       'attr' => 'report_id'
      );
    $this->dataStructure=array(
      'table_name' => 'reports',
        'id_field' => 'id',
          'fields' => $tmp,
          'depend' => $tmp2
    );

    $this->fillDataStructure();

    $this->periods = array(
      'day'=>array(),
      'week'=>array(),
      'month'=>array(),
      'year'=>array()
    );
  }

  /**
  * Looks up table 'reports' to find out the name of the class it should
  * instantiate, and tries to instantiate that class
  */

  function onLoad()
  {
    $this->instantiated = FALSE;
    if ($this->id) {
      // Should add some error checking if class cannot be loaded,
      // and also some functionality for dynamically loading the classes
      $my_class = $this->getAttr('class_name');
      LFX_include($my_class);
      //$this->report = new $my_class($id,$db_id);
      if($this->getAttr('spot_report')) {
        $this->report= new $my_class($this->id,$this->db_id);
      } else {
        $this->report = new $my_class();
      }
      // Find the earliset date when this report should run
      $this->earliest = $this->getAttr('report_start');
      if(!$this->earliest){
        echo 'Determining earliest date for report '.$this->getAttr('name')."\n";
        $this->earliest = $this->report->getEarliest();
        echo "earliest is: ".$this->us($this->earliest)."\n";
        $this->setAttr('report_start',$this->earliest);
        $this->save();
      }
      if($this->earliest==0){
        $this->debug('LFX_report_wrapper::onLoad(): '.
          'No data currently available to build the report.',4);
      }
      $this->instantiated = TRUE;
    }
  }

  /**
  * This is the 'main' function of the report wrapper.
  *
  * Purpose: it will write into the reports_data table the reports for the
  * timeperiods needed. If the last time it was run is n days ago, it will update
  * all reports from then until now.
  *
  * How it works:
  *
  * 1. Checks to see whether a report class is instantiated (it needs one)
  *
  * 1a. If $this->earliest is 0, it means there's no data available to build
  *    the report and we exit graciously (this is not an error)
  *
  * 1b. Inactive reports just return TRUE
  *
  * 2. If you are trying to run a report for a date before the date specified in
  *    the report table as the earliest date that this report should run, it won't
  *    let you do that.
  *
  * 3. For every period (as defined in array periods), it does the following:
  *
  *    a. Looks to see whether it has a dataset for the specific report and period
  *       (say, report 'alerts' and period 'day')
  *
  *    b. If it finds no such dataset, or if it finds one but the end timestamp is too old
  *       (e.g. if the report was for a day which ended before the report_start as defined in the
  *       reports table) then it assumes that it has found no valid dataset which
  *       to update and will create a new one. (Jump to point d)
  *
  *    c. If it finds a candidate dataset, it checks to see whether it's not trying to
  *       update it for a period for which it was already updated.
  *       Then, it checks whether the report for that period is "complete"
  *       by looking at the date it was last updated versus the end date of the report.
  *         i. If last_updated is smaller than end_timestamp, it means that we last updated this
  *            report *during* that period, and therefore our dataset is designated as 'work in progress'
  *            and we keep it in $old (because we will update with data from last_updated until end_timestamp)
  *        ii. If the last_updated is greater than end_timestamp, then the report is considered
  *            complete for that period and we start with with the immediately next period.
  *            This means that we make our old datasset NULL. (Jump to point d)
  *
  *    d. We loop and create as many 'reports' until we reach 'today' (whatever 'today' means in the context of
  *       that period--eg this week, month or year)
  *
  *    e. We run the loop one more time for the 'actual' period. If the period is a day,
  *       then we run it from today 00:00:00h until whatever the time is now. If we run it
  *       for a week and it is now Tuesday 14:34:08h, then we query from Monday 00:00:00h until
  *       Tuesday 14:34:08h. You get the point...:)
  *
  * Low-prioirity wish: as things go now, if someone deletes my yearly report, then the class
  * will see that it's missing and will create it. However, it's likely that it won't have the data
  * for the whole year (the data would have been archived.) Therefore, it would be nice if it created
  * the report from the other periods (eg add up days or whatever it has.)
  *
  * Medium-priority: in many cases we receive 'delayed' data, due to the nature of the 'pull' data-collection method
  * It would be nice to then update reports 'retroactively' (e.g. if today I receive data for yesterday, then
  * I should update yesterday's report as well.)
  *
  * @return: TRUE on succes or FALSE on failure.
  */

  function updateReport(){
    // 1 -- See function documentation above
    if(!$this->instantiated){
      $this->addError("Method LFX_report_wrapper::updateReport(): Don't know ".
        "which report to update. Please instantiate the report wrapper with ".
        "a report id (as defined in table reports) and execute method ".
        "load() while checking that the method succeeds.");
      return(FALSE);
    }

    // 1a
    if($this->earliest==0){
      echo $this->report->getAttr('name')." does not have any data to work with.";
      return true;
      exit;
    }

    //1b
    if($this->getAttr('active')==0){
      echo 'Report '.$this->getAttr('name')." is set to inactive and will NOT be updated";
      return(TRUE);
    }

    // 2
    if($this->timestamp < $this->earliest){
      $this->addError("Method LFX_reports_generic::updateReport(): ".
        "You are trying to instantiate a report with a date (".
        $this->us($this->timestamp).") smaller than the earliest start date (".
        $this->us($this->earliest)."), as set in table reports, field ".
        "start_date. Could not update report.");
      return(FALSE);
    }

    // 3
    // BeLooped!--loop counter for debugging
    $bl=1;
    while(list($k,$v)=each($this->periods)){
      $this->debug("<b>LOOP $bl, Period: $k</b>",4);
      $bl++;

      // Will use it to load report datasets
      $data = new LFX_reports_generic;

      // 3a (Look for a dataset)
      $sql = "SELECT id, UNIX_TIMESTAMP( last_updated ) last_updated, UNIX_TIMESTAMP( end_timestamp ) end_timestamp
             FROM reports_data
             WHERE reports_id=".$this->id." AND period='$k'
             ORDER BY start_timestamp DESC LIMIT 1";
      $this->debug("Function updateReport: sql:<br> $sql ",6);
      $this->query($sql);
      $this->db->next_record();
      $id = $this->db->f('id');
      $last = $this->db->f('last_updated');
      $end_time = $this->db->f('end_timestamp');
      $this->debug("id is $id, end_timestamp is ".$this->us($end_time)."<br>"."last updated is ".$this->us($last),4);

      // The force_new flag tells me whether I have to create a new dataset or not
      // Presumption of innocence
      unset($force_new);

      if(($id>0 && $end_time<$this->earliest)||(!($id>0))){
        // 3b (dataset too old or no dataset, make new one)
        $start = $this->earliest;
        $this->debug("Method LFX_report_wrapper::updateReport(): ".
          "Found either a record older than report_start or no records ".
          "in reports_data table, for report \"".$this->getAttr("name").
          "\", period \"$k\"",4);

        $old = NULL;
        $force_new = 1;
      }else{
        // 3c (found candidate dataset)
        $data->load($id);

        // We gotta check see if we don't overwrite an existing dataset
        $this->debug("Comparing wrapper initialization timestamp: ".$this->us($this->timestamp)." with the last_updated date of the
        found report:".$this->us($data->getAttr('last_updated')),4);
        if($this->timestamp<$data->getAttr('last_updated')){
          $this->debug("The \"".$this->getAttr("name")."\" report for $k starting ".$this->us($this->timestamp)."
          was updated earlier than the current timestamp with which you are trying to update it, <B>SKIPPING</B>",4);
          continue;
        }

        $this->debug("Method LFX_report_wrapper::updateReport(): ".
          "Found a valid record (id=$id) for report \"".$this->getAttr("name").
          "\" period \"$k\"",4);


        if(!$data->getAttr('is_complete')){
          // 3ci (work in progress)
          $start = $last;
          $this->debug("Work in progress, last updated is ".$this->us($last),4);

          $old = $data->getAttr("data");
        }else{
          // 3cii (complete dataset, start new one)
          $start = $this->myDates($data->getAttr("end_timestamp"),$k,"end")+1;
          $this->debug("Starting a new $k at ".$this->us($start),4);
          $old = NULL;
          $force_new=1;
        }
      }

      $i=0; // Loop counter

      // Keep loping until we reach the current period
      cecho("Comparing end: ".$this->us($this->myDates($start,$k,"end"))." with now: ".$this->us($this->timestamp),true);

      while(!($this->myDates($start,$k,"end")>$this->timestamp)){
        $i++;
        $this->debug("Running update $i",4);
        if($force_new==1){
          // This happens in cases 3b) or 3cii)
          $data = new LFX_reports_generic;
        }
        $end = $this->myDates($start,$k,"end");

        // Call your report's gatherData method for period starting at $start and ending at $end
        $new = $this->report->gatherData($start,$end);

        // Use your report's addData method to add the new data to the old data
        $mydata = $this->doAddData($old,$new);

        // Set the attributes
        $data->setAttr("reports_id",$this->id);
        $data->setAttr("period",$k);
        $data->setAttr("start_timestamp",$this->myDates($start,$k,"start"));
        $data->setAttr("end_timestamp",$end);
        $data->setAttr("data",$mydata);
        $data->setAttr('is_complete',1);

        // Remember, last_updated is just 'symbolic'
        $data->setAttr('last_updated',$end+1);

        $this->debug('<b>Now we should be saving status!!!!</b>',4);
        $data->save();

        // This is the ONLY place where full updates are made, i.e. the only
        // place where periodical reports are FINISHED (not merely updated).
        // So we'll take this opportunity and also send the associated
        // Per Event subscriptions.
        $data->sendEventSubs();

        // Set the flag to 1, from here on we need a new dataset anyway
        $force_new=1;

        // Moving to the next period, I'm adding one second to the unix
        // timestamp of the current period, which will give me the new period
        $start = $end+1;

        // We need $old only for the first time we run this loop,
        // after that we just set it to null since we do new periods
        $old = NULL;

      }

      // Now we reached the current period, and we run the update one last time
      $this->debug("Executing single update",4);

      if($force_new==1){
        // This happens either directly from case 3cii or when coming from the above loop
        $data = new LFX_reports_generic;
        $data->setAttr("reports_id",$this->id);
        $data->setAttr("period",$k);
        $data->setAttr("start_timestamp",$this->myDates($start,$k,"start"));
        $data->setAttr("end_timestamp",$this->myDates($start,$k,"end"));
      }

      cecho("Setting last updated to: ".$this->us($this->timestamp),true);
      $data->setAttr("last_updated",$this->timestamp);
      $end = $this->timestamp;
      $new = $this->report->gatherData($start,$end);
      $mydata = $this->doAddData($old,$new);
      $data->setAttr("data",$mydata);
      $data->save();
    }
  }

  /**
  * A wrapper around the report's addData, so you don't have to write
  * this into every report.
  *
  */

  function doAddData($old,$new){
    if(($old === NULL) && ($new === NULL)) {
      $this->debug('<b>BOTH ARE NULL</b>',4);
      $this->debug('LFX_report_data::doAddData: Both are NULL, returning NULL<br>',4);
      return(NULL);
    }elseif($old === NULL){
      return($new);
    }elseif($new === NULL){
      return($old);
    }else{
      $mydata = $this->report->addData($old,$new);
      return($mydata);
    }
  }

  /**
  * Calculates start and end timestapms for the periods (as defined in the constructor)
  *
  * Usage: for a given timestamp, it will return the points (start, end or name) corresponding to the
  * period in which the timestamp is.
  *
  * E.g.:
  * $now = time(); // current time
  * myDates($now,"month","start") // returns the unix timestamp corresponding to the star of the current month
  * myDates($now,"year","end") // returns the end of the current year
  * myDates($now,"week","name") // returns the string 13--the number of the week
  * myDates($now,"day","name") // returns the string "Wednesday"--the name of the day
  *
  * @return: unix_timestamp for the desired period for points "start" or "end", string period_name for point "name" or FALSE
  */

  function myDates($timestamp=NULL,$period=NULL,$point=NULL) {
    // If no timestamp is set (which is the normal case), then I just use the current time
    $timestamp=($timestamp==NULL)?$this->timestamp:$timestamp;

    // If you pass aninvalid period, we exit with false
    if(!array_key_exists($period,$this->periods)){
      $this->addError("Period ($period) passed to ".
        "LFX_report_wrapper::myDates() is invalid, it must be in the ".
        "\$this->periods array as defined in class LFX_reports_wrapper.");
      return(FALSE);
    }

    // Accept only valid time points
    if (!($point=="start"||$point=="end"||$point=="name")){
      $this->addError("Point ($point) passed to function myDates is invalid,
      it must be either \"start\" or \"end\".<Br>");
      return(FALSE);
    }

    // Straightforward date-time calculations, consult the corresponding php functions documentation
    $day_of_week = date("w",$timestamp);
    $cd = array (0=>6,1=>0,2=>1,3=>2,4=>3,5=>4,6=>5);
    $day_of_week = $cd[$day_of_week];
    $day_of_month = date("j",$timestamp);

    switch($period){
      case "day":
        switch ($point){
          case "start":
            $date = mktime (0,0,0,date("m",$timestamp),date("d",$timestamp),date("Y",$timestamp));
            return($date);
          case "end":
            $date = mktime (23,59,59,date("m",$timestamp),date("d",$timestamp),date("Y",$timestamp));
            return($date);
          case "name":
            $date = date(j,$timestamp);
            return($date);
        }
      case "week":
        switch ($point){
          case "start":
            $date = mktime (0,0,0,date("m",$timestamp),date("d",$timestamp)-$day_of_week,date("Y",$timestamp));
            return($date);
          case "end":
            $date = mktime (23,59,59,date("m",$timestamp),date("d",$timestamp)+6-$day_of_week,date("Y",$timestamp));
            return($date);
          case "name":
            $date = date(W,$timestamp);
            return($date);
        }
      case "month":
        switch ($point){
          case "start":
            $date = mktime (0,0,0,date("m",$timestamp),1,date("Y",$timestamp));
            return($date);
          case "end":
            $date = mktime (0,0,-1,date("m",$timestamp)+1,1,date("Y",$timestamp));
            return($date);
          case "name":
            $date = date(F,$timestamp);
            return($date);
        }
      case "year":
        switch ($point){
          case "start":
            $date =mktime (0,0,0,1,1,date("Y",$timestamp));;
            return($date);
          case "end":
            $date = mktime (0,0,-1,1,1,date("Y",$timestamp)+1);;
            return($date);
          case "name":
            $date = date(Y,$timestamp);
            return($date);
        }
    }
  }

  /**
  * Prints a string representation of the current timestamp in the format "Monday, 12 February 2003, 12:04:15"
  * @return date string
  */

  function us($unix_timestamp){
    $date = date("l, j F Y, H:i:s",$unix_timestamp);
    return($date);
  }

  /**
  * Deprecated, not used anymore
  *
  * Creates html page with clickable reports, see /reporting/time_reports/browser.php for output
  *
  * Primitive implementation just for testing, user should be able to go back and forth along the timeline
  */

  function reportSummary($date=NULL){
    $date=($date==NULL)?$this->myDates($this->timestamp,"day","start"):$date;
    //echo $this->us($date)."<br>";
    $day_of_month = date("j",$date);
    //echo "day is $day_of_month<br>";
    $month_start = $this->myDates($date,"month","start");
    $sql = "select *, unix_timestamp(start_timestamp) start from reports_data where start_timestamp>='".$this->sqlDate($month_start)."'";
    //echo "$sql<br>";
    $this->query($sql);
    $num = $this->db->num_rows();
    for($i=0;$i<$num;$i++){
      $this->db->next_record();
      $e[$this->db->f("period")][$this->db->f("start")]=$this->db->f("id");
    }
    //echo"<pre>";print_r($e);
    $tmp=array_reverse($this->periods);
    while(list($k,$v)=each($tmp)){
      pecho("$k<br>");
      $found=0;
      while(list($k1,$v1)=@each($e[$k])){
        //pecho("$k1 -> $v1&nbsp;$nbsp;&nbsp;"); //$this->myDates($k1,"start","name")." ";
        pecho("<a href='render.php?id=".$v1."'>".$this->myDates($k1,$k,"name")."</a>&nbsp;&nbsp;");
        $found=1;
      }
      if($found==0){
        pecho("<i>No records found. Make sure you are updating your report.</i>");
      }
      pecho("<hr>");
    }
  }

  /**
  * Creates HTML or PDF for a given datased loaded from reports_data
  *
  * This essenstially calls function LFX_report_warpper::renderHeader and then
  * calls yourReport::renderReport. Your function renderReport receives parameter
  * $db_id, it reads the corresponding record from reports_data and then creates
  * the corresponding HTML or PDF.
  *
  * @return: TRUE on succes or FALSE on failure
  */



  function render($id=NULL,$db_id=NULL,$ver='HTML'){
    // We need to know which report to work with, and generate
    // the corresponding error messages if we don't
    if(isset($this->id)){
      $id=$this->id;
    }
    if($id==NULL){
      $this->addError("Method LFX_report_wrapper::render(): ".
        "Please set a report id to render (as per table reports)");
      return(NULL);
    }
    if($db_id==NULL){
      $this->addError("Method LFX_report_wrapper::render(): ".
        "Please pass a report db_id id to render (as per table reports_data)");
      return(NULL);
    }
    $data = new LFX_reports_generic($db_id);
    $d = $data->getAttr('data');

    // We got FALSE rather than NULL. FALSE means an empty record and
    // LFX_reports_generic will create an error, so we just die here. If we
    // got NULL, then it means that the report ran but there was no data
    // (next if statement, after this)
    if($d == FALSE && $d!==NULL){
      pdie();
    }

    $this->reportHeader($data,$ver);
    if(!$d){
      pecho($this->no_data_msg);
      return(FALSE);
    }
    //echo("Report ID ".$this->id.", db_id $db_id: ");
    if (!$this->report) {
      return(NULL);
    //} else {
      //echo("Report found, rendering.\n");
    }
    // Call the report's own render method
    global $_LFX;
    $_LFX['page']['report_rendering']=true;
    $reportResult=$this->report->renderReport($d);
    $_LFX['page']['report_rendering']=false;

    if ($reportResult) {
      return(true);
    } else {
      return(false);
    }
  }

  function reportHeader($data,$ver){
    $name = $this->getAttr("name");
    if (LFX_getDisposition()=='html') {
      pecho(LFX_echo(
        "<table border=0 width=100%>".
          "<tr>".
            "<td class=number>".
              "<a href='#' onClick='window.print();return false;'>".
                "<b>Print Report</b>".
              "</a>".
            "</td>".
          "</tr>".
        "</table>"
      ));
    }
    pecho(LFX_echo("<br><h4>$name for the ".$data->getAttr("period")." starting ".$this->us($data->getAttr("start_timestamp"))."</h4>"));
    pecho(LFX_echo("Last updated ".$this->us($data->getAttr("last_updated"))."<br>"));
  }

  function getSubscription($subId,$data_id=0,$email_specific=0)
  {
    global $_page;
    $tmp=$_page;
    $_page=new LFX_page();
    $_page->user=$tmp->user;
    if($data_id) {
      $this->db_id=$data_id;
      $this->load($this->id);
    } else {
      $this->load($this->id);
    }
    // Ok, now let's get the last report of this type and with the
    // period ('day', 'week', 'month' or 'year') equal to $subId
    // (Since you're here, check out the freaky if() chain below :)
    if($data_id && $email_specific) {
      $result=$this->report->renderMail();
      $result->subject=$this->getAttr('name');
      return($result);
    }
    if (!$this->query('
      SELECT id
        FROM reports_data
        WHERE reports_id='.$this->id.' AND period=\''.$subId.'\'
        ORDER BY start_timestamp DESC
        LIMIT 1')) {
      $this->addError('Failed to query database for last report (id='.
        $this->id.', subId='.$subId.')');
      $result=false;
    } elseif (!$this->db->next_record()) {
      // The current report hasn't been generated yet - nothing to send
      $result=NULL;
    } elseif ($this->render('',$this->db->f(0))) {
      $result=$_page->content;
    } else {
      // Should be this way, but the reports are messed up and don't
      // return proper status codes...
      // $result=NULL;
      $result=$_page->content;
    }
    $_page=$tmp;
    return($result);
  }

  /*
  * This takes two arrays of the form (count, value) and outputs the weighted
  * average
  */

  function addAverage($arr1=NULL, $arr2=NULL, $precision=NULL)
  {
  	$arr1 = (is_array($arr1))?$arr1:array(0,0);
  	$arr2 = (is_array($arr2))?$arr2:array(0,0);
  	$count = $arr1[0]+$arr2[0];
    if(!($count>0)){
      $result = array(0,0);
      return($result);
    }
    $wa = ($arr1[0]*$arr1[1] + $arr2[0]*$arr2[1])/($count);
    $wa = round($wa,10);
    $result = array($count,$wa);

    return($result);
  }

  /*
  * Returns an indexed array containing the elements of arr2 which are NOT in arr1
  */
  
  function getNewElements($arr1=NULL,$arr2=NULL){
  	$arr1 = (is_array($arr1))?$arr1:array();
  	$arr2 = (is_array($arr2))?$arr2:array();
    while(list($k,$v)=each($arr2)){
      if(!array_key_exists($k,$arr1)){
        $result[$k]=$arr2[$k];
      }
    }
    return($result);
  }
  
  /*
  * Adds the elements of arr2 to arr1
  */
  
  function addNewElements($arr1=NULL,$arr2=NULL){
  	$arr1 = (is_array($arr1))?$arr1:array();
  	$arr2 = (is_array($arr2))?$arr2:array();
    while(list($k,$v)=each($arr2)){
      $arr1[$k]=$v;
    }
    return($arr1);
  }

  function formatPercent($a=NULL,$b=NULL){
    if($b==0||$b==NULL){
      return('n/a');
    }
    $pc = ($a/$b)*100;
    return(round($pc).'%');
  }
  
  function formatDivision($a=NULL,$b=NULL,$precision=NULL){
    if($precision==NULL)$precision=2;
    if($b==NULL){
      return ('n/a');
    }
    $result = round(($a/$b),$precision);
    return $result;
  }

  // This comes from www.php.net--Andrei
  // Based on the other notes given before
  // Sorts an array (you know the kind) by key
  // and by the comparison operator you prefer.

  // Note that instead of most important criteron first, it's
  // least important criterion first.

  // The default sort order is ascending, and the default sort
  // type is strnatcmp.

  // function multisort($array[, $key, $order, $type]...)

  function multisort($array)
  {
     for($i = 1; $i < func_num_args(); $i += 3)
     {
         $key = func_get_arg($i);
       
         $order = true;
         if($i + 1 < func_num_args())
             $order = func_get_arg($i + 1);
       
         $type = 0;
         if($i + 2 < func_num_args())
             $type = func_get_arg($i + 2);

         switch($type)
         {
             case 1: // Case insensitive natural.
                 $t = 'strcasenatcmp($a[' . $key . '], $b[' . $key . '])';
                 break;
             case 2: // Numeric.
                 $t = '$a[' . $key . '] - $b[' . $key . ']';
                 break;
             case 3: // Case sensitive string.
                 $t = 'strcmp($a[' . $key . '], $b[' . $key . '])';
                 break;
             case 4: // Case insensitive string.
                 $t = 'strcasecmp($a[' . $key . '], $b[' . $key . '])';
                 break;
             default: // Case sensitive natural.
                 $t = 'strnatcmp($a[' . $key . '], $b[' . $key . '])';
                 break;
         }

         uasort($array, create_function('$a, $b', 'return ' . ($order ? '' : '-') . '(' . $t . ');'));
     }

     return $array;
  }

}
?>
