<?

  /**
  * LFX_Subscription class
  * Bogdan Stancescu <bogdan@lanifex.com>, March 2003
  * $Id: LFX_subscription.php,v 1.20 2005/08/31 10:00:28 dieter Exp $
  * Use this class to store subscriptions and deliver them
  * @package LFX
  */

  _checkMeOut(__FILE__);

  if (class_exists('lfx_access')) {
    $_noCode=true;
  }
  
  $result=array('LFX_Access', 'LFX_Mail', 'LFX_Subscriber');
  if (!_checkIncludes($result)) {
    return($result);
  }

  class LFX_Subscription extends LFX_Access
  {
    function LFX_Subscription($id=0)
    {
      $this->dbData['dbDataSet']='eh';
      $this->LFX_Object($id);
      
      // == DEFINING ATTRIBUTES ==
      $tmp['name']=
      $tmp['objectClass']=
      $tmp['objectId']=
      $tmp['objectSubId']=
      $tmp['dueDate']=
      $tmp['data_id']=
      $tmp['email_specific']=
        array();
        
      // == DEFINING DEPENDENCIES ==
      $tmp2['subscriber']=array(
        'class'=>'LFX_Subscriber',
        'attr'=>'subscription');
        
      // == DEFINING DATASTRUCTURE ==
      $this->dataStructure=array(
        'table_name'=>'subscription',
        'id_field'=>'id',
        'fields'=>$tmp,
        'depend'=>$tmp2);
        
      $this->fillDataStructure();
    }

      /**
      * Private - checks if a date pattern element matches the value passed.
      * Used by {@link shouldDeliver}. This function should be sent
      * the full element, not the atoms (it manages atom lists itself)
      * Will not issue errors even if the pattern is invalid, because the
      * ONLY truly invalid pattern is * / 0, but nobody will issue that,
      * let's get serious. Apart from that, logically invalid patterns are
      * 61 for minute or 32 for day of the month, but that is the
      * administrator's concern. Well, maybe in a future version...
      * @param string value the value to match
      * @param string pattern the pattern to match against
      * @return boolean true on match or false otherwise
      */
    function _match($value,$pattern) {
      // echo("Trying to match $value to $pattern: ");
      $patt=explode(',',$pattern);
      while(list(,$pattern)=each($patt)) {
        // Yes, it could all be one single if block, but who would
        // want to debug that?
        if (($pattern=='*') || ($pattern==$value)) {
          // echo("It's a match! (pattern was * or exact match)<br>");
          return(true);
        }
        if (((count($dashed=explode('-',$pattern)))==2) &&
        (($value>=$dashed[0]) && ($value<=$dashed[1]))) {
          // echo("It's a match! (pattern was matching range)<br>");
          return(true);
        }
        if (((count($slashed=explode('/',$pattern)))==2) &&
        (($slashed[0]=='*') && ($slashed[1]) && ($value % $slashed[1]==0))) {
          // echo("It's a match! (pattern was modulo match)<br>");
          return(true);
        }
      }
      // echo("No match<br>");
      return(false);
    }

      /**
      * Checks if the subscription should be delivered.
      * Please note this method uses the server's time zone to check,
      * and also that it does NOT care about the last time this subscription
      * has been delivered. It only checks if the CURRENT time matches the
      * crontab-like pattern specified in the dueDate attribute.
      * The pattern follows the same format as crontab, only leaving out the
      * minutes section. So then, the format is a space separated string of four
      * elements: the hour, day of month, month, day of week.
      * The following values are valid:
      * - comma separated list of atoms (no spaces!)
      * - numeric atom: 1, 4, 30 are all valid atoms
      * - wildcard atom: * matches anything
      * - range atom: 2-30 will match all values between 2 and 30 INCLUDING 2 and 30
      * - modulo atom: * / 5 (no spaces -- added because * / would end this comment)
      *                this will match all multiples of 5 (0, 5, 10, 15 etc)
      * @param integer id the id of the subscription to check, or 0 for the current one
      * @return mixed true if due to deliver, NULL if not or false on error.
      */
    function shouldDeliver($id=0)
    {
      $sub=&$this->defaultObject($id);
      $now=getdate();
      $due=$sub->getAttr('dueDate');
      if (count($dueX=explode(' ',$due))!=4) {
        $this->addError('Bad format for date specification in this object: '.$due);
        return(false);
      }
      $due=array('hours'=>$dueX[0], 'mday'=>$dueX[1], 'mon'=>$dueX[2], 'wday'=>$dueX[3]);
      // echo("Due: ".serialize($due)."<br>");
      // echo("Now: ".serialize($now)."<br>");
      return(($this->_match($now['hours'],$due['hours'])) &&
             ($this->_match($now['mday'],$due['mday'])) &&
             ($this->_match($now['mon'],$due['mon'])) &&
             ($this->_match($now['wday'],$due['wday'])));
    }

      /**
      * This method does everything related to this subscription:
      * checks if it should be delivered and delivers it.
      * You may deliver a subscription even if it's not due, by calling
      * this function with the force parameter set to true.
      * In this version of this class, the subscription is only delivered
      * by mail, in a single format. In future versions, per user and
      * per subscription delivery types will be implemented.
      * @param boolean force true to deliver relardless of date or false
      *   to only deliver on due dates.
      * @return mixed true if delivered, NULL if not due or false on error.
      */
    function digestDeliver($id=0, $force=false)
    {
      $sub=&$this->defaultObject($id);
      // echo("Checking if sub {$this->id} should be delivered<br>");
      if ($force || ($sub->getAttr('dueDate')=='25 * * *') || ($sub->shouldDeliver())) {
        // echo("Delivering by mail sub {$this->id}<br>");
        return($sub->mailDeliver());
      } else {
        return(NULL);
      }
    }
    
      /**
      * The method to deliver an event subscription
      * @param mixed message is one of the following:
      *   - a string:
      *       the string is delivered as HTML to the subscribers
      *   - an LFX_Mail object:
      *       the address(es) are assigned and the message is delivered as
      *       received by this method
      *   - an LFX_Object descendant:
      *       method getEventSubscription() is called on the respective object
      *       for every user the subscription has to be delivered to.
      *       The LFX_Subscription class makes sure to set the id of global
      *       $_page->user to the user the message is being delivered to,
      *       and it also sets the proper internationalization settings.
      *       That way the message can be rendered using translation functions
      *       for each subscriber to receive in their preferred language.
      * @return boolean true if delivered or false on error
      */
    function deliver($message)
    {
      global $_LFX_globals;
      $this->message=$message;
      if ($_LFX_globals->getSetting('send_sms')) {    
        $this->smsDeliver($message);
      }
      $this->mailDeliver();
    }

    /**
     * Delivering sms messages for all subscribers 
     * @param HD_request/HD_Process request/process that caused notification sending
     * return 
     */
    function smsDeliver($obj)
    {
      switch (strtolower(get_class($obj))) {
        case "hd_request":
          $request = $obj;
          $process = $request->getObject('process');
          $requests = $process->getMyRequests();
          $firstRq = $requests[0];

          $forms = $firstRq->getObjects('forms');
          for ($j = 0; $j < count($forms); $j++) {
            $form_data = &$forms[$j];
            $bp_form = $form_data->getObject('form');
            $data = $form_data->getData();
          }
          $description = addslashes($data[4]);	//correspodent incident description
          $sensor = $data[2];

          //subscription object
          $sub = &$this->defaultObject($id);
          $subscribers = $sub->getObjects('subscriber');

          for ($i = 0; $i < count($subscribers); $i++) {
            //$user is of type LFX_User
            $user = $subscribers[$i]->getObject('user');
            $exec_str = "/home/httpd/EH-Master/SNTlib/sms_send " . $request->getAttr('process') . " " . $request->id . " " . $user->id . " " . $user->getAttr('uname') . " \"" . $user->getAttr('mphone') . "\" " . "\"$description\"" . " " . $sensor . " \"" . $request->getAttr('title') . "\"" . " " . $request->getAttr('escalation_level');
            exec($exec_str);
          }
          break;

        case "hd_process":
            $process = $obj;
            $requests = $process->getMyRequests();
            $firstRq = $requests[0];

            $forms = $firstRq->getObjects('forms');
            for ($j = 0; $j < count($forms); $j++) {
              $form_data = &$forms[$j];
              $bp_form = $form_data->getObject('form');
              $data = $form_data->getData();
            }
            $description = addslashes($data[4]);	//correspodent incident description
            $sensor = $data[2];

            //subscription object
            $sub = $process->mySubscription();
            $subscribers = $sub->getObjects('subscriber');
            for ($i = 0; $i < count($subscribers); $i++) {
              //$user is of type LFX_User
              $user = $subscribers[$i]->getObject('user');
              $exec_str = "/home/httpd/EH-Master/SNTlib/sms_send " . $process->id . " " . $firstRq->id . " " . $user->id . " " . $user->getAttr('uname') . " \"" . $user->getAttr('mphone') . "\" " . "\"$description\"" . " " . $sensor . " \"" . $firstRq->getAttr('title') . "\"" . " " . $firstRq->getAttr('escalation_level');
              exec($exec_str);
            }
          break;
      }
    } 


      /**
      * Delivers subscription via e-mail.
      * Check {@link getMessage} to see what the delivered message is, how to
      * set it for your objects and how to override it if you want to send
      * custom messages.
      * @param integer $id the id of the subscription to deliver or 0 for the current
      * @return boolean true if delivered, NULL if not needed
      *     (e.g. if the getSubscription method on the associated object returned false)
      *     or false on error
      */
    function mailDeliver($id=0)
    {
      global $_LFX_globals,$_LFX_charset,$_page;
      $sub=&$this->defaultObject($id);
      $mail=$sub->getMessage();
      if (($mail===NULL) || ($mail===false)) {
        $this->debug('LFX_Subscription::mailDeliver() called LFX_Subscription::getMessage() which failed to provide a proper message.');
        return($mail);
      }
      $myAddresses=$sub->getAddresses();
      $tmpUser=new LFX_User;
      $tmpUser->id=$_page->user->id;
      if ($preflang=$_LFX_globals->getAttr('system_language')) {
        $tmpUser->setAttr('preflang',$preflang);
      } else {
        $tmpUser->setAttr('preflang',$_page->user->getAttr('preflang'));
      }
      $success=true;
      // This is not the nicest way to do it because the message is rendered once
      // for every user, instead of once for every language. But it can be put
      // to good use in the future, since the actual user who receives the
      // message can be determined using $_page->user, so very
      // customized messages can be sent, according to user permissions.
      while(list(,$subscriber)=@each($myAddresses)) {
        if (is_object($subscriber)) {
          $subscriber->setIntl();
          $_page->user->id=$subscriber->id;
        } else {
          $tmpUser->setIntl();
          $_page->user->id=$tmpUser->id;
        }
        $mail=$sub->getMessage();
        $mail->textCharset=$_LFX_charset;
        $mail->htmlCharset=$_LFX_charset;
        $mail->addresses=array($subscriber);
        $success=$success && $mail->send();
      }
      $_page->user->id=$tmpUser->id;
      return($success);
    }

      /**
      * Returns the e-mail addresses associated with this subscription
      * as required by the LFX_Mail() class (i.e. LFX_User instances or
      * strings where LFX_User instances are not available)
      * @param integer id the id of the subscription to get the addresses for
      * @return array the addresses
      */
    function getAddresses($id=0)
    {
      $sub=&$this->defaultObject($id);
      $addresses=array();

      $subscribers=$sub->getObjects('subscriber');

      for($i=0;$i<count($subscribers);$i++) {
        $addresses[]=$subscribers[$i]->getAddress();
      }
      return($addresses);
    }

      /**
      * Returns the subscription message associated with this subscription,
      * either from $this->message if available or by running the associated
      * object's getSubscription() method.
      * @param integer in the id of the subscription to return the message for
      * @return object LFX_Mail object with all except the email addresses set correctly,
      *     false on error or NULL if that's what the associated object's getSubscription()
      *     method returned (i.e. subscription delivery not needed or n/a)
      */
    function getMessage($id=0)
    {
      $sub=&$this->defaultObject($id);
      if ($sub->message) {
        $message=$sub->message;
      } else {
        if (!LFX_include_class($sub->getAttr('objectClass'))) {
          $this->addError("Failed to include class ".
            $sub->getAttr('objectClass').
            " for this subscription for delivery! Skipping subscription.");
          return(false);
        }
        $objClass=$sub->getAttr('objectClass');
        $obj=new $objClass($sub->getAttr('objectId'));
        $message=$obj->getSubscription($sub->getAttr('objectSubId'),$sub->getAttr('data_id'),$sub->getAttr('email_specific'));
      }
      if ($message===NULL) {
        $this->debug("No message to deliver in ".
          "LFX_Subscription::getMessage(). Returning NULL result.");
        return(NULL);
      }
      if (is_string($message)) {
        $result=new LFX_Mail;
        $result->subject=$sub->getAttr('name');
        $result->html=$message;
        return($result);
      } elseif (is_object($message) && (get_class($message)=='lfx_mail')) {
        return($message);
      } elseif (is_object($message) && ($this->getBaseClass($message)=='lfx_object')) {
        $myClass=get_class($this);
        $tmpObject=new $myClass;
        $tmpObject->message=$message->getEventSubscription($this->getAttr('objectSubId'));
        return($tmpObject->getMessage());
      } else {
        $this->addError("Unknown message to deliver! Message must be either ".
          "a string or a LFX_Mail object, and it's neither. ".
          "(in LFX_Subscription::getMessage())");
        return(false);
      }
    }
    
      /**
      * Checks whether the user specified, or the current user if no user specified,
      * is subscribed to this subscription.
      * You may also specify an e-mail address to subscribe, but be warned that
      * string comparison is performed when checking if the email is subscribed,
      * so you might potentially end up with false negatives if you specify it
      * as "foo@bar.com" when "Foo Bar foo@bar.com" is subscribed.
      * @param integer user the user to check
      * @return boolean true if subscribed or false if not
      */
    function isSubscribed($user=NULL)
    {
      if ($user===NULL) {
        global $_page;
        $user=$_page->user->id;
      }
      $tmp=new LFX_Subscriber;
      if (is_numeric($user)) {
        return(0!=count($tmp->search(array('subscription','user'),array($this->id,$user))));
      } else {
        return(0!=count($tmp->search(array('subscription','email'),array($this->id,$user))));
      }
    }

      /**
      * Subscribes the whole group specified. If the second argument evaluates
      * true then only the users in the respective group will be subscribed.
      * If it evaluates false or if it's not specified then all users
      * in or under this group are added recursively.
      *
      * @param mixed group the group ID or the instantiated LFX_Group object
      * @param boolean notRecursive inhibit recursive searching for users
      * @return boolean true on success or false on failure
      */
    function groupSubscribe($group,$notRecursive=false)
    {
      if (is_numeric($group)) {
        $group=new LFX_Group($group);
      } elseif (is_object($group)) {
        if (get_class($group)!='lfx_group') {
          $this->addError('LFX_Subscription::groupSubscribe() has been called with '.
            'an instantiated object of the wrong class as a parameter');
          return(false);
        }
      } else {
        $this->addError('LFX_subscription::groupSubscribe() has been called with '.
          'an unknown parameter type ('.gettype($group).')');
        return(false);
      }
      if ($group->getAttr('email')) {
        return($this->subscribe($group->getAttr('email')));
      }
      $users=$group->getGroupMembersID();
      $ok=true;
      for ($i=0;$i<count($users['user']);$i++) {
        if ($this->subscribe($users['user'][$i][0])===false) {
          $this->debug('LFX_Subscription::groupSubscribe() called '.
            'LFX_Subscription::subscribe() which failed');
          $ok=false;
        }
      }
      return($ok);
    }
    
      /**
      * Subscribes the current user, or the user specified if any,
      * to this subscription.
      * You may also specify an e-mail address to subscribe, but be warned that
      * string comparison is performed when checking if the email is subscribed,
      * so you might potentially end up with an e-mail being subscribed twice
      * if you specify it once as "foo@bar.com" and once as "Foo Bar foo@bar.com".
      * @param mixed user the user id, the instantiated user object
      *   or the email address to subscribe
      * @return boolean true if subscribed, NULL if user already subscribed or false on error
      */
    function subscribe($user=NULL)
    {
      if ($user===NULL) {
        global $_page;
        $user=$_page->user->id;
      } elseif (is_object($user)) {
        if (get_class($user)!='lfx_user') {
          $this->addError("You passed a ".niceClass(get_class($user)).
            " object LFX_Subscription::subscribe() ".
            "instead of a LFX_User object or id.");
          return(false);
        }
        $user=$user->id;
      }
      if($this->isSubscribed($user)) {
        return(NULL);
      }
      $tmp=new LFX_Subscriber();
      $tmp->setAttr('subscription',$this->id);
      if (is_numeric($user)) {
        $tmp->setAttr('user',$user);
      } else {
        $tmp->setAttr('email',$user);
      }
      $tmp->save();
      return(true);
    }

      /**
      * Unsubscribes the current user, or the user specified if any,
      * from this subscription.
      * @param integer user the user id to subscribe
      * @return boolean true if unsubscribed, NULL if user already unsubscribed or false on error
      */
    function unsubscribe($user=NULL)
    {
      if ($user===NULL) {
        global $_page;
        $user=$_page->user->id;
      }
      $tmp=new LFX_Subscriber;
      $tmp=$tmp->search(array('subscription','user'),array($this->id,$user));
      if (!$tmp) {
        return(NULL);
      }
      $tmp[0]->delete();
      return(true);
    }
    
    function getFormattedAttr($att_name)
    {
      $attVal=$this->getAttr($att_name);
      if (($att_name=='dueDate') && ($attVal=='25 * * *')) {
        return(__L('Per Event'));
      }
      return($attVal);
    }
  }

?>
