<?
  /**
  * "The" LFX User class
  * Andrei Tinca <andrei@lanifex.com> (groups)
  * Bogdan Stancescu <bogdan@lanifex.com> (infrastructure & authentication)
  * Lucian Silistru <lucian@moongate.ro> (maintenance)
  * December 2002
  * $Id: LFX_user.php,v 1.62 2005/11/08 10:31:46 bogdan Exp $
  * @package LFX
  */

  _checkMeOut(__FILE__);

  $result=array('LFX_Object', 'LFX_Access', 'LFX_User_Group');
  if (!_checkIncludes($result)) {
    return($result);
  }

  class LFX_User extends LFX_Access
  {

    /**
    * Session timeout in seconds
    * This is the system default; it can be overridden by setting gllobal
    * session_timeout_seconds in module LFX to any other non-null value.
    */
    var $sessionTimeout=900;
    var $includeKey='LFX_User';

    /**
    * This is a caching variable, and holds boolean true or false since
    * after the first call to LFX_User::{@link isSuperuser}(). Use
    * LFX_User::{@link clearPermissionCache}() to clear all permission caching
    * variables for this object.
    */
    var $isSuperuserCache;

    /**
    * This is a caching variable, and holds an associative array of the form
    * [group name][access]=boolean, thus caching the result of function
    * LFX_User::{@link getRights}(). Use
    * LFX_User::{@link clearPermissionCache}() to clear all permission caching
    * variables for this object.
    */
    var $myDigestedAccessList=array();

    /**
    * This is a caching variable, and holds the array returned by
    * LFX_User::{@link access_list}(). Use
    * LFX_User::{@link clearPermissionCache}() to clear all permission caching
    * variables for this object.
    */
    var $myAccessList=array();
    var $fullACL=array();

    function LFX_User($id=0)
    {
      $this->LFX_Access($id);
      $tmp['fname']=
      $tmp['lname']=
      $tmp['mname']=
      $tmp['sal']=
      $tmp['position']=
      $tmp['company']=
      $tmp['email']=
      $tmp['addr']=
      $tmp['mphone']=
      $tmp['wphone']=
      $tmp['fax']=
      $tmp['hphone']=
      $tmp['uname']=
      $tmp['pwd']=
      $tmp['preflang']=
      $tmp['lastlog']=
      $tmp['settings']=
      $tmp['iscontact']=
      $tmp['status']=
      $tmp['prevlog']=
      $tmp['notes']=
      $tmp['ispayment']=
      $tmp['deactivated']=
      $tmp['date_created']=
      $tmp['external_provider']= 
      array();
      
      // Defining FLAGS
      $tmp['date_created']['flags']['sqlDate']=true;

      $tmp['lastlog']['flags']['sqlDate']=true;
      $tmp['lastlog']['glags']['forceSave']=true;

      $tmp['prevlog']['flags']['sqlDate']=true;
      $tmp['prevlog']['glags']['forceSave']=true;

      // Defining DEPENDENCIES
      $tmp2['user_group']=array(
        'class'=>'LFX_user_group',
        'attr'=>'child_user',
        'on_mod'=>'MISTRESS');
      $tmp2['files']=array(
        'class'=>'LFX_File',
        'attr'=>'src');
      $tmp2['logs']=array(
        'class'=>'LFX_Log',
        'attr'=>'user');
      $tmp2['subscriptions']=array(
        'class'=>'LFX_Subscriber',
        'attr'=>'user');
      $tmp2['forum_messages']=array(
        'class'=>'LFX_Forum_Message',
        'attr'=>'user');
      $tmp2['forum_message_notifs']=array(
        'class'=>'LFX_Forum_Message_Notif',
        'attr'=>'notif_person');
      $tmp2['personal_global']=array(
        'class'=>'LFX_Personal_Global',
        'attr' =>'user');
      $tmp2['message']=array(
        'class'=>'LFX_Message',
        'attr' =>'user');
      $tmp2['file_notification']=array(
        'class'=>'LFX_File_Notif',
        'attr'=>'notif_person');

      $this->dataStructure=array(
        'table_name' => 'people',
          'id_field' => 'id',
            'fields' => $tmp,
            'depend' => $tmp2,
       'object_page' => '/access_manager/show_user?id=%d'
      );

      $this->fillDataStructure();
      $this->showAttr=array(
        'attributes' => array(
          'fname' => array(
            'header' => 'First Name'
          ),
          'lname' => array(
            'header' => 'Last Name'
          ),
          'email' => array(
            'header' => 'E-mail'
          ),
          'iscontact' => array(
            'header' => 'Contact'
          )
        )
      );

      global $_LFX_globals;
      if ((int) $_LFX_globals->getSetting('session_timeout_seconds')) {
        $this->sessionTimeout=
          (int) $_LFX_globals->getSetting('session_timeout_seconds');
      }
    }

    function beforeSave($new)
    {
      if ($new) {
        $this->setAttr('date_created',time());
      }
      // Enforcing unique uname
      $sql = "SELECT count(*) FROM people WHERE uname='".$this->getAttr('uname')."'";
      if(!$new){
        $sql .= " AND id!=".$this->id;
      }
      $obj = new LFX_Object;
      $obj->query($sql);
      $obj->db->next_record();
      if($obj->db->f(0)){
        $this->addError(__L("Duplicate user name. User names should be unique system-wide."));
        return false;
      }
      return(true);
    }
    
      /**
      * This returns the ID's of the groups to which the user belongs, non-recursive.
      * Optionally, the groups can be filtered by type, and the extra info
      * which identified the access rights per group can be stripped via the
      * $simple parameter.
      *
      * Depending on whether the $simple parameter evaluates false or true,
      * the result is either a "complete" array of the form
      * array(array(<group_id>,<access>),array(<group_id>,<access>),...)
      * or a "simple" array of the form
      * array(<group_id>,<group_id>,...)
      *
      * @param string $type the type of the groups to filter by
      * @param boolean $simple whether to strip the access data
      * @param integer $id the id of the user to return the membership info for
      * @return mixed array if ok or false on error
      */
    function getUserGroupsID($type=NULL,$simple=false,$id=0)
    {
      if (!$id) {
        $id=$this->id;
      }
      $id=abs($id);
      if (!$id) {
        $this->addError("User groups required for a user with no ID set!");
        return(false);
      }
      $tug=new LFX_User_Group;
      $ugt=$tug->getTableName();
      if ($type) {
        $tg=new LFX_Group;
        $gt=$tg->getTableName();
        $colGroupsSQL="
          SELECT
            ug.parent_group_id id,
            ug.access access
          FROM
            $gt g,
            $ugt ug
          WHERE
            g.id=ug.parent_group_id AND
            ug.child_user_id=$id AND
            g.type='".addslashes($type)."'
        ";
      } else {
        $colGroupsSQL="
          SELECT
            ug.parent_group_id id,
            ug.access access
          FROM
            $ugt ug
          WHERE
            ug.child_user_id=$id
        ";
      }
      if (!$this->query($colGroupsSQL)) {
        // SC::query() will complain
        return(false);
      }
      $colGroups=array();
      while ($this->db->next_record()) {
        if ($simple) {
          $colGroups[]=$this->db->f('id');
        } else {
          $colGroups[]=array($this->db->f('id'),$this->db->f('access'));
        }
      }
      return($colGroups);
    }

      /**
      * This returns the ID's of the groups to which the user belongs,recursive
      * (descends into groups)
      * @return Indexed array of the same form as function accessList
      */
    function getUserGroupsIDRecursive($id=0)
    {
      $user=&$this->defaultObject($id);
      if ($user->groupsIDRecursive) {
        return($user->groupsIDRecursive);
      }
      $access_list = $user->accessList($id);
      $groups = array();
      while(list($k,$v) = each($access_list)){
        $groups[] = $v;
      }
      $user->groupsIDRecursive=$groups;
      return($groups);
    }

    /**
    * Checks whether the user is active or not
    * @return true if user is active and false if the user is deactivated
    */
    function isActive($id=0)
    {
      $user=&$this->defaultObject($id);
      return(!$user->getAttr('deactivated'));
    }
    
      /**
      * Checks the parent groups of the user, and then recurses into all the other
      * groups which parent the user's parent.
      * Returns an array with group id's and access levels for the corresponding
      * group.
      * @return array of the form $list = array(0=>array(group_id,access_level),
      *                                         .
      *                                         .
      *                                         n=>array(group_id,access_level))
      */
    function accessList($id=0,$lvl=1)
    {
      static $groupMapping, $currentAccessList;
      //$lvl=$level;
      // If we are at first level then there is no level set at this time.
      // This is the only time the entity for which we are trying to determine
      // the rights is the base user and not an including group.
      // Here we check if the user is a Superuser. If he is we return
      // full rights to all groups.
      if ($lvl==1) {
        $user=&$this->defaultObject($id);
        if ($user->myAccessList) {
          return($user->myAccessList);
        }
        $id=$user->id;
        $groupMapping=array();
        $currentAccessList=array();
        if ($user->isSuperuser()) {
          $groups = new LFX_Group;
          $groups = $groups->searchID(NULL,NULL,'name');
          while(list(,$gid) = each($groups)) {
            $access_list[] = array($gid,'*');
          }
          $user->myAccessList = $access_list;
          return($access_list);
        } else {
          // We'll need to continue. We set the level to 1
          $lvl=1;
        }
      }
      //echo(vardump($groupMapping));
      // If first level then we are determinig rights for a user, so $attr='child_user'
      switch ($lvl) {
        case 1:
          $attr='child_user';
          break;
        default:
          $attr='child_group';
          break;
      }

      // Getting list of memberships for current id
      $link=new LFX_User_Group;
      $links=$link->search($attr,$id);

      // For each of the memberships
      while(list($k,$v)=@each($links)) {
        // We get the parent group and access to it
        if (!$group=$v->getObject('parent_group')) {
          continue;
        }
        $tmp_acc=$v->getAttr('access');

        // If the group is active then we calculate rights
        if ($group->isActive()) {
          // If this is the first level ($id belongs to a user)
          // there is no need to perform the intersections. We simply add the access.
          if ($lvl==1) {
            $acc=$tmp_acc;
            //echo("simple adding: $acc<br />\n");
          } else {
            // Otherwise, we find the access the base user has on the current group
            // We found it before as it is on a lower level
            $tmp_list=array();
            foreach ($currentAccessList as $key => $value) {
              $tmp_list[$key]=$value[0];
            }
            $access_idx=array_search($id,$tmp_list);
            unset($tmp_list);
            $parentAccess=strtoupper($currentAccessList[$access_idx][1]);
            $childAccess=strtoupper($tmp_acc);
            $acc=$this->permsFilter($childAccess,$parentAccess);
          }
          if(isset($groupMapping[$group->id])) {
            $tmp1=$currentAccessList[$groupMapping[$group->id]][1];
            $currentAccessList[$groupMapping[$group->id]][1]=LFX_permsMerge($acc,$currentAccessList[$groupMapping[$group->id]][1]);
          } else {
            $currentAccessList[] = array($group->id,$acc);
            $groupMapping[$group->id]=count($currentAccessList)-1;
          }
        }
        $currentAccessList=$this->accessList($group->id,$lvl+1);
      }
      $lvl--;
      if (!$lvl) {
        $user->myAccessList=$currentAccessList;
      }
      return($currentAccessList);
    }

      /**
      * Checks whether user $id has right $access to group called $group_name
      *
      * See also: LFX_User::{@link rightsToGroup}()
      *
      * @return TRUE if user has access, or FALSE if not
      */
    function getRights($group_name,$access='R',$id=0)
    {
      //echo("Checking for [$access] rights on group $group_name.<br />\n");
      if (!$this->id) {
        return(false);
      }
      if ($this->isSuperuser()) {
        return(true);
      }
      $user=&$this->defaultObject($id);

      if (isset($user->myDigestedAccessList[$group_name][$access])) {
        return($user->myDigestedAccessList[$group_name][$access]);
      }
      $access_list=$this->accessList();

      LFX_include('LFX_Group');
      $g = new LFX_Group;
      $group_id = $g->searchID('name',$group_name);
      if (!$group_id = $group_id[0]) {
        $user->myDigestedAccessList[$group_name][$access]=false;
        return(false);
      }

      while (list(,$v) = each($access_list)) {
        if($group_id == $v[0]) {
          //echo("Found $group_name with $v[1].<br />\n");
          //                (!$access) - for simple membership tests;
          // LFX_permsFilter($v[1],$access) - for proper access level tests.
          if ((!$access) || LFX_permsFilter($v[1],$access)) {
            $user->myDigestedAccessList[$group_name][$access]=true;
            return(true);
          }
          break;
        }
      }
      $user->myDigestedAccessList[$group_name][$access]=false;
      return(false);
    }

    /**
    * Works like getRights but with a group_id instead of a group_name.
    *
    * @return TRUE if the user with the id $id has the access level $access in
    * the group with id $id.
    */
    function getRightsID($group_id,$access="R",$id=0)
    {
      $user=&$this->defaultObject($id);

      // If the user belongs to the Supergroup then we grant access no matter what
      if($user->isSuperuser()) {
        return(true);
      }

      $access_list = $user->accessList($id);

      while (list(,$v) = each($access_list)) {
        if($group_id == $v[0]) {
          $rights[] = $v[1];
          if (strstr($v[1],$access) || (strstr($v[1],'W') && ($access=='U' || $access=='C'))) {
            return(TRUE);
          }
        }
      }

      if (sizeof($rights)==0) {
        return(FALSE);
      }
    }

    /**
    * Checks whether the user belongs to the Superuser group
    * @return TRUE if the user is a member of the superuser group.
    */
    function isSuperuser($id=0)
    {
      $user=&$this->defaultObject($id);
      if (isset($this->isSuperuserCache)) {
        return($this->isSuperuserCache);
      }
      $groups=$user->getUserGroupsID();
      for ($i=0;$i<count($groups);$i++) {
        $groups[$i]=$groups[$i][0];
      }
      $this->isSuperuserCache=in_array(1,$groups);
      return($this->isSuperuserCache);
    }

      /**
      * Clears all permission caches on this object. See the variables defined
      * in this class for a comprehensive list.
      * @return boolean true
      */
    function clearPermissionCache()
    {
      unset($this->isSuperuserCache,$this->myDigestedAccessList,$this->myAccessList);
      return(true);
    }
    
      /**
      * Makes $this the current user, by also performing the authentication.
      *
      * Please don't confuse this with {@link authenticate}. This one performs
      * the invisible part of the authentication you don't have to worry about.
      * That one is a more techinal part (which you don't need to use), and that's
      * the one which actually receives the username and the password and checks
      * those against the database.
      *
      * This function will also set $this->id to the authenticated user ID.
      *
      * @return integer the id of the current user or false if user failed to
      *   authenticate. Well, actually you never get to receive any result if the
      *   user failed to authenticate. :)
      */
    function current()
    {
      if ($_SESSION['LFX_user_id']) {
        if (time()>$_SESSION['LFX_user_lasthit']+$this->sessionTimeout) {
          $_SESSION['LFX_user_id']=0;
        }
        $_SESSION['LFX_user_lasthit']=time();
      }
      
      // We first check if a user class is defined for this module.
      // If there is, then we give it complete control over the login
      // process. Mind you, *complete* control, it has to manage everything
      // except updating the session timeouts. But if that's also needed then
      // it can also override that by setting session variables itself,
      // please note that although the code above sets the session variables
      // needed for session management, the custom class code is executed below
      // nevertheless. Therefore that code can easily store its own data in the
      // session and use it to revert a timed-out user to logged in or vice-versa.
      
      // If the module's user class doesn't exist, or if it returns false,
      // global alternate_login_classes is looked up; this is supposed to
      // contain a comma-separated list of classes. If this global is available,
      // those classes are instantiated, and their "current()" method is executed,
      // with the same expected results as for the module user class.
      
      // Please note however that if all custom login fail, authentication
      // falls back to LFXlib. In other words, your module can NOT inhibit
      // specific LFXlib users login, they WILL log in if you use the same login
      // form and the username/passwords match with a known LFXlib user. If you
      // want to restrict access to your module you'll have to do that via
      // LFXlib permissions -- groups or whatever. An ugly hackish solution
      // would also be to use other variables in your custom login form, but
      // that's not recommended.
      global $_LFX;
      $altUser=$_LFX['global']['application'].'_user';
      if (($_LFX['global']['application']!='LFX') && (class_exists($altUser))) {
        $tmp=new $altUser();
        if ($current=$tmp->current()) {
          $this->load($current);
          $this->setIntl();
          return($current);
        }
      }

      // The module class doesn't exist or it returned false; let's check
      // using the alternate login classes.
      global $_LFX_globals;
      if ($alt=$_LFX_globals->getSetting('alternate_login_classes')) {
        $alt=explode(',',$alt);
        for($i=0;$i<count($alt);$i++) {
          $curAlt=trim($alt[$i]);
          if (!LFX_include_class($curAlt)) {
            continue;
          }
          $tmp=new $curAlt;
          if ($current=$tmp->current()) {
            $this->load($current);
            $this->setIntl();
            return($current);
          }
        }
      }
      
      // Ok, there is no custom user login management, falling back to LFXlib
      // default.
      if (!$_SESSION['LFX_user_id']) {
        if ($_POST['username'] && $_POST['password']) {
          if ($me=$this->authenticate($_POST['username'],$_POST['password'])) {
            $_SESSION['LFX_user_id']=$me;
            $_SESSION['LFX_user_lasthit']=time();
            if ($_POST['lang']) {
              $me=new LFX_User($me);
              $me->quickLoad();
              $me->setAttr('preflang',$_POST['lang']);
              $me->save();
            }
          }
        }
      }

      // Do NOT collapse this and the previous test in one block!
      if (!$_SESSION['LFX_user_id']) {

        // Setting the system language intl settings here, if available
        global $_LFX_globals, $_LFX;
        if ($syslang=$_LFX_globals->getSetting('system_language')) {
          LFX_include('LFX_Language');
          $tmp=new LFX_Language;
          $tmp=$tmp->search('code',$syslang);
          $lng=$tmp[0];
          if ($lng && $lng->getAttr('encoding')) {
            global $_LFX_charset;
            $_LFX_charset=$lng->getAttr('encoding');
            header('Content-type: text/html; charset='.$_LFX_charset);

            putenv('LANG='.$syslang);
            setlocale(LC_ALL,$syslang);
            $_LFX['page']['language']=$syslang;
          }
        }
        if (!$this->showLogin()) {
          echo('Sorry, unable to display the login form!');
          HORRIFIC_prependError(100);
        }
        // Yes, exit -- we just showed the login form, and we don't want to
        // risk executing any other page code before the user authenticates!
        exit;
      }
      $this->load($_SESSION['LFX_user_id']);
      $this->setIntl();
      return($_SESSION['LFX_user_id']);
    }

      /**
      * This method sets the current system-wide locales for this user.
      *
      * This method needs to remain in place here, separated from the page
      * code for subscriptions, where in one single page several messages
      * are sent to several users, each with their own language settings.
      *
      * @return mixed true on success, false on error or NULL if couldn't
      *   find the proper language (that's not an error, will fall back to En)
      */
    function setIntl($id=0)
    {
      $user=&$this->defaultObject($id);
      LFX_include('LFX_Language');
      $tmp=new LFX_Language;
      $tmp=$tmp->search('code',$user->getAttr('preflang'));
      $lng=$tmp[0];
      if ($lng && $lng->getAttr('encoding')) {
        global $_LFX_charset;
        $_LFX_charset=$lng->getAttr('encoding');
        header('Content-type: text/html; charset='.$_LFX_charset);

        putenv('LANG='.$user->getAttr('preflang'));
        setlocale(LC_ALL,$user->getAttr('preflang'));
        global $_LFX;
        $_LFX['page']['language']=$user->getAttr('preflang');
      } else {
        return(NULL);
      }
      return(true);
    }
    
      /**
      * This method simply shows the appropriate login form for this
      * module, or the generic LFXlib login if the current module has
      * defined none. The only possible error is that no login form exists.
      * @return bool true on success or false on error
      */
    function showLogin()
    {
      global $_LFX;
      $module_var='_'.$_LFX['global']['application'];
      global $$module_var;
      $moduleVar=$$module_var;
      if (($moduleVar['global']['paths']['login_form']) &&
      (is_file($moduleVar['global']['paths']['login_form']))){
        include($moduleVar['global']['paths']['login_form']);
      } elseif (is_file($_LFX['global']['paths']['inc'].'/login_form.php')) {
        include($_LFX['global']['paths']['inc'].'/login_form.php');
      } else {
        return(false);
      }
      return(true);
    }

      /**
      * A method to log out the current user.
      */
    function logout()
    {
      session_unset();
      session_destroy();
    }
      /**
      * Performs the actual database authentication of a user - you
      * don't need this.
      *
      * @param string $uname the username entered by the user
      * @param string $pwd the password entered by the user
      * @return integer user id on success or false on failure
      */
    function authenticate($uname, $pwd)
    {
      /*

        TO DO LDAP:
          * Review all combinations of existing LDAP and/or LFXlib /
            non-existing LDAP and/or LFXlib /
            activated/deactivated

      */
      $uname=$this->gpcUnslash($uname);
      $pwd=$this->gpcUnslash($pwd);
      list($person)=$this->search(array('uname','pwd','deactivated'),array($uname,md5($pwd),0));
      if ($person) {
        // Ok, we have a "clean" LFXlib user; returning him
        return($person->id);
      }

      // No LFXlib user matches this username and password!
      // Checking alternate login methods

      # ldap external users provider
      LFX_include_class('LFX_LDAP_User');
      $ldap_obj = new LFX_LDAP_User();
      $match = array(
        'uname'   => $uname,
        'password'=> $pwd
      );
      $res = $ldap_obj->match_against_ldap($match);
      $traceUser = new LFX_User();
      $resTrace = $traceUser->search(array('uname', 'deactivated', 'external_provider'), array($uname, 0, 'ldap'));
      if (!$res['count'] && $resTrace) {
        // We have an active user in LFXlib with this username,
        // but we have none in LDAP -- let's deactivate the LFXlib user then.
        $utbd = $resTrace[0];
        $utbd->setAttr('deactivated', '1');
        if (!$utbd->save()) {
          $this->addError(
            'The user couldn\'t be deactivated, '.
            'although s/he doesn\'t exist in LDAP anymore! '.
            '(unknown cause, failed to save user)'
          );
          return(false);
        }
      }

      if (!$res['count']) {
        // Regardless whether we had an active user in LFXlib or not,
        // we didn't have one in LDAP -- no luck, sorry
        return(false);
      }

      // Ok, at this point we know for sure that we have a matching user
      // in LDAP -- let's check whether he exists in LFXlib, and create him
      // if he doesn't:
      if ($resTrace) {
        // Ok, we already have an active user in LFXlib -- let's just
        // make sure he has the proper settings, save him and return his ID
        $currID = $resTrace[0]->id;
        $uu = $resTrace[0];
        $uu->setAttr('external_provider', 'ldap');
        $uu->setAttr('deactivated', '0');
        if (!$uu->save()) {
          return(false);
        }
      } else {
        // We DON'T have a user in LFXlib with a matching username:
        // creating him and returning his ID.
        $this->setAttr('uname', $uname);
        $this->setAttr('external_provider', 'ldap');
        $this->setAttr('deactivated', '0');
        if (!$this->save()) {
          return(false);
        }
        $currID = $this->id;
        $group = new LFX_Group();
        $res = $group->search('name', 'External_Users');
        if (!$res) {
          $group->setAttr('name', 'External_Users');
          $gid = $group->save();
          if (!$gid) {
            $this->addError('Could not create External_Users Group, '.
              'although this is mandatory for ldap users!');
            return(false);
          }
        } else {
          $gid = $res[0]->id;
        }
        $ugroup = new LFX_User_Group();
        $ugroup->setAttr('parent_group', $gid);
        $ugroup->setAttr('child_user', $currID);
        $ugroup->setAttr('comments', 'LDAP User');
        $ugroup->setAttr('access', '*');
        if (!$ugroup->save()) {
          return(false);
        }
      }

      return($currID);
    }

      /**
      * This method looks at the "Colleague" group(s) in which this user is
      * a direct member and returns the ID's of all other users who are direct
      * members in the same group(s).
      *
      * Please note that this method excludes the current user from the list.
      *
      * @param integer id the id of the user to return the colleagues for
      * @return an indexed array with user ID's or false on error
      */
    function getColleaguesID($id=0)
    {
      $user=&$this->defaultObject($id);
      $colGroupIDs=$user->getUserGroupsID('Colleague',true);
      if ($colGroupIDs===false) {
        // getUserGroupsID() will complain
        return(false);
      }
      $colleagues=array();
      for($i=0;$i<count($colGroupIDs);$i++) {
        $tg=new LFX_Group($colGroupIDs[$i]);
        $tgMembers=$tg->getGroupMembersIDRecursive();
        foreach($tgMembers['user'] as $userID) {
          if (($userID[0]!=$user->id) && (!in_array($userID[0],$colleagues))) {
            $colleagues[]=$userID[0];
          }
        }
      }
      return($colleagues);
    }
    
      /**
      * This method returns the instantiated users returned by {@link getColleaguesID}.
      * Please check on that method for more info.
      *
      * @param integer $id the id of the user to return the colleagues for
      * @return mixed indexed array of LFX_User objects or false on error
      */
    function getColleagues($id=0)
    {
      $colIDs=$this->getColleaguesID($id);
      if ($colIDs===false) {
        return(false);
      }
      return($this->_doLoad($colIDs,false));
    }

      /**
      * Tells you whether the user with the id $id is member in at least one user group
      * with the other user(($other_id).
      *
      * If you omit the second parameter, you test with the user that is
      * currently logged in. This is useful to test whether to show the details
      * of a certain user to the logged-in user.
      *
      * NOTE from Bogdan: This is NOT the right way to check if a user should
      * be visible to another -- the simple fact that both CEO's from two rival
      * companies can edit DMO instances (i.e. they're members in group DMO_instance)
      * shouldn't mean that they can see each other. A better way would be for each
      * user to be a member in group LFX_user_<user ID>, and this group should be a
      * member of group LFX_company_<company ID> as well as a member of group LFX_Users.
      * Anyone with access in either LFX_company_<company ID>, LFX_Users or LFX_User
      * should be able to see a given user.
      *
      * @return True if you are a colleague, false if not
      */
    function canSeeUser($other_id,$id=0)
    {
      $user=&$this->defaultObject($id);

      $colleagues = $user->getColleaguesID();

      return(($user->id==$other_id) || in_array($other_id,$colleagues));
    }

      /**
      * This method returns the explicit rights this user has in the group
      * with name $group_name.
      *
      * The difference between this method and LFX_User::{@link getRights}()
      * is that this method returns the actual rights (e.g. 'RW', '*', etc),
      * whereas getRights() checks for a specific right and returns true or
      * false, depending on whether the user does or does not have that right.
      *
      * @param string $group_name the group to retrieve the rights for
      * @param integer $id the user id to check the rights for (by default this
      *   user)
      * @return string the user's rights on the respective group, or boolean
      *   false on error.
      */
    function rightsToGroup($group_name,$id=0)
    {
      if ($this->isSuperuser()) {
        return('*');
      }
      $user=&$this->defaultObject($id);
      
      if (!$user->id) {
        $this->addError("LFX_User::rightsToGroup() called on user with no ID!");
        return(false);
      }

      $access_list = $user->accessList($id);

      $g = new LFX_Group;
      $group_id = $g->searchID('name',$group_name);
      
      if (!$group_id) {
        return('');
      }
      
      // Group names are unique in the database
      $group_id = $group_id[0];
      $rights=array();
      
      //echo "Group id is $group_id<br>";
      //echo"<pre>";print_r($group_id);echo"</pre>";

      while (list(,$v) = each($access_list)) {
        if($v[0]==$group_id) {
          $rights[] = $v[1];
        }
      }

      if (sizeof($rights)==0) {
        return('');
      } else {
        $right="";
        for ($i=0;$i<count($rights);$i++) {
          //echo("Merging ".$right." with ".$rights[$i]." on $group_name<br />\n");
          $right=$this->permsMerge($right,$rights[$i]);
        }
        return($right);
      }
    }
    
    function permsMerge($perm1,$perm2)
    {
      return(LFX_permsMerge($perm1,$perm2));
    }
    
    function permsFilter($perm,$filter)
    {
      return(LFX_permsFilter($perm,$filter));
    }
    
    function getNameH()
    {
      global $_LFX_globals, $_LFX;
      if (isset($_LFX['page']['data_cache']['users_getNameH'][$this->id])) {
        return($_LFX['page']['data_cache']['users_getNameH'][$this->id]);
      }
      $fname=$this->getAttrH('fname');
      $lname=$this->getAttrH('lname');
      if (!$_LFX_globals->getSetting('last_name_first')) {
        $result="$fname $lname";
      } else {
        $result="$lname $fname";
      }
      $_LFX['page']['data_cache']['users_getNameH'][$this->id]=$result;
      return($result);
    }
    
    function getName()
    {
      global $_LFX_globals;
      if (!$_LFX_globals->getSetting('last_name_first')) {
        return($this->getAttr('fname').' '.$this->getAttr('lname'));
      } else {
        return($this->getAttr('lname').' '.$this->getAttr('fname'));
      }
    }

    function _filePerm()
    {
      global $_page;
      if ($_page->user->id==$this->id) {
        return('RWUD');
      }
      return(NULL);
    }
   
      /**
      * Returns the first group in which this user was assigned, 
      * useful for reporting on the activity of the user, when
      * you don't know in which group the user belongs
      */

    function getPrimaryGroup()
    {
      $groups = $this->getUserGroupsID();
      $group = $groups[0][0];
      return($group);
    }
    
      /**
      * Retrieves the value for one of this user's personal globals.
      * See class LFX_Personal_Global for the structure of the data in the database.
      * @param string $global the name of the global to retrieve
      * @return mixed the value of the global or NULL if it isn't set
      */
    function getSetting($global)
    {
      LFX_include_class('LFX_Personal_Global');
      $globalObject=new LFX_Personal_Global;
      $globals=$globalObject->search(array('user','global_name'),array($this->id,$global));
      if ($globals) {
        return($globals[0]->getAttr('value'));
      } else {
        return(NULL);
      }
    }
    
      /**
      * Saves a personal global for this user.
      * If the global exists, it's updated; if it doesn't exist, it's created.
      * See class LFX_Personal_Global for the structure of the data in the database.
      * @param string $global the name of the global to change/create
      * @param string $value the value for the global; please note that you
      *   have to serialize/unserialize objects yourself if you need to pass
      *   anything else except strings and numbers
      * @return mixed the result of SC::save() on the LFX_Personal_Global object
      */
    function saveSetting($global,$value)
    {
      LFX_include_class('LFX_Personal_Global');
      $globalObject=new LFX_Personal_Global;
      $globals=$globalObject->search(array('user','global_name'),array($this->id,$global));
      if ($globals) {
        $myGlobal=$globals[0];
      } else {
        $myGlobal=new LFX_Personal_Global;
      }
      $myGlobal->setAttrs(
        array(
          'user'=>$this->id,
          'global_name'=>$global,
          'value'=>$value
        )
      );
      return($myGlobal->save());
    }
  }
?>
