<?

  /**
  * This is the group which handles the table which links users and groups
  *
  * Use this to create/delete inclusion (pareting) relationhips.
  *
  * @author Andrei Tinca
  * @since November 2002
  * @package LFX
  */

  _checkMeOut(__FILE__);

  $result=array('LFX_Object', 'LFX_Access');
  if (!_checkIncludes($result)) {
    return($result);
  }

  class LFX_User_Group extends LFX_Access {

      /**
      * If this is set to true, membership tests are skipped in beforeSave()
      */
    var $skipMembershipTests=false;
    
      /**
      * Class constructor
      */
      
    function LFX_User_Group($id=0) {
        $this->LFX_Access($id);

        $tmp["parent_group"]=
        $tmp["child_user"]=
        $tmp["child_group"]=
        $tmp["access"]=
        $tmp["comments"]=
        $tmp["date_added"]=
        array();

        // Defining DATABASE FIELD names--for those which have different names
        $tmp["child_user"]["fld_name"]="child_user_id";
        $tmp["child_group"]["fld_name"]="child_group_id";
        $tmp["parent_group"]["fld_name"]="parent_group_id";

        // Defining FLAGS
        $tmp['date_added']['flags']['sqlDate']=true;
        
        // Defining LINKS
        $tmp["parent_group"]["link_class"]="LFX_Group";
        $tmp["child_group"]["link_class"]="LFX_Group";
        $tmp["child_user"]["link_class"]="LFX_User";

        $this->dataStructure=array(
            "table_name"=>"user_group",
            "id_field"=>"id",
            "fields"=>$tmp
        );

        $this->fillDataStructure();
    }
    
     /**
     *  Internal method called only in {@link beforeSave}
     *  $critical_groups = array of groups considered to be critical 
     *  For any new member added(group or user) in any critical group
     *  a notify is sent via email to all SuperUsers.
     */
    
    function notifyBeforeSave() {
       /* critical groups modifications -  notifying SuperUsers*/      
      LFX_include_class("LFX_mail");
      
       /* SuperUsers id -> always 1 */
      /* List of critical_groups -> should be continued */
      $critical_groups = 
      array(
            "LFX_SQL_Ops",
            "DMO_Access",
            "DMO_Template"
      );
      
      $parent = $this->getAttr('parent_group');
      $obj = new LFX_Group($parent);      
      
      $send_mail = false;
      if (($parent == '1') || (in_array($obj->getAttr('name'), $critical_groups))) {
        $send_mail = true;
      } else {
          $rec_parents_ids = $obj->getGroupParentsIDRecursive();
          for($i=0; $i<count($rec_parents_ids); $i++) {
            $this_id = $rec_parents_ids[$i][0];
            $this_parent_obj = new LFX_group($this_id);
            if (($this_id == '1') || (in_array($this_parent_obj->getAttr('name'), $critical_groups))) {
              $send_mail = true;
              break;
            }
          }
        }
        /** email sending */
        if ($send_mail) {
        
        /* get SuperUsers to notify */
          $su = new LFX_Group(1); // SuperUsers group
          $susers = $su->getMemberUsersID();
          for($i=0; $i<count($susers); $i++) {
            $user = new LFX_user($susers[$i][0]);
            $addresses[$i]['email'] = $user->getAttr('email');
            $addresses[$i]['name'] = $user->getAttr('uname');
          }
          $text = '';
          $subject = "Member added to Group: [".$obj->getAttr('name')."]";
          if ($uid = $this->getAttr('child_user')) {
            $user = new LFX_user($uid);
            $text =  "User with details : \r\n
                      Email : [".$user->getAttr('email')."]\r\n
                      Username: [".$user->getAttr('uname')."]\r\n
                      First Name: [".$user->getAttr('fname')."]\r\n
                      Last Name: [".$user->getAttr('lname')."]\r\n
                      is now member of Group [".$obj->getAttr('name')."]
                      ";              
          }
          if ($gr_id = $this->getAttr('child_group')) {
            $gr = new LFX_group($gr_id);
            $text = "Group [".$gr->getAttr('name')."] is now member of Group [".$obj->getAttr('name')."]";
          }
          
          $mail = new LFX_mail();
          $mail->addresses = $addresses;
          $mail->text = $text;
          $mail->subject = $subject;
          $mail->send('toAll');
        }
        /** done email sending */
      }
    
      /**
      * This checks whether a group can really parent the passed children.
      *
      * It's done by checking whether the child is among
      * the possible children of this group by calling
      * {@link getPossibleMembersID}
      *
      * @return true on success or false
      */
    
    function beforeSave($new, $id, $force)
    {
      
      if ($this->skipMembershipTests) {
        $this->notifyBeforeSave();
        return(true);
      }

      if (!$new) {
        $tmp=new LFX_user_group($id);
        if (
          ($tmp->getAttr('child_user')==$this->getAttr('child_user')) &&
          ($tmp->getAttr('child_group')==$this->getAttr('child_group')) &&
          ($tmp->getAttr('parent_group')==$this->getAttr('parent_group'))
        ) {
          $this->notifyBeforeSave();
          return(true);
        }
      }
      
      //echo "This is beforeSave!<br>";
      //return(false);
      if (($this->getAttr(child_user)>0)&&($this->getAttr(child_group)>0)) {
          $this->addError("You are trying to add both a user and a group. ".
            "Only one at a time please!");
          return(false);
      }

      if (!$this->getAttr('parent_group')) {
        $this->addError('Har, har, you are trying to create a relationship '.
          'between something and no group! Please specify a parent group!');
        return(false);
      }
      $group = new LFX_group($this->getAttr("parent_group"));
      $possible_members = $group->getPossibleMembersID($this->getAttr("parent_group"));

      if ($this->getAttr("child_user")>0) {
        if(!in_array($this->getAttr("child_user"),$possible_members["user"])) {
          $this->addError("Cannot add this user ".
            "(#".$this->getAttr("child_user").") to the group ".
            "(#".$this->getAttr("parent_group")."). ".
            "The user is already a member of this group.");
          return(false);
        }
      } elseif ($this->getAttr("child_group")>0) {
        if(!in_array($this->getAttr("child_group"),$possible_members["group"])) {
          $this->addError("Cannot add this group ".
            "(#".$this->getAttr("child_group").") to the group ".
            "(#".$this->getAttr("parent_group")."). ".
            "It already is a member, or already a parent, ".
            "or the group/role/function combination is impossible ".
            "(functions don't accept any members at all, ".
            "and roles only accept functions or other roles ".
            "as possible members).");
          return(false);
        }
      } else {
        $this->addError('You must specify either a child group or a child user, '.
          "you can't have both empty!");
        return(false);
      }
      $this->setAttr("date_added",time());
      $this->notifyBeforeSave();
      return(true);
    }

    function onSave()
    {
      global $_LFX_globals;
      if ($child=$this->getAttr('child_user')) {
        $child_user=new LFX_User($child);
        $child_user->saveSetting('user_rights_changed',time());
      } elseif ($child=$this->getAttr('child_group')) {
        $_LFX_globals->saveSetting('global_rights_changed',time());
      }
    }
    
    function beforeDelete()
    {
      global $_LFX_globals;
      if ($child=$this->getAttr('child_user')) {
        $child_user=new LFX_User($child);
        $child_user->saveSetting('user_rights_changed',time());
      } elseif ($child=$this->getAttr('child_group')) {
        $_LFX_globals->saveSetting('global_rights_changed',time());
      }
      return(true);
    }      
     
     /**
     * Executes an sql to find the id of a record in the user_group table
     *
     * This function is useful when removing a member from a group.
     * @return the id of the searched record.
     */


    function getLinkID($id,$child_id,$child_type) {
      switch ($child_type) {
        case "User":
          $ugs=$this->search(array('parent_group','child_user'),array($id,$child_id));
          break;
        case "Group":
          $ugs=$this->search(array('parent_group','child_group'),array($id,$child_id));
          break;
      }
      if (($rows=count($ugs))>1) {
        echo "Found $rows links instead of 1. The link table has integrity problems:
             group $id has multiple links to $child_type $child_id. Aborting.<br>";
        return(false);
      } elseif ($rows) {
        return($ugs[0]->id);
      } else {
        return(0);
      }
    }
  }
?>
