<?
  /**
  * LFX_XML class
  * Bogdan Stancescu <bogdan@moongate.ro>, May 2003
  * $Id: LFX_xml.php,v 1.7 2005/01/20 18:12:40 bogdan Exp $
  * @package LFX
  */
  
  class LFX_XML extends LFX_Object
  {
      /**
      * This is where this object's XML structure will be stored.
      */
    var $xml=NULL;
      /**
      * The indent character(s)
      */
    var $indentStr='  ';
    
      /**
      * The constructor -- you may optionally pass it a string to
      * parse. If you need to parse a file you'll have to do it in two
      * steps -- instantiate and then call {@link fParse}
      */
    function LFX_XML($data=NULL)
    {
      GLOBAL $_LFX;
      $this->dbData['dbDataSet']=$_LFX['global']['applications']['LFX']['app_dbdata'];
      $this->LFX_Object();
      if ($data!==NULL) {
        $this->sParse($data);
      }
    }
    
      /**
      * Very similar to {@link byTag}, this method returns all matching
      * last level tags in the path. This means that the path is walked
      * with byTag until the last element (thus only processing the first match)
      * where all matching tags are returned in an indexed array.
      *
      * @param mixed path the path to walk, either as an indexed array or as
      *   a string for single level path
      * @param xml the XML structure to parse; if not present the last parsed
      *   xml structure is used; only {@link sParse} and {@link fParse} affect
      *   the default structure.
      * @return array indexed array of matching tags or false if not found
      */
    function byTags($path,$xml=NULL)
    {
      if ($xml===NULL) {
        $xml=$this->xml;
      }
      if ($xml===NULL) {
        $this->addError('No XML to walk! (in LFX_XML::byTags())');
        return(false);
      }
      if (is_array($path)) {
        $myTag=strToUpper(array_pop($path));
        $xml=$this->byTag($path,$xml);
      } elseif (is_string($path)) {
        $myTag=strToUpper($path);
      } else {
        $this->addError('Unknown data type ('.(getType($path)).
          ') passed as path in LFX_XML::getXMLByTags())');
        return(false);
      }
      $xmlChildren=$xml['children'];
      $result=array();
      for ($i=0;$i<count($xmlChildren);$i++) {
        if ($xmlChildren[$i]['tag']==$myTag) {
          $result[]=$xmlChildren[$i];
        }
      }
      return($result);
    }

      /**
      * Method to easily retrieve a tag by path in the XML tree.
      * Returns the FIRST matching tag -- use this only if you're certain
      * you only have one matching tag, or if you only need the first one.
      * Otherwise use {@link byTags} for all matching tags.
      *
      * See {@link sParse} for usage example.
      *
      * @param mixed path the path to the tag to retrieve as indexed array of
      *   tag names, or a string for single level path
      * @param xml the XML structure to parse; if not present the last parsed
      *   xml structure is used; only {@link sParse} and {@link fParse} affect
      *   the default structure.
      * @return array the first matching tag or false if not found
      */
      /*
        Please note that changing the definition to &byTag(), which would allow
        building XML structures breaks nmap-remote, probably due to too many
        cross-dependencies or something. "Breaks" as in generating a segfault.
        So another solution must be found until PHP 5 which will hopefully fix this.
      */
    function byTag($path, $xml=NULL)
    {
      if ($xml===NULL) {
        $myXML=$this->xml;
      } else {
        $myXML=$xml;
      }
      if ($myXML===NULL) {
        $this->addError('No XML structure to walk! (in LFX_XML::byTag())');
        return(false);
      }
      // A nice way to help people: we'll check if this is a node as opposed
      // to a children array as we expect -- if it is, we go inside the children
      // array, thus helping developers avoid confusion. There are no risks
      // involved -- child arrays are indexed arrays, so the 'children' key
      // is never set for those.
      if (isset($xml['children'])) {
        $myXML=$xml['children'];
      }
      if (is_array($path)) {
        list(,$tag)=each($path);
      } elseif (is_string($path)) {
        $tag=$path;
      } else {
        $this->addError('Unknown data type ('.(getType($path)).
          ') passed as path in LFX_XML::getXMLByTag())');
        return(false);
      }
      $tag=strToUpper($tag);
      $count=count($myXML);
      $found=false;
      for ($i=0;(!$found) && ($i<$count);$i++) {
        if ($myXML[$i]['tag']==$tag) {
          $myXML=$myXML[$i];
          $found=true;
        }
      }
      while ($found && (list(,$tag)=each($path))) {
        $tag=strToUpper($tag);
        $count=count($myXML['children']);
        $found=false;
        for ($i=0;(!$found) && ($i<$count);$i++) {
          if ($myXML['children'][$i]['tag']==$tag) {
            $myXML=$myXML['children'][$i];
            $found=true;
          }
        }
      }
      if (!$found) {
        return(false);
      }
      return($myXML);
    }

      /**
      * Parses an XML file and returns an array representing it.
      *
      * The core and idea for this function copied from
      * http://www.php.net/manual/en/function.xml-parse-into-struct.php##10069
      * gdemartini@bol.com.br -- 11-Dec-2000 03:49
      *
      * The result is an indexed array made of a associative arrays representing
      * the topmost XML tags. Please note the XML standards actually restrict
      * the number of topmost tags to one and only one, but people tend to forget
      * that.
      *
      * Each resulting node has the following structure:
      * node -+-> tag        [string]
      *       |-> attributes [associative array <att name> => <att value>]
      *       `-> children   [indexed array of this XML tag's child XML tags]
      *
      * Keep in mind that given the format of the result, the original array you
      * get is not exactly the one depicted above, but rather the format of the
      * "children" array (i.e. you don't get one node in the result, but rather
      * an indexed array of nodes, just like the "children" array).
      *
      * Also note that valid XML documents MUST have one single root, so this
      * indexed array should typically only have one element for valid XML
      * documents.
      *
      * Let's take an example XML and how you'd use this class to manipulate it:
      *
      * The XML string you have in your code in $myXML:
      * <country name='Austria'>
      *   <city name='Vienna'>
      *     <company name='CSO Lanifex'>
      *       <server name='Monkey' />
      *       <server name='Loom' />
      *       <person name='Paul Gillingwater' />
      *     </company>
      *   </city>
      * </country>
      *
      * The code (the PHP tags mangled so we don't escape from PHP here):
      * < ?
      *   $xml=new LFX_XML($myXML);
      *   $cityTag=$xml->byTag(array('country','city'));
      *   $companyTag=$xml->byTag($cityTag,'company');
      *   $serverTags=$xml->byTags($companyTag,'server');
      * ? >
      *
      * If all you wanted were the servers, you might as well have used
      * < ?
      *   $xml=new LFX_XML($myXML);
      *   $serverTags=$xml->byTags($companyTag,
      *     array('country','city','company','server');
      * ? >
      * but the first example is better at showing all the manipulation you
      * generally need in production where the data tends to be parsed
      * sequentially, passed through loops, conditional blocks, functions etc.
      *
      * @param string data the data to parse
      * @return array
      */
    function sParse($data)
    {
      $p = xml_parser_create();
      xml_parser_set_option($p, XML_OPTION_SKIP_WHITE, 1);
      xml_parse_into_struct($p, $data, $vals, $index);
      xml_parser_free($p);
      $tree = array();
      $i = 0;
      array_push($tree, array(
        'tag' => $vals[$i]['tag'],
        'attributes' => $vals[$i]['attributes'],
        'children' => $this->_getChildren($vals, $i)));
      $this->xml=$tree;
      return($this->xml);
    }

      /**
      * Parses an XML file and returns an array representing it.
      * Basically does the dirty work for you: checks whether the file exists,
      * is readable, reads it and feeds it to LFX_XML::sParse().
      *
      * See {@link sParse} for more info on the resulting array format.
      *
      * @param string file the file to parse
      * @return array
      */
    function fParse($file)
    {
      if (!is_file($file)) {
        $this->addError("File $file doesn't exist (in LFX_XML::fParse())");
        return(false);
      }
      if (!is_readable($file)) {
        $this->addError("Permission denied to read file $file (in LFX_XML::fParse())");
        return(false);
      }
      $data = implode('', file($file));
      return($this->sParse($data));
    }

      /**
      * This is a private method you shouldn't need to use directly.
      *
      * The core and idea for this function copied from
      * http://www.php.net/manual/en/function.xml-parse-into-struct.php
      * gdemartini@bol.com.br -- 11-Dec-2000 03:49
      *
      * @return array
      */
    function _getChildren($vals, &$i)
    {
      $children = array();
      if ($vals[$i]['value'])
        array_push($children, $vals[$i]['value']);

      // Might as well have while(true) ;)
      while (++$i < count($vals)) {
        switch ($vals[$i]['type']) {
          case 'cdata':
            array_push($children, $vals[$i]['value']);
          break;

          case 'complete':
            array_push($children, array(
              'tag' => $vals[$i]['tag'],
              'attributes' => $vals[$i]['attributes'],
              'value' => $vals[$i]['value']));
          break;

          case 'open':
            array_push($children, array(
              'tag' => $vals[$i]['tag'],
              'attributes' => $vals[$i]['attributes'],
              'children' => $this->_getChildren($vals, $i)));
          break;

          case 'close':
            return $children;
        }
      }
    }
    
      /**
      * This method renders an XML structured to a string; if no structure
      * is specified, the current structure is used.
      * @param array $xml an XML structure, defined as explained at {@link sParse}
      * @return string the rendered XML
      */
    function toString($xml=NULL)
    {
      if ($xml===NULL) {
        $myXML=$this->xml;
      } else {
        $myXML=$xml;
      }
      if ($myXML===NULL) {
        $this->addError('No XML structure to render! (in LFX_XML::toString())');
        return(false);
      }
      $result='';
      $this->indent=0;
      for($i=0;$i<count($xml);$i++) {
        $result.=$this->_renderTag($xml[$i]);
      }
    }
    
      /**
      * Private, used by {@link toString} to recursively render the tags.
      * @param array $tag the tag to render, with all its children
      * @return string the respective tags in text
      */
    function _renderTag($tag)
    {
      $result=str_repeat($this->indentStr,$this->indent);
      $result.="<".strToUpper($tag['tag']);
      if (is_array($tag['attributes'])) {
        while(list($attName,$attVal)=each($tag['attributes'])) {
          $result.=' '.strToUpper($attName).'="'.$this->escape($attVal).'"';
        }
      }
      if (is_array($tag['children'])) {
        $result.=">\n";
        $this->indent++;
        for($i=0;$i<count($tag['children']);$i++) {
          $result.=$this->_renderTag($tag['children'][$i]);
        }
        $this->indent--;
        $result.=str_repeat($this->indentStr,$this->indent);
        $result.="</".strToUpper($tag['tag']).">\n";
      } else {
        $result.=" />\n";
      }
      return($result);
    }
    
      /**
      * A simple method to escape values in attributes; first escapes any
      * "&" characters, and then escapes ">", "<" and """, all being escaped
      * to the corresponding HTML entities ("&amp;", "&gt;", "&lt;" and "&quot;").
      */
    function escape($str)
    {
      $str=str_replace('&','&amp;',$str);
      $str=str_replace('>','&gt;',$str);
      $str=str_replace('<','&lt;',$str);
      $str=str_replace('"','&quot;',$str);
      return($str);
    }
  }
?>
