<?

  /**
  * SQL Diff functions
  * Bogdan Stancescu <bogdan@lanifex.com>, July 2005
  * $Id: include.php,v 1.2 2005/11/14 12:05:23 bogdan Exp $
  * @package LFX
  */

  _checkMeOut(__FILE__);

  function parseSQL($sql)
  {
    if (!is_array($sql)) {
      $sql=explode("\n",$sql);
    }
    $curTable='';
    $parsed=array();
    for($i=0;$i<count($sql);$i++) {
      $line=trim($sql[$i]);
      if ($line==="") {
        continue;
      }
      if (substr($line,0,2)=='--') {
        continue;
      }
      if (substr($line,-1)===',') {
        $line=substr($line,0,-1);
      }
      if (substr($line,0,strlen('CREATE TABLE'))=='CREATE TABLE') {
        $aux=substr($line,strlen('CREATE TABLE '));
        $curTable=nameCleanup(substr($aux,0,strpos($aux,' ')));
        $parsed[$curTable]=array();
      } elseif (substr($line,0,1)==')') {
        // Should parse table type
        if (substr($line,0,strlen(") TYPE="))==") TYPE=") {
          $parsed[$curTable]['type']=substr($line,strlen(") TYPE="),-1);
        } elseif (substr($line,0,strlen(") ENGINE="))==") ENGINE=") {
          $parsed[$curTable]['type']=substr($line,strlen(") ENGINE="),-1);
        }
        $curTable='';
      } elseif (!$curTable) {
        //decho("Statement outside table! Ignoring. ($line)<br />\n");
      } elseif (preg_match("/^(PRIMARY|UNIQUE)?[ ]?KEY[ ]+/i",$line)) {
        //preg_match('/^(PRIMARY|UNIQUE)?[ ]?KEY[ ]+([^ ]+)[ ]?([^ ]+)?/',$line,$matches);
        preg_match('/^(PRIMARY|UNIQUE)?[ ]?KEY[ ]+([^ ]+)[ ]?(.+)?/',$line,$matches);
        $keyType=strtoupper($matches[1]);
        if($keyType=='PRIMARY') {
            $keyName='';
            $keyDef=$matches[2];
        } else {
            $keyName=nameCleanup($matches[2]);
            $keyDef=$matches[3];
            $keyDef=str_replace(' ','',$keyDef);
        }
        $keyDef=substr($keyDef,1,-1); // getting rid of brackets
        //decho("$i.name for ".$line.": ".$keyName);
        //decho("$i.def for ".$line.": ".$keyDef);
        //decho("$i.key type for $line: $keyType");
        $explodedDef=explode(',',$keyDef);
        for($j=0;$j<count($explodedDef);$j++) {
          if (preg_match('/^([^(]+)\(([^)]+)\)$/',$explodedDef[$j],$matches)) {
            $explodedDef[$j]=nameCleanup($matches[1])."($matches[2])";
          } else {
            $explodedDef[$j]=nameCleanup($explodedDef[$j]);
          }
        }
        $keyDef=implode(',',$explodedDef);
        if ($keyType=='PRIMARY') {
          $parsed[$curTable]['primary_key']=$keyDef;
        } else {
          $parsed[$curTable]['keys'][$keyName]=array(
            'type'=>$keyType?$keyType:'regular',
            'definition'=>$keyDef
          );
        }
      } else {
        // Parse field
        $fieldName=substr($line,0,strpos($line,' '));
        $parsed[$curTable]['fields'][nameCleanup($fieldName)]=substr($line,strlen($fieldName)+1);
      }
    }
    return($parsed);
  }

  function nameCleanup($name)
  {
    if (strpos(' '.$name,',')) {
      $names=explode(',',$name);
      for($i=0;$i<count($names);$i++) {
        $names[$i]=nameCleanup($names[$i]);
      }
      return(implode(',',$names));
    }
    if ((substr($name,0,1)==='`') && (substr($name,-1)==='`')) {
      return($name);
    } else {
      return("`$name`");
    }
  }

  function dumpSQL($dbKey)
  {
    global $_LFX_dbData;
    $dbObject=new LFX_Object;
    $db=$dbObject->_doDbInit($_LFX_dbData[$dbKey]);
    if(!$db->query("SHOW TABLES")) {
      return(false);
    }
    $dump='';
    $tables=array();
    while ($db->next_record()) {
      $tables[]=$db->f(0);
    }
    for($i=0;$i<count($tables);$i++) {
      $db->query("SHOW CREATE TABLE {$tables[$i]}");
      $db->next_record();
      $dump.=$db->f(1).";\n\n";
    }
    return($dump);
  }

  function diffSQL($old,$new)
  {
    //decho("<font color='red'>".vardump($old)."</font>");
    //decho("<font color='green'>".vardump($new)."</font>");
    $result=array();
    while(list($table,$tableData)=each($new)) {
      if (!isset($old[$table])) {
        $result[]=array(
          'action'=>'create_table',
          'table'=>$table,
          'data'=>$tableData
        );
      } else {
        while(list($field,$fieldData)=each($tableData['fields'])) {
          if (!$old[$table]['fields'][$field]) {
            $result[]=array(
              'action'=>'create_field',
              'table'=>$table,
              'field'=>$field,
              'data'=>$fieldData
            );
          } elseif ($old[$table]['fields'][$field]!=$fieldData) {
            $result[]=array(
              'action'=>'update_field',
              'table'=>$table,
              'field'=>$field,
              'old_data'=>$old[$table]['fields'][$field],
              'data'=>$fieldData
            );
          }
          if (isset($old[$table]['fields'][$field])) {
            unset($old[$table]['fields'][$field]);
          }
        }
        while(list($oldField,$oldFieldData)=@each($old[$table]['fields'])) {
          $result[]=array(
            'action'=>'drop_field',
            'table'=>$table,
            'field'=>$oldField,
            'old_data'=>$oldFieldData
          );
        }
        if ($tableData['primary_key']!==$old[$table]['primary_key']) {
          $drop=$create=false;
          if ($tableData['primary_key'] && $old[$table]['primary_key']) {
            $drop=true;
            $create=true;
          } elseif ($tableData['primary_key']) {
            $create=true;
          } else {
            $drop=true;
          }

          if ($drop) {
            $result[]=array(
              'action'=>'drop_primary_key',
              'table'=>$table,
              'old_data'=>$old[$table]['primary_key']
            );
          }
          if ($create) {
            $result[]=array(
              'action'=>'create_primary_key',
              'table'=>$table,
              'data'=>$new[$table]['primary_key']
            );
          }
        }
        while(list($key,$keyData)=@each($tableData['keys'])) {
          if (!isset($old[$table]['keys'][$key])) {
            $result[]=array(
              'action'=>'create_key',
              'table'=>$table,
              'key'=>$key,
              'data'=>$keyData
            );
          } elseif ($old[$table]['keys'][$key]!==$keyData) {
            $result[]=array(
              'action'=>'drop_key',
              'table'=>$table,
              'key'=>$key,
              'old_data'=>$old[$table]['keys'][$key]
            );
            $result[]=array(
              'action'=>'create_key',
              'table'=>$table,
              'key'=>$key,
              'data'=>$keyData
            );
          }
          unset($old[$table]['keys'][$key]);
        }
        while (list($oldKey,$oldKeyData)=@each($old[$table]['keys'])) {
          $result[]=array(
            'action'=>'drop_key',
            'table'=>$table,
            'key'=>$oldKey,
            'old_data'=>$oldKeyData
          );
        }
      }
      unset($old[$table]);
    }
    while(list($table,$tableData)=@each($old)) {
      $result[]=array(
        'action'=>'drop_table',
        'table'=>$table,
        'old_data'=>$tableData
      );
    }
    // Ok, we now have the final diff. One last preparation: we move all
    // drop_key entries at the top of their respective table action list,
    // as to avoid doing a drop_field followed by a drop_key on the same field
    // (the key obviously doesn't exist any more)
    $current_table='';
    $current_table_index=0;
    for($i=0;$i<count($result);$i++) {
      if ($result[$i]['table']!=$current_table) {
        // We entered a new table
        $current_table=$result[$i]['table'];
        $current_table_index=$i;
        // Nothing to do, this is the first entry, and we don't care about it.
        continue;
      }
      // We're guaranteed to be in a known table, at the 2nd or more diff.
      if ($result[$i]['action']!='drop_key') {
        continue;
      }
      // Ok, here we go: we "pull" all actions between the first one in this
      // table and the one above the current entry one place higher in the
      // array, as to make room for the current one at the top:
      $my_action=$result[$i];
      for($j=$i;$j>$current_table_index;$j--) {
        $result[$j]=$result[$j-1];
      }
      $result[$current_table_index]=$my_action;
      // ok, now we move the "current_table_index" pointer one position higher,
      // as to keep the same order for dropping indexes as in the original
      // (otherwise it may be confusing when confirming)
      $current_table_index++;
    }
    return($result);
  }

  function buildSQLfromForm()
  {
    $result='';
    for($i=0;$i<count($_POST['sql']);$i++) {
      if ($_POST['sql'][$i]) {
        $result.=LFX_Object::gpcUnslash($_POST['sql'][$i]).";\n\n";
      }
    }
    return($result);
  }
?>
