<?
  /**
  * Database consistency check
  * Lucian Silistru <lucian@moongate.ro>, November 2003
  * Bogdan Stancescu <bogdan@lanifex.com>, February 2004
  * $Id: consist_enforce.php,v 1.3 2005/06/22 09:07:20 bogdan Exp $
  * @package LFX
  */
    
  require(dirname(dirname(__FILE__)).'/LFX_lib.php');
  ob_end_clean();

  // Comment out the following line for debugging
  $_LFX['page']['debug_level']=LFX_DL_QUIET;

  $verbose=false;
  $update_interval=1000;
  $report=array(
    'links_found'=>0, // Number of inconsistent links found
    'deps_found'=>0,  // Number of inconsistent many to many dependencies found
    'links_del'=>0, // Number of links deleted (attribute set to 0)
    'deps_del'=>0,  // Number of deps deleted (entry in link table deleted)
    'obj_del'=>0,   // Number of objects deleted
    'ignored'=>0    // Number of ignored inconsistencies due to STRANGER on_mod
  );

  if (in_array('-h',$argv)) {
    echo('By default the script only shows missing items in the database for each class it scans'."\n");
    echo('Arguments are:'."\n");
    echo('    -v  - The script also outputs informations about current link/dependency it scans.'."\n");
    echo('    -d# - By default # is 1000, that means that every 1000 objects in the current class the script outputs the % of the class it has completed.');
    exit;
  }

  for ($i=1;$i<$argc;$i++) {
    if ($argv[$i]=='-v') {
      $verbose=true;
    } elseif (substr($argv[$i],0,2)=='-d') {
      if (!preg_match('/^-d([0-9]+)$/',$argv[$i])) {
        echo("Wrong parameter $argv[$i] -- proper format is -d#, where # is a number. Ignoring\n");
      } else {
        $update_interval=substr($argv[$i],2);
      }
    } else {
      echo("Warning! Unknown parameter $argv[$i]. Ignoring.\n");
    }
  }
  
  $_incReq=array_keys($_LFX["global"]["master_include_array"]);
  if (!include($_incMgr)) {
    echo("Unable to perform required includes. Please note this is the consistency checker, ".
      "which has to load ALL dependencies, so this might not be a problem at all for the whole of the system.");
    exit;
  }
  
  $classes=get_declared_classes();
  asort($classes);
  $deps=array();
  $links=array();

  $exclude_classes=array(
    'hd_graphical_report',
    'lfx_cache_rights'
  );
  // First off, read the dependency data
  while (list(,$class)=each($classes)) {
    if (in_array($class,$exclude_classes)) {
      continue;
    }
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }
    if (!class_exists($class)) {
      continue;
    }
    $tmp = new $class;
    if (!$tmp->dataStructure) {
      continue;
    }
    $deps[$class]=array();
    $links[$class]=array();
    @reset($tmp->dataStructure["fields"]);
    while (list($attName, $attDesc)=@each($tmp->dataStructure["fields"])) {
      if (!$attDesc["link_class"]) {
        continue;
      }
      if (!class_exists($attDesc['link_class'])) {
        echo('*Link class "'.niceClass($attDesc['link_class'])."\" for attribute $attName defined in class ".niceClass($class)." does not exist! Skipping link.\n");
        continue;
      }
      $links[$class][]=array(
        "attr" => $attName,
        "class" => $attDesc["link_class"]
      );
    }
    if (!count($links[$class])) {
      unset($links[$class]);
    }
    $deps[$class]=$tmp->dataStructure["depend"];
    while(list($depName,$depData)=@each($tmp->dataStructure['depend'])) {
      if (!class_exists($depData['class'])) {
        echo('*Dependency class '.niceClass($depData['class']).' linked to class "'.niceClass($class).'" does not exist. Skipping dependecy test.'."\n");
        unset($deps[$class][$depName]);
        continue;
      }
      if (!$deps[$class]) {
        unset($deps[$class]);
      }
    }
  }
  
  if ($verbose) {
    echo("========= Link Consistency Check =========\n");
  }
  while (list($class,$link_list)=@each($links)) {
    $tmp = new $class;
    $entryCount=$tmp->searchCount();
    if ($verbose) {
      echo('--- Class '.niceClass($class)." ($entryCount objects) ---\n");
      for ($j=0;$j<count($link_list);$j++) {
        echo('  Attribute "'.$link_list[$j]['attr'].'" is class '.niceClass($link_list[$j]['class'])."\n");
      }
    }
    if (($tmp->disableConsistency=='link') || ($tmp->disableConsistency=='all')) {
      if ($verbose) {
        echo("  Class has links consistency checking disabled.\n");
      }
      continue;
    }
    $leftCount=$entryCount;
    $offsetCount=0;
    while ($leftCount) {
      $id_fld=$tmp->dataStructure['id_field'];
      $tbl_name=$tmp->dataStructure['table_name'];
      $tmp->query("SELECT $id_fld FROM $tbl_name ORDER BY $id_fld LIMIT $offsetCount,$update_interval");
      $tmp_array=array();
      while ($tmp->db->next_record()) {
        $tmp_array[]=$tmp->db->f(0);
      }
      $updInt=min(count($tmp_array),$update_interval);
      $leftCount=$leftCount-$updInt;
      if ($verbose) {
        echo(" (".((int) (($offsetCount+count($tmp_array))/$entryCount*100))."% retrieved ".count($tmp_array)." objects [$offsetCount..".($offsetCount+$updInt)."], $leftCount left)\n");
      }
      $offsetCount+=$updInt;
      $object_list=$tmp->instantiate($tmp_array);
      for ($j=0;$j<count($link_list);$j++) {
        $link_class = new $link_list[$j]['class'];
        $link_attr=$link_list[$j]['attr'];
        for ($i=0;$i<count($object_list);$i++) {
          $link_id=$object_list[$i]->getAttr($link_attr);
          if (!$link_class->probe($link_id) && $link_id!=0) {
            echo('*'.niceClass($class).' #'.$object_list[$i]->id.', attr "'.$link_attr.'" points to '.niceClass($link_list[$j]['class']).' #'.$link_id.', which is missing'."\n");
            $report['links_found']++;
            if ($myDep=$deps[strtolower($link_list[$j]['class'])]) {
              $msg=false;
              while(list($depName,$depDetails)=each($myDep)) {
                if ((strtolower($depDetails['class'])==strtolower($class)) && ($depDetails['attr']==$link_attr)) {
                  if ($depDetails['on_mod']=='WIFE') {
                    if ($object_list[$i]->delete()) {
                      $msg=true;
                      echo(" + Object deleted, as recommended by reverse dependency $depName.\n");
                      $report['obj_del']++;
                      $offsetCount--;
                    } else {
                      $msg=true;
                      echo(" ! Failed to delete object as recommended by reverse dependency $depName!\n");
                    }
                  } elseif ($depDetails['on_mod']=='MISTRESS') {
                    $object_list[$i]->setAttr($link_attr,0);
                    if ($object_list[$i]->save()) {
                      $msg=true;
                      echo(" + Link deleted, as recommended by reverse dependency $depName.\n");
                      $report['link_del']++;
                    } else {
                      $msg=true;
                      echo(" ! Failed to delete link as recommended by reverse dependency $depName!\n");
                    }
                  } elseif ($depDetails['on_mod']=='STRANGER') {
                    $msg=true;
                    echo(" ! Reverse dependency $depName recommends that this inconsistency should be left alone!\n");
                    $report['ignored']++;
                  } else {
                    $msg=true;
                    echo(" ! Unknown on_mod type for reverse dependency $depName!\n");
                  }
                  break;
                }
              }
              if (!$msg) {
                echo(" ! Failed to find a dependency involving ".niceClass($class)." in class ".niceClass($link_list[$j]['class']).".\n");
              }
            } else {
              echo(" ! Failed to find any dependencies whatsoever for class ".niceClass($link_list[$j]['class'])."\n");
            }
          }
        }
      }
    }
  }
  
  if ($verbose) {
    echo("========= Dependency Consistency Check =========\n");
  }
  while (list($class,$dep_list)=@each($deps)) {
    $tmp = new $class;
    $entryCount=$tmp->searchCount();
    if ($verbose) {
      echo('--- Class '.niceClass($class)." ($entryCount objects) ---\n");
      while (list($dep_name,$dep_data)=@each($dep_list)) {
        $dep_class = new $dep_data['class'];
        if (strtolower($dep_data['type'])!='many') {
          unset($dep_list[$dep_name]);
          continue;
        }
        echo('  Dependency class '.niceClass($dep_data['class'])."\n");
      }
      @reset($dep_list);
    }
    if (($tmp->disableConsistency=='depend') || ($tmp->disableConsistency=='all')) {
      if ($verbose) {
        echo("  Class has dependency consistency checking disabled.\n");
      }
      continue;
    }
    $leftCount=$entryCount;
    $offsetCount=0;
    while ($leftCount) {
      $id_fld=$tmp->dataStructure['id_field'];
      $tbl_name=$tmp->dataStructure['table_name'];
      $tmp->query("SELECT $id_fld FROM $tbl_name ORDER BY $id_fld LIMIT $offsetCount,$update_interval");
      $tmp_array=array();
      while ($tmp->db->next_record()) {
        $tmp_array[]=$tmp->db->f(0);
      }
      $updInt=min(count($tmp_array),$update_interval);
      $leftCount=$leftCount-$updInt;
      if ($verbose) {
        echo(" (".((int) (($offsetCount+count($tmp_array))/$entryCount*100))."% retrieved ".count($tmp_array)." objects [$offsetCount..".($offsetCount+$updInt)."], processing...)\n");
      }
      $offsetCount+=$updInt;
      $object_list=$tmp->instantiate($tmp_array);
      while (list($dep_name,$dep_data)=@each($dep_list)) {
        $dep_class = new $dep_data['class'];
        for ($i=0;$i<count($object_list);$i++) {
          $dep_links=$object_list[$i]->getObjects($dep_name);
          for ($j=0;$j<count($dep_links);$j++) {
            if (!$dep_class->probe($dep_links[$j]->id)) {
              echo('*'.niceClass($class).' #'.$object_list[$i]->id." has a dependency $dep_name to ".niceClass($dep_data['class']).' #'.$dep_links[$j]->id.', which is missing.'."\n");
              $report['deps_found']++;
              if ($dep_data['on_mod']=='WIFE') {
                if ($object_list[$i]->delete()) {
                  echo(" + Object deleted, as recommended by dependency.\n");
                  $offsetCount--;
                  $report['obj_del']++;
                } else {
                  echo(" ! Failed to delete object as recommended by dependency!\n");
                }
              } elseif ($dep_data['on_mod']=='MISTRESS') {
                if ($object_list[$i]->dropLink($dep_name,$dep_links[$j]->id)) {
                  echo(" + Link deleted, as recommended by dependency.\n");
                  $report['dep_del']++;
                } else {
                  echo(" ! Failed to delete link as recommended by dependency!\n");
                }
              } elseif ($dep_data['on_mod']=='STRANGER') {
                echo(" - Dependency recommends that this inconsistency should be left alone!\n");
                $report['ignored']++;
              } else {
                echo(" ! Unknown on_mod type for dependency!\n");
              }
            }
          }
        }
      }
    }
  }
  if ($verbose) {
    echo("\n========= Summary =========\n");
    echo("Inconsistent links: {$report['links_found']}\n");
    echo("Inconsistent dependencies: {$report['deps_found']}\n");
    echo("Total inconsistencies found: ".($report['links_found']+$report['deps_found'])."\n");
    echo("Deleted links: {$report['links_del']}\n");
    echo("Deleted dependencies: {$report['deps_del']}\n");
    echo("Deleted objects: {$report['obj_del']}\n");
    echo("Total inconsistencies fixed: ".($report['links_del']+$report['deps_del']+$report['obj_del'])."\n");
    echo("Inconsistencies ignored due to data structure which allows them: {$report['ignored']}\n");
    echo("Remaining inconsistencies which should be fixed: ".($report['links_found']+$report['deps_found']-$report['links_del']-$report['deps_del']-$report['obj_del']-$report['ignored'])."\n");
    echo("\n");
    echo("Consistency checker finished. Clean exit.\n");
  }
?>
