<?

  /**
  * SC Class Consistency Check
  * Bogdan Stancescu <bogdan@lanifex.com>, December 2002
  * $Id: consist_show_criminal.php,v 1.5 2005/01/11 14:32:28 bogdan Exp $
  * @package LFX
  */

  $_page->title=__L("Class Consistency Checker (Criminal View)");

  $_incReq=array('LFX_menu_init');
  include($_incMgr);
  $_page->menu->focus('Class Consistency');

  while(list($inc)=@each($_LFX["global"]["master_include_array"])) {
    $_incReq[]=$inc;
  }
  if (!include($_incMgr)) {
    $_page->addError("Unable to perform required includes. Please note this is the consistency checker, ".
      "which has to load ALL dependencies, so this might not be a problem at all for the whole of the system.");
    exit;
  }

  pecho("<p>".__L("This view groups the inconsistencies by 'criminal'. ".
    "A 'victim' in this context is the 'innocent' class: the one which ".
    "contains the proper links or dependencies; the 'criminal' is the 'guilty' ".
    "class, the one which doesn't reciprocate the link or dependency.")."</p>");
  pecho("<p>".sprintf(__L("This view is useful for identifying the classes which ".
    "need to be fixed in this regard. If you want to fix the ".
    "database inconsistencies, you might want to switch to %svictim view%s, ".
    "that one helps you identify the types of entries which might be left ".
    "laying around in the database."),
    "<a href='$_LFX_url/util/consist_show_victim.php'>","</a>")."</p>");

  $col_same_module="#ee0000";
  $col_other_module="#aaaa00";
  $col_missing="#00aa00";

  pecho(LFX_startTables());
  pecho(LFX_str());
  pecho(LFX_td("Class", "thead"));
  pecho(LFX_td("Links", "thead"));
  pecho(LFX_td("Deps", "thead"));
  pecho(LFX_td("Messages", "thead"));
  pecho(LFX_etr());
  $classes=get_declared_classes();
  asort($classes);
  $deps=array();
  $links=array();

  // First off, read the dependency data
  while (list(,$class)=each($classes)) {
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }
    $deps[$class]=array();
    $links[$class]=array();
    $deps[$class]=array();
    $links[$class]=array();
    $tmp = new $class;
    @reset($tmp->dataStructure["fields"]);
    while (list($attName, $attDesc)=@each($tmp->dataStructure["fields"])) {
      if (!$attDesc["link_class"]) {
        continue;
      }
      $links[$class][]=array(
        "attr" => $attName,
        "class" => $attDesc["link_class"]
      );
    }
    $deps[$class]=$tmp->dataStructure["depend"];
    $module[$class]=$tmp->module;
  }

  // Now parsing dependencies and distributing messages to classes
  // which have to be acted upon. Please note we don't output anything yet!
  $msgs=array();
  $msgPri=array();
  reset($classes);
  while (list(,$class)=each($classes)) {
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }

    // Checking dependencies
    @reset($deps[$class]);
    while (list($depName,$dep)=@each($deps[$class])) {
      $ok=false;
      $dep["class"]=strtolower($dep["class"]);
      if ($dep['type']=='many') {
        $otherDep=$deps[$dep['class']];
        while (list(,$tmp)=@each($otherDep)) {
          if (
          (strtolower($tmp['class'])==$class) &&
          ($tmp['type']=='many') &&
          ($tmp['table_name']==$dep['table_name']) &&
          ($tmp['link_fld_name']==$dep['my_fld_name']) &&
          ($tmp['my_fld_name']==$dep['link_fld_name'])
          ) {
            $ok=true;
            break;
          }
        }
      } else {
        @reset($links[$dep["class"]]);
        while (list(,$tmp)=@each($links[$dep["class"]])) {
          if ((strtolower($tmp["class"])==$class) && ($tmp["attr"]==$dep["attr"])) {
            $ok=true;
            break;
          }
        }
      }
      if (!$ok) {
        if ($module[$class]==$module[$dep['class']]) {
          $emphasis=array("<font color='$col_same_module'>",'</font>');
          $pri=0;
        } elseif (class_exists($dep["class"])) {
          $emphasis=array("<font color='$col_other_module'>",'</font>');
          $pri=1;
        } else {
          $emphasis=array("<font color='$col_missing'>",'</font>');
          $pri=2;
        }
        if ($dep['type']=='many') {
          $msg='<b>Many to many dependency</b> <i>%1$s</i> in class '.
            '<b>%2$s</b> not reciprocated in this class.';
        } else {
          $msg='<b>Dependency</b> <i>%1$s</i> in class <b>%2$s</b> '.
            'not reciprocated as a link in this class.';
        }
        $msgs[$dep['class']][]="$emphasis[0]".
          sprintf(__L($msg),$depName,niceClass($class))."$emphasis[1]<br />";
        $msgPri[$dep['class']][]=$pri;
      }
    }

    // Checking links
    @reset($links[$class]);
    while (list(,$link)=@each($links[$class])) {
      $ok=false;
      $link["class"]=strtolower($link["class"]);
      @reset($deps[$link["class"]]);
      while (list(,$tmp)=@each($deps[$link["class"]])) {
        if ((strtolower($tmp["class"])==$class) && ($tmp["attr"]==$link["attr"])) {
          $ok=true;
        }
      }
      if ($module[$class]==$module[$link['class']]) {
        $emphasis=array("<font color='$col_same_module'>",'</font>');
        $pri=0;
      } elseif (class_exists($link['class'])) {
        $emphasis=array("<font color='$col_other_module'>",'</font>');
        $pri=1;
      } else {
        $emphasis=array("<font color='$col_missing'>",'</font>');
        $pri=2;
      }
      if (!$ok) {
        $msgs[$link['class']][]="$emphasis[0]".
          sprintf(__L('<b>Link</b> on attribute <i>%1$s</i> '.
            'in class <b>%2$s</b> not reciprocated as a dependency '.
            'in this class.'),$link["attr"],niceClass($class)).
          "$emphasis[1]<br />\n";
        $msgPri[$link['class']][]=$pri;
      }
    }
  }
  
  // Ok, we're finally ready to output the data we collected
  reset($classes);
  while (list(,$class)=each($classes)) {
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }
    pecho(LFX_str());
    pecho(LFX_rth("<A HREF='#".niceClass($class)."'>".niceClass($class)."</A>"));
    pecho(LFX_td(count($links[$class])));
    pecho(LFX_td(count($deps[$class])));

    $msg='';
    if ($msgPri[$class]) {
      asort($msgPri[$class]);
      while(list($index)=each($msgPri[$class])) {
        $msg.=$msgs[$class][$index];
      }
      unset($msgPri[$class],$msgs[$class]);
    }
    pecho(LFX_td("<A name='".niceClass($class)."'></A>".($msg?$msg:"OK")));
    pecho(LFX_etr());
  }
  
  // Let's now check for warnings in inexistent classes
  while (list($class,$messages)=each($msgs)) {
    pecho(LFX_str());
    pecho(LFX_rth(niceClass($class)));
    pecho(LFX_td('??'));
    pecho(LFX_td('??'));
    asort($msgPri[$class]);
    while(list($index)=each($msgPri[$class])) {
      $msg.=$msgs[$class][$index];
    }
    pecho(LFX_td($msg?$msg:"OK"));
    pecho(LFX_etr());
  }
  
  pecho(LFX_endTables());

  pecho("<br />\n");

  pecho(LFX_startTables());
  pecho(LFX_swatchHeader(array("title"=>__L("Legend"))));
  pecho(LFX_str());
  $legend="<font color='$col_same_module'><b>".__L("Missing link/dependency for existing classes in the same module")."</b></font><br />\n";
  $legend.="<i>".__L("This is really urgent to fix, you have inconsistencies within one single package.")."</i><br /><br />\n";
  $legend.="<font color='$col_other_module'><b>".__L("Missing link/dependency for existing classes in distinct modules")."</b></font><br />\n";
  $legend.="<i>".__L("This is important to fix, you must register the dependency via LFX_register_dependency().")."</i><br /><br />\n";
  $legend.="<font color='$col_missing'><b>".__L("Missing link/dependency for inexistent classes")."</b></font><br />\n";
  $legend.="<i>".__L("This is impossible to fix at the moment, since the \"other\" class isn't defined yet. Just make sure to remember this when you create that class.")."</i>\n";
  pecho(LFX_td($legend));
  pecho(LFX_etr());
  pecho(LFX_endTables());

  $_page->show();
?>
