<?

  /**
  * SC Class Consistency Check
  * Bogdan Stancescu <bogdan@lanifex.com>, December 2002
  * $Id: consist_show_victim.php,v 1.4 2005/01/11 14:33:09 bogdan Exp $
  * @package LFX
  */

  $_page->title=__L("Class Consistency Checker (Victim View)");

  $_incReq=array('LFX_menu_init');
  include($_incMgr);
  $_page->menu->focus('Class Consistency');

  while(list($inc)=@each($_LFX["global"]["master_include_array"])) {
    $_incReq[]=$inc;
  }
  if (!include($_incMgr)) {
    $_page->addError("Unable to perform required includes. Please note this is the consistency checker, ".
      "which has to load ALL dependencies, so this might not be a problem at all for the whole of the system.");
    exit;
  }

  pecho("<p align='center'><b><i>PLEASE BE AWARE THAT THIS SCRIPT DOES NOT WORK PROPERLY WITH MANY TO MANY RELATIONSHIPS, IN THAT IT ISSUES FALSE POSITIVES ON THOSE! FOR AN ACCURATE VIEW OF THE CURRENT SITUATION, PLEASE USE THE 'CRIMINAL' VIEW!</i></b></p>");
  pecho("<p>".__L("This view groups the inconsistencies by 'victim'. ".
    "A 'victim' in this context is the 'innocent' class: the one which ".
    "contains the proper links or dependencies; the 'criminal' is the 'guilty' ".
    "class, the one which doesn't reciprocate the link or dependency.")."</p>");
  pecho("<p>".sprintf(__L("This view is useful for identifying entries which might ".
    "have been left laying around in the database. If you want to fix the ".
    "actual class code, you might want to switch to %scriminal view%s, ".
    "that one helps you identify classes which need fixes so you don't have ".
    "to jump back and forth between them."),
    "<a href='$_LFX_url/util/consist_show_criminal.php'>","</a>")."</p>");

  $col_same_module="#ee0000";
  $col_other_module="#aaaa00";
  $col_missing="#00aa00";

  pecho(LFX_startTables());
  pecho(LFX_str());
  pecho(LFX_td("Class", "thead"));
  pecho(LFX_td("Links", "thead"));
  pecho(LFX_td("Deps", "thead"));
  pecho(LFX_td("Messages", "thead"));
  pecho(LFX_etr());
  $classes=get_declared_classes();
  asort($classes);
  $deps=array();
  $links=array();

  // First off, read the dependency data
  while (list(,$class)=each($classes)) {
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }
    $deps[$class]=array();
    $links[$class]=array();
    $deps[$class]=array();
    $links[$class]=array();
    $tmp = new $class;
    @reset($tmp->dataStructure["fields"]);
    while (list($attName, $attDesc)=@each($tmp->dataStructure["fields"])) {
      if (!$attDesc["link_class"]) {
        continue;
      }
      $links[$class][]=array(
        "attr" => $attName,
        "class" => $attDesc["link_class"]
      );
    }
    $deps[$class]=$tmp->dataStructure["depend"];
    $module[$class]=$tmp->module;
  }

  // Debug stuff
  /*
  echo("<pre>\n");
  echo("<b>Dependencies</b>\n");
  print_r($deps);
  echo("\n<b>Links</b>\n");
  print_r($links);
  echo("</pre>\n");
  */

  reset($classes);
  while (list(,$class)=each($classes)) {
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }
    pecho(LFX_str());
    pecho(LFX_rth(niceClass($class)));
    pecho(LFX_td(count($links[$class])));
    pecho(LFX_td(count($deps[$class])));

    // Start checking
    $msgs=array();
    $msgPri=array();

    // Checking dependencies
    @reset($deps[$class]);
    while (list($depName,$dep)=@each($deps[$class])) {
      $ok=false;
      $dep["class"]=strtolower($dep["class"]);
      // $msg.="Starting to look into \$links[{$dep['class']}]<br />\n";
      @reset($links[$dep["class"]]);
      while (list(,$tmp)=@each($links[$dep["class"]])) {
        // $msg.="Checking class ".$dep["class"]." for link to $class on attribute ".$dep["attr"]."<br />";
        if ((strtolower($tmp["class"])==$class) && ($tmp["attr"]==$dep["attr"])) {
          $ok=true;
        }
      }
      if (!$ok) {
        if ($module[$class]==$module[$dep['class']]) {
          $emphasis=array("<font color='$col_same_module'>",'</font>');
          $pri=0;
        } elseif (class_exists($dep["class"])) {
          $emphasis=array("<font color='$col_other_module'>",'</font>');
          $pri=1;
        } else {
          $emphasis=array("<font color='$col_missing'>",'</font>');
          $pri=2;
        }
        $msgs[]="$emphasis[0]".
          sprintf(__L('<b>Dependency</b> <i>%1$s</i> to class <b>%2$s</b> '.
            'not reciprocated as a link in that class.'),
            $depName,niceClass($dep["class"])).
          "$emphasis[1]<br />";
        $msgPri[]=$pri;
      }
    }

    // Checking links
    @reset($links[$class]);
    while (list(,$link)=@each($links[$class])) {
      $ok=false;
      $link["class"]=strtolower($link["class"]);
      @reset($deps[$link["class"]]);
      while (list(,$tmp)=@each($deps[$link["class"]])) {
        if ((strtolower($tmp["class"])==$class) && ($tmp["attr"]==$link["attr"])) {
          $ok=true;
        }
      }
      if ($module[$class]==$module[$link['class']]) {
        $emphasis=array("<font color='$col_same_module'>",'</font>');
        $pri=0;
      } elseif (class_exists($link['class'])) {
        $emphasis=array("<font color='$col_other_module'>",'</font>');
        $pri=1;
      } else {
        $emphasis=array("<font color='$col_missing'>",'</font>');
        $pri=2;
      }
      if (!$ok) {
        $msgs[]="$emphasis[0]".
          sprintf(__L('<b>Link</b> on attribute <i>%1$s</i> '.
            'to class <b>%2$s</b> not reciprocated as a dependency '.
            'in that class.'),$link["attr"],niceClass($link["class"])).
          "$emphasis[1]<br />\n";
        $msgPri[]=$pri;
      }
    }
    asort($msgPri);
    $msg='';
    while(list($index)=each($msgPri)) {
      $msg.=$msgs[$index];
    }
    pecho(LFX_td($msg?$msg:"OK"));
    pecho(LFX_etr());
  }
  pecho(LFX_endTables());

  pecho("<br />\n");

  pecho(LFX_startTables());
  pecho(LFX_swatchHeader(array("title"=>__L("Legend"))));
  pecho(LFX_str());
  $legend="<font color='$col_same_module'><b>".__L("Missing link/dependency for existing classes in the same module")."</b></font><br />\n";
  $legend.="<i>".__L("This is really urgent to fix, you have inconsistencies within one single package.")."</i><br /><br />\n";
  $legend.="<font color='$col_other_module'><b>".__L("Missing link/dependency for existing classes in distinct modules")."</b></font><br />\n";
  $legend.="<i>".__L("This is important to fix, you must register the dependency via LFX_register_dependency().")."</i><br /><br />\n";
  $legend.="<font color='$col_missing'><b>".__L("Missing link/dependency for inexistent classes")."</b></font><br />\n";
  $legend.="<i>".__L("This is impossible to fix at the moment, since the \"other\" class isn't defined yet. Just make sure to remember this when you create that class.")."</i>\n";
  pecho(LFX_td($legend));
  pecho(LFX_etr());
  pecho(LFX_endTables());

  $_page->show();
?>
