<?

  /**
  * SC Data Type converter
  * Bogdan Stancescu <bogdan@lanifex.com>, October 2004
  * $Id: data_type_converter.php,v 1.1 2004/11/09 10:45:42 bogdan Exp $
  * @package LFX
  */

  require(dirname(dirname(__FILE__)).'/LFX_lib.php');
  ob_end_clean();

  LFX_subsystemStart();
  
  if (!$_SERVER['argv'][1]) {
    cecho("Please specify the name of a class from the list below as the first ".
      "parameter to this script (please ignore the PHP headers warning(s) below):",true);

    while(list($inc)=@each($_LFX["global"]["master_include_array"])) {
      $_incReq[]=$inc;
    }
    if (!include($_incMgr)) {
      cecho("Unable to perform required includes. Please note this is the data type converter, ".
        "which has to load ALL dependencies, so this might not be a problem at all for the whole of the system.");
      LFX_subsystemEnd(true);
    }

    $classes=get_declared_classes();
    asort($classes);

    while (list(,$class)=each($classes)) {
      $root=$class;
      while ($new_root=get_parent_class($root)) {
        $root=$new_root;
      }
      if ($root!='lfx_object') {
        continue;
      }
      cecho(niceClass($class),true);
    }
    LFX_subsystemEnd(true);
  }
  
  $class=$_SERVER['argv'][1];
  
  if (!LFX_include_class($class)) {
    cecho("Failed to include class $class!");
    LFX_subsystemEnd(true);
  } else {
    $tmp=new $class;
    $ds=$tmp->dataStructure;
    if ($tmp->query("SELECT MIN({$ds['id_field']}) FROM {$ds['table_name']}")) {
      $tmp->db->next_record();
      $minID=$tmp->db->f(0);
      
      $tmp->query("SELECT MAX({$ds['id_field']}) FROM {$ds['table_name']}");
      $tmp->db->next_record();
      $maxID=$tmp->db->f(0);
      
      $tmp->query("SELECT COUNT(*) FROM {$ds['table_name']}");
      $tmp->db->next_record();
      $totalIDs=$tmp->db->f(0);
      
      cecho("Please be warned that depending on the number of objects ".
        "in the class you select, the volume of data and your server's ".
        "performance, this process can take a considerable amount of time.",true);
      cecho("During that time, the system usage in parallel will be slowed down ".
        "but safe, no side effects should be generated by this operation, ".
        "even while running.",true);
      cecho("If you decide to interrupt this operation, ".
        "you can do it safely by interrupting this process using your ".
        "system's specific controls (usually CTRL+C) at any time.",true);

      $myID=$minID-1;
      // We don't have to worry about new entries in the database being
      // inserted while we do this because we already have the code which
      // uses the new data types, so they will be inherently inserted in
      // the proper format.
      startWatch('data_type_converter');
      $counter=0;
      while($myID<$maxID) {
        $tmp->query("SELECT MIN({$ds['id_field']}) FROM {$ds['table_name']} WHERE {$ds['id_field']}>$myID");
        $tmp->db->next_record();
        $tmp=new $class($tmp->db->f(0));
        $tmp->noLogging=true;
        $tmp->load();
        $tmp->touchAllAttrs();
        if (!$tmp->save()) {
          cecho("Failed at ID {$tmp->id} while processing class ".niceClass($class)."!");
          LFX_subsystemEnd();
        }
        $myID=$tmp->id;
        $counter++;
        if (($counter==10) || !($counter%200)) {
          $timeSoFar=getWatch('data_type_converter');
          $totalTime=$totalIDs*$timeSoFar/$counter-$timeSoFar;
          $estFinalTime=date('r',time()+round($totalTime));
          if ($totalTime>3600) {
            $totalTime=number_format($totalTime/3600,2)." hours";
          } elseif ($totalTime>60) {
            $totalTime=number_format($totalTime/60,2)." minutes";
          } else {
            $totalTime=number_format($totalTime,2)." seconds";
          }
          if ($timeSoFar>3600) {
            $timeSoFar=number_format($timeSoFar/3600,2)." hours";
          } elseif ($timeSoFar>60) {
            $timeSoFar=number_format($timeSoFar/60,2)." minutes";
          } else {
            $timeSoFar=number_format($timeSoFar,2)." seconds";
          }
          cecho("$counter out of $totalIDs ".niceClass($class).
            " objects converted in $timeSoFar. ".
            "Estimated time left: $totalTime (Est time: $estFinalTime)",true);
        }
      }
      cecho("Class ".niceClass($class)." converted successfully!");
    } else {
      cecho("Failed to execute basic query on class ".niceClass($class)."!");
      LFX_subsystemEnd(true);
    }
  }

  LFX_subsystemEnd();
?>
