<?
  /**
  * A script to remove the empty data files.
  * This script only needs to run on selected systems, and only once.
  * Bogdan Stancescu <bogdan@lanifex.com>, July 2004
  * $Id: remove_empty_data_files.php,v 1.2 2004/08/02 11:52:38 bogdan Exp $
  * @package LFX
  */
    
  require(dirname(__FILE__).'/../LFX_lib.php');
  ob_end_clean();

  // Comment out the following line for debugging
  $_LFX['page']['debug_level']=LFX_DL_QUIET;

  $verbose=false;

  if (in_array('-h',$argv)) {
    echo('This script transparently removes empty data files. '.
      'It accepts no parameters. '.
      "So, if that's what you need, just run it again with no parameters.\n");
    exit;
  }

  while(list($inc)=@each($_LFX["global"]["master_include_array"])) {
    $_incReq[]=$inc;
  }
  if (!include($_incMgr)) {
    echo("Unable to perform required includes. Please note this is an ".
      "utility script which has to load ALL dependencies, so this might not ".
      "be a problem at all for the whole of the system.");
    exit;
  }
  
  echo("All includes succeeded. Starting to browse the database. A vertical ".
    "bar represents a new class being parsed; a dot represents a successfully ".
    "fixed zero-length file. An exclamation mark represents a failure to fix ".
    "such a file. A brief report will be presented at the end. Here we go:\n");
  $classes=get_declared_classes();
  asort($classes);
  $deps=array();
  $links=array();

  // First off, read the fields
  $stats=array(
    'total'=>0, // Total number of files found
    'zero' =>0, // Number of zero-length files found
    'fixed'=>0, // Number of files fixed
    'failed'=>0 // Number of files failed
  );
  while (list(,$class)=each($classes)) {
    if (!class_exists($class)) {
      continue;
    }
    echo('|');
    $root=$class;
    while ($new_root=get_parent_class($root)) {
      $root=$new_root;
    }
    if ($root!='lfx_object') {
      continue;
    }
    $tmp = new $class;
    if (!$tmp->dataStructure) {
      continue;
    }
    @reset($tmp->dataStructure["fields"]);
    while (list($attName, $attDesc)=@each($tmp->dataStructure["fields"])) {
      if (!$attDesc['flags']['file']) {
        continue;
      }
      $stats['total']++;
      //echo("Got one! Class $class, attribute $attName ({$attDesc['fld_name']})!\n");
      $tmp->query("SELECT ".$tmp->dataStructure['id_field'].
        " FROM ".$tmp->dataStructure['table_name']);
      while ($tmp->db->next_record()) {
        $tmp->id=$tmp->db->f(0);
        $fname=$tmp->_getDataFile($attName);
        if (!is_file($fname)) {
          continue;
        }
        if(!filesize($fname)) {
          $stats['zero']++;
          //echo("Empty one: $fname; fixing... ");
          $tmp2=new $class;
          if ($tmp2->query("UPDATE ".$tmp->dataStructure['table_name'].
            " SET {$attDesc['fld_name']}=0 ".
            "WHERE ".$tmp->dataStructure['id_field']."=".$tmp->db->f(0)) &&
            unlink($fname)) {
            //echo("Done!\n");
            $stats['fixed']++;
            echo('.');
          } else {
            //echo("FAILED!\n");
            $stats['failed']++;
            echo('!');
          }
        }
      }
    }
  }
  echo("\n");
  if ($stats['zero']) {
    echo("Found {$stats['total']} LFXlib data files on the system, ".
      "of which {$stats['zero']} had zero size. Of these, {$stats['fixed']} ">
      "were fixed and {$stats['failed']} have failed.\n");
  } else {
    echo("Found {$stats['total']} LFXlib data files on the system, ".
      "but none was empty.\n");
  }
?>
