<?php

 /**
  * This script will check the consistency of EH and DMO databases.
  * 1. will check the synchronization between EH_sensors and DMO_sensors. 
  * 2. will check for orphan SUBNETS - that is, they have no correspondence to DMO sensors.
  * 3. Will check for orphan HOSTS - that is, they have no correspondence to DMO Subnets.
  * Edward Surubaru <eddie@moongate.ro>, October 2005
  * $Id: checksensors.php,v 1.1 2005/10/12 13:41:57 bogdan Exp $
  */  


  $_incReq=array('DMO_basic', 'DMO_Definition', 'DMO_Instance',
      'EH_Sensor', 'DMO_Instattr', 'EH_Event', 'DMO_Attribute');
  include($_incMgr);

  $tmpDef=new DMO_Definition;
  list($sensorDef)=$tmpDef->search('name','Sensor');
  list($subnetDef)=$tmpDef->search('name','Subnet');
  list($hostDef)=$tmpDef->search('name','Host');
  list($unknDef)=$tmpDef->search('name','Unknown');
  
  $att=new DMO_Attribute;
  $attrScanEnable = $att->byNameID('ScanEnable');
  $attrStatus = $att->byNameID('Host Status');
  $attrSubnet     = $att->byNameID('Subnet Address');
  $attrIP         = $att->byNameID('IP Address');
  $attrLastActive = $att->byNameID('Active Date');
  
  $ins = new DMO_instance();
  $ia = new DMO_instattr();
// Will get all DMO orphan sensors - that is, they have no correspondence to EH sensors.
  $hosts = new DMO_object();
  $sql = "SELECT sensor.instance_ID AS DMO_SENSOR_ID, sensor.instance_name AS DMO_SENSOR_NAME
          FROM instance sensor 
            LEFT JOIN eh_idw.sensor eh_sensor ON (sensor.instance_ID = eh_sensor.dmo_id)
            WHERE sensor.definition_ID = {$sensorDef->id} AND eh_sensor.sensor_ID is NULL";
  $hosts->query($sql);

  if ($hosts->db->num_rows()) {
    echo("<BR><div align='center'><B>Warning! Detected the following DMO Sensor(s) unsychronized (not found in EH-Sensor):</B></div>");
    echo("<table border=1 width='90%' align='center'>
          <tr align='center'>
            <td colspan=2 >SENSORS</td>
            <td colspan=3>ATTRIBUTES</td>
            <td colspan=2>CHILDREN</td>
          </tr>
          <tr align='center'>
            <td>DMO_ID</td>
            <td>DMO_NAME</td>
            <td>SCAN_ENABLE</td>
            <td>HOST_STATUS</td>
            <td>IP</td>
            <td>SUBNET(S)</td>
            <td>NO_OF_HOSTS</td>
          </tr>");
    while ($hosts->db->next_record()) {
      echo("<tr align='center'>");
      echo("<td>".$hosts->db->f("DMO_SENSOR_ID")."</td><td>".$hosts->db->f("DMO_SENSOR_NAME")."</td>");

      $scanEnbl = $ia->search(array('instance','attribute'),array($hosts->db->f("DMO_SENSOR_ID"), $attrScanEnable));
      echo("<td>".($scanEnbl ? $scanEnbl[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");
 
      $attStatus = $ia->search(array('instance','attribute'),array($hosts->db->f("DMO_SENSOR_ID"), $attrStatus));
      echo("<td>".($attStatus ? $attStatus[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");

      $attIP = $ia->search(array('instance','attribute'),array($hosts->db->f("DMO_SENSOR_ID"), $attrIP));
      echo("<td>".($attIP ? $attIP[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");

      //Show children
      $subN = $ins->search('parent', $hosts->db->f("DMO_SENSOR_ID"));
      echo("<td>");
      if ($subN) {
        $kids_no = 0;
        for ($i = 0; $i < count($subN); $i++) {
          echo($subN[$i]->getAttr('name'));
          if ($i != count($subN)-1) echo("<BR>");
          $kids = $ins->search('parent', $subN[$i]->id);
          $kids_no += count($kids);
        }
      } else {
        echo("<font color='#A00003'>Not found!</font>");
      }
      echo("<td>".($kids_no ? $kids_no : "None")."</td>");
      echo("</tr>");
      unset($kids_no);
    }
    echo("</table>");
  } else {
    echo(" *** DMO Sensors integrity... OK.<BR>");
  }


  // Will get all EH orphan sensors - that is, they have no correspondence to DMO sensors.
  $hosts = new DMO_object();
  $sql = "SELECT eh_sensor.sensor_ID AS EH_SENSOR_ID, eh_sensor.sensor_name AS EH_SENSOR_NAME,
            eh_sensor.sensor_status AS SENSOR_STATUS, eh_sensor.sensor_ip AS SENSOR_IP, eh_sensor.sensor_function AS SENSOR_FUNCTION, eh_sensor.dmo_id AS DMO_ID
          FROM eh_idw.sensor eh_sensor 
            LEFT JOIN instance sensor ON (sensor.instance_ID = eh_sensor.dmo_id)
            WHERE sensor.instance_ID is NULL";
  $hosts->query($sql);

  if ($hosts->db->num_rows()) {
    echo("<BR><div align='center'><B>Warning! Detected the following EH Sensor(s) unsychronized (not found in DMO-Sensor):</B></div>");
    echo("<table border=1 width='90%' align='center'>
          <tr align='center'>
            <td>EH_ID</td>
            <td>EH_NAME</td>
            <td>SENSOR_STATUS</td>
            <td>IP</td>
            <td>FUNCTION</td>
            <td>DMO_ID</td>
          </tr>");
    while ($hosts->db->next_record()) {
      echo("<tr align='center'>");
      echo("<td>".$hosts->db->f("EH_SENSOR_ID")."</td>");
      echo("<td>".$hosts->db->f("EH_SENSOR_NAME")."</td>");
      echo("<td>".$hosts->db->f("SENSOR_STATUS")."</td>");
      echo("<td>".$hosts->db->f("SENSOR_IP")."</td>");
      echo("<td>".$hosts->db->f("SENSOR_FUNCTION")."</td>");
      echo("<td><font color='#A00003'>(".$hosts->db->f("DMO_ID").")</font></td>");
      echo("</tr>");
    }
    echo("</table>");
  } else {
    echo("<BR> *** EH Sensors integrity... OK.<BR>");
  }

  // Will get all orphan SUBNETS - that is, they have no correspondence to DMO sensors.
  $hosts = new DMO_object();
  $sql = "select subnet.instance_ID AS SUBNET_ID, subnet.instance_name AS SUBNET_NAME
	FROM instance subnet
	LEFT JOIN instance sensor ON (subnet.instance_parent = sensor.instance_ID)
	WHERE subnet.definition_ID = {$subnetDef->id}
	AND sensor.instance_ID IS NULL";
  $hosts->query($sql);

  if ($hosts->db->num_rows()) {
    echo("<BR><div align='center'><B>Warning! Detected the following orphan Subnet(s) (no parent DMO-Sensor):</B></div>");
    echo("<table border=1 width='90%' align='center'>
          <tr align='center'>
            <td>DMO_ID</td>
            <td>DMO_NAME</td>
            <td>SCAN_ENABLE</td>
            <td>HOST_STATUS</td>
            <td>IP</td>
            <td>HOST_NAMES</td>
            <td>NO_OF_HOSTS</td>
          </tr>");
    while ($hosts->db->next_record()) {
      echo("<tr align='center'>");
      echo("<td>".$hosts->db->f("SUBNET_ID")."</td><td>".$hosts->db->f("SUBNET_NAME")."</td>");

      $scanEnbl = $ia->search(array('instance','attribute'),array($hosts->db->f("SUBNET_ID"), $attrScanEnable));
      echo("<td>".($scanEnbl ? $scanEnbl[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");
 
      $attStatus = $ia->search(array('instance','attribute'),array($hosts->db->f("SUBNET_ID"), $attrStatus));
      echo("<td>".($attStatus ? $attStatus[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");

      $attIP = $ia->search(array('instance','attribute'),array($hosts->db->f("SUBNET_ID"), $attrIP));
      echo("<td>".($attIP ? $attIP[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");

      //Show children
      $subN = $ins->search('parent', $hosts->db->f("SUBNET_ID"));
      echo("<td>");
      if ($subN) {
        $kids_no = 0;
        for ($i = 0; $i < count($subN); $i++) {
          echo($subN[$i]->getAttr('name'));
          if ($i != count($subN)-1) echo("<BR>");
        }
        $kids_no = count($subN);
      } else {
        echo("<font color='#A00003'>Not found!</font>");
      }
      echo("<td>".($kids_no ? $kids_no : "None")."</td>");
      echo("</tr>");
      unset($kids_no);
    }
    echo("</table>");
  } else {
    echo("<BR> *** SUBNETS integrity... OK.<BR>");
  }

  // Will get all orphan HOSTS - that is, they have no correspondence to DMO Subnets.
  $hosts = new DMO_object();
  $sql = "SELECT def.definition_name AS DEFINITION, host.instance_ID AS HOST_ID, host.instance_name AS HOST_NAME
          FROM definition def
            LEFT JOIN instance host   ON (host.definition_ID = def.definition_ID)
            LEFT JOIN instance subnet ON (host.instance_parent = subnet.instance_ID)
            WHERE (def.definition_parent = {$hostDef->id} OR host.definition_ID = {$unknDef->id}) 
              AND host.instance_ID IS NOT NULL
              AND subnet.instance_ID IS NULL";
  $hosts->query($sql);

  if ($hosts->db->num_rows()) {
    echo("<BR><div align='center'><B>Warning! Detected the following orphan Host(s) (no parent DMO-Subnet):</B></div>");
    while ($hosts->db->next_record()) {
    echo("<table border=1 width='90%' align='center'>
          <tr align='center'>
            <td>[DEFINITION]</td>   
            <td>DMO_ID</td>
            <td>DMO_NAME</td>
            <td>SCAN_ENABLE</td>
            <td>HOST_STATUS</td>
            <td>IP</td>
          </tr>");
    while ($hosts->db->next_record()) {
      echo("<tr align='center'>");

      echo("<td>[".$hosts->db->f("DEFINITION")."]</td>");

      echo("<td>".$hosts->db->f("HOST_ID")."</td><td>".$hosts->db->f("HOST_NAME")."</td>");

      $scanEnbl = $ia->search(array('instance','attribute'),array($hosts->db->f("HOST_ID"), $attrScanEnable));
      echo("<td>".($scanEnbl ? $scanEnbl[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");
 
      $attStatus = $ia->search(array('instance','attribute'),array($hosts->db->f("HOST_ID"), $attrStatus));
      echo("<td>".($attStatus ? $attStatus[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");

      $attIP = $ia->search(array('instance','attribute'),array($hosts->db->f("HOST_ID"), $attrIP));
      echo("<td>".($attIP ? $attIP[0]->getAttr('text') : "<font color='#A00003'>Not found!</font>")."</td>");
      echo("</tr>");
    }
    echo("</table>");
    }
  } else {
    echo("<BR> *** HOSTS integrity... OK.<BR>");
  }



//===================================================================================

?>
