<?
  /*
  * Attribute attachment handler
  * Author: Paul Gillingwater (C) CSO Lanifex GmbH
  * Date: January 2004
  * $Id: test2.php,v 1.14 2005/06/23 15:00:01 dieter Exp $
  * @package DMO
  */
  
  # This code is used to manage a directory of files, which are stored
  # under directories and with names that correspond to the key (id) taken
  # from a database record, rather than the original filename.  We will
  # retain the extension (to make it easier to manage MIME type operations)
  # but store other meta information (such as the filename before the 
  # extension) and its MIME type in the database.
  
  # The purpose of doing this is to prevent filename collisions, as most
  # people will choose common names which are likely to clash.  By saving
  # the file as a index number, we can quickly access it from the database
  # just by using the correct path.
  
  # Because we have potentially tens of thousands of files, we don't want
  # to store them all in one directory, as the traversal of the directory
  # will take too long.  Therefore, we use subdirectories, which by limiting
  # path component names to two characters, will limit directory contents.
  
  # Note there is a security issue.   An attacker with access to the Web
  # site might be able to iterate through all files, and retrieve them
  # without control of permissions.  Therefore, maybe we should store
  # these files in a place where only the script can access them, and
  # only allow the script to retrieve and present them.
  
  $_incReq=array('DMO_Definition','DMO_Instance','DMO_Attribute','DMO_Instattr',
                'LFX_User','LFX_Group','LFX_User_Group','LFX_Language','DMO_Page','DMO_Gui','DMO_intl');
  include($_incMgr);
  
  $me=$_page->user;
  $_page=new DMO_Page(true);
  $_page->user=$me;
  $_page->noMenu=true;
  unset($me);
  
  $LFX_col_lt_lt='#fffdec';
  $LFX_col_lt='#fffad6';
  $LFX_col_lt_green='#B5F0AF';
  $LFX_col_md='#ffeebb';
  $LFX_col_dk='#ffd36c';
  $LFX_col_dk_dk='#6c4c00';
  
  # Let's try to find out what attribute we are using.  The attribute instance
  # id will be stored in a hidden field in the form.

  if ($insattr > 0) {
    $tmp=new DMO_Instattr($insattr);
    $name=$tmp->getAttr("text");
    $fileid = $tmp->getAttr("dmo_file_id");
  } else {
    $_page->addError("Instance attribute not found to attach to.<br/>\n");
    $_page->addError("Please create instance attribute before attaching file.\n");
    exit(0);
  }
  
  // Retrieve POST items if they are present
  $filesize = $_FILES['userfile']['size'];
  $filetype = $_FILES['userfile']['type'];
  $filetemp = $_FILES['userfile']['tmp_name'];
  $filename = $_FILES['userfile']['name'];
  $uploaddir = '/tmp/';
  $uploadfile = $uploaddir . $filename;

  //$base_dir = '/home/httpd/EH-Master/data/DMO_files';
  $base_dir=$_DMO_globals->getSetting('dataDir');
  if (!$base_dir) {
    $_page->addError("Please define the global setting dataDir");
    exit;
  }
  
  // We will use DMO functions to allow files to be uploaded, then attached
  // to instance attributes.   
  
  // Recursive mkdir that will ensure preceding directories are made
  function mkdirs($strPath, $mode)
  {
    if (is_dir($strPath)) return true;

    $pStrPath = dirname($strPath);
    if (!mkdirs($pStrPath, $mode)) return false;
    return mkdir($strPath);
  }
  
  function create_file ($fileid, $filepath, $filename) {
    /* $id is the id number used to define where the file is stored, taken from the database .
       $filepath is the original of the file (usually taken from a temporary directory)
       $filename is the original filename, which we use to get the extension
    */
    global $base_dir, $tmp,$_DMO_url;
    $s = (string) $fileid;
    $path = $base_dir;
    # We take the last two characters of the id as the filename
    $len = strlen($s);
    $file = $s[$len-2] . $s[$len-1];
  
    # Each directory should be distinguished from a file with the name
    # number.  Normally, files will have extensions -- inherited from 
    # the original filename.  If it doesn't have an extension, then
    # we might end up with a filename without an extension.  Our 
    # default will be to make the default .dat for any file without
    # an extension, when stored in the filesystem.
  
    $id = floor($fileid / 100);
    $id=$fileid;
    $part = '';
    while ($id >= 1) { // loop through until we consume all digits in id
      $digits = $id % 100;
      // pad leading zero when necessary
      if ($digits > 9) {
        $digits = (string) $digits;
      } else {
        $digits = '0' . (string) $digits; // prepend leading zero
      }
      $part = '/' . $digits . $part;
      $id = floor($id / 100);
    }
    $path .= $part;
    if (!file_exists($path))
      mkdirs($path,0770);
    // Now check the original filename, to look for an extension
    $ext = substr($filename, strrpos($filename, '.') + 1);
    if (strlen($ext) == 0) $ext = "txt";  // make this the default extension
    //echo "create using $path/$file.$ext<br>\n";
    if (!copy($filepath, "$path/$file.$ext")) {
      pecho(LFX_echo("failed to copy $filepath to $path/$file.$ext<br />\n"));
    } else {
      $tmp->setAttr("dmo_file_id",$fileid);
      $tmp->save();
      pecho(LFX_echo("<br/>File successfully attached <a target='_new' href='$_DMO_url/data/serve_file.php?id=$fileid'>here</a> to view the file"));
      pecho(LFX_echo($tmp->id . "<p><input type='submit' value='Close' name='Close' onClick='window.close()'/>"));
    }
    //$f = fopen ("$path/$file.dat", 'w');
    //fwrite ($f, $contents);
    //fclose ($f);
  }
  
  function id_to_filename ($id)
  {
        $base_dir = '/data/DMO_files';
  
  	$s = (string) $id;
  	$path = $base_dir;
          # We take the last two characters of the id as the filename
          $len = strlen($s);
          $file = $s[$len-2] . $s[$len-1];
  	
          # Now build the path components from right to left, to ensure
          # that we use two character names (not one) for the ones on right
          //$id = floor($id / 100);
          $part = '';
          while ($id >= 1) { // loop through until we consume all digits in id
            $digits = $id % 100;
            // pad leading zero when necessary
            if ($digits > 9) {
              $digits = (string) $digits;
            } else {
              $digits = '0' . (string) $digits; // prepend leading zero
            }
            $part = '/' . $digits . $part;
            $id = floor($id / 100);
          }
          $path .= $part;
  		
        //  echo "id_filename using $path/$file<br>\n";
  	return $path . '/' . $file;
  }
  
  function get_file ($id)
  {
  	$path = id_to_filename ($id);
  
  	if (!file_exists ($path))
  		return null;
  	else
  		return file_get_contents ($path);
  }
  
  function purge ($files)
  {
  	foreach ($files as $v)
  		unlink (id_to_filename ($v));
  }
  
if ($deleteattachment) {
  pecho(LFX_echo("We want to delete the attachment $insattr. (Not yet fully implemented.)"));
  // We do this in three steps -- first the file, then the table pointing
  // to the image, then the attribute in the instance pointing to the table entry
  // Initially, we will just remove the instance attribute pointer
  if ($insattr > 0) {
    $tmp=new DMO_Instattr($insattr);
    if ($tmp) {
      $tmp->setAttr("dmo_file_id",'0');
      $tmp->save();
      pecho(LFX_echo("Removed pointer to file from instance attribute $insattr.\n"));
    } else {
      pecho(LFX_echo("Can't find instance attribute #$insattr\n"));
    }
  } else {
    pecho(LFX_echo("Instance attribute is zero\n"));
  }
  exit(0);
}

if ($filename) {
  pecho(LFX_echo("<h3>Attaching File $filename to Attribute $name ($insattr)</h3>"));
  if ($filesize > 0) {
    if (move_uploaded_file($_FILES['userfile']['tmp_name'], $uploadfile)) {
      pecho(LFX_echo("File is valid, and was successfully uploaded. Size is $filesize and type is $filetype<br/>"));
      // Now we can insert this file information into the database, and
      // attach it to the attribute instance
      $req=new DMO_Object;
      $query="INSERT INTO dmo_file (dmo_file_name,dmo_file_mimetype,dmo_file_desc) VALUES ('$filename','$filetype','$description')";
      $req->query($query);
      $dmo_file_id = mysql_insert_id();
      //print "Insert ID was $dmo_file_id<br/>";
      create_file($dmo_file_id,$uploadfile,$filename);
    } else {
      pecho(LFX_echo("Possible file upload attack!  Here's some debugging info:\n"));
      pecho(vardump($_FILES));
    }
  } else {
    pecho(LFX_echo("File size was zero.  Ignoring the upload of $filename."));
  }
} else {
  
  // MAIN STARTS HERE
  // This form will allow the user to select a file and upload it, 
  // It will also allow the deletion of the currently attached file.
  // attaching it to the currently selected instance attribute.
  // This file will be stored in the file system, and will also
  // have meta information inserted into the database.
  
    //echo(LFX_sform("$PHP_SELF","POST"));
    pecho(LFX_echo("<script language='Javascript'>window.focus()</script>\n"));
    pecho(LFX_echo("This will add a new file attachment to the current instance attribute, or replace any existing one.<br/>"));
    pecho(LFX_echo("<div align='center'>"));
    pecho(LFX_echo("<form enctype='multipart/form-data' action='$PHP_SELF' method='post'>\n"));
    pecho(LFX_startTables());
    pecho(LFX_str());
    pecho(LFX_td(__D("File details"),'thead',NULL,2));
    pecho(LFX_etr());    
    // If we have a filename that exists, we can display info on it.
    if ($fileid > 0) {
      // we need to get the old filename
      $req=new DMO_Definition;
      $sql="SELECT * FROM dmo_file WHERE dmo_file_id = $fileid";
      $req->query($sql);
      $req->db->next_record();
      $filename = $req->db->f(1);
      $filetype = $req->db->f(2);
      $filedesc = $req->db->f(5);
      $ext = substr($filename, strrpos($filename, '.') + 1);
      if (strlen($ext) == 0) $ext = "txt";  // make this the default extension
      switch($filetype) {
        case 'image/pjpeg':
        case 'image/jpeg':
        case 'image/gif':
        case 'image/png':
          $img = "<img align='right' width='150' border=0 src='" . id_to_filename($fileid) . ".$ext' />";
      }
      pecho(LFX_str());
      pecho(LFX_echo("<td style='text-align: left' align='left'>$img <b>Current attachment:</b> <a href='" . id_to_filename($fileid) . ".$ext'>$filename</a> ($filetype) <br/><b>Description:</b> $filedesc<hr/><input type='checkbox' name='deleteattachment'/>Click here to delete the attachment.</td>\n"));
      pecho(LFX_etr());
    }
    pecho(LFX_str());
    pecho(LFX_rth("Please choose a file"));
    pecho(LFX_td("<input type='file' name='userfile' size='20' /></td>\n"));
    pecho(LFX_estr());
    pecho(LFX_rth("Description"));
    pecho(LFX_td("<textarea name='description' rows='5' cols='40'>$filedesc</textarea>"));
    pecho(LFX_estr());
    pecho(LFX_std(NULL,NULL,2));
    pecho(LFX_echo("<input type='hidden' name='MAX_FILE_SIZE' value='20000000' />\n"));
    pecho(LFX_echo("<input type='hidden' name='insattr' value='$insattr'>\n"));
    pecho(LFX_echo("<div align='center'><input type='submit' name='save' value='Send file' />\n"));
    pecho(LFX_echo("<input onClick='window.close()' type='Reset' name='Close' value='Close' /></div>"));
    pecho(LFX_etd());
    pecho(LFX_etr());
  
    pecho(LFX_endTables());
    pecho(LFX_eform());
    pecho(LFX_echo("</div>"));
  
  }
  
  $_page->show();
?>
