<?

  /**
  * DMO_menu class
  * Bogdan Stancescu <bogdan@lanifex.com>, January 2003
  * $Id: DMO_menu.php,v 1.5 2003/05/06 10:49:49 bogdan Exp $
  * @package DMO
  */
    /**
    * The DMO_Menu class used to manage the main DMO menu
    */
  class DMO_Menu
  {
    var $children=NULL;
    var $focused=false;
    var $id=0;
    var $fullId='0';
    var $title='[no title]';
    var $url='#';
    var $tdTag='';
    var $tip='';
  
      /**
      * The constructor. You can pass a menu description to the constructor
      * in order to build the whole menu structure. You could alternatively
      * use the class methods for the same purpose, but the recommended way
      * is to pass the menu description to the constructor.
      *
      * Each entry in the menu description array must have the following structure:
      * "title" (string) => the title of the menu entry to be displayed
      * "url" (string) => the URL to link the respective entry to
      * "tip" (string) => the "tip" to be displayed when the user hovers over the link
      * "focused" (boolean) => [optional] if true, the respective item will be focused
      * "children" (array) => indexed array of entries like this one, for submenu items.
      *
      * @param array $menuDesc a menu description
      */
    function DMO_Menu($menuDesc=NULL)
    {
      global $_DMO_images;
      $this->tdTag="<td style=\"text-align: center; background: url('$_DMO_images/menu_bar.jpg')\" width='1%'>";
      if (is_array($menuDesc)) {
        $this->title='DMOMenuRoot';
        $this->loadArray($menuDesc);
      }
    }
    
      /**
      * The function used to load the menu entries for a menu item from a menu
      * description array. Please check {@link DMO_Menu} for the array structure.
      *
      * @param array $menuDesc a menu description
      * @param boolean $preserve if true, child menus will NOT be deleted prior to loading the description
      */
    function loadArray($menuDesc, $preserve=false)
    {
      if (!$preserve) {
        unset($this->children);
      }
      while(list(,$menuEntry)=each($menuDesc)) {
        $tmp=new DMO_Menu();
        if ($menuEntry['title']) {
          $tmp->title=$menuEntry['title'];
        }
        if ($menuEntry['url']) {
          $tmp->url=$menuEntry['url'];
        }
        $tmp->tip=$menuEntry['tip'];
        if ($menuEntry['focused']) {
          $tmp->focused=true;
        }
        $this->addChild($tmp);
        if (is_array($menuEntry['children'])) {
          $this->children[count($this->children)-1]->loadArray($menuEntry['children']);
        }
        unset($tmp);
      }
    }
    
      /**
      * Method to add a DMO_Menu object as a child to this object
      *
      * @param object $menu a DMO_Menu object to add
      * @return string the full path to the new child
      */
    function addChild(&$menu)
    {
      $menu->id=count($this->children);
      $menu->fullId=$this->fullId.'|'.$menu->id;
      $this->children[]=$menu;
      return($this->id.'|'.$menu->id);
    }
    
      /**
      * Sets focus on this menu item
      */
    function setFocus()
    {
      $this->focused=true;
    }
    
      /**
      * Returns the full path to the focused item BELOW this one.
      * Please make sure you only run this on the root of the menu - checking
      * is only performed among the children of the current entry.
      *
      * @return mixed the full path to the focused item or false if none
      */
    function getFocus()
    {
      if ($this->focused) {
        return($this->fullId);
      }
      for($i=0;$i<count($this->children);$i++) {
        if ($tmp=$this->children[$i]->getFocus()) {
          return($tmp);
        }
      }
      return(false);
    }
    
      /**
      * Returns a reference to the menu item specified by path.
      *
      * NOTE ON REFERENCES
      * Although the value is returned by reference, you still have
      * to call this method like $foo=&$bar->getById($myPath) in order
      * to retrieve a reference to the respective object, as opposed to
      * a copy of it. If it's a copy you want, then simply call the method
      * without the ampersand.
      *
      * The id path is simply the zero-based index of every child in the
      * children array sequence path to the desired element separated by vertical
      * bars. For instance, "0|1|6" would be the seventh child of the second child
      * of the first child of the root.
      *
      * Since the menus will most certainly move around, you should not hardcode
      * these values, but rather use {@link getByTitle} with a single value (not path).
      * If that's not possible (if there are duplicates in the titles) then use
      * getByTitle with a path.
      *
      * @param string $id the path to the menu entry to retrieve
      * @return mixed the respective object or false if not found
      */
    function &getById($id)
    {
      if (substr($id,0,strlen($this->fullId))!=$this->fullId) {
        return(false);
      }
      $id=substr($id,strlen($this->fullId)+1);
      $ids=explode('|',$id);
      $tmp=&$this;
      for($i=0;$i<count($ids);$i++) {
        if (!is_object($tmp)) {
          return(false);
        }
        $tmp=&$tmp->children[$ids[$i]];
      }
      return($tmp);
    }
    
      /**
      * Returns a reference to the menu item specified by name.
      *
      * If there are duplicate menu item titles in the menu structure
      * you can pass a name path to this function. Name paths are the same
      * as id paths (documented at the {@link getById} method), only they
      * contain menu titles separated by vertical bars. Please note only the
      * English text is used for paths, so make sure you don't translate the
      * titles before building the path.
      *
      * Please read the note on references at {@link getById}
      *
      * @param string $title either a simple title or the full path to a title
      * @param string $path if you don't call this method on the root element,
      * then pass the full path to the current object here (but exclude the current object)
      * @return mixed the reference to the object or false if not found
      */
    function &getByTitle($title, $path='')
    {
      if ($this->title==$title) {
        return($this);
      }
      $titles=explode('|','DMOMenuRoot|'.$title);
      if ((count($titles)>1) &&
        ($this->title==array_pop($titles)) &&
        ($path==implode('|',$titles))) {
        return($this);
      }
      for($i=0;$i<count($this->children);$i++) {
        $tmp=&$this->children[$i]->getByTitle($title,$path.($path?'|':'').$this->title);
        if ($tmp!==false) {
          return($tmp);
        }
      }
      return(false);
    }

      /**
      * Renders a single menu cell - you shouldn't need to call this manually.
      *
      * @return string the rendered cell (one HTML table cell)
      */
    function renderCell()
    {
      $title=str_replace(' ','&nbsp;',htmlspecialchars($this->title,ENT_QUOTES));
      $result=$title;
      $tip=htmlspecialchars($this->tip,ENT_QUOTES);
      if ($this->focused) {
        $id='menu_focused';
      } elseif($this->getFocus()) {
        $id='menu_focused_path';
      } else {
        $id='menu';
      }
      $result="&nbsp;<a href='{$this->url}' title='$tip' id='$id'>$result</a>&nbsp;";
      $result="{$this->tdTag}$result</td>";
      return(LFX_echo($result));
    }
    
      /**
      * Renders the children of the current menu entry - you shouldn't need
      * to call this manually.
      *
      * @return string the rendered children (one HTML table).
      */
    function renderChildren()
    {
      global $_DMO_images;
      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      global $_DMO_images;
      $result.="{$this->tdTag}<img src='$_DMO_images/spacer.gif' alt='' height='23' width='2'></td>";
      for($i=0;$i<count($this->children);$i++) {
        $result.=$this->children[$i]->renderCell();
        if ($i<count($this->children)-1) {
          $result.=LFX_echo($this->tdTag."<img src='$_DMO_images/menu_sep.gif' alt='|'></td>");
        }
      }
      $result.=LFX_etr();
      $result.=LFX_echo('</table>');
      return($result);
    }
    
      /**
      * This is really private, and you should NEVER call this directly.
      *
      * Due to the particular nature of the DMO menu, the top most menu level
      * is significantly different from the rest in terms of HTML tags, because
      * it's made of three cells by two, as opposed to two by two as the other
      * levels are rendered (an extra cell is needed to render the shadow below
      * the DMO header bar). This method renders the "other" levels (the child
      * menu levels) while {@link render} performs the root level rendering.
      * {@link render} also calls this method, which is why you should never
      * need to call this method directly.
      *
      * @return string the current level and the levels below up to one level below
      * the focused item, if any - otherwise only the current level - rendered in HTML
      */
    function renderChildLevel()
    {
      global $_DMO_images;
      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      $result.=LFX_echo('<td>');
      $result.=$this->renderChildren();
      $result.=LFX_echo('</td>');
      $result.=LFX_td("<img src='$_DMO_images/menu_end.jpg' alt=''>");
      $result.=LFX_etr();

      $result.=LFX_str();
      if (($focused=$this->getFocus()) && (!$this->focused)) {
        $focused=substr($focused,strlen($this->fullId)+1);
        list($focusChild)=explode('|',$focused);
        if (!count($this->children[$focusChild]->children)) {
          $result.=LFX_echo("<td background='$_DMO_images/menu_empty_thin.jpg' width='100%'><img src='$_DMO_images/spacer.gif' height='4' alt=''></td>");
        } else {
          $result.=LFX_echo('<td>');
          $result.=$this->children[$focusChild]->renderChildLevel();
          $result.=LFX_echo('</td>');
        }
      } else {
        $result.=LFX_echo("<td background='$_DMO_images/menu_empty_thin.jpg' width='100%'><img src='$_DMO_images/spacer.gif' height='4' alt=''></td>");
      }
      $result.=LFX_echo("<td valign='top'><img src='$_DMO_images/menu_end_empty_thin.jpg' alt=''></td>");
      $result.=LFX_etr();

      $result.=LFX_echo('</table>');
      return($result);
    }
    
      /**
      * The method to render the menu item and all its children.
      *
      * If interested in technical details, you can check the note on {@link renderChildLevel}
      *
      * @return string the whole menu rendered in HTML, with respect to the focused item.
      */
    function render()
    {
      global $_DMO_images;
      $result=LFX_echo('<!-- Rendering menu -->');
      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      $result.=LFX_echo('<td>');

      $result.=LFX_echo('<table border=\'0\' cellspacing=\'0\' cellpadding=\'0\' width=\'100%\'>');
      $result.=LFX_str();
      $result.=LFX_echo('<td>');
      $result.=$this->renderChildren();
      $result.=LFX_echo('</td>');
      $result.=LFX_etr();
      $result.=LFX_echo('</table>');

      $result.=LFX_echo('</td>');
      $result.=LFX_td("<img src='$_DMO_images/menu_end.jpg' alt=''>");
      $result.=LFX_echo("<td background='$_DMO_images/menu_empty.jpg' width='100%'>&nbsp;</td>");
      $result.=LFX_etr();

      $result.=LFX_str();
      if (($focused=$this->getFocus())) {
        $focused=substr($focused,strlen($this->fullId)+1);
        list($focusChild)=explode('|',$focused);
        if (!count($this->children[$focusChild]->children)) {
          $result.=LFX_echo("<td background='$_DMO_images/menu_empty_thin.jpg'><img src='$_DMO_images/spacer.gif' height='3' alt=''></td>");
        } else {
          $result.=LFX_echo('<td>');
          $result.=$this->children[$focusChild]->renderChildLevel();
          $result.=LFX_echo('</td>');
        }
      } else {
        $result.=LFX_echo("<td background='$_DMO_images/menu_empty_thin.jpg'>&nbsp;</td>");
      }
      // Might be rendered in the middle of the cell in older agents (non-CSS) - oh well...
      $result.=LFX_echo("<td style='align:left; vertical-align:top'><img src='$_DMO_images/menu_end_empty_thin.jpg' alt=''></td>");
      $result.=LFX_echo("<td width='100%'><img src='$_DMO_images/spacer.gif' height='3' alt=''></td>");
      $result.=LFX_etr();
      $result.=LFX_echo('</table>');

      $result.=LFX_echo('<!-- Finished menu -->');

      return($result);
    }
    
      /**
      * Unfocuses whatever focused item there may be below this menu item.
      *
      * @return mixed path to the previously focused item if there was any, false otherwise
      */
    function unfocus()
    {
      if($focus=$this->getFocus()) {
        $focused=&$this->getById($focus);
        $focused->focused=false;
        return($focus);
      }
      return(false);
    }
    
      /**
      * Focuses an item by title.
      *
      * See comment on {@link getByName} for more information on how to pass titles.
      *
      * Any previously focused item in the menu is naturally unfocused before setting
      * the focus on the desired item - but bear in mind that you can end up with
      * multiple items focused it you call this method on any other item but the menu root.
      *
      * @return boolean true on success or false on failure
      */
    function focus($title)
    {
      $this->unfocus();
      $tmp=&$this->getByTitle($title);
      if ($tmp!==false) {
        $tmp->focused=true;
        return(true);
      }
      return(false);
    }

  }

?>
