<?

  /**
  * Generic network object DMO instance
  * Bogdan Stancescu <bogdan@lanifex.com>, May 2005
  * $Id: DMO_network_object_instance.php,v 1.5 2005/09/16 10:08:27 bogdan Exp $
  * @package DMO
  */

  $result=array('DMO_Instance');
  if (!_checkIncludes($result)) {
    return($result);
  }

    /**
    * This class is a descendant of DMO_Instance, instantiated
    * by DMO_Instance::instanceFactory().
    */
  class DMO_Network_Object_Instance extends DMO_Instance
  {
      /**
      * This method tries to return the ID of the sensor associated with this
      * instance.
      *
      * It first tries to look at the parents of this instance recursively,
      * looking for a Sensor instance; if it finds one, it tries to look up
      * its "Sensor ID" attribute and return its value.
      *
      * If any of the steps above fails, it falls back to looking up the
      * "ScanEnable" attribute of the current object, and returns its numeric
      * value.
      *
      * If both attempts fail, it returns false.
      *
      * @return mixed false on error or the sensor ID on success
      */
    function getSensorID()
    {
      if (!LFX_include_class('DMO_Attribute','DMO_InstAttr')) {
        return(false);
      }
      $cObj=$this;
      while(
        ($cObj=$cObj->getObject('parent')) &&
        ($pDef=$cObj->getObject('definition')) &&
        (strtolower($pDef->getAttr('name'))!='sensor')
      );
      
      if ($parent=$cObj) {
        $att=new DMO_Attribute;
        $attrSensorID = $att->byNameID('Sensor ID');
        if ($attrSensorID) {
          $ia=new DMO_InstAttr;
          $ia=$ia->search(array('instance','attribute'),array($parent->id,$attrSensorID));
          if ($ia) {
            return($ia[0]->getAttr('text'));
          }
        }
      }
      $att=new DMO_Attribute;
      $attrScanEnable=$att->byNameID('ScanEnable');
      if ($attrScanEnable) {
        $ia=new DMO_InstAttr;
        $ia=$ia->search(array('instance','attribute'),array($this->id,$attrScanEnable));
        if ($ia) {
          return($ia[0]->getAttr('value'));
        }
      }
      return(false);
    }
    
      /**
      * This function uses method {@link getSensorID} to find the sensor ID,
      * and instantiates an EH_Sensor object with the respective ID.
      *
      * @return mixed false on error or an instantiated EH_Sensor object on
      *   success.
      */
    function getSensor()
    {
      if (!LFX_include_class('EH_Sensor')) {
        return(false);
      }
      if (($sensorID=$this->getSensorID())===false) {
        return(false);
      }
      $sensor=new EH_Sensor($sensorID);
      return($sensor);
    }
    
    function getScanPattern()
    {
      if (!LFX_include_class('DMO_Attribute','DMO_InstAttr')) {
        return(false);
      }
      $att=new DMO_Attribute;
      $attScanPattern=$att->byNameID('ScanPattern');
      if (!$attScanPattern) {
        return(false);
      }
      $ia=new DMO_InstAttr;
      $ia=$ia->search(array('instance','attribute'),array($this->id,$attScanPattern));
      if (!$ia) {
        return(NULL);
      }
      return($ia[0]->getAttr('text'));
    }
    
    function getLastScan($type)
    {
      if (!LFX_include_class('EH_scan_data')) {
        return(false);
      }
      $sd=new EH_Scan_Data;
      $sd=$sd->search(array('instance','scan_type'),array($this->id,$type));
      if (!$sd) {
        return(NULL);
      }
      return($sd[0]);
    }
    
    // $timestamp is used for debugging purposes only!
    function shouldScan($type,$timestamp=NULL)
    {
      // First, let's determine whether we should use a pattern in the first place
      if (!($pattern=$this->getScanPattern())) {
        $varName=$type."_scan_hour";
        global $_EH_globals;
        if (!($hour=$_EH_globals->getSetting($varName))) {
          // No pattern, no global -- no luck, we use hardcoded "3 am, each day".
          $pattern='3 *';
        } else {
          $pattern="$hour *";
        }
      }
      /*
      pecho(LFX_startTables());
      pecho(LFX_str());
      pecho(LFX_rth("Pattern"));
      pecho(LFX_td($pattern));
      */
      // Ok, we now have the pattern in $pattern. Let's see when we successfully
      // ran the last time.
      $sd=$this->getLastScan($type);
      if (!$sd) {
        /*
          Ok, here's the deal: we never ran before, and don't yet know whether
          to run now or not. You'll say "ok, match that pattern to the current
          time and let's find out!" Not so fast, cowboy! What if we were supposed
          to run an hour ago, but the scan script was busy? And consider this can
          happen again the next time this particular scan is due! No, we'll scan
          unconditionally right now, in order to create a precedent. After that,
          we'll have a reference point to look at, and decide whether to run or
          not, using the usual algorithm.
        */
        return(true);
      }
      
      // All right, we have the pattern and we have the last scan, let's do some
      // magic!
      $last_scan_date=$sd->getAttr('last_scan_date');
      /*
      pecho(LFX_estr());
      pecho(LFX_rth("Last scan date"));
      pecho(LFX_td('<pre>'.date('r',$last_scan_date).'</pre>'));
      */
      $last_scan_array=getdate($last_scan_date);
      $last_scan_crontab="{$last_scan_array['hours']} {$last_scan_array['wday']}";
      //decho("Last scan date (getdate): ".vardump($last_scan_array));
      $next_date_crontab=$sd->next_match(
        $last_scan_crontab,
        $pattern,
        '24 7'
      );
      list($next_date_crontab_array['hours'],$next_date_crontab_array['wday'])=explode(' ',$next_date_crontab);
      $next_date_real_array=$last_scan_array;
      $next_date_real_array['mday']+=$next_date_crontab_array['wday']-$last_scan_array['wday'];
      $next_date_real_array['hours']=$next_date_crontab_array['hours'];
      $next_date_real_array['minutes']=0;
      $next_date_real_array['seconds']=0;

      //decho("Next date (getdate): ".vardump($next_date_real_array));
      $next_date_real=mktime(
        $next_date_real_array['hours'],
        $next_date_real_array['minutes'],
        $next_date_real_array['seconds'],
        $next_date_real_array['mon'],
        $next_date_real_array['mday'],
        $next_date_real_array['year']
      );
      /*
      pecho(LFX_estr());
      pecho(LFX_rth("Next date"));
      pecho(LFX_td('<pre>'.date('r',$next_date_real).'</pre>'));
      */

      while($next_date_real<$last_scan_date) {
        $next_date_real+=7*24*3600;
        /*
        pecho(LFX_estr());
        pecho(LFX_rth("Next date (after wrapping)"));
        pecho(LFX_td('<pre>'.date('r',$next_date_real).'</pre>'));
        */
      }

      if (!$timestamp) {
        $timestamp=time();
      }
      /*
      pecho(LFX_estr());
      pecho(LFX_rth("Right now"));
      pecho(LFX_td('<pre>'.date('r').'</pre>'));
      pecho(LFX_etr());
      pecho(LFX_endTables());
      */
      return($next_date_real<$timestamp);
    }
    
    // $timestamp can be used legitimately here,
    // but typically you shouldn't need to
    function saveScanData($type,$timestamp=NULL)
    {
      if (!$timestamp) {
        $timestamp=time();
      }
      if ($sd=$this->getLastScan($type)) {
        $sd->setAttr('last_scan_date',$timestamp);
        return($sd->save());
      }
      if (!LFX_include_class('EH_Scan_Data')) {
        return(false);
      }
      $sd=new EH_Scan_Data;
      $sd->setAttrs(array(
        'scan_type'=>$type,
        'instance'=>$this->id,
        'last_scan_date'=>$timestamp
      ));
      return($sd->save());
    }
  }
?>
