<?php

  /**
  * The DMO report class
  * Bogdan Stancescu <bogdan@lanifex.com>, November 2003
  * $Id: DMO_report.php,v 1.6 2003/11/05 13:31:52 bogdan Exp $
  * @package EH
  */

$result=array('DMO_Object');
if (!_checkIncludes($result)) {
  return($result);
}

class DMO_report extends DMO_Object
{

  /**
  * Constructor
  *
  */

  function DMO_report() {
    $this->DMO_Object();
  }

  /*
  * This is called just once when the report is updated for the first time,
  * to determine which is the earliest date when data is available to start
  * creating the reports.
  * Returning 0 means there is no object to report, delay asking for earliest.
  */


  function getEarliest() {
    $this->query("SELECT unix_timestamp(min(instance_date)) earliest FROM instance");
    if (!$this->db->next_record()) {
      return(0);
    }
    return($this->db->f('earliest'));
  }

  /**
  * This is the data collection function
  *
  * This function is called by EH_report_wrapper::updateReport. In this function, we
  * execute the SQL queries. If we need to execute more than one query (which usually is the case)
  * then we create other functions for these queries and call them from within this function.
  * Then, we stick all results in array $mydata and return it. The wrapper class takes care from that
  * point on.
  *
  * @parameter unix timestamp $start and unix timestamp $end
  * @return array $mydata contining data for the report
  */

  function gatherData($start,$end) {
    $result=array();
    $this->query("SELECT COUNT(*) c FROM instance WHERE instance_date>='".
      $this->sqlDate($start)."' AND instance_date<'".$this->sqlDate($end)."'");
    $this->db->next_record();
    $result['new']=$this->db->f('c');
    $this->query("SELECT COUNT(*) c FROM instance WHERE instance_date<'".
      $this->sqlDate($end)."'");
    $this->db->next_record();
    $result['count']=$this->db->f('c');
    $result['del']=0;
    $this->query("SELECT COUNT(*) c FROM instance WHERE lastedit>='".
      $this->sqlDate($start)."' AND lastedit<'".$this->sqlDate($end)."'");
    $this->db->next_record();
    $result['edits']=$this->db->f('c');
    $result['definitions']=array(array());
    $reportDefs=$this->getReportDefs();
    for ($i=0;$i<count($reportDefs);$i++) {
      $result['definitions'][$reportDefs[$i]]=$this->defGatherData($start,$end,$reportDefs[$i]);
    }
    return($result);
  }
  
  function addData($old,$new) {
    $result=array();
    $result['count']=$new['count'];
    $result['del']=abs($new['count']-$new['new']-$old['count']);
    $result['new']=$old['new']+$new['new'];
    /*
      INTRODUCTION
      (skip if you only want to know HOW this works, this explains WHY)

      Ok, with the edits it's a bit trickier: we don't know
      *which* instances have been edited in the two periods, $old and $new.
      We only know how many edits there have been. The same instance
      may have been edited twice in the two periods, so we can't just
      add the edit count. Some of the instances edited in $old may have
      been deleted since, so we can't perform a counting query either.

      Please note we'll always count the number of instances which have
      been edited, not the actual edits. An instance which has been edited
      three times in a report period will count as one. Two instances edited
      5,000 times in one report period will count as two. That's because
      we can't tell how many times an instance has been edited during one
      reporting period -- and we don't really care, either, the time periods
      are small enough that we don't care if someone's clumsy and edited
      an instance fifty times within an hour, we're much more interested
      if someone's agile and edited fifty instances in an hour. This makes
      things even trickier when adding edit counts, because simply adding
      edit counts between reporting periods would result in counting
      apples on one hand, and then adding them as sheep on the other.
      We'd end up reporting applosheep, and since the word doesn't exist
      we can't do it.

      So all we can do is infer an edit count range from the data we have.

      IMPLEMENTATION

      (1) If there are no edits within either $new or $old, we can safely
      say only the other one applies, so no worries there. This is the
      only clear case in the code below. The rest of the logic explained
      here is implemented in a too compact manner to make comments in the
      actual code.
      (2) If changes have been made both in $old and in $new, then we can
      say that the number of edits is at most the total number of edits
      and at least the HIGHEST between the number of edits in $old and $new.
      In that case, we say the result is "min-max", and actually return
      a string of that form, where min and max are integers, and the result
      is a string containing the two values with a dash between them.
      (3) But what if either of $old or $new -- or indeed both -- already
      contain a "min-max" value? In that case, all we know is that:
        - the minimum number of edits can't be smaller than the highest
          value of the two "min" values for the total period;
        - the maximum number of edits can't be larger than the sum of the
          two "max" values for the total period.
        - the maximum number of edits can't be larger than the total number
          of instances at time $new - remember we count edited instances,
          not instance edits.
        - since min(x,x)=max(x,x)=x, we can safely transform a clear value
          (not min-max) to min-max, by providing the same value for min
          and max, and apply the algorithm above even if one of the $new
          and $old is using clear values and the other one is using min-max
          values.
    */

    // Start (1)
    if ($old['edits']===0) {
      $result['edits']=$new['edits'];
    } elseif ($new['edits']===0) {
      $result['edits']=$old['edits'];
    // End (1)
    } else {
      if (strpos($old['edits'],'-')) {
        list($minold,$maxold)=explode('-',$old['edits']);
      } else {
        $minold=$maxold=$old['edits'];
      }
      if (strpos($new['edits'],'-')) {
        list($minnew,$maxnew)=explode('-',$new['edits']);
      } else {
        $minnew=$maxnew=$new['edits'];
      }
      $minres=max($minold,$minnew);
      $maxres=min($maxold+$maxnew,$new['count']);
      if ($minres==$maxres) {
        $result['edits']=$minres;
      } else {
        $result['edits']="$minres-$maxres";
      }
    }
    
    $reportDefs=$this->getReportDefs();
    $result['definitions']=array(array());
    for ($i=0;$i<count($reportDefs);$i++) {
      $result['definitions'][$reportDefs[$i]]['count']=$new['definitions'][$reportDefs[$i]]['count'];
      $result['definitions'][$reportDefs[$i]]['del']=$new['definitions'][$reportDefs[$i]]['count']-$new['definitions'][$reportDefs[$i]]['new']-$old['definitions'][$reportDefs[$i]]['count'];
      $result['definitions'][$reportDefs[$i]]['new']=$old['definitions'][$reportDefs[$i]]['new']+$new['definitions'][$reportDefs[$i]]['new'];

      // Start (1)
      if ($old['definitions'][$reportDefs[$i]]['edits']===0) {
        $result['definitions'][$reportDefs[$i]]['edits']=$new['definitions'][$reportDefs[$i]]['edits'];
      } elseif ($new['definitions'][$reportDefs[$i]]['edits']===0) {
        $result['definitions'][$reportDefs[$i]]['edits']=$old['definitions'][$reportDefs[$i]]['edits'];
      // End (1)
      } else {
        if (strpos($old['definitions'][$reportDefs[$i]]['edits'],'-')) {
          list($minold,$maxold)=explode('-',$old['definitions'][$reportDefs[$i]]['edits']);
        } else {
          $minold=$maxold=$old['definitions'][$reportDefs[$i]]['edits'];
        }
        if (strpos($new['definitions'][$reportDefs[$i]]['edits'],'-')) {
          list($minnew,$maxnew)=explode('-',$new['definitions'][$reportDefs[$i]]['edits']);
        } else {
          $minnew=$maxnew=$new['definitions'][$reportDefs[$i]]['edits'];
        }
        $minres=max($minold,$minnew);
        $maxres=min($maxold+$maxnew,$new['definitions'][$reportDefs[$i]]['count']);
        if ($minres==$maxres) {
          $result['definitions'][$reportDefs[$i]]['edits']=$minres;
        } else {
          $result['definitions'][$reportDefs[$i]]['edits']="$minres-$maxres";
        }
      }      
    }    
    return($result);
  }

  function renderReport($data) {
    pecho(LFX_echo("<p>"));
    pecho(LFX_echo("Total instances count: {$data['count']}<br />"));
    if ($data['new']) {
      pecho(LFX_echo("New instances: <FONT color='red'>{$data['new']}</FONT><br />"));
    } else {
      pecho(LFX_echo("New instances: {$data['new']}<br />"));
    }
    pecho(LFX_echo("Deleted instances: {$data['del']}<br />"));
    pecho(LFX_echo("Edited instances: {$data['edits']}"));
    pecho(LFX_echo("</p>"));
    $reportDefs=$this->getReportDefs();
    $tmp=$data['definitions'];
    foreach ($tmp as $id => $sdata) {
      $sarray[$id]=$sdata['edits'];
    }
    unset($tmp);
    asort($sarray);
    reset($sarray);
    while (list($defID,)=@each($sarray)) {
      $defData=$data['definitions'][$defID];
      if ($defID && in_array($defID,$reportDefs) && $data['definitions'][$defID]['count']) {
        LFX_include('DMO_Definition');
        $def=new DMO_Definition($defID);
        $defName=$def->getAttr('name');
        pecho(LFX_echo("<p>"));
        pecho(LFX_echo('Report for definition '.$defName.'<br />'));
        pecho(LFX_echo("Total instances count: {$data['definitions'][$defID]['count']}<br />"));
        if ($data['definitions'][$defID]['new']) {
          pecho(LFX_echo("New instances: <FONT color='red'>{$data['definitions'][$defID]['new']}</FONT><br />"));
        } else { 
          pecho(LFX_echo("New instances: {$data['definitions'][$defID]['new']}<br />"));
        }
        pecho(LFX_echo("Deleted instances: {$data['definitions'][$defID]['del']}<br />"));
        pecho(LFX_echo("Edited instances: {$data['definitions'][$defID]['edits']}"));      
        pecho(LFX_echo("</p>"));
      }
    }
    return($_page);
  }
  
  function defGatherData($start,$end,$def)
  {
    $result=array();
    $this->query("SELECT COUNT(*) c FROM instance WHERE instance_date>='".
      $this->sqlDate($start)."' AND instance_date<'".$this->sqlDate($end)."' AND definition_ID='$def'");
    $this->db->next_record();
    $result['new']=$this->db->f('c');
    $this->query("SELECT COUNT(*) c FROM instance WHERE instance_date<'".
      $this->sqlDate($end)."' AND definition_ID='$def'");
    $this->db->next_record();
    $result['count']=$this->db->f('c');
    $result['del']=0;
    $this->query("SELECT COUNT(*) c FROM instance WHERE lastedit>='".
      $this->sqlDate($start)."' AND lastedit<'".$this->sqlDate($end)."' AND definition_ID='$def'");
    $this->db->next_record();
    $result['edits']=$this->db->f('c');
    return($result);    
  }
  
  function getReportDefs()
  {
    GLOBAL $_DMO_globals;
    
    LFX_include('DMO_Definition');    
    $srcDef=new DMO_Definition;    
    if ($_DMO_globals->getSetting('Report All')) {
      return($srcDef->searchId());
    }
    return($srcDef->searchId('enable_report',1));
  }    
}
?>